﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace PatchSto.Building
{
	public class CreateUnitButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		string typename = string.Empty;
		Unit.CommonUnitParameter parameter = null;

		public CreateUnitButton(string name, Type type, int x, int y)
		{
			this.typename = type.FullName;

			Position = Tuple.Create((ButtonPosition)x, (ButtonPosition)y);
			ButtonImagePath = "PatchSto/Image/Button/Unit/" + type.FullName.Split('.').Last();
			ButtonName = name;

			Kind = ButtonKind.CreateUnit;

			CreateUnitEvent = (building, isAppend) =>
			{
				var param = GetParameter(building.Faction.World);
				
				if (building.Faction.GetResource(0) < param.Costs[0]) return;

				var unit = Utl.PutUnitAroundBuilding(building.Faction.World, building.Faction, param.GetType().FullName, building);
				if (unit != null)
				{
					building.Faction.DecResource(0, param.Costs[0]);
					param.UnitCountMax++;
				}
			};

			MouseOverTextFunc = (faction) =>
				{
					var param = GetParameter(faction.World);

					int count = faction.World.UnitManager.GetUnits(_ => _.Parameter == param).Count();
					int countMax = param.UnitCountMax;
					int level = param.Level + 1;
					int cost = param.Costs[0];
					var skills = param.Skills.Take(level).Where(_ => _.Type != SkillType.None);

					var exec = faction.GetResource(0) > param.Costs[0];

					var costText = new ColorTextLine(new ColorText("コスト : " + cost, exec ? new Color(1, 1, 1) : new Color(1, 0, 0))); 

					var countText = new ColorTextLine("(現在の数/最大数) " + count.ToString("000") + " / " + countMax.ToString("000"));
					
					var levelText = new ColorTextLine("レベル : " + level.ToString("00"));

					ColorTextLine typeText = null;
					if (param.UnitType_ == UnitType.ShortRange)
					{
						typeText = new ColorTextLine("近距離ユニット");
					}

					if (param.UnitType_ == UnitType.MiddleRange)
					{
						typeText = new ColorTextLine("中距離ユニット");
					}

					if (param.UnitType_ == UnitType.LongRange)
					{
						typeText = new ColorTextLine("遠距離ユニット");
					}

					var skillStr = string.Empty;
					foreach(var s in skills)
					{
						skillStr += SkillUtls.GetName(s) + " ";
					}

					var skillText = new ColorTextLine("スキル : " + skillStr);
					return new ColorTextLine[] { costText, countText, typeText, levelText, skillStr };
				};
		
			CanExecute = (faction, _, building) =>
			{
				var param = GetParameter(faction.World);

				return new ExecutableInfo(faction.GetResource(0) >= param.Costs[0], new ColorTextLine("リソースが足りません。"));
			};
		}

		Unit.CommonUnitParameter GetParameter(thrts.IWorld world)
		{
			if (parameter != null) return parameter;
			parameter = world.Factions[0].GetUnitParameter(typename) as Unit.CommonUnitParameter;
			return parameter;
		}
	}

	public class LevelUpButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		string typename = string.Empty;
		Unit.CommonUnitParameter parameter = null;

		public LevelUpButton(string name, Type type, int x, int y)
		{
			this.typename = type.FullName;

			Position = Tuple.Create((ButtonPosition)x, (ButtonPosition)y);
			ButtonImagePath = "PatchSto/Image/Button/Unit/" + type.FullName.Split('.').Last();
			ButtonName = name + " のレベルアップ";

			Kind = ButtonKind.Normal;

			NormalButtonEvent = (building, isAppend) =>
				{
					var param = GetParameter(building.Faction.World);

					if (building.Faction.GetResource(0) < param.GetNextCost()) return;

					building.Faction.DecResource(0, param.GetNextCost());

					param.LevelUp();
				};

			MouseOverTextFunc = (faction) =>
				{
					var param = GetParameter(faction.World);
					int level = param.Level + 1;
					int cost = param.GetNextCost();

					var exec = faction.GetResource(0) >= param.GetNextCost();

					var costText = new ColorTextLine("コスト : " + (cost < int.MaxValue / 10 ? cost.ToString() : "∞"));

					var countText = new ColorTextLine(level + " -> " + (level + 1));

					return new ColorTextLine[] { costText, countText };
				};

			CanExecute = (faction, _, building) =>
			{
				var param = GetParameter(faction.World);
				if (param.Level + 1 >= Unit.CommonUnitParameter.MaxLevel)
				{
					return new ExecutableInfo(false, new ColorTextLine("最大レベルです。"));
				}

				return new ExecutableInfo(faction.GetResource(0) >= param.GetNextCost(), new ColorTextLine("リソースが足りません。"));
			};
		}

		Unit.CommonUnitParameter GetParameter(thrts.IWorld world)
		{
			if (parameter != null) return parameter;
			parameter = world.Factions[0].GetUnitParameter(typename) as Unit.CommonUnitParameter;
			return parameter;
		}
	}

	public class SelectShujinkouUnitsButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		public SelectShujinkouUnitsButton()
		{
			Position = Tuple.Create((ButtonPosition)0, (ButtonPosition)0);
			ButtonImagePath = "PatchSto/Image/Button/SelectShujinkou";
			ButtonName = "主人公勢の選択";

			Kind = ButtonKind.Normal;

			NormalButtonEvent = (building, isAppend) =>
			{
				var library = building.Parameter as Library;
				library.ChangeShujinkouUnits(building.Faction, true);
				library.IsUnitsSelected = true;
			};
		}
	}

	public class SelectKoumakanUnitsButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		public SelectKoumakanUnitsButton()
		{
			Position = Tuple.Create((ButtonPosition)1, (ButtonPosition)0);
			ButtonImagePath = "PatchSto/Image/Button/SelectKoumakan";
			ButtonName = "紅魔館勢の選択";

			Kind = ButtonKind.Normal;

			NormalButtonEvent = (building, isAppend) =>
			{
				var library = building.Parameter as Library;
				library.ChangeKoumakanUnits(building.Faction, true);
				library.IsUnitsSelected = true;
			};
		}
	}

	public class SelectYouyoumuUnitsButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		public SelectYouyoumuUnitsButton()
		{
			Position = Tuple.Create((ButtonPosition)2, (ButtonPosition)0);
			ButtonImagePath = "PatchSto/Image/Button/SelectYouyoumu";
			ButtonName = "妖々夢勢の選択";

			Kind = ButtonKind.Normal;

			NormalButtonEvent = (building, isAppend) =>
			{
				var library = building.Parameter as Library;
				library.ChangeYouyoumuUnits(building.Faction, true);
				library.IsUnitsSelected = true;
			};
		}
	}

	public class SelectSeirensenUnitsButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		public SelectSeirensenUnitsButton()
		{
			Position = Tuple.Create((ButtonPosition)3, (ButtonPosition)0);
			ButtonImagePath = "PatchSto/Image/Button/SelectSeirensen";
			ButtonName = "星蓮船勢の選択";

			Kind = ButtonKind.Normal;

			NormalButtonEvent = (building, isAppend) =>
			{
				var library = building.Parameter as Library;
				library.ChangeSeirensenUnits(building.Faction, true);
				library.IsUnitsSelected = true;
			};
		}
	}

	public class SelectYouseiUnitsButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		public SelectYouseiUnitsButton()
		{
			Position = Tuple.Create((ButtonPosition)4, (ButtonPosition)0);
			ButtonImagePath = "PatchSto/Image/Button/SelectYousei";
			ButtonName = "妖精勢の選択";

			Kind = ButtonKind.Normal;

			NormalButtonEvent = (building, isAppend) =>
			{
				var library = building.Parameter as Library;
				library.ChangeYouseiUnits(building.Faction, true);
				library.IsUnitsSelected = true;
			};
		}
	}


	public class Library : thrts.BuildingParameter
	{
		/// <summary>
		/// 更新対象のユニット
		/// </summary>
		public List<Unit.CommonUnitParameter> UnitParameters = new List<Unit.CommonUnitParameter>();

		/// <summary>
		/// 高速回復する時間
		/// </summary>
		public int WaterTime = 0;

		/// <summary>
		/// 高速移動する時間
		/// </summary>
		public int WindTime = 0;

		/// <summary>
		/// 防御力があがる時間
		/// </summary>
		public int SoilTime = 0;

		public IEffect WaterEffect = null;

		List<thrts.ButtonParameterHolder<thrts.IBuilding>> buttonsShujinkou = new List<ButtonParameterHolder<IBuilding>>();

		List<thrts.ButtonParameterHolder<thrts.IBuilding>> buttonsKoumakan = new List<ButtonParameterHolder<IBuilding>>();

		List<thrts.ButtonParameterHolder<thrts.IBuilding>> buttonsYouyoumu = new List<ButtonParameterHolder<IBuilding>>();

		List<thrts.ButtonParameterHolder<thrts.IBuilding>> buttonsSeirensen = new List<ButtonParameterHolder<IBuilding>>();

		List<thrts.ButtonParameterHolder<thrts.IBuilding>> buttonsYousei = new List<ButtonParameterHolder<IBuilding>>();

		List<thrts.ButtonParameterHolder<thrts.IBuilding>> buttonsSelectUnits = new List<ButtonParameterHolder<IBuilding>>();

		public bool IsUnitsSelected = false;

		public Library ()
		{
			IconPath = "LotusCraft/Image/Button/Black/Building/BlackSmith";
			Name = "図書館";
            ButtonDescription += "全ての中心である。破壊されたら敗北する。";
			ShortcutKeyword = "BuildLaboratory";
			Size = 3;

			MakingTime = 25*30;
			Bullets = new string[] { };
			HP = 2500;
			Armor = 20;
			Supply = 1000;
			Costs = new [] {0,0,0,0};

			ViewRange = thrts.Def.MapTipLengthOnField * 128;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Laboratory.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			Animation = new TextureAnimation();
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;
		}
		public override void OnLoad(IFaction faction)
		{
			buttonsShujinkou.Add(new ButtonParameterHolder<IBuilding> (new CreateUnitButton("博麗霊夢", typeof(Unit.Reimu), 0, 0)));
			buttonsShujinkou.Add(new ButtonParameterHolder<IBuilding> (new LevelUpButton("博麗霊夢", typeof(Unit.Reimu), 0, 1)));
				
			buttonsShujinkou.Add(new ButtonParameterHolder<IBuilding> (new CreateUnitButton("霧雨魔理沙", typeof(Unit.Marisa), 1, 0)));
			buttonsShujinkou.Add(new ButtonParameterHolder<IBuilding> (new LevelUpButton("霧雨魔理沙", typeof(Unit.Marisa), 1, 1)));

			buttonsShujinkou.Add(new ButtonParameterHolder<IBuilding> (new CreateUnitButton("伊吹萃香", typeof(Unit.Suika), 2, 0)));
			buttonsShujinkou.Add(new ButtonParameterHolder<IBuilding> (new LevelUpButton("伊吹萃香", typeof(Unit.Suika), 2, 1)));

			buttonsShujinkou.Add(new ButtonParameterHolder<IBuilding> (new CreateUnitButton("アリス・マーガトロイド", typeof(Unit.Alice), 3, 0)));
			buttonsShujinkou.Add(new ButtonParameterHolder<IBuilding> (new LevelUpButton("アリス・マーガトロイド", typeof(Unit.Alice), 3, 1)));

			buttonsShujinkou.Add(new ButtonParameterHolder<IBuilding> (new CreateUnitButton("東風谷早苗", typeof(Unit.Sanae), 4, 0)));
			buttonsShujinkou.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("東風谷早苗", typeof(Unit.Sanae), 4, 1)));

			// 紅
			buttonsKoumakan.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("紅美鈴", typeof(Unit.Meirin), 0, 0)));
			buttonsKoumakan.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("紅美鈴", typeof(Unit.Meirin), 0, 1)));
				
			buttonsKoumakan.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("パチュリー・ノーレッジ", typeof(Unit.Patchouli), 1, 0)));
			buttonsKoumakan.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("パチュリー・ノーレッジ", typeof(Unit.Patchouli), 1, 1)));

			buttonsKoumakan.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("十六夜咲夜", typeof(Unit.Sakuya), 2, 0)));
			buttonsKoumakan.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("十六夜咲夜", typeof(Unit.Sakuya), 2, 1)));
				 
			buttonsKoumakan.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("レミリア・スカーレット", typeof(Unit.Remilia), 3, 0)));
			buttonsKoumakan.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("レミリア・スカーレット", typeof(Unit.Remilia), 3, 1)));
				
			buttonsKoumakan.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("フランドール・スカーレット", typeof(Unit.Flandre), 4, 0)));
			buttonsKoumakan.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("フランドール・スカーレット", typeof(Unit.Flandre), 4, 1)));

			// 妖
			buttonsYouyoumu.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("八雲橙", typeof(Unit.Chen), 0, 0)));
			buttonsYouyoumu.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("八雲橙", typeof(Unit.Chen), 0, 1)));
				
			buttonsYouyoumu.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("八雲藍", typeof(Unit.Ran), 1, 0)));
			buttonsYouyoumu.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("八雲藍", typeof(Unit.Ran), 1, 1)));
				   
			buttonsYouyoumu.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("魂魄妖夢", typeof(Unit.Youmu), 2, 0)));
			buttonsYouyoumu.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("魂魄妖夢", typeof(Unit.Youmu), 2, 1)));

			buttonsYouyoumu.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("西行寺幽々子", typeof(Unit.Yuyuko), 3, 0)));
			buttonsYouyoumu.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("西行寺幽々子", typeof(Unit.Yuyuko), 3, 1)));

			buttonsYouyoumu.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("八雲紫", typeof(Unit.Yukari), 4, 0)));
			buttonsYouyoumu.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("八雲紫", typeof(Unit.Yukari), 4, 1)));


			// 星
			buttonsSeirensen.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("ナズーリン", typeof(Unit.Nazrin), 0, 0)));
			buttonsSeirensen.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("ナズーリン", typeof(Unit.Nazrin), 0, 1)));

			buttonsSeirensen.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("村紗水蜜", typeof(Unit.Minamitsu), 1, 0)));
			buttonsSeirensen.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("村紗水蜜", typeof(Unit.Minamitsu), 1, 1)));

			buttonsSeirensen.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("虎丸星", typeof(Unit.Shou), 2, 0)));
			buttonsSeirensen.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("虎丸星", typeof(Unit.Shou), 2, 1)));

			buttonsSeirensen.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("聖白蓮", typeof(Unit.Byakuren), 3, 0)));
			buttonsSeirensen.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("聖白蓮", typeof(Unit.Byakuren), 3, 1)));

			buttonsSeirensen.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("UFO", typeof(Unit.UFO), 4, 0)));
			buttonsSeirensen.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("UFO", typeof(Unit.UFO), 4, 1)));

			// 妖精
			buttonsYousei.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("サニーミルク", typeof(Unit.SunnyMilk), 0, 0)));
			buttonsYousei.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("サニーミルク", typeof(Unit.SunnyMilk), 0, 1)));
				
			buttonsYousei.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("ルナチャイルド", typeof(Unit.LunarChild), 1, 0)));
			buttonsYousei.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("ルナチャイルド", typeof(Unit.LunarChild), 1, 1)));
				 
			buttonsYousei.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("スターサファイア", typeof(Unit.StarSapphire), 2, 0)));
			buttonsYousei.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("スターサファイア", typeof(Unit.StarSapphire), 2, 1)));
		
			buttonsYousei.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("大妖精", typeof(Unit.HighFairy), 3, 0)));
			buttonsYousei.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("大妖精", typeof(Unit.HighFairy), 3, 1)));
				  
			buttonsYousei.Add(new ButtonParameterHolder<IBuilding>(new CreateUnitButton("毛玉", typeof(Unit.Kedama), 4, 0)));
			buttonsYousei.Add(new ButtonParameterHolder<IBuilding>(new LevelUpButton("毛玉", typeof(Unit.Kedama), 4, 1)));


			buttonsSelectUnits.Add(new ButtonParameterHolder<IBuilding>(new SelectShujinkouUnitsButton()));

			buttonsSelectUnits.Add(new ButtonParameterHolder<IBuilding>(new SelectKoumakanUnitsButton()));

			buttonsSelectUnits.Add(new ButtonParameterHolder<IBuilding>(new SelectYouyoumuUnitsButton()));

			buttonsSelectUnits.Add(new ButtonParameterHolder<IBuilding>(new SelectSeirensenUnitsButton()));

			buttonsSelectUnits.Add(new ButtonParameterHolder<IBuilding>(new SelectYouseiUnitsButton()));

			List<thrts.ButtonParameterHolder<thrts.IBuilding>> buttons = new List<ButtonParameterHolder<IBuilding>>();

			buttons.AddRange(buttonsShujinkou);
			buttons.AddRange(buttonsKoumakan);
			buttons.AddRange(buttonsYouyoumu);
			buttons.AddRange(buttonsSeirensen);
			buttons.AddRange(buttonsYousei);

			buttons.AddRange(buttonsSelectUnits);

			buttons.Add(new ButtonParameterHolder<IBuilding> (new Bomb.Fire()));
			buttons.Add(new ButtonParameterHolder<IBuilding> (new Bomb.Water()));
			buttons.Add(new ButtonParameterHolder<IBuilding> (new Bomb.Wind()));
			buttons.Add(new ButtonParameterHolder<IBuilding> (new Bomb.Soil()));
			buttons.Add(new ButtonParameterHolder<IBuilding>(new Bomb.Gold()));

			ButtonParams = buttons.ToArray();
		}
		public override void OnLoadAfterLoadingButton(IFaction faction)
		{
			ChangeShujinkouUnits(faction, false);
			ChangeSeirensenUnits(faction, false);
			ChangeYouyoumuUnits(faction, false);
			ChangeKoumakanUnits(faction, false);
			ChangeYouseiUnits(faction, false);
		}

		/// <summary>
		/// 主人公勢のユニットの生産可不可を切り替える。
		/// </summary>
		/// <param name="faction"></param>
		/// <param name="value"></param>
		public void ChangeShujinkouUnits(IFaction faction, bool value)
		{
			if (value)
			{
				foreach (var b in buttonsSelectUnits)
				{
					faction.SetBuildingButtonVisible(typeof(Library).FullName, b, false);
				}

				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Reimu).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Marisa).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Suika).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Alice).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Sanae).FullName) as Unit.CommonUnitParameter);
			}

			foreach (var b in buttonsShujinkou)
			{
				faction.SetBuildingButtonVisible(typeof(Library).FullName, b, value);
			}
		}

		/// <summary>
		/// 紅魔館勢のユニットの生産可不可を切り替える。
		/// </summary>
		/// <param name="faction"></param>
		/// <param name="value"></param>
		public void ChangeKoumakanUnits(IFaction faction, bool value)
		{
			if (value)
			{
				foreach (var b in buttonsSelectUnits)
				{
					faction.SetBuildingButtonVisible(typeof(Library).FullName, b, false);
				}

				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Meirin).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Patchouli).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Sakuya).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Remilia).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Flandre).FullName) as Unit.CommonUnitParameter);
			}

			foreach (var b in buttonsKoumakan)
			{
				faction.SetBuildingButtonVisible(typeof(Library).FullName, b, value);
			}
		}

		/// <summary>
		/// 妖々夢勢のユニットの生産可不可を切り替える。
		/// </summary>
		/// <param name="faction"></param>
		/// <param name="value"></param>
		public void ChangeYouyoumuUnits(IFaction faction, bool value)
		{
			if (value)
			{
				foreach (var b in buttonsSelectUnits)
				{
					faction.SetBuildingButtonVisible(typeof(Library).FullName, b, false);
				}

				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Chen).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Ran).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Youmu).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Yuyuko).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Yukari).FullName) as Unit.CommonUnitParameter);
			}

			foreach (var b in buttonsYouyoumu)
			{
				faction.SetBuildingButtonVisible(typeof(Library).FullName, b, value);
			}
		}

		/// <summary>
		/// 妖精勢のユニットの生産可不可を切り替える。
		/// </summary>
		/// <param name="faction"></param>
		/// <param name="value"></param>
		public void ChangeYouseiUnits(IFaction faction, bool value)
		{
			if (value)
			{
				foreach (var b in buttonsSelectUnits)
				{
					faction.SetBuildingButtonVisible(typeof(Library).FullName, b, false);
				}

				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.SunnyMilk).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.LunarChild).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.StarSapphire).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.HighFairy).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Kedama).FullName) as Unit.CommonUnitParameter);
			}

			foreach (var b in buttonsYousei)
			{
				faction.SetBuildingButtonVisible(typeof(Library).FullName, b, value);
			}
		}
		/// <summary>
		/// 星蓮船勢のユニットの生産可不可を切り替える。
		/// </summary>
		/// <param name="faction"></param>
		/// <param name="value"></param>
		public void ChangeSeirensenUnits(IFaction faction, bool value)
		{
			if (value)
			{
				foreach (var b in buttonsSelectUnits)
				{
					faction.SetBuildingButtonVisible(typeof(Library).FullName, b, false);
				}

				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Nazrin).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Minamitsu).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Shou).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.Byakuren).FullName) as Unit.CommonUnitParameter);
				UnitParameters.Add(faction.GetUnitParameter(typeof(Unit.UFO).FullName) as Unit.CommonUnitParameter);
			}

			foreach (var b in buttonsSeirensen)
			{
				faction.SetBuildingButtonVisible(typeof(Library).FullName, b, value);
			}
		}

		public override void OnVanish (IBuilding building)
		{
			/*
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName,building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
			*/
		}
		public override void OnStartConstruction (IBuilding building)
		{
			/*
			IEffect effect = null;
			var offset = new Vector3D(0,0,1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName,building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale,scale,scale));
			building.Completed += () => {if(effect != null){effect.Stop ();effect=null;}};
			building.Destoroyed += () => {
				if(effect != null)
				{
					effect.Stop ();
					effect=null;
				}
			};
			*/
		}
		public override void OnCompleted (IBuilding building)
		{
			/*
			if(building.Faction.World.CurrentTime > 10)
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.CreatedBuilding).FullName, building.Pos);
			base.OnCompleted (building);
			*/

			building.Updating += () =>
				{
					Update(building);
				};
		}

		void Update(IBuilding building)
		{
			foreach (var param in UnitParameters)
			{
				// 回復
				if (param.NextChargeTime <= 0)
				{
					int count = this.Faction.World.UnitManager.GetUnits(_ => _.Parameter == param).Count();
					int countMax = param.UnitCountMax;

					if (count < countMax)
					{
						var unit = Utl.PutUnitAroundBuilding(building.Faction.World, building.Faction, param.GetType().FullName, building);
					}

					param.NextChargeTime = param.ChargeTime;
				}
				else
				{
					if (WaterTime > 0)
					{
						param.NextChargeTime -= 10;
					}
					else
					{
						param.NextChargeTime--;
					}
				}
			}

			if (WindTime > 0 && WindTime % thrts.Def.FPS == 0)
			{
				var units = building.Faction.World.UnitManager.GetUnits(_ => _.Faction == building.Faction);

				foreach (var unit in units)
				{
					if (unit.AbilityManager.ContainsAbility(Ability.WindAbility.AbilityName))
					{
					}
					else
					{
						unit.AbilityManager.AppendAbility(new Ability.WindAbility(building.Parameter as Library));
					}
				}
			}

			if (SoilTime > 0 && SoilTime % thrts.Def.FPS == 0)
			{
				var units = building.Faction.World.UnitManager.GetUnits(_ => _.Faction == building.Faction);

				foreach (var unit in units)
				{
					if (unit.AbilityManager.ContainsAbility(Ability.SoilAbility.AbilityName))
					{
					}
					else
					{
						unit.AbilityManager.AppendAbility(new Ability.SoilAbility(building.Parameter as Library));
					}
				}
			}

			if (WaterTime > 0)
			{
				WaterTime--;
				if (WaterTime == 0 && WaterEffect != null)
				{
					WaterEffect.Stop();
					WaterEffect = null;
				}
			}

			if (WindTime > 0)
			{
				WindTime--;
			}

			if (SoilTime > 0)
			{
				SoilTime--;
			}
		}
	}
}

