﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PatchSto.Bullet
{
	public class NormalBullet : thrts.BulletParameter
	{
		public void SetBulletType(UnitType unitType)
		{
			if (unitType == PatchSto.UnitType.ShortRange)
			{
				DamageBonus = new float[thrts.AttackBonusSet.None.Length];
				DamageBonus[(int)thrts.TargetObjectType.Clay] = 1.0f;
				DamageBonus[(int)thrts.TargetObjectType.Hero] = 1.25f;
				DamageBonus[(int)thrts.TargetObjectType.Water] = 0.75f;
				DamageBonus[(int)thrts.TargetObjectType.Structure] = 1.0f;
			}

			if (unitType == PatchSto.UnitType.MiddleRange)
			{
				DamageBonus = new float[thrts.AttackBonusSet.None.Length];
				DamageBonus[(int)thrts.TargetObjectType.Clay] = 0.75f;
				DamageBonus[(int)thrts.TargetObjectType.Hero] = 1.0f;
				DamageBonus[(int)thrts.TargetObjectType.Water] = 1.25f;
				DamageBonus[(int)thrts.TargetObjectType.Structure] = 1.0f;
			}

			if (unitType == PatchSto.UnitType.LongRange)
			{
				DamageBonus = new float[thrts.AttackBonusSet.None.Length];
				DamageBonus[(int)thrts.TargetObjectType.Clay] = 1.25f;
				DamageBonus[(int)thrts.TargetObjectType.Hero] = 0.75f;
				DamageBonus[(int)thrts.TargetObjectType.Water] = 1.00f;
				DamageBonus[(int)thrts.TargetObjectType.Structure] = 1.0f;
			}
		}

		public NormalBullet()
		{
			SpaceType = thrts.BulletSpaceType.Earth;
			OnHit += Hit;
		}

		public void SetPenetrationSkill()
		{
			DoesHitOnlyTarget = false;
			HitType = thrts.BulletHitType.Pierce;
		}

		public void LevelUp(int level)
		{
			LevelUpInternal(level);
		}

		protected virtual void LevelUpInternal(int level)
		{
		}

		void Hit(thrts.IBullet bullet, thrts.ITargetObject target)
		{
			var unit = bullet.Generator as thrts.IUnit;
			Unit.CommonUnitParameter normalUnitParam = null;
			if (unit != null)
			{
				normalUnitParam = unit.Parameter as Unit.CommonUnitParameter;
			}

			// 必殺スキル
			if (normalUnitParam != null && normalUnitParam.HasInstantDeathSkill)
			{
				int probability = 20;
				
				// 無効
				if (unit.AbilityManager.ContainsAbility(Ability.VoidAbility.AbilityName))
				{
					probability = 80;
				}

				var world = unit.Faction.World;
				var num = world.NextRand(probability);

				if (num == 0)
				{
					var tu = target as thrts.IUnit;
					if (tu != null)
					{
						tu.HP = 0;
					}
				}
			}

			// 吸血スキル
			if (normalUnitParam != null && normalUnitParam.HasLifeSteelSkill)
			{
				int probability = 10;

				// 無効
				if (unit.AbilityManager.ContainsAbility(Ability.VoidAbility.AbilityName))
				{
					probability = 40;
				}

				var world = unit.Faction.World;
				var num = world.NextRand(probability);

				if (num == 0)
				{
					// (与えたダメージが取得できないので)固定値回復
					unit.HP += unit.MaxHP / 10;
				}
			}

			// 遅延スキル
			if (normalUnitParam != null && normalUnitParam.HasDelaySkill)
			{
				int probability = 10;

				// 無効
				if (unit.AbilityManager.ContainsAbility(Ability.VoidAbility.AbilityName))
				{
					probability = 40;
				}

				var world = unit.Faction.World;
				var num = world.NextRand(probability);

				if (num == 0)
				{
					var tu = target as thrts.IUnit;
					if (tu != null)
					{
						if (tu.AbilityManager.ContainsAbility(Ability.DelayAbility.AbilityName))
						{
							unit.AbilityManager.ExtendLifetimeWithName(thrts.Def.FPS, Ability.DelayAbility.AbilityName, thrts.Def.FPS + 1);
						}
						else
						{
							tu.AbilityManager.AppendAbility(new Ability.DelayAbility());
						}
					}
				}
			}

			// 凍結スキル
			if (normalUnitParam != null && normalUnitParam.HasFreezeSkill)
			{
				int probability = 20;

				// 無効
				if (unit.AbilityManager.ContainsAbility(Ability.VoidAbility.AbilityName))
				{
					probability = 80;
				}

				var world = unit.Faction.World;
				var num = world.NextRand(probability);

				if (num == 0)
				{
					var tu = target as thrts.IUnit;
					if (tu != null)
					{
						if (tu.AbilityManager.ContainsAbility(Ability.FreezeAbility.AbilityName))
						{
						}
						else
						{
							tu.AbilityManager.AppendAbility(new Ability.FreezeAbility());
						}
					}
				}
			}

			// 反魂
			if (normalUnitParam != null && normalUnitParam.HasResurrectionSkill)
			{
				int probability = 5;

				// 無効
				if (unit.AbilityManager.ContainsAbility(Ability.VoidAbility.AbilityName))
				{
					probability = 20;
				}

				var world = unit.Faction.World;
				var num = world.NextRand(probability);

				if (num == 0)
				{
					var tu = target as thrts.IUnit;
					if (tu != null)
					{
						if (tu.AbilityManager.ContainsAbility(Ability.ResurrectionAbility.AbilityName))
						{
							unit.AbilityManager.ExtendLifetimeWithName(3, Ability.ResurrectionAbility.AbilityName, 4);
						}
						else
						{
							tu.AbilityManager.AppendAbility(new Ability.ResurrectionAbility(unit.Faction));
						}
					}
				}
			}

			// 反射
			if (target is thrts.IUnit)
			{
				var tu = target as thrts.IUnit;
				var tp = tu.Parameter as Unit.CommonUnitParameter;

				if (tp.HasReflectionSkill)
				{
					int probability = 5;

					// 無効
					if (tu.AbilityManager.ContainsAbility(Ability.VoidAbility.AbilityName))
					{
						probability = 20;
					}

					var world = unit.Faction.World;
					var num = world.NextRand(probability);

					if (num == 0)
					{
						tu.CreateBullet(this.GetType().FullName, unit, tu.Pos);
					}
				}
			}
		}
	}
}
