﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace PatchSto
{
	public enum SkillType
	{ 
		None,
		Defence,
		Void,
		SelfExplosion,
		Acceleration,
		InstantDeath,
		LifeSteel,
		Delay,
		Evasion,
		Fission,
		Doll,
		AreaAttack,
		Penetration,
		Fear,
		Recovery,
		Freeze,
		Reflection,
		Resurrection,
		Flying,
	}

	public class SkillUtls
	{
		public static string GetName(SkillType type)
		{
			if (type == SkillType.Defence) return "防御";
			if (type == SkillType.Void) return "無効";
			if (type == SkillType.SelfExplosion) return "自爆";
			if (type == SkillType.Acceleration) return "加速";
			if (type == SkillType.InstantDeath) return "即死";
			if (type == SkillType.LifeSteel) return "吸収";
			if (type == SkillType.Delay) return "遅延";
			if (type == SkillType.Evasion) return "回避";
			if (type == SkillType.Fission) return "分裂";
			if (type == SkillType.Doll) return "人形";
			if (type == SkillType.AreaAttack) return "範囲";
			if (type == SkillType.Penetration) return "貫通";
			if (type == SkillType.Fear) return "威圧";
			if (type == SkillType.Recovery) return "回復";
			if (type == SkillType.Freeze) return "凍結";
			if (type == SkillType.Reflection) return "反射";
			if (type == SkillType.Resurrection) return "反魂";
			if (type == SkillType.Flying) return "飛行";
			
			return string.Empty;
		}

		public static string GetName(SkillInformation info)
		{
			return GetName(info.Type);
		}
	}

	public struct SkillInformation
	{
		/// <summary>
		/// スキルの種類
		/// </summary>
		public SkillType Type;

		/// <summary>
		/// スキルに半径が必要な場合の情報
		/// </summary>
		public float Radius;

		/// <summary>
		/// スキルにレベルが必要な場合の情報
		/// </summary>
		public int Level;

		public SkillInformation(SkillType type)
		{
			Type = type;
			Radius = 0;
			Level = 0;
		}

		public SkillInformation(SkillType type, float radius)
		{
			Type = type;
			Radius = radius;
			Level = 0;
		}

		public SkillInformation(SkillType type, float radius, int level)
		{
			Type = type;
			Radius = radius;
			Level = level;
		}
	}

	public class PatchStoMissionEasy : PatchStoMission
	{
		public PatchStoMissionEasy()
		{
			Name = Name + "(Easy)";
			Level = 0;
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] { "PatchSto/Map/Easy.map" };
		}
	}

	public class PatchStoMissionNormal : PatchStoMission
	{
		public PatchStoMissionNormal()
		{
			Name = Name + "(Normal)";
			Level = 1;
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] { "PatchSto/Map/Normal.map" };
		}
	}

	public class PatchStoMissionHard : PatchStoMission
	{
		public PatchStoMissionHard()
		{
			Name = Name + "(Hard)";
			Level = 2;
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] { "PatchSto/Map/Hard.map" };
		}
	}

	public class PatchStoMissionLunatic : PatchStoMission
	{
		public PatchStoMissionLunatic()
		{
			Name = Name + "(Lunatic)";
			Level = 3;
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] { "PatchSto/Map/Lunatic.map" };
		}
	}

	public abstract class PatchStoMission : Mission
	{
		/// <summary>
		/// 難易度
		/// </summary>
		public static int Level = 0;

		public override string[] GetEnableFactions()
		{
			return new string[] { 
				"PatchSto.Faction.White", 
			};
		}

		int nextAttack = 15 * thrts.Def.FPS;

		List<Wave> waves = new List<Wave>();

		IBuilding library = null;

		public PatchStoMission()
			: base()
		{
			Name = "ぱちぇ☆スト";
			Description = "大量のユニットを操作して図書館を守るゲームです。";
			Mode = MissionMode.Single;

			TheNumberOfSelectableTeams = 0;
			TheNumberOfMaximumSelectablePlayersInTeam = 0;
		}
		public override void OnInitialize(IWorld world)
		{
			world.Factions[0].IncResource(0, 1500*1);

			world.Factions[1].DoesNotLose = true;

			List<Type> unitTypes = new List<Type>();
			unitTypes.Add(typeof(Unit.Reimu));
			unitTypes.Add(typeof(Unit.Marisa));
			unitTypes.Add(typeof(Unit.Suika));
			unitTypes.Add(typeof(Unit.Alice));
			unitTypes.Add(typeof(Unit.Sanae));

			unitTypes.Add(typeof(Unit.Nazrin));
			unitTypes.Add(typeof(Unit.Minamitsu));
			unitTypes.Add(typeof(Unit.Shou));
			unitTypes.Add(typeof(Unit.Byakuren));
			unitTypes.Add(typeof(Unit.UFO));

			unitTypes.Add(typeof(Unit.Meirin));
			unitTypes.Add(typeof(Unit.Sakuya));
			unitTypes.Add(typeof(Unit.Patchouli));
			unitTypes.Add(typeof(Unit.Remilia));
			unitTypes.Add(typeof(Unit.Flandre));

			unitTypes.Add(typeof(Unit.Chen));
			unitTypes.Add(typeof(Unit.Ran));
			unitTypes.Add(typeof(Unit.Youmu));
			unitTypes.Add(typeof(Unit.Yuyuko));
			unitTypes.Add(typeof(Unit.Yukari));

			unitTypes.Add(typeof(Unit.SunnyMilk));
			unitTypes.Add(typeof(Unit.StarSapphire));
			unitTypes.Add(typeof(Unit.LunarChild));
			unitTypes.Add(typeof(Unit.HighFairy));
			unitTypes.Add(typeof(Unit.Kedama));

			List<Type> bossUnitTypes = new List<Type>();
			bossUnitTypes.Add(typeof(Unit.BossReimu));
			bossUnitTypes.Add(typeof(Unit.BossRemilia));
			bossUnitTypes.Add(typeof(Unit.BossAlice));

			for (int w = 0; w < 8; w++)
			{
				int level = w;

				waves.Add(new Wave(unitTypes[world.NextRand(unitTypes.Count)], 13 + level, level, 15, world.NextRand(8)));
				waves.Add(new Wave(unitTypes[world.NextRand(unitTypes.Count)], 15 + level, level, 15, world.NextRand(8)));
				waves.Add(new Wave(unitTypes[world.NextRand(unitTypes.Count)], 15 + level, level, 15, world.NextRand(8)));
				waves.Add(new Wave(unitTypes[world.NextRand(unitTypes.Count)], 18 + level, level, 15, world.NextRand(8)));
				waves.Add(new Wave(unitTypes[world.NextRand(unitTypes.Count)], 18 + level, level, 15, world.NextRand(8)));
				waves.Add(new Wave(unitTypes[world.NextRand(unitTypes.Count)], 20 + level, level, 15, world.NextRand(8)));
				waves.Add(new Wave(bossUnitTypes[world.NextRand(bossUnitTypes.Count)], 2, level, 15, world.NextRand(8)));
			}

			// 初期視点設定
			var home = world.BuildingManager.GetBuildings(_ => true).First();
			var basePos = home.Pos;
			world.Factions[0].Parameter.DefaultScreenPosition = new Point2D((int)basePos.X, (int)basePos.Y);
		}

		public override System.Collections.IEnumerable Update(IWorld world)
		{
			var ui = world.StoryUI;

			library = world.BuildingManager.GetBuildings( _ => true ).First();

			int target = 0;

			while (true)
			{
				if ((library.Parameter as Building.Library).IsUnitsSelected)
				{
					break;
				}
				else
				{
					ui.ShowConditionText("図書館からユニットを選択してください。");
					yield return 0;
				}
			}

			while (true)
			{
				if (target < waves.Count)
				{
					int restTime = nextAttack / thrts.Def.FPS;
					var wave = waves[target];

					var nextParam = world.Factions[1].GetUnitParameter(wave.UnitType.FullName) as Unit.CommonUnitParameter;
					var name = nextParam.Name;

					var text = string.Empty;
					text += "残りウェーブ" + (waves.Count - target).ToString("00") + "回\n";
					text += "次の敵出現まで後" + restTime.ToString("00") + "秒\n";

					text += "次の敵 : " + name + "　レベル" + (wave.Level + 1).ToString("00");

					if (nextParam.UnitType_ == UnitType.ShortRange)
					{
						text += "\n(近距離ユニット)";
					}

					if (nextParam.UnitType_ == UnitType.MiddleRange)
					{
						text += "\n(中距離ユニット)";
					}

					if (nextParam.UnitType_ == UnitType.LongRange)
					{
						text += "\n(遠距離ユニット)";
					}
					
					ui.ShowConditionText(text);

					if (nextAttack % (thrts.Def.FPS * 2) == 0)
					{
						var areas = world.AllPurposeArea;
						var area = areas[wave.Position % areas.Count()] as thrts.AreaRectangle;
						var x = (area.Start.X + area.End.X) / 2 * thrts.Def.MapTipLengthOnField;
						var y = (area.Start.Y + area.End.Y) / 2 * thrts.Def.MapTipLengthOnField;
						var pos = new Vector3D(x, y, 120f);
						world.EffectManager.CreateTextureEffect(typeof(Effect.NextTextureEffect).FullName, pos);
					}

					if (nextAttack == 0 && target < waves.Count)
					{	
						target++;

						SendWave(world, wave);
					}

					nextAttack--;
				}
				else
				{
					ui.ShowConditionText("全敵ユニット出現済み");
					world.Factions[1].DoesNotLose = false;
				}

				
				yield return 0;
			}
		}

		void SendWave(IWorld world, Wave wave)
		{
			var areas = world.AllPurposeArea;

			var param = world.Factions[1].GetUnitParameter(wave.UnitType.FullName) as Unit.CommonUnitParameter;

			// レベルアップ
			while (param.Level < wave.Level)
			{
				param.LevelUp();
			}

			var units = PutUnitsOnArea(world, world.Factions[1], wave.UnitType, areas[wave.Position % areas.Count()], wave.Count);

			foreach (var unit in world.UnitManager.GetUnits(_=>_.Faction == world.Factions[1]))
			{
				world.UnitManager.AttackMove(new int[] { unit.UniqueID }, library, false);
			}
			nextAttack = wave.WaveTime;
		}

		/// <summary>
		/// ユニットをエリア上に生成する。
		/// </summary>
		/// <param name="world"></param>
		/// <param name="faction"></param>
		/// <param name="paramType"></param>
		/// <param name="area"></param>
		/// <param name="count"></param>
		/// <returns></returns>
		public IEnumerable<IUnit> PutUnitsOnArea(IWorld world, IFaction faction, Type paramType, Area area, int count)
		{
			var ar = area as AreaRectangle;
			if (ar == null) return new IUnit[0];

			List<IUnit> units = new List<IUnit>();

			var sx = Math.Min(ar.Start.X, ar.End.X);
			var sy = Math.Min(ar.Start.Y, ar.End.Y);
			var ex = Math.Max(ar.Start.X, ar.End.X);
			var ey = Math.Max(ar.Start.Y, ar.End.Y);

			List<Tuple<int, thrts.Point2D>> points = new List<Tuple<int, thrts.Point2D>>();

			for (var y = sy; y <= ey; y++)
			{
				for (var x = sx; x <= ex; x++)
				{
					points.Add(Tuple.Create(world.NextRand(),new Point2D(x,y)));
				}
			}

			points.Sort((v1, v2) => v1.Item1 - v2.Item1);

			foreach (var p in points)
			{
				var u =
						world.UnitManager.Create(paramType.FullName, faction, (p.Item2.X + 0.5f) * thrts.Def.MapTipLengthOnField, (p.Item2.Y + 0.5f) * thrts.Def.MapTipLengthOnField, faction);
				if (u != null) units.Add(u);

				if (units.Count >= count) break;
			}

			return units;
		}

		class Wave
		{
			public Type UnitType { get; private set; }
			public int Count { get; private set; }
			public int Level { get; private set; }
			public int WaveTime { get; private set; }
			public int Position { get; private set; }

			public Wave(Type unitType, int count, int level, int waveTime, int position)
			{
				UnitType = unitType;
				Count = count;
				Level = level;
				WaveTime = waveTime * thrts.Def.FPS;
				Position = position;
			}
		}
	}
}
