﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PatchSto.Unit
{
	public class BossReimu : BossUnitParameter
	{
		public BossReimu()
		{
			const int HPBase = 900;
			const int HPUp = 240;
			const int ATKBase = 40;
			const int ATKUp = 20;
			const int ArmorsBase = 30;
			const int ArmorsUp = 13;

			float SpeedBase = 0.5f;

			const int LevelUpCostBase = 16000;
			const int LevelUpCostUp = 7000;

			const int LevelUpCostByUnitsBase = 700;
			const int LevelUpCostByUnitsUp = 700;

			const int CostToCreateBase = 1400;

			IconPath = "PatchSto/Image/Button/Unit/BossReimu";

			Name = "博麗霊夢(ボス)";

			Speed = SpeedBase * BasisSpeed;

			UnitUtls.SetBossSizeAnimation(this, "LotusCraft/Image/Unit/White/Unique/reimu");

			HPs[0] = HPBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				HPs[i] = HPs[i - 1] + HPUp;
			}

			ATKs[0] = ATKBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				ATKs[i] = ATKs[i - 1] + ATKUp;
			}

			Armors[0] = ArmorsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				Armors[i] = Armors[i - 1] + ArmorsUp;
			}

			LevelUpCosts[0] = LevelUpCostBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCosts[i] = LevelUpCosts[i - 1] + LevelUpCostUp;
			}

			LevelUpCostsByUnits[0] = LevelUpCostByUnitsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCostsByUnits[i] = LevelUpCostsByUnits[i - 1] + LevelUpCostByUnitsUp;
			}

			CostsToCreate[0] = CostToCreateBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				CostsToCreate[i] = CostsToCreate[i - 1] + LevelUpCostByUnitsUp;
			}

			for (int i = 0; i < MaxLevel; i++)
			{
				Rewards[i] = (int)(CostsToCreate[i] * 0.32f);
			}

			ChargeTime = 2 * thrts.Def.FPS;

			Bullets = new string[] { 
				typeof(BossReimuBullet).FullName,
			};

			SetUnitType(PatchSto.UnitType.LongRange);
		}

		public override void OnLoad(thrts.IFaction faction)
		{
			InitializeParameters();
		}
	}

	public class BossReimuBullet : Bullet.NormalBullet
	{
		public BossReimuBullet()
		{
			Type = thrts.BulletType.LongRange;

			Speed = 10.0f * BasisSpeed;
			Gravity = 0.1f;
			MaxRange = 6 * thrts.Def.MapTipLengthOnField;

			AttackDelay = (int)(0.0f * thrts.Def.FPS);
			AttackRigor = 0;
			AttackSpeed = AttackDelay + (int)(0.1f * thrts.Def.FPS);

			OnHit += OnHitEffect;

			TextureAnimation = new thrts.TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/ofuda";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = thrts.CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = thrts.TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 8;
		}
		public static void OnHitEffect(thrts.IBullet bullet, thrts.ITargetObject target)
		{
			EffectUtls.ShowCommonHitEffect(bullet, target, HitEffectType.NormalRed);
		}
	}
}
