﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PatchSto.Unit
{
	public class CommonUnitParameter : thrts.UnitParameter
	{
		public UnitType UnitType_ { get; private set; }

		public void SetUnitType(UnitType unitType)
		{
			if (unitType == PatchSto.UnitType.ShortRange)
			{
				UnitType = new thrts.TargetObjectType[1];
				UnitType[0] = thrts.TargetObjectType.Clay;
			}

			if (unitType == PatchSto.UnitType.MiddleRange)
			{
				UnitType = new thrts.TargetObjectType[1];
				UnitType[0] = thrts.TargetObjectType.Hero;
			}

			if (unitType == PatchSto.UnitType.LongRange)
			{
				UnitType = new thrts.TargetObjectType[1];
				UnitType[0] = thrts.TargetObjectType.Water;
			}

			UnitType_ = unitType;
		}

		public bool IsBoss = true;

		public const int MaxLevel = 30;

		public int[] HPs = new int[MaxLevel];

		/// <summary>
		/// レベルアップコスト
		/// </summary>
		public int[] LevelUpCosts = new int[MaxLevel];

		/// <summary>
		/// ユニット1体あたりのレベルアップコスト
		/// </summary>
		public int[] LevelUpCostsByUnits = new int[MaxLevel];

		/// <summary>
		/// 生産コスト
		/// </summary>
		public int[] CostsToCreate = new int[MaxLevel];

		public int[] ATKs = new int[MaxLevel];

		public int[] Armors = new int[MaxLevel];

		public int[] Rewards = new int[MaxLevel];

		public SkillInformation[] Skills = new SkillInformation[MaxLevel];

		public int ChargeTime = 0;

		/// <summary>
		/// 防御スキルを持っているか?
		/// </summary>
		bool HasDefenceSkill = false;

		/// <summary>
		/// 無効スキルを持っているか?
		/// </summary>
		bool HasVoidSkill = false;

		/// <summary>
		/// 自爆スキルを持っているか?
		/// </summary>
		bool HasSelfExplosionSkill = false;

		/// <summary>
		/// 加速スキルを持っているか?
		/// </summary>
		bool HasAccelerationSkill = false;

		/// <summary>
		/// 必殺スキルを持っているか?
		/// </summary>
		public bool HasInstantDeathSkill { get; private set; }

		/// <summary>
		/// 吸血スキルを持っているか?
		/// </summary>
		public bool HasLifeSteelSkill { get; private set; }

		/// <summary>
		/// 遅延スキルを持っているか?
		/// </summary>
		public bool HasDelaySkill { get; private set; }

		/// <summary>
		/// 回避スキルを持っているか?
		/// </summary>
		bool HasEvasionSkill = false;

		/// <summary>
		/// 人形スキルを持っているか?
		/// </summary>
		bool HasDollSkill = false;

		/// <summary>
		/// 分裂スキルを持っているか?
		/// </summary>
		bool HasFissionSkill = false;

		/// <summary>
		/// 威圧スキルを持っているか?
		/// </summary>
		bool HasFearSkill = false;

		/// <summary>
		/// 回復スキルを持っているか?
		/// </summary>
		bool HasRecoverySkill = false;

		/// <summary>
		/// 凍結スキルを持っているか?
		/// </summary>
		public bool HasFreezeSkill = false;

		/// <summary>
		/// 反射スキルを持っているか?
		/// </summary>
		public bool HasReflectionSkill = false;

		/// <summary>
		/// 反魂スキルを持っているか?
		/// </summary>
		public bool HasResurrectionSkill = false;

		/// <summary>
		/// 飛行スキルを持っているか?
		/// </summary>
		public bool HasFlyingSkill = false;

		/// <summary>
		/// 現在のレベル(0スタート)
		/// </summary>
		public int Level = 0;

		/// <summary>
		/// ユニットの最大数
		/// </summary>
		public int UnitCountMax = 0;

		/// <summary>
		/// 次に回復可能になるまでの時間
		/// </summary>
		public int NextChargeTime = 0;

		/// <summary>
		/// 防御スキルの範囲
		/// </summary>
		float radiusDefence = 0;

		/// <summary>
		/// 無効の範囲
		/// </summary>
		float radiusVoid = 0;

		/// <summary>
		/// 自爆の範囲
		/// </summary>
		float radiusSelfExplosion = 0;

		/// <summary>
		/// 威圧の範囲
		/// </summary>
		float radiusFear = 0;

		/// <summary>
		/// 回復の範囲
		/// </summary>
		float radiusRecovey = 0;

		/// <summary>
		/// 人形のレベル
		/// </summary>
		int dollLevel = 0;

		public CommonUnitParameter()
		{
			for (int i = 0; i < MaxLevel; i++)
			{
				HPs[i] = 1;
				LevelUpCosts[i] = 0;
			}

			Radius = 8;

			ViewRange = 10 * thrts.Def.MapTipLengthOnField;

			this.SetCutinAnimation("LotusCraft/Image/Cutin/Patch");
		}

		void SetCutinAnimation(string basePath)
		{
			Cutin.Path800 = basePath + "/Main800";
			Cutin.BackgroundPath800 = basePath + "/Back800";
			Cutin.ForegroundPath800 = basePath + "/Fore800";

			Cutin.Count800.X = 5;
			Cutin.Count800.Y = 3;

			Cutin.Path1280 = basePath + "/Main1280";
			Cutin.BackgroundPath1280 = basePath + "/Back1280";
			Cutin.ForegroundPath1280 = basePath + "/Fore1280";

			Cutin.Count1280.X = 5;
			Cutin.Count1280.Y = 3;

			Cutin.BackgroundBlack = 0.5f;
		}

		/// <summary>
		/// パラメーターを初期化する。
		/// </summary>
		public void InitializeParameters()
		{
			var world = Faction.World;
			
			// 敵補正
			if (Faction == Faction.World.Factions[1])
			{
				if (PatchStoMission.Level == 0)
				{
					for (int i = 0; i < MaxLevel; i++)
					{
						HPs[i] = (int)(HPs[i] * 0.5f);
						ATKs[i] = (int)(ATKs[i] * 0.7f);
						Armors[i] = (int)(Armors[i] * 0.6f);
					}

					Speed = Speed * 0.6f;
				}
				else if (PatchStoMission.Level == 1)
				{
					for (int i = 0; i < MaxLevel; i++)
					{
						HPs[i] = (int)(HPs[i] * 0.6f);
						ATKs[i] = (int)(ATKs[i] * 0.7f);
						Armors[i] = (int)(Armors[i] * 0.7f);
					}

					Speed = Speed * 0.6f;
				}
				else if (PatchStoMission.Level == 2)
				{
					for (int i = 0; i < MaxLevel; i++)
					{
						HPs[i] = (int)(HPs[i] * 0.7f);
						ATKs[i] = (int)(ATKs[i] * 0.8f);
						Armors[i] = (int)(Armors[i] * 0.7f);
					}

					Speed = Speed * 0.7f;
				}
				else if (PatchStoMission.Level == 3)
				{
					for (int i = 0; i < MaxLevel; i++)
					{
						HPs[i] = (int)(HPs[i] * 0.8f);
						ATKs[i] = (int)(ATKs[i] * 0.9f);
						Armors[i] = (int)(Armors[i] * 0.8f);
					}

					Speed = Speed * 0.7f;
				}
			}

			HP = HPs[Level];
			Armor = Armors[Level];
			Costs[0] = CostsToCreate[Level];

			if (Bullets.Length > 0)
			{
				var bulletName = Bullets[0];
				Faction.GetBulletParameter(bulletName).Attack = ATKs[Level];
			}

			LevelUpSkill();

			var bullet = GetNormalBullet();
			if (bullet != null)
			{
				bullet.SetBulletType(UnitType_);
			}
		}

		/// <summary>
		/// 次にレベルアップするためのコストを取得する。
		/// </summary>
		/// <returns></returns>
		public int GetNextCost()
		{
			if (Level >= MaxLevel) return int.MaxValue;
			return LevelUpCosts[Level] + UnitCountMax * LevelUpCostsByUnits[Level];
		}

		/// <summary>
		/// レベルアップを行う。
		/// </summary>
		/// <returns>成否</returns>
		public bool LevelUp()
		{
			if (Level + 1 >= MaxLevel) return false;
			Level++;

			var world = Faction.World;

			HP = HPs[Level];
			Armor = Armors[Level];
			Costs[0] = CostsToCreate[Level];

			if (Bullets.Length > 0)
			{
				var bulletName = Bullets[0];
				var bullet = Faction.GetBulletParameter(bulletName) as Bullet.NormalBullet;
				bullet.Attack = ATKs[Level];
				bullet.LevelUp(Level);
			}

			LevelUpSkill();

			LevelUpInternal(Level);

			// ユニットのパラメーター全更新
			foreach (var u in world.UnitManager.GetUnits(_ => _.Parameter == this))
			{
				u.SetParameter(this);
			}

			return true;
		}

		/// <summary>
		/// ユニット特有のレベルアップ処理を行う。
		/// </summary>
		/// <param name="level"></param>
		protected virtual void LevelUpInternal(int level)
		{ 
		}

		void LevelUpSkill()
		{
			var skill = Skills[Level];
			if (skill.Type == SkillType.None)
			{
				return;
			}
			else if (skill.Type == SkillType.Defence)
			{
				SetDefenceSkill(skill.Radius);
			}
			else if (skill.Type == SkillType.Acceleration)
			{
				SetAccelerationSkill();
			}
			else if (skill.Type == SkillType.Void)
			{
				SetVoidSkill(skill.Radius);
			}
			else if (skill.Type == SkillType.Evasion)
			{
				SetEvasionSkill();
			}
			else if (skill.Type == SkillType.AreaAttack)
			{
				SetAreaAttackSkill(skill.Radius);
			}
			else if (skill.Type == SkillType.Doll)
			{
				SetDollSkill(skill.Level);
			}
			else if (skill.Type == SkillType.Fission)
			{
				SetFissionSkill();
			}
			else if (skill.Type == SkillType.SelfExplosion)
			{
				SetSelfExplosionSkill(skill.Radius);
			}
			else if (skill.Type == SkillType.Penetration)
			{
				var b = GetNormalBullet();
				if (b != null)
				{
					b.SetPenetrationSkill();
				}
			}
			else if (skill.Type == SkillType.Fear)
			{
				SetFearSkill(skill.Radius);
			}
			else if (skill.Type == SkillType.Recovery)
			{
				SetRecoverySkill(skill.Radius);
			}
			else if (skill.Type == SkillType.Freeze)
			{
				SetFreezeSkill();
			}
			else if (skill.Type == SkillType.Reflection)
			{
				SetReflectionSkill();
			}
			else if (skill.Type == SkillType.Resurrection)
			{
				SetResurrectionSkill();
			}
			else if (skill.Type == SkillType.Flying)
			{
				SetFlyingSkill();
			}
		}

		/// <summary>
		/// 範囲攻撃スキルを有効にする。
		/// </summary>
		/// <param name="radius"></param>
		public void SetAreaAttackSkill(float radius)
		{
			if (Bullets.Length > 0)
			{
				var bulletName = Bullets[0];
				var bullet = Faction.GetBulletParameter(bulletName) as Bullet.NormalBullet;
				bullet.HasSplash = true;
				bullet.IsSplashMissHit = true;
				bullet.SplashDamageRates = new Dictionary<float, float>();
				bullet.SplashDamageRates.Add(radius, 1.0f);
			}
		}

		/// <summary>
		/// 防御スキルを有効にする。
		/// </summary>
		public void SetDefenceSkill(float radius)
		{
			radiusDefence = radius;
			HasDefenceSkill = true;
		}

		/// <summary>
		/// 無効スキルを有効にする。
		/// </summary>
		public void SetVoidSkill(float radius)
		{
			radiusVoid = radius;
			HasVoidSkill = true;
		}

		/// <summary>
		/// 自爆スキルを有効にする。
		/// </summary>
		public void SetSelfExplosionSkill(float radius)
		{
			HasSelfExplosionSkill = true;
			radiusSelfExplosion = radius;
		}

		/// <summary>
		/// 加速スキルを有効にする。
		/// </summary>
		public void SetAccelerationSkill()
		{
			HasAccelerationSkill = true;
		}

		/// <summary>
		/// 回避スキルを有効にする。
		/// </summary>
		public void SetEvasionSkill()
		{
			HasEvasionSkill = true;
		}

		/// <summary>
		/// 必殺スキルを有効にする。
		/// </summary>
		public void SetInstantDeathSkill()
		{
			HasInstantDeathSkill = true;
		}
		
		/// <summary>
		/// 吸血スキルを有効にする。
		/// </summary>
		public void SetLifeSteelSkill()
		{
			HasLifeSteelSkill = true;
		}

		/// <summary>
		/// 人形スキルを有効にする。
		/// </summary>
		/// <param name="level"></param>
		public void SetDollSkill(int level)
		{
			HasDollSkill = true;
			dollLevel = level;
		}

		/// <summary>
		/// 分裂スキルを有効にする。
		/// </summary>
		public void SetFissionSkill()
		{
			HasFissionSkill = true;
		}

		/// <summary>
		/// 威圧スキルを有効にする。
		/// </summary>
		public void SetFearSkill(float radius)
		{
			HasFearSkill = true;
			radiusFear = radius;
		}

		/// <summary>
		/// 回復スキルを有効にする。
		/// </summary>
		/// <param name="radius"></param>
		public void SetRecoverySkill(float radius)
		{
			HasRecoverySkill = true;
			radiusRecovey = radius;
		}

		/// <summary>
		/// 凍結スキルを有効にする。
		/// </summary>
		public void SetFreezeSkill()
		{
			HasFreezeSkill = true;
		}

		/// <summary>
		/// 反射スキルを有効にする。
		/// </summary>
		public void SetReflectionSkill()
		{
			HasReflectionSkill = true;
		}

		/// <summary>
		/// 反魂スキルを有効にする。
		/// </summary>
		public void SetResurrectionSkill()
		{
			HasReflectionSkill = true;
		}

		/// <summary>
		/// 飛行スキルを有効にする。
		/// </summary>
		public void SetFlyingSkill()
		{
			HasFlyingSkill = true;
			CanJump = true;
		}

		public override void OnAppear(thrts.IUnit unit)
		{
			unit.Updating += () =>
				{
					Update(unit, this);
				};

			unit.Vanishing += (u) =>
				{

				};

			unit.Vanished += (u) =>
				{
					Vanished(unit, this);
				};

			// 召喚エフェクト
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.CreatedUnit).FullName, unit.Pos);
			var effect = unit.Faction.World.EffectManager.CreateEffect(typeof(Effect.CreateUnitEffect).FullName, unit.Pos);
			effect.SetScale(new thrts.Vector3D(3, 3, 3));

			unit.StartLoadCutin();

			// 人形スキル
			if(HasDollSkill)
			{
				for (int i = 0; i < dollLevel + 1; i++)
				{
					thrts.Utl.PutUnitAroundPoint(
						unit.Faction.World,
						unit.Faction,
						typeof(Unit.Doll).FullName,
						new thrts.Vector2D(unit.Pos.X, unit.Pos.Y));
				}
			}
		}

		public override void OnVanish(thrts.IUnit unit)
		{
			var faction = unit.Faction;
			var offset = new thrts.Vector3D(0, 0, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(Effect.DeadUnitEffect).FullName, unit.Pos + offset);
			var scale = new thrts.Vector3D(8, 8, 8);
			effect.SetScale(scale);
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.PCH).FullName, unit.Pos);
			base.OnVanish(unit);

			// 分裂スキル
			if (HasFissionSkill)
			{
				for (int i = 0; i < 2; i++)
				{
					thrts.Utl.PutUnitAroundPoint(
						unit.Faction.World,
						unit.Faction,
						typeof(Unit.SuikaSmall).FullName,
						new thrts.Vector2D(unit.Pos.X, unit.Pos.Y));
				}
			}
		}

		static void Update(thrts.IUnit unit, CommonUnitParameter param)
		{
			// 防御スキル
			if (param.HasDefenceSkill)
			{
				var time = unit.TimeAfterCreated;
				var world = unit.Faction.World;

				// ライフが一定以下で発動
				if (time % thrts.Def.FPS == 0)
				{
					if ((float)unit.HP / (float)unit.MaxHP < 0.5f)
					{
						var targets = world.GetObjectFromCircle(
							new thrts.Circle2D(param.radiusDefence, new thrts.Vector2D(unit.Pos.X, unit.Pos.Y))).Where(_ => _ != null);

						foreach (var u in targets.OfType<thrts.IUnit>().Where(_ => _.Faction == unit.Faction))
						{
							if (u.AbilityManager.ContainsAbility(Ability.DefenceAbility.AbilityName))
							{
								u.AbilityManager.ExtendLifetimeWithName(thrts.Def.FPS, Ability.DefenceAbility.AbilityName, thrts.Def.FPS + 1);
							}
							else
							{
								u.AbilityManager.AppendAbility(new Ability.DefenceAbility());
							}
						}
					}
				}
			}

			// 無効スキル
			if (param.HasVoidSkill)
			{
				var time = unit.TimeAfterCreated;
				var world = unit.Faction.World;

				if (time % thrts.Def.FPS == 0)
				{
					var targets = world.GetObjectFromCircle(
						new thrts.Circle2D(param.radiusVoid, new thrts.Vector2D(unit.Pos.X, unit.Pos.Y))).Where(_ => _ != null);

					foreach (var u in targets.OfType<thrts.IUnit>().Where(_ => _.Faction != unit.Faction))
					{
						if (u.AbilityManager.ContainsAbility(Ability.VoidAbility.AbilityName))
						{

						}
						else
						{
							u.AbilityManager.AppendAbility(new Ability.VoidAbility());
						}
					}
				}
			}

			// 加速スキル
			if (param.HasAccelerationSkill)
			{
				var time = unit.TimeAfterCreated;
				var world = unit.Faction.World;

				if (time % thrts.Def.FPS == 0)
				{
					if (unit.AbilityManager.ContainsAbility(Ability.AccelerationAbility.AbilityName))
					{
						unit.AbilityManager.ExtendLifetimeWithName(thrts.Def.FPS, Ability.AccelerationAbility.AbilityName, thrts.Def.FPS + 1);
					}
					else
					{
						unit.AbilityManager.AppendAbility(new Ability.AccelerationAbility());
					}
				}
			}

			// 回避スキル
			if (param.HasEvasionSkill)
			{
				var time = unit.TimeAfterCreated;
				var world = unit.Faction.World;

				if (time % thrts.Def.FPS == 0)
				{
					if (unit.AbilityManager.ContainsAbility(Ability.EvasionAbility.AbilityName))
					{
						unit.AbilityManager.ExtendLifetimeWithName(thrts.Def.FPS, Ability.EvasionAbility.AbilityName, thrts.Def.FPS + 1);
					}
					else
					{
						unit.AbilityManager.AppendAbility(new Ability.EvasionAbility());
					}
				}
			}

			// 威圧スキル
			if (param.HasFearSkill)
			{
				var time = unit.TimeAfterCreated;
				var world = unit.Faction.World;

				if (time % thrts.Def.FPS == 0)
				{

					var targets = world.GetObjectFromCircle(
						new thrts.Circle2D(param.radiusFear, new thrts.Vector2D(unit.Pos.X, unit.Pos.Y))).Where(_ => _ != null);

					foreach (var u in targets.OfType<thrts.IUnit>().Where(_ => _.Faction != unit.Faction))
					{
						if (u.AbilityManager.ContainsAbility(Ability.FearAbility.AbilityName))
						{
							u.AbilityManager.ExtendLifetimeWithName(thrts.Def.FPS, Ability.FearAbility.AbilityName, thrts.Def.FPS + 1);
						}
						else
						{
							u.AbilityManager.AppendAbility(new Ability.FearAbility());
						}
					}
				}
			}

			// 回復
			if (param.HasRecoverySkill)
			{
				var time = unit.TimeAfterCreated;
				var world = unit.Faction.World;

				if (time % thrts.Def.FPS == 0)
				{

					var targets = world.GetObjectFromCircle(
						new thrts.Circle2D(param.radiusRecovey, new thrts.Vector2D(unit.Pos.X, unit.Pos.Y))).Where(_ => _ != null);

					foreach (var u in targets.OfType<thrts.IUnit>().Where(_ => _.Faction == unit.Faction))
					{
						u.HP += (int)((float)unit.MaxHP / 100.0f);
					}
				}
			}
		}

		static void Vanishing(thrts.IUnit unit, CommonUnitParameter param)
		{
			if (unit.AbilityManager.ContainsAbility(Ability.ResurrectionAbility.AbilityName))
			{
				var ablity = unit.AbilityManager.GetAbilityWithName(Ability.ResurrectionAbility.AbilityName) as Ability.ResurrectionAbility;
				unit.HP = unit.MaxHP;
				unit.ChangeFaction(ablity.TargetFaction);
			}
		}

		static void Vanished(thrts.IUnit unit, CommonUnitParameter param)
		{
			// 自爆スキル
			if (param.HasSelfExplosionSkill)
			{
				var world = unit.Faction.World;
				var atk = param.ATKs[param.Level];
				var bullet = param.GetNormalBullet();

				if (bullet != null)
				{
					var targets = world.GetObjectFromCircle(
							new thrts.Circle2D(param.radiusSelfExplosion, new thrts.Vector2D(unit.Pos.X, unit.Pos.Y))).Where(_ => _ != null);

					foreach (var u in targets.OfType<thrts.IUnit>().Where(_ => _.Faction != unit.Faction))
					{
						u.BeDamaged(unit, atk, bullet.DamageBonus);
					}

					world.EffectManager.CreateTextureEffect(typeof(Effect.BombHitTextureEffect).FullName, unit.Pos);
					world.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom01).FullName, unit.Pos);
				}
			}

			// 報酬
			if (unit.Faction.World.Factions[1] == unit.Faction)
			{
				unit.Faction.World.Factions[0].IncResource(0, param.Rewards[param.Level]);
			}
		}

		/// <summary>
		/// 基本的な弾を取得する。
		/// </summary>
		/// <returns></returns>
		Bullet.NormalBullet GetNormalBullet()
		{
			if (Bullets.Length > 0)
			{
				var bulletName = Bullets[0];
				var bullet = Faction.GetBulletParameter(bulletName) as Bullet.NormalBullet;
				return bullet;
			}
			return null;
		}
	}
}
