﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PatchSto.Unit
{
	public class HighFairy : CommonUnitParameter
	{
		public HighFairy()
		{
			const int HPBase = 120;
			const int HPUp = 80;
			const int ATKBase = 28;
			const int ATKUp = 15;
			const int ArmorsBase = 11;
			const int ArmorsUp = 6;

			float SpeedBase = 1.0f;

			const int LevelUpCostBase = 1500;
			const int LevelUpCostUp = 600;

			const int LevelUpCostByUnitsBase = 70;
			const int LevelUpCostByUnitsUp = 70;

			const int CostToCreateBase = 90;

			const PatchSto.UnitType unitType = PatchSto.UnitType.LongRange;

			ChargeTime = (int)(1.0f * thrts.Def.FPS);

			IconPath = "PatchSto/Image/Button/Unit/HighFairy";

			Name = "大妖精";

			Speed = SpeedBase * BasisSpeed;

			UnitUtls.SetAnimation(this, "PatchSto/Image/Unit/HighFairy");

			HPs[0] = HPBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				HPs[i] = HPs[i - 1] + HPUp;
			}

			ATKs[0] = ATKBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				ATKs[i] = ATKs[i - 1] + ATKUp;
			}

			Armors[0] = ArmorsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				Armors[i] = Armors[i - 1] + ArmorsUp;
			}

			LevelUpCosts[0] = LevelUpCostBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCosts[i] = LevelUpCosts[i - 1] + LevelUpCostUp;
			}

			LevelUpCostsByUnits[0] = LevelUpCostByUnitsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCostsByUnits[i] = LevelUpCostsByUnits[i - 1] + LevelUpCostByUnitsUp;
			}

			CostsToCreate[0] = CostToCreateBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				CostsToCreate[i] = CostsToCreate[i - 1] + LevelUpCostByUnitsUp;
			}

			for (int i = 0; i < MaxLevel; i++)
			{
				Rewards[i] = (int)(CostsToCreate[i] * 0.32f);
			}

			Skills[0] = new SkillInformation(SkillType.Recovery, thrts.Def.MapTipLengthOnField * 3);

			Skills[2] = new SkillInformation(SkillType.Freeze);

			Skills[5] = new SkillInformation(SkillType.Penetration);

			Bullets = new string[] { 
				typeof(HighFairyBullet).FullName,
			};

			SetUnitType(unitType);
		}

		public override void OnLoad(thrts.IFaction faction)
		{
			InitializeParameters();
		}
	}


	public class HighFairyBullet : Bullet.NormalBullet
	{
		public HighFairyBullet()
		{
			Type = thrts.BulletType.LongRange;

			Speed = 10.0f * BasisSpeed;
			Gravity = 0.1f;
			MaxRange = 6 * thrts.Def.MapTipLengthOnField;

			AttackDelay = (int)(0.0f * thrts.Def.FPS);
			AttackRigor = 0;
			AttackSpeed = AttackDelay + (int)(0.5f * thrts.Def.FPS);

			OnHit += OnHitEffect;

			TextureAnimation = new thrts.TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/ball_blue";
			TextureAnimation.AlphaBlend = thrts.AlphaBlend.Add;
			TextureAnimation.Width = 32;
			TextureAnimation.Height = 32;
			TextureAnimation.Center.X = 16;
			TextureAnimation.Center.Y = 16;
			TextureAnimation.CenterPosition = thrts.CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 4;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = thrts.TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 4;
		}
		public static void OnHitEffect(thrts.IBullet bullet, thrts.ITargetObject target)
		{
			EffectUtls.ShowCommonHitEffect(bullet, target, HitEffectType.NormalBlue);
		}
	}
}
