﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PatchSto.Unit
{
	public class Patchouli : CommonUnitParameter
	{
		public Patchouli()
		{
			const int HPBase = 170;
			const int HPUp = 90;
			const int ATKBase = 36;
			const int ATKUp = 20;
			const int ArmorsBase = 16;
			const int ArmorsUp = 11;

			float SpeedBase = 0.8f;

			const int LevelUpCostBase = 1800;
			const int LevelUpCostUp = 950;

			const int LevelUpCostByUnitsBase = 110;
			const int LevelUpCostByUnitsUp = 110;

			const int CostToCreateBase = 170;

			const PatchSto.UnitType unitType = PatchSto.UnitType.LongRange;

			ChargeTime = 2 * thrts.Def.FPS;

			IconPath = "PatchSto/Image/Button/Unit/Patchouli";

			Name = "パチュリー・ノーレッジ";

			Speed = SpeedBase * BasisSpeed;

			UnitUtls.SetAnimation(this, "PatchSto/Image/Unit/Patchouli");

			HPs[0] = HPBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				HPs[i] = HPs[i - 1] + HPUp;
			}

			ATKs[0] = ATKBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				ATKs[i] = ATKs[i - 1] + ATKUp;
			}

			Armors[0] = ArmorsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				Armors[i] = Armors[i - 1] + ArmorsUp;
			}

			LevelUpCosts[0] = LevelUpCostBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCosts[i] = LevelUpCosts[i - 1] + LevelUpCostUp;
			}

			LevelUpCostsByUnits[0] = LevelUpCostByUnitsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCostsByUnits[i] = LevelUpCostsByUnits[i - 1] + LevelUpCostByUnitsUp;
			}

			CostsToCreate[0] = CostToCreateBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				CostsToCreate[i] = CostsToCreate[i - 1] + LevelUpCostByUnitsUp;
			}

			for (int i = 0; i < MaxLevel; i++)
			{
				Rewards[i] = (int)(CostsToCreate[i] * 0.32f);
			}

			Skills[0] = new SkillInformation(SkillType.AreaAttack, thrts.Def.MapTipLengthOnField * 1);

			Skills[2] = new SkillInformation(SkillType.Void, thrts.Def.MapTipLengthOnField * 3);

			Bullets = new string[] { 
				typeof(PatchouliBullet).FullName,
			};

			SetUnitType(unitType);
		}

		public override void OnLoad(thrts.IFaction faction)
		{
			InitializeParameters();
		}
	}


	public class PatchouliBullet : Bullet.NormalBullet
	{
		public PatchouliBullet()
		{
			Type = thrts.BulletType.LongRange;

			Speed = 10.0f * BasisSpeed;
			Gravity = 0.1f;
			MaxRange = 7 * thrts.Def.MapTipLengthOnField;

			AttackDelay = (int)(0.0f * thrts.Def.FPS);
			AttackRigor = 0;
			AttackSpeed = AttackDelay + (int)(0.5f * thrts.Def.FPS);

			OnHit += OnHitEffect;

			TextureAnimation = new thrts.TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 0;
			TextureAnimation.Center.Y = 0;
			TextureAnimation.CenterPosition = thrts.CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = thrts.TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 1;
			TextureAnimation.Offset.X = 128 * 2 + 16 * 0;
			TextureAnimation.Offset.Y = 128 * 1 + 16 * 4;
			TextureAnimation.AlphaBlend = thrts.AlphaBlend.Add;
		}
		public static void OnHitEffect(thrts.IBullet bullet, thrts.ITargetObject target)
		{
			EffectUtls.ShowCommonHitEffect(bullet, target, HitEffectType.ColumnRed);
		}
	}
}
