﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PatchSto.Unit
{
	public class Sakuya: CommonUnitParameter
	{
		public Sakuya()
		{
			const int HPBase = 160;
			const int HPUp = 90;
			const int ATKBase = 38;
			const int ATKUp = 16;
			const int ArmorsBase = 13;
			const int ArmorsUp = 8;

			float SpeedBase = 1.0f;

			const int LevelUpCostBase = 1700;
			const int LevelUpCostUp = 900;

			const int LevelUpCostByUnitsBase = 100;
			const int LevelUpCostByUnitsUp = 100;

			const int CostToCreateBase = 140;

			const PatchSto.UnitType unitType = PatchSto.UnitType.MiddleRange;

			ChargeTime = 2 * thrts.Def.FPS;

			IconPath = "PatchSto/Image/Button/Unit/Sakuya";

			Name = "十六夜咲夜";

			Speed = SpeedBase * BasisSpeed;

			UnitUtls.SetAnimation(this, "PatchSto/Image/Unit/Sakuya");

			HPs[0] = HPBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				HPs[i] = HPs[i - 1] + HPUp;
			}

			ATKs[0] = ATKBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				ATKs[i] = ATKs[i - 1] + ATKUp;
			}

			Armors[0] = ArmorsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				Armors[i] = Armors[i - 1] + ArmorsUp;
			}

			LevelUpCosts[0] = LevelUpCostBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCosts[i] = LevelUpCosts[i - 1] + LevelUpCostUp;
			}

			LevelUpCostsByUnits[0] = LevelUpCostByUnitsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCostsByUnits[i] = LevelUpCostsByUnits[i - 1] + LevelUpCostByUnitsUp;
			}

			CostsToCreate[0] = CostToCreateBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				CostsToCreate[i] = CostsToCreate[i - 1] + LevelUpCostByUnitsUp;
			}

			for (int i = 0; i < MaxLevel; i++)
			{
				Rewards[i] = (int)(CostsToCreate[i] * 0.32f);
			}

			Skills[0] = new SkillInformation(SkillType.Delay);

			Skills[2] = new SkillInformation(SkillType.Reflection);

			Skills[5] = new SkillInformation(SkillType.Penetration);

			Bullets = new string[] { 
				typeof(SakuyaBullet).FullName,
			};

			SetUnitType(unitType);
		}

		public override void OnLoad(thrts.IFaction faction)
		{
			InitializeParameters();
		}
	}


	public class SakuyaBullet : Bullet.NormalBullet
	{
		public SakuyaBullet()
		{
			Type = thrts.BulletType.LongRange;

			Speed = 10.0f * BasisSpeed;
			Gravity = 0.1f;
			MaxRange = 4 * thrts.Def.MapTipLengthOnField;

			AttackDelay = (int)(0.0f * thrts.Def.FPS);
			AttackRigor = 0;
			AttackSpeed = AttackDelay + (int)(0.5f * thrts.Def.FPS);

			OnHit += OnHitEffect;

			TextureAnimation = new thrts.TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = thrts.CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = thrts.TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
			TextureAnimation.Offset.X = 128 * 1 + 16 * 3;
			TextureAnimation.Offset.Y = 128 * 0 + 16 * 0;
		}
		public static void OnHitEffect(thrts.IBullet bullet, thrts.ITargetObject target)
		{
			EffectUtls.ShowCommonHitEffect(bullet, target, HitEffectType.NormalYellow);
		}
	}
}
