﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PatchSto.Unit
{
	public class Sanae : CommonUnitParameter
	{
		public Sanae()
		{
			const int HPBase = 130;
			const int HPUp = 80;
			const int ATKBase = 32;
			const int ATKUp = 20;
			const int ArmorsBase = 11;
			const int ArmorsUp = 6;

			float SpeedBase = 1.1f;

			const int LevelUpCostBase = 2000;
			const int LevelUpCostUp = 760;

			const int LevelUpCostByUnitsBase = 76;
			const int LevelUpCostByUnitsUp = 76;

			const int CostToCreateBase = 120;

			const PatchSto.UnitType unitType = PatchSto.UnitType.LongRange;

			ChargeTime = 2 * thrts.Def.FPS;

			IconPath = "PatchSto/Image/Button/Unit/Sanae";

			Name = "東風谷早苗";

			Speed = SpeedBase * BasisSpeed;

			UnitUtls.SetAnimation(this, "PatchSto/Image/Unit/Sanae");

			HPs[0] = HPBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				HPs[i] = HPs[i - 1] + HPUp;
			}

			ATKs[0] = ATKBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				ATKs[i] = ATKs[i - 1] + ATKUp;
			}

			Armors[0] = ArmorsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				Armors[i] = Armors[i - 1] + ArmorsUp;
			}

			LevelUpCosts[0] = LevelUpCostBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCosts[i] = LevelUpCosts[i - 1] + LevelUpCostUp;
			}

			LevelUpCostsByUnits[0] = LevelUpCostByUnitsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCostsByUnits[i] = LevelUpCostsByUnits[i - 1] + LevelUpCostByUnitsUp;
			}

			CostsToCreate[0] = CostToCreateBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				CostsToCreate[i] = CostsToCreate[i - 1] + LevelUpCostByUnitsUp;
			}

			for (int i = 0; i < MaxLevel; i++)
			{
				Rewards[i] = (int)(CostsToCreate[i] * 0.32f);
			}

			Skills[2] = new SkillInformation(SkillType.AreaAttack, thrts.Def.MapTipLengthOnField * 1.0f);

			Skills[5] = new SkillInformation(SkillType.Acceleration);

			Bullets = new string[] { 
				typeof(SanaeBullet).FullName,
			};

			SetUnitType(unitType);
		}

		public override void OnLoad(thrts.IFaction faction)
		{
			InitializeParameters();
		}
	}


	public class SanaeBullet : Bullet.NormalBullet
	{
		public SanaeBullet()
		{
			Type = thrts.BulletType.LongRange;

			Speed = 10.0f * BasisSpeed;
			Gravity = 0.1f;
			MaxRange = 6 * thrts.Def.MapTipLengthOnField;

			AttackDelay = (int)(0.0f * thrts.Def.FPS);
			AttackRigor = 0;
			AttackSpeed = AttackDelay + (int)(0.5f * thrts.Def.FPS);

			OnHit += OnHitEffect;

			TextureAnimation = new thrts.TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/sanae";
			TextureAnimation.AlphaBlend = thrts.AlphaBlend.Add;
			TextureAnimation.Width = 128;
			TextureAnimation.Height = 64;
			TextureAnimation.Center.X = 64;
			TextureAnimation.Center.Y = 32;
			TextureAnimation.CenterPosition = thrts.CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 6;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = thrts.TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 2;
		}
		public static void OnHitEffect(thrts.IBullet bullet, thrts.ITargetObject target)
		{
			EffectUtls.ShowCommonHitEffect(bullet, target, HitEffectType.NormalRed);
		}
	}
}
