﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PatchSto.Unit
{
	public class SunnyMilk : CommonUnitParameter
	{
		public SunnyMilk()
		{
			const int HPBase = 130;
			const int HPUp = 90;
			const int ATKBase = 30;
			const int ATKUp = 16;
			const int ArmorsBase = 12;
			const int ArmorsUp = 7;

			float SpeedBase = 1.0f;

			const int LevelUpCostBase = 1500;
			const int LevelUpCostUp = 700;

			const int LevelUpCostByUnitsBase = 70;
			const int LevelUpCostByUnitsUp = 70;

			const int CostToCreateBase = 100;

			const PatchSto.UnitType unitType = PatchSto.UnitType.ShortRange;

			ChargeTime = (int)(1.0f * thrts.Def.FPS);

			IconPath = "PatchSto/Image/Button/Unit/SunnyMilk";

			Name = "サニーミルク";

			Speed = SpeedBase * BasisSpeed;

			UnitUtls.SetAnimation(this, "PatchSto/Image/Unit/SunnyMilk");

			HPs[0] = HPBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				HPs[i] = HPs[i - 1] + HPUp;
			}

			ATKs[0] = ATKBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				ATKs[i] = ATKs[i - 1] + ATKUp;
			}

			Armors[0] = ArmorsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				Armors[i] = Armors[i - 1] + ArmorsUp;
			}

			LevelUpCosts[0] = LevelUpCostBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCosts[i] = LevelUpCosts[i - 1] + LevelUpCostUp;
			}

			LevelUpCostsByUnits[0] = LevelUpCostByUnitsBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				LevelUpCostsByUnits[i] = LevelUpCostsByUnits[i - 1] + LevelUpCostByUnitsUp;
			}

			CostsToCreate[0] = CostToCreateBase;
			for (int i = 1; i < MaxLevel; i++)
			{
				CostsToCreate[i] = CostsToCreate[i - 1] + LevelUpCostByUnitsUp;
			}

			for (int i = 0; i < MaxLevel; i++)
			{
				Rewards[i] = (int)(CostsToCreate[i] * 0.32f);
			}

			Skills[0] = new SkillInformation(SkillType.Reflection);

			Skills[2] = new SkillInformation(SkillType.Evasion);

			Skills[5] = new SkillInformation(SkillType.Acceleration);

			Bullets = new string[] { 
				typeof(SunnyMilkBullet).FullName,
			};

			SetUnitType(unitType);
		}

		public override void OnLoad(thrts.IFaction faction)
		{
			InitializeParameters();
		}
	}

	public class SunnyMilkBullet : Bullet.NormalBullet
	{
		public SunnyMilkBullet()
		{
			Type = thrts.BulletType.ShortRange;
			MaxRange = 0.5f * thrts.Def.MapTipLengthOnField;

			OnHit += OnHitEffect;

			AttackDelay = (int)(0.0f * thrts.Def.FPS);
			AttackRigor = 0;
			AttackSpeed = AttackDelay + (int)(0.5f * thrts.Def.FPS);
		}
		public static void OnHitEffect(thrts.IBullet bullet, thrts.ITargetObject target)
		{
			EffectUtls.ShowCommonHitEffect(bullet, target, HitEffectType.NormalOrange);
		}
	}
}
