using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.Ability
{
	/// <summary>
	/// ĂGtFNg
	/// </summary>
	class FollowEffectAbility : EffectContinuousAbility
	{
		public Vector3D Offset { get; set; }
		public bool StopEffectOnVanish { get; set; }

		public bool IsDirectionChanged { get; set; }
		public float FrontOffset { get; set; }
		public bool StopEffectRootOnVanish { get; set; }

		public FollowEffectAbility()
		{
			IsDirectionChanged = false;

			this.Name = string.Empty;
			this.DurationTime = 0;
			this.Effect = null;
			this.Offset = new Vector3D(0, 0, 0);
			this.StopEffectOnVanish = true;
		}
		public FollowEffectAbility(IEffect effect, Vector3D offset, int duration, string name, bool stopEffect = true)
		{
			this.Name = name;
			this.DurationTime = duration + 2;
			this.Effect = effect;
			this.Offset = offset;
			this.StopEffectOnVanish = stopEffect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case AbilityOrder.OtherExecs:
					if (Effect != null)
					{
						if (FrontOffset == 0.0f)
						{
							Effect.SetPosition(unit.Pos + Offset);
						}
						else
						{
							var front = unit.GetFrontDirection();
							Effect.SetPosition(unit.Pos + Offset + new Vector3D(front.X * FrontOffset, front.Y * FrontOffset, 0));
						}

						if (IsDirectionChanged)
						{
							Effect.SetRotation(new Vector3D(0, 0, unit.GetFrontAngle()));
						}
					}
					break;
			}
			return true;
		}
		public override bool ExecBuildingAbility(thrts.IBuilding building, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case AbilityOrder.OtherExecs:
					if (Effect != null)
						Effect.SetPosition(building.Pos + Offset);
					break;
			}
			return true;
		}
		public override bool ExecBulletAbility(thrts.IBullet bullet, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case AbilityOrder.OtherExecs:
					if (Effect != null)
						Effect.SetPosition(bullet.Position + Offset);
					break;
			}
			return true;
		}
		public override bool OnVanish(IBullet bullet)
		{
			if (StopEffectRootOnVanish && Effect != null)
			{
				Effect.StopRoot();
				Effect = null;
			}
			else if (StopEffectOnVanish && Effect != null)
			{
				Effect.Stop();
				Effect = null;
			}

			return true;
		}
		public override bool OnVanish(IUnit unit)
		{
			if (StopEffectRootOnVanish && Effect != null)
			{
				Effect.StopRoot();
				Effect = null;
			}
			else if (StopEffectOnVanish && Effect != null)
			{
				Effect.Stop();
				Effect = null;
			}

			return true;
		}
		public override bool OnVanish(IBuilding building)
		{
			if (StopEffectOnVanish && Effect != null)
			{
				Effect.Stop();
				Effect = null;
			}

			return true;
		}
	}
}
