using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Common;
using LotusCraft.Ability;
using LotusCraft.Ability.Common;

namespace LotusCraft.Bomb
{
	public class Attack : BombParameter
	{
		const float affectRadius = 5;
		const int duration = 2*Def.FPS;
		const int damage = 40;
		bool used = false;
		// 霊撃 : 周囲の敵に40ダメージ、2秒間無敵
		public Attack ()
		{
			ImageOnSelection = "System/Bomb/bomb_attack";
			DescriptionOnSelection = "霊撃「紫電一閃」\n\n1stヒロインの周囲に範囲ダメージを生じ、\n1stヒロインが短時間のあいだ無敵になる。\n1stヒロインが存在しない場合は使用できない。\n\nコスト\n　　酒150\n効果\n　　即時、敵兵ユニットに40のダメージを与える。防御力による軽減不可\n　　1stヒロイン自身に2秒間、AI・手動問わず敵の攻撃の目標にならない効果\n範囲\n　　1stヒロインの周囲、半径5。";

			Name = "霊撃";
			Costs[0] = 150;
			Description = "周囲の敵に"+damage+"ダメージ、" + duration/Def.FPS + "秒間無敵";
			this.ButtonParameter = new Button.BombButton(this,BombEvent,CanExec,"button_reigeki");

			used = false;

			BombCutin800.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/800/White/white_reigeki");
			BombCutin1280.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/1280/White/white_reigeki");

			BombCutin800.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/800/Red/red_reigeki");
			BombCutin1280.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/1280/Red/red_reigeki");

			BombCutin800.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/800/Green/green_reigeki");
			BombCutin1280.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/1280/Green/green_reigeki");

#if !_TRIAL
			BombCutin800.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/800/Blue/blue_reigeki");
			BombCutin1280.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/1280/Blue/blue_reigeki");

			BombCutin800.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/800/Black/black_reigeki");
			BombCutin1280.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/1280/Black/black_reigeki");
#endif
		}

		public ExecutableInfo CanExec (IFaction faction, BombButtonArguments args)
		{
			var cantExecuteMessage = new List<ColorTextLine> ();
			if (null == faction) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("勢力がnullです", new Color (1, 0, 0))));
			} else if(!args.CreatedOnceForGame) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインが生産されていません", new Color (1, 0, 0))));
			} else if(null == args.FirstHiroin || !args.FirstHiroin.Exists() || args.FirstHiroin.IsEscaping) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインがマップに存在しません", new Color (1, 0, 0))));
			} else if(used) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("ボムは1回しか使えません", new Color (1, 0, 0))));
			} else if (Enumerable.Range (0, Costs.Length).Any (i => faction.GetResource (i) < Costs [i])) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("リソースが足りません", new Color (1, 0, 0))));
			}
			return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
		}
		public void BombEvent(IBomb bomb, BombButtonArguments args, bool flag)
		{
			if (BombAction(args.CreatedOnceForGame, args.FirstHiroin, bomb.Faction))
			{
				if(null != bomb.Faction)
				{
					for(int i=0;i<Costs.Length;++i)
						bomb.Faction.DecResource(i,Costs[i]);
				}
				bomb.ShowCutin();
				used = true;
			}
		}

		public bool BombAction (bool has1stHero, IUnit unit, IFaction faction)
		{
			if (unit == null)
				return false;

			var selfFaction = unit.Faction;
			var world = selfFaction.World;

			var ability = unit.AbilityManager.GetAbilityWithName (AttackAbility.AbilityName);



			if (!unit.AbilityManager.ContainsAbility (AttackAbility.AbilityName)) {
				IEffect effect = null;
				Vector3D offset = new Vector3D (2.0f, 2.0f, 1.0f);
				{
					var e = world.EffectManager.CreateEffect (typeof(AttackBombEffect).FullName, unit.Pos + offset);
					e.SetScale (new Vector3D (5, 5, 5));
					effect = e;
				}
				if (!unit.AbilityManager.AppendAbility (new AttackAbility (effect, offset, duration))) {
					if (effect != null) {
						effect.Stop ();
						effect = null;
					}
				}
			} else {
				ability.DurationTime = duration;
			}
			if (!unit.AbilityManager.ContainsAbility ("attackBombAreaEffect")) {
				IEffect effect = null;
				Vector3D offset = new Vector3D (2.0f, 2.0f, -20.0f);
				{
					var e = world.EffectManager.CreateEffect (typeof(AttackBombAreaEffect).FullName, unit.Pos + offset);
					var effectScale = 6f * affectRadius;
					e.SetScale (new Vector3D (effectScale, effectScale, effectScale));
					effect = e;
				}

				if (!unit.AbilityManager.AppendAbility (new EffectContinuousAbility(effect,duration,"attackBombAreaEffect"))) {
					if (effect != null) {
						effect.Stop ();
						effect = null;
					}
				}
			}

			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Burst).FullName, unit.Pos);

			// get targets
			var targets = world.GetObjectFromCircle(new Circle2D( affectRadius*Def.MapTipLengthOnField,new Vector2D( unit.Pos.X, unit.Pos.Y ))).Where(_ => _ != null);
			foreach (var t in targets.Where(_=>!world.IsSameTeam(selfFaction,_.Faction)))
			{
				var u = t as IUnit;
				var b = t as IBuilding;

				if(null != u)
				{
					// 自分には効果無し
//					if(u.UniqueID == unit.UniqueID)
//						continue;
					// 農民にはダメージ無し
					if(u.Parameter.IsFarmer)
						continue;

					Unit.Utls.ShowHitEffect(t, Unit.Utls.HitEffectType.NormalPurple);
					u.BeDamaged(unit,damage,AttackBonusSet.None,true);
				}
				if(null != b)
				{
					Unit.Utls.ShowHitEffect(t, Unit.Utls.HitEffectType.NormalPurple);
					b.BeDamaged(unit,damage,AttackBonusSet.None,true);
				}
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Common
{
	class AttackAbility : FollowEffectAbility
	{
		public const string AbilityName = "AttackBomb";
		public AttackAbility(IEffect effect,Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.IsntFoundByEnemy = true;
				unit.IsntHitWithBullet = true;
				unit.IsPreventedFromFindingEnemy = true;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool OnVanish(IUnit unit)
		{
			unit.IsntFoundByEnemy = false;
			unit.IsntHitWithBullet = false;
			unit.IsPreventedFromFindingEnemy = false;
			return base.OnVanish(unit);
		}
	}
}