using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Common;
using LotusCraft.Ability;
using LotusCraft.Ability.Common;

namespace LotusCraft.Bomb
{
	public class Increase : BombParameter
	{
		const float affectRadius = 5;
		const int duration = 15*Def.FPS;
		const int interval = 10;
		const string actionAbilityName = "IncreaseBombActionAbility";
		bool used = false;
		// 増力 : 15秒、攻撃力+20%
		public Increase ()
		{
			ImageOnSelection = "System/Bomb/bomb_increase";
			DescriptionOnSelection = "増力「獅子奮迅」\n\n1stヒロインの周囲に居る味方ユニットの攻撃力を上昇させる。\n1stヒロインが存在しない場合は使用できない。\n\nコスト\n　　酒150\n効果\n　　15秒間、味方ユニットの攻撃力を1.25倍にする\n範囲\n　　1stヒロインの周囲、半径5。";

			Name = "増力";
			Costs[0] = 150;
			Description = "15秒、攻撃力+"+IncreaseAbility.IncAttackRatio.ToString("###.%");
			this.ButtonParameter = new Button.BombButton(this,BombEvent,CanExec,"button_zouryoku");
			used = false;

			BombCutin800.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/800/White/white_zouryoku");
			BombCutin1280.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/1280/White/white_zouryoku");

			BombCutin800.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/800/Red/red_zouryoku");
			BombCutin1280.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/1280/Red/red_zouryoku");

			BombCutin800.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/800/Green/green_zouryoku");
			BombCutin1280.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/1280/Green/green_zouryoku");

#if !_TRIAL
			BombCutin800.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/800/Blue/blue_zouryoku");
			BombCutin1280.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/1280/Blue/blue_zouryoku");

			BombCutin800.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/800/Black/black_zouryoku");
			BombCutin1280.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/1280/Black/black_zouryoku");
#endif
		}
		public ExecutableInfo CanExec (IFaction faction, BombButtonArguments args)
		{
			var cantExecuteMessage = new List<ColorTextLine> ();
			if (null == faction) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("勢力がnullです", new Color (1, 0, 0))));
			} else if(!args.CreatedOnceForGame) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインが生産されていません", new Color (1, 0, 0))));
			} else if(null == args.FirstHiroin || !args.FirstHiroin.Exists() || args.FirstHiroin.IsEscaping) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインがマップに存在しません", new Color (1, 0, 0))));
			} else if(used) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("ボムは1回しか使えません", new Color (1, 0, 0))));
			} else if (Enumerable.Range (0, Costs.Length).Any (i => faction.GetResource (i) < Costs [i])) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("リソースが足りません", new Color (1, 0, 0))));
			}
			return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
		}
		public void BombEvent (IBomb bomb, BombButtonArguments args, bool flag)
		{
			if (BombAction (args.CreatedOnceForGame, args.FirstHiroin, bomb.Faction)) {
				if(null != bomb.Faction)
				{
					for(int i=0;i<Costs.Length;++i)
						bomb.Faction.DecResource(i,Costs[i]);
				}

				IEffect effect = null;
                Vector3D offset = new Vector3D(0, 0, -20.0f);
				// エフェクトを描画
				{
					var unit = args.FirstHiroin;
					var selfFaction = unit.Faction;
					var world = selfFaction.World;
					var e = world.EffectManager.CreateEffect (typeof(IncreaseBombAreaEffect).FullName, unit.Pos + offset);
					var effectScale = 6f*affectRadius;
					e.SetScale (new Vector3D (effectScale, effectScale, effectScale));
					effect = e;
				}
				args.FirstHiroin.AbilityManager.AppendAbility(new SpellContinuousAbility((IUnit unit) => BombAction (args.CreatedOnceForGame, args.FirstHiroin, bomb.Faction),effect,offset,duration,actionAbilityName,interval));
				args.FirstHiroin.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, args.FirstHiroin.Pos);

				bomb.ShowCutin();
				used = true;
			}
		}

		public bool BombAction (bool has1stHero, IUnit unit, IFaction faction)
		{
			if(unit == null)
				return false;

			var selfFaction = unit.Faction;
			var world = selfFaction.World;
			// get targets
			var targets = world.GetObjectFromCircle(new Circle2D( affectRadius*Def.MapTipLengthOnField,new Vector2D( unit.Pos.X, unit.Pos.Y ))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>world.IsSameTeam(selfFaction,_.Faction)))
			{
				// 自分には効果無し
//				if(u.UniqueID == unit.UniqueID)
//					continue;

				if(!u.AbilityManager.ContainsAbility(IncreaseAbility.AbilityName))
				{
					IEffectCommon effect = null;
					Vector3D offset = new Vector3D (2.0f, 2.0f, 4.0f);
					if(u != unit)
					{
						// エフェクトを描画
						{
							var e = world.EffectManager.CreateTextureEffect(typeof(IncreaseBombEffect).FullName, unit.Pos + offset);
							effect = e;
						}
					}
					if(!u.AbilityManager.AppendAbility(new IncreaseAbility(effect,offset,interval*2)))
					{
						if(effect != null)
						{
							effect.Stop();
							effect = null;
						}
					}
				}
				else
				{
					u.AbilityManager.ExtendLifetimeWithName(interval,IncreaseAbility.AbilityName,interval*2);
				}
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Common
{
	class IncreaseAbility : AbilityTemplate
	{
		public const float IncAttackRatio = 0.25f;
		public const string AbilityName = "IncreaseBomb";
		public IEffectCommon Effect = null;
		Vector3D offset;
		public IncreaseAbility(IEffectCommon effect,Vector3D offset,int duration)
		{
			this.offset = offset;
			Effect = effect;
			Name = AbilityName;
			DurationTime = duration;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				foreach(var b in unit.ModifiedBulletParameters)
					b.Attack += (int)Math.Ceiling(b.Attack*IncAttackRatio);

				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if (Effect != null)
					Effect.SetPosition(unit.Pos + offset);
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool OnVanish(IUnit unit)
		{
			if (Effect != null)
			{
				Effect.Stop();
				Effect = null;
			}
			return base.OnVanish(unit);
		}
	}
}