using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Common;
using LotusCraft.Ability;
using LotusCraft.Ability.Common;

namespace LotusCraft.Bomb
{
	public class Reinforce : BombParameter
	{
		const float radius = 15;
		bool used = false;
		Type homeType;
		

		// 増援 : 増援ユニットを召喚します
		public Reinforce ()
		{
			ImageOnSelection = "System/Bomb/bomb_reinforce";
			DescriptionOnSelection = "援軍「千軍万馬」\n\n15secの間生存する増援ユニットを召喚する。\n増援ユニットから経験値は得られない。\n\nコスト\n　　酒125\n効果\n　　15秒間、増援ユニット(映姫、傭兵×2)を召喚\n範囲\n　　初期拠点の近くにいる1stヒロインの周囲に出現。\n　　どちらも存在しない場合は使用不可。";

			Name = "援軍";
			Costs[0] = 125;
			Description = "1stヒロインの近くに増援ユニットを召喚。\n初期拠点の近くでしか使えない。";
			this.ButtonParameter = new Button.BombButton(this,BombEvent,CanExec,"button_engun");
			used = false;
			homeType = null;

			BombCutin800.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/800/White/white_engun");
			BombCutin1280.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/1280/White/white_engun");

			BombCutin800.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/800/Red/red_engun");
			BombCutin1280.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/1280/Red/red_engun");

			BombCutin800.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/800/Green/green_engun");
			BombCutin1280.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/1280/Green/green_engun");

#if !_TRIAL
			BombCutin800.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/800/Blue/blue_engun");
			BombCutin1280.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/1280/Blue/blue_engun");

			BombCutin800.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/800/Black/black_engun");
			BombCutin1280.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/1280/Black/black_engun");
#endif
		}
		public ExecutableInfo CanExec (IFaction faction, BombButtonArguments args)
		{
			var cantExecuteMessage = new List<ColorTextLine> ();

			if (homeType == null) {
				if (faction.Parameter.GetType () == typeof(Faction.White))
					homeType = typeof(Building.White.HomeBase);
				else if (faction.Parameter.GetType () ==  typeof(Faction.Red))
					homeType = typeof(Building.Red.HomeBase);
				else if (faction.Parameter.GetType () == typeof(Faction.Green))
					homeType = typeof(Building.Green.HomeBase);
				else if (faction.Parameter.GetType () == typeof(Faction.Blue))
					homeType = typeof(Building.Blue.HomeBase);
				else if (faction.Parameter.GetType () == typeof(Faction.Black))
					homeType = typeof(Building.Black.HomeBase);
				else
					homeType = null;
			}

			if (null == faction) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("勢力がnullです", new Color (1, 0, 0))));
			} else if(!args.CreatedOnceForGame) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインが生産されていません", new Color (1, 0, 0))));
			} else if(null == args.FirstHiroin || !args.FirstHiroin.Exists() || args.FirstHiroin.IsEscaping 
			          || !faction.World.GetObjectFromCircle(new Circle2D(radius*Def.MapTipLengthOnField,new Vector2D(args.FirstHiroin.Pos.X,args.FirstHiroin.Pos.Y)))
			          .OfType<IBuilding>()
			          .Any(_=> _.IsComplete && _.Faction == faction && _.Parameter.GetType() == homeType && _.UserValues.ContainsKey(Building.General.BaseStructure.FirstBuildingKey))) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインが初期拠点の近くに居ません", new Color (1, 0, 0))));
			} else if(used) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("ボムは1回しか使えません", new Color (1, 0, 0))));
			} else if (Enumerable.Range (0, Costs.Length).Any (i => faction.GetResource (i) < Costs [i])) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("リソースが足りません", new Color (1, 0, 0))));
			}
			return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
		}
		public void BombEvent(IBomb bomb, BombButtonArguments args, bool flag)
		{
			if (BombAction(args.CreatedOnceForGame, args.FirstHiroin, bomb.Faction))
			{
				if(null != bomb.Faction)
				{
					for(int i=0;i<Costs.Length;++i)
						bomb.Faction.DecResource(i,Costs[i]);
				}

				bomb.ShowCutin();
				used = true;
			}
		}

		static public bool BombAction (bool has1stHero, IUnit unit, IFaction faction)
		{
			if (null == unit )
				return false;

			Vector2D targetPos = new Vector2D (unit.Pos.X, unit.Pos.Y);

			// summon reinforce
//			var spells = faction.GetSpellWithName(typeof(Spell.Common.EikiSkill).FullName);
//			if(spells != null && !spells.IsEmpty() && spells.All(_=>_.Level < 1))
			{
				/// lvup eiki skill
//				faction.LvUpSpellWithName(typeof(Spell.Common.EikiSkill).FullName);
				
				/// call mercenary units
				var u0 = Utl.PutUnitAroundPoint(faction.World,faction,typeof(Unit.Common.Mercenary01).FullName,targetPos);
//				var u1 = Utl.PutUnitAroundPoint(faction.World,faction,typeof(Unit.Common.Mercenary01).FullName,targetPos);
				var u2 = Utl.PutUnitAroundPoint(faction.World,faction,typeof(Unit.Common.Mercenary02).FullName,targetPos);
//				var u3 = Utl.PutUnitAroundPoint(faction.World,faction,typeof(Unit.Common.Mercenary02).FullName,targetPos);
				var u4 = Utl.PutUnitAroundPoint(faction.World,faction,typeof(Unit.Common.Eiki).FullName,targetPos);
				/*
				if( faction.Parameter.GetType() == typeof(Faction.Green))
				{
					u0.AbilityManager.AppendAbility(new Ability.Green.OnGardenEffectAbility());
//					u1.AbilityManager.AppendAbility(new Ability.Green.OnGardenEffectAbility());
					u2.AbilityManager.AppendAbility(new Ability.Green.OnGardenEffectAbility());
//					u3.AbilityManager.AppendAbility(new Ability.Green.OnGardenEffectAbility());
					u4.AbilityManager.AppendAbility(new Ability.Green.OnGardenEffectAbility());
				}
				//*/
			}

			return true;
		}
	}
}
