using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Common;
using LotusCraft.Ability;
using LotusCraft.Ability.Common;

namespace LotusCraft.Bomb
{
    public class Revive : BombParameter
    {
        const int bonusExp = 0;
        bool used = false;
        // 再生 : 1stヒロインを即時再生産 + 0 EXP
        public Revive()
        {
			ImageOnSelection = "System/Bomb/bomb_revive";
			DescriptionOnSelection = "再生「百折不撓」\n\n1stヒロインが死亡している場合、\n1stヒロインを1stベースから即時に出現させる\n1stヒロインを生産していなかったり、\n1stヒロインが死亡していない場合は使用できない。\n\nコスト\n　　酒0\n効果\n　　即時、1stヒロインの再生産\n範囲\n　　なし";

            Name = "再生";
            Costs[0] = 0;
            Description = "1stヒロインを即時再生産";
            this.ButtonParameter = new Button.BombButton(this, BombEvent, CanExec, "button_saisei");
            used = false;

            BombCutin800.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/800/White/white_saisei");
            BombCutin1280.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/1280/White/white_saisei");

            BombCutin800.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/800/Red/red_saisei");
            BombCutin1280.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/1280/Red/red_saisei");

            BombCutin800.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/800/Green/green_saisei");
            BombCutin1280.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/1280/Green/green_saisei");

#if !_TRIAL
            BombCutin800.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/800/Blue/blue_saisei");
            BombCutin1280.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/1280/Blue/blue_saisei");

            BombCutin800.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/800/Black/black_saisei");
            BombCutin1280.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/1280/Black/black_saisei");
#endif
        }
        public ExecutableInfo CanExec(IFaction faction, BombButtonArguments args)
        {
            var cantExecuteMessage = new List<ColorTextLine>();
            if (null == faction)
            {
                cantExecuteMessage.Add(new ColorTextLine(new ColorText("勢力がnullです", new Color(1, 0, 0))));
            }
            else if (!args.CreatedOnceForGame)
            {
                cantExecuteMessage.Add(new ColorTextLine(new ColorText("1stヒロインが生産されていません", new Color(1, 0, 0))));
            }
            else if (null != args.FirstHiroin && args.FirstHiroin.Exists())
            {
                cantExecuteMessage.Add(new ColorTextLine(new ColorText("1stヒロインがマップに存在します", new Color(1, 0, 0))));
            }
            else if (used)
            {
                cantExecuteMessage.Add(new ColorTextLine(new ColorText("ボムは1回しか使えません", new Color(1, 0, 0))));
            }
            else if (Enumerable.Range(0, Costs.Length).Any(i => faction.GetResource(i) < Costs[i]))
            {
                cantExecuteMessage.Add(new ColorTextLine(new ColorText("リソースが足りません", new Color(1, 0, 0))));
            }
            return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
        }
        public void BombEvent(IBomb bomb, BombButtonArguments args, bool flag)
        {
            if (BombAction(args.CreatedOnceForGame, args.HiroinParameterName, args.FirstHiroin, bomb.Faction))
            {
                if (null != bomb.Faction)
                {
                    for (int i = 0; i < Costs.Length; ++i)
                        bomb.Faction.DecResource(i, Costs[i]);
                }

                bomb.ShowCutin();
                used = true;
            }
        }

        static public bool BombAction(bool has1stHero, string unitKindName, IUnit unit, IFaction faction)
        {
            if (null != unit || faction == null)
                return false;

            var world = faction.World;

            Type homeType;

            IBuilding home = null;
            if (faction.UserValues.ContainsKey(Building.General.BaseStructure.FirstBuildingKey))
            {
                home = faction.UserValues[Building.General.BaseStructure.FirstBuildingKey] as IBuilding;
            }
            if (null == home || !home.Exists())
            {
                if (faction.Parameter.GetType() == typeof(Faction.White))
                    homeType = typeof(Building.White.HomeBase);
                else if (faction.Parameter.GetType() == typeof(Faction.Red))
                    homeType = typeof(Building.Red.HomeBase);
                else if (faction.Parameter.GetType() == typeof(Faction.Green))
                    homeType = typeof(Building.Green.HomeBase);
                else if (faction.Parameter.GetType() == typeof(Faction.Blue))
                    homeType = typeof(Building.Blue.HomeBase);
                else if (faction.Parameter.GetType() == typeof(Faction.Black))
                    homeType = typeof(Building.Black.HomeBase);
                else
                    homeType = null;

                if (null == homeType)
                    return false;

                var homes = world.BuildingManager.GetBuildings(_ => _.IsComplete && _.Faction == faction && _.Parameter.GetType() == homeType);

                home = homes.FirstOrDefault();
            }

            if (null == home)
                return false;


            var u = Utl.PutUnitAroundBuilding(world, faction, unitKindName, home);


            if (null != u)
            {
                u.AddExperience(bonusExp);

                IEffect effect = null;
                Vector3D offset = new Vector3D(2, 2, 1);
                // エフェクトを描画
                {
                    var e = world.EffectManager.CreateEffect(typeof(ReviveBombEffect).FullName, u.Pos + offset);
                    var effectScale = 15.0f;
                    if (e == null)
                        throw new Exception("ReviveBombEffect");
                    e.SetScale(new Vector3D(effectScale, effectScale, effectScale));
                    effect = e;
                }
                u.AbilityManager.AppendAbility(new EffectContinuousAbility(effect, 1 * Def.FPS, "BombReviveeffect"));
                u.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin01).FullName, u.Pos);

            }
            else
            {
                return false;
            }

			world.BuildingManager.GetBuildings(_ => _.IsComplete && _.Faction == faction && _.Parameter == home.Parameter)
				.Where(h => 0 < h.GetCountInProductionQueue(unitKindName))
				.Take(1)
				.ForEach(h => h.CanselCreationOfUnitQueue(unitKindName));

            return true;
        }
    }
}
