using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Common;
using LotusCraft.Ability;
using LotusCraft.Ability.Common;

namespace LotusCraft.Bomb
{
	public class TownPortal : BombParameter
	{
		const float affectRadius = 5;
		const int duration = 15*Def.FPS;
		const int delay = 3*Def.FPS;
		bool used = false;
		// 帰還 : 1stヒロインとその周囲のユニットを最寄りの拠点に転送します
		public TownPortal  ()
		{
			ImageOnSelection = "System/Bomb/bomb_return";
			DescriptionOnSelection = "帰還「神出鬼没」\n\n1stヒロインと自軍のユニットを、最寄りの拠点に瞬間移動させる。\n1stヒロインが存在しない場合は使用できない。\n\nコスト\n　　酒100\n効果\n　　即時、自軍ユニットを最寄りの拠点まで瞬間移動させる\n範囲\n　　1stヒロインの周囲、半径5。";

			Name = "帰還";
			Costs[0] = 100;
			Description = "1stヒロインとその周囲のユニットを最寄りの拠点に転送";
			this.ButtonParameter = new Button.BombButton(this,BombEvent,CanExec,"button_kikan");
			used = false;

			BombCutin800.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/800/White/white_kikan");
			BombCutin1280.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/1280/White/white_kikan");

			BombCutin800.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/800/Red/red_kikan");
			BombCutin1280.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/1280/Red/red_kikan");

			BombCutin800.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/800/Green/green_kikan");
			BombCutin1280.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/1280/Green/green_kikan");

#if !_TRIAL
			BombCutin800.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/800/Blue/blue_kikan");
			BombCutin1280.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/1280/Blue/blue_kikan");

			BombCutin800.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/800/Black/black_kikan");
			BombCutin1280.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/1280/Black/black_kikan");
#endif

		}
		public ExecutableInfo CanExec (IFaction faction, BombButtonArguments args)
		{
			var cantExecuteMessage = new List<ColorTextLine> ();
			if (null == faction) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("勢力がnullです", new Color (1, 0, 0))));
			} else if(!args.CreatedOnceForGame) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインが生産されていません", new Color (1, 0, 0))));
			} else if(null == args.FirstHiroin || !args.FirstHiroin.Exists() || args.FirstHiroin.IsEscaping) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインがマップに存在しません", new Color (1, 0, 0))));
			} else if(used) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("ボムは1回しか使えません", new Color (1, 0, 0))));
			} else if (Enumerable.Range (0, Costs.Length).Any (i => faction.GetResource (i) < Costs [i])) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("リソースが足りません", new Color (1, 0, 0))));
			}
			return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
		}
		public void BombEvent(IBomb bomb, BombButtonArguments args, bool flag)
		{
			if (BombAction(args.CreatedOnceForGame, args.FirstHiroin, bomb.Faction))
			{
				if(null != bomb.Faction)
				{
					for(int i=0;i<Costs.Length;++i)
						bomb.Faction.DecResource(i,Costs[i]);
				}

				bomb.ShowCutin();
				used = true;
			}
		}

		static public bool BombAction (bool has1stHero, IUnit unit, IFaction faction)
		{
			if (null == unit)
				return false;

			if (null == faction)
				faction = unit.Faction;

			var world = faction.World;

			Type homeType;

			if (faction.Parameter.GetType () == typeof(Faction.White))
				homeType = typeof(Building.White.HomeBase);
			else if (faction.Parameter.GetType () ==  typeof(Faction.Red))
				homeType = typeof(Building.Red.HomeBase);
			else if (faction.Parameter.GetType () == typeof(Faction.Green))
				homeType = typeof(Building.Green.HomeBase);
			else if (faction.Parameter.GetType () == typeof(Faction.Blue))
				homeType = typeof(Building.Blue.HomeBase);
			else if (faction.Parameter.GetType () == typeof(Faction.Black))
				homeType = typeof(Building.Black.HomeBase);
			else
				homeType = null;

			var homes = world.BuildingManager.GetBuildings (_ => _.IsComplete && _.Faction == faction && _.Parameter.GetType () == homeType).OrderBy (__ => (__.Pos.X - unit.Pos.X) * (__.Pos.X - unit.Pos.X) + (__.Pos.Y - unit.Pos.Y) * (__.Pos.Y - unit.Pos.Y));

			IBuilding home = homes.FirstOrDefault ();
			
			if (null == home)
				return false;

			IEffect effect = null;
			Vector3D offset = new Vector3D(2, 2, 1);
			// エフェクトを描画
			{
				var selfFaction = unit.Faction;
				var e = world.EffectManager.CreateEffect(typeof(TownPortalBombEffect).FullName, unit.Pos + offset);
				var effectScale = 15.0f;
				e.SetScale(new Vector3D(effectScale, effectScale, effectScale));
				effect = e;
			}

			var targetPos = new Vector2D(home.Pos.X,home.Pos.Y);

			unit.AbilityManager.AppendAbility(new TownPortalAbility(affectRadius,targetPos,effect, delay));
			unit.HoldPosition(false);

			return true;
		}
	}
}

namespace LotusCraft.Ability.Common
{
	class TownPortalAbility : EffectContinuousAbility
	{
		Vector2D targetPos;
		float affectRadius = 4;
		public const string AbilityName = "TownPortalBomb";
		public TownPortalAbility(float affectRadius,Vector2D targetPos,IEffect effect,int duration)
			:base(effect,duration,AbilityName)
		{
			this.affectRadius = affectRadius;
			this.targetPos = targetPos;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				// 確定で停止+押しのけ不可
				unit.Speed = 0;
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool OnVanish (IUnit unit)
		{
			if (DurationTime < 1 && null != unit && unit.Exists ()) {
				unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin02).FullName, unit.Pos);
				
				var faction = unit.Faction;
				var world = faction.World;
				// get targets
				var targets = world.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (unit.Pos.X, unit.Pos.Y))).Where (_ => _ != null);
				foreach (IUnit u in targets.OfType<IUnit>().Where(_=>faction == _.Faction)) {
					// show effect
					
					// warp
					u.Warp(targetPos,true);
				}

			}
			return base.OnVanish(unit);
		}
	}
}