﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Black;

namespace LotusCraft.Building.Black
{
	public class BlackSmith : General.BaseStructure
	{
		public BlackSmith ()
		{
			IconPath = "LotusCraft/Image/Button/Black/Building/BlackSmith";
			Name = "研究所";
            ButtonDescription += "ユニットを強化できる研究所を建築する。\n";
			ShortcutKeyword = "BuildLaboratory";
			Size = 3;

			MakingTime = 25*30;
			Bullets = new string[] { };
			HP = 2500;
			Armor = 2;
			Supply = 10;
			Costs = new [] {125,0,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/Black/BlackSmith";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Laboratory.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			RequireBuildings = new string[]{
				typeof(HomeBase).FullName
			};
		}
		public override void OnLoad(IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new LotusCraft.AutoCast.DissolveAutoCast(),
			};

			var buildingHpTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.BuildingHpLv1Tech()),
			};
			var buildingArmorTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.BuildingArmorLv1Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.BuildingArmorLv2Tech()),
			};
			var infantryArmorTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.InfantryArmorLv1Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.InfantryArmorLv2Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.InfantryArmorLv3Tech()),
			};
			var gunTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.GunLv1Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.GunLv2Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.GunLv3Tech()),
			};
			var cannonArmorTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.CannonArmorLv1Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.CannonArmorLv2Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.CannonArmorLv3Tech()),
			};
			var cannonAtkTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.CannonAtkLv1Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.CannonAtkLv2Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.CannonAtkLv3Tech()),
			};

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding>(buildingArmorTechButtons[0],buildingArmorTechButtons),
				new ButtonParameterHolder<IBuilding>(buildingHpTechButtons[0],buildingHpTechButtons),
				new ButtonParameterHolder<IBuilding>(infantryArmorTechButtons[0],infantryArmorTechButtons),
				new ButtonParameterHolder<IBuilding>(gunTechButtons[0],gunTechButtons),
				new ButtonParameterHolder<IBuilding>(cannonAtkTechButtons[0],cannonAtkTechButtons),
				new ButtonParameterHolder<IBuilding>(cannonArmorTechButtons[0],cannonArmorTechButtons),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			

			TechnologyParams = buildingHpTechButtons
				.Concat(buildingArmorTechButtons)
				.Concat(infantryArmorTechButtons)
				.Concat(gunTechButtons)
				.Concat(cannonArmorTechButtons)
				.Concat(cannonAtkTechButtons)
				.Select(_=>_.TechParam);

		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName,building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}
		public override void OnStartConstruction (IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0,0,1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName,building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale,scale,scale));
			building.Completed += () => {if(effect != null){effect.Stop ();effect=null;}};
			building.Destoroyed += () => {
				if(effect != null)
				{
					effect.Stop ();
					effect=null;
				}
			};
		}
		public override void OnCompleted (IBuilding building)
		{
			if(building.Faction.World.CurrentTime > 10)
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.CreatedBuilding).FullName, building.Pos);
			base.OnCompleted (building);
		}
	}
}
