﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect;
using LotusCraft.Effect.Black;
using LotusCraft.Unit.Black;
using LotusCraft.Bullet.Black;

namespace LotusCraft.Bullet.Black
{
	public class TowerBullet : thrts.BulletParameter
	{
		public TowerBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Both;
			Attack = 20;
			Speed = 10.0f * BasisSpeed;
			Gravity = 0;
			
			MaxRange = 11 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.8f * Def.FPS);
			AttackRigor = (int)(0.0f * Def.FPS);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit += OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/uroko";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			var offset = new Vector3D(-8,-8,3);
			var effect = faction.World.EffectManager.CreateEffect(typeof(HitRemoteWhiteEffect).FullName,faction,target.Pos+offset);
			var scale = new Vector3D(8,8,8);
			effect.SetScale(scale);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
		}
	}
}

namespace LotusCraft.Building.Black
{
	public class Tower : General.BaseStructure
	{
		public Tower ()
		{
			IconPath = "LotusCraft/Image/Button/Black/Building/Tower";
			Name = "塔";
            ButtonDescription += "広い視界を持ち、敵ユニットを自動的に攻撃する塔を建築する。\n";
			ShortcutKeyword = "BuildBulwark";
			Size = 2;
			ViewRange = 14*32;


			MakingTime = 20*Def.FPS;
			Bullets = new string[] { };
			HP = 1000;
			Armor = 2;
			Supply = 5;
			Costs = new [] {100,0,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/Black/Tower";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Tower.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned2.mdl";

			RequireBuildings = new string[]{
				typeof(BlackSmith).FullName,
			};

			Bullets = new string[] { typeof(TowerBullet).FullName };

			Capacity = 4;
			StationableUnits = new string[] {
				typeof(Farmer).FullName,
			};
			Crenelle = 1;
			CanStayedUnitsAttack = false;

		}
		public override void OnLoad (IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new LotusCraft.AutoCast.DissolveAutoCast(),
			};

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.UnStationButton()),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			
		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, faction, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}
		public override void OnStartConstruction (IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0,0,1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName,building.Faction,building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale,scale,scale));
			building.Completed += () => {if(effect != null){effect.Stop ();effect=null;}};
			building.Destoroyed += () => {
				if(effect != null)
				{
					effect.Stop ();
					effect=null;
				}
			};
		}
		public override void OnCompleted (IBuilding building)
		{
			if(building.Faction.World.CurrentTime > 10)
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.CreatedBuilding).FullName, building.Pos);
			building.Updating += () =>{building.Crenelle += building.StayedUnitCount;};
			base.OnCompleted (building);
		}
	}
}
