﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Unit.Blue;
using LotusCraft.Effect.Blue;

namespace LotusCraft.Building.Blue
{
	public class BigTree : General.BaseStructure
	{
		public BigTree()
		{
			IconPath = "LotusCraft/Image/Button/Blue/Building/BigTree";
			Name = "大木";
			ButtonDescription += "中にユニットを格納し、ユニットを保護することができる。\nサニー、ルナチャ、スターの生産に必要である。";
			ShortcutKeyword = "BuildExhibition1";
			Size = 3;

			MakingTime = 75 * Def.FPS;
			HP = 1000;
			Armor = 2;
			Supply = 10;
			Costs = new[] { 150, 100, 0, 0 };

			Animation = new TextureAnimation();
			Animation.Path = "LotusCraft/Image/Building/Blue/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/BigTree.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			Bullets = new string[] { };

			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
			};

			Capacity = 8;
			StationableUnits = new string[] {
				typeof(Farmer).FullName,
				typeof(Low02).FullName,
				typeof(Low03).FullName,
				typeof(Sunny).FullName,
				typeof(Lunar).FullName,
				typeof(Star).FullName,
				typeof(Iku).FullName,
			};
			CanStayedUnitsAttack = true;
		}
		public override void OnLoad(IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new EnqueueProductionUnitAutoCast(typeof(Sunny).FullName,faction.GetUnitParameter(typeof(Sunny).FullName).MakingTime, ButtonPosition._1, ButtonPosition._0),
				new EnqueueProductionUnitAutoCast(typeof(Lunar).FullName,faction.GetUnitParameter(typeof(Lunar).FullName).MakingTime, ButtonPosition._2, ButtonPosition._0),
				new EnqueueProductionUnitAutoCast(typeof(Star).FullName,faction.GetUnitParameter(typeof(Star).FullName).MakingTime, ButtonPosition._3, ButtonPosition._0),
				new DissolveAutoCast(),
			};
			var ShortcutKeywords = new string[]
			{
				"CreateSpecial2",
				"CreateSpecial3",
				"CreateSpecial4",
			};
			/*
			var SunnySkillTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Blue.SunnySkillTech(ButtonPosition._0,ButtonPosition._0)),
			};
			var LunarSkillTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Blue.LunarSkillTech(ButtonPosition._1,ButtonPosition._0)),
			};
			var StarSkillTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Blue.StarSkillTech(ButtonPosition._2,ButtonPosition._0)),
			};
*/
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				/*
				new ButtonParameterHolder<IBuilding> (
					SunnySkillTechButtons[0],SunnySkillTechButtons
					),
				new ButtonParameterHolder<IBuilding> (
					LunarSkillTechButtons[0],LunarSkillTechButtons
					),
				new ButtonParameterHolder<IBuilding> (
					StarSkillTechButtons[0],StarSkillTechButtons
					),
					*/
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[0].Name), AutocasterParams[0].ButtonPosX, AutocasterParams[0].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[0]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[1].Name), AutocasterParams[1].ButtonPosX, AutocasterParams[1].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[1]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[2].Name), AutocasterParams[2].ButtonPosX, AutocasterParams[2].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[2]
					)
					),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.UnStationButton()),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			/*
			TechnologyParams = SunnySkillTechButtons
					.Concat(LunarSkillTechButtons)
					.Concat(StarSkillTechButtons)
					.Select (_=>_.TechParam);
					*/

		}
		public override void OnVanish(IBuilding building)
		{
			building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
			var faction = building.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
			var scale = new Vector3D(25, 25, 25);
			effect.SetScale(scale);
		}

		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}
		public override void OnCompleted(IBuilding building)
		{
			BuildingUtls.ShowComplete(building);
			base.OnCompleted(building);
		}
	}
}
