﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.Building.General
{
	public class ProduceResources<HomeBaseParam, CollectResource> : General.BaseStructure where HomeBaseParam : BuildingParameter
	{
		public const string WorkerPositionsKey = "WorkerPositions";

		Dictionary<int, int> produceResources = new Dictionary<int, int>(){
			{0,5},
			{1,5},
			{2,0},
			{3,0},
		};
		protected float correctPerSec = 0;
		protected float incPerFarmer = 0.25f;
		protected float searchRadius = 9 * Def.MapTipLengthOnField;

		public ProduceResources()
		{
			IconPath = "LotusCraft/Image/Button/ProduceResType0";
			Name = "";
			Size = 2;
			ViewRange = 6 * Def.MapTipLengthOnField;


			MakingTime = 20 * 30;
			Costs = new[] { 50, 0, 0, 0 };
			//			Bullets = new string[] { typeof(ResType0Bullet).FullName };
			HP = 1500;

			Exp = 150;

			EnableBuildingOnResourceKindSequence = new[] { 0 };

			Animation = new TextureAnimation();
			Animation.Path = "LotusCraft/Image/Building/White/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/AlchoholeCollector.mdl";
			//BuildingObjectPath = "LotusCraft/3D/Model/BuildingUnderConstruction2.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned2.mdl";
		}
		public override void OnLoad(IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]{
				new LotusCraft.AutoCast.DissolveAutoCast(),
			};

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};

		}

		public override void OnVanish(IBuilding building)
		{
			building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
		}

		public override void OnDecidePlan (IBuilding building)
		{
			bool ret = false;
			var selfFaction = building.Faction;
			var world = selfFaction.World;

			var homes = world.GetObjectFromCircle (new Circle2D (searchRadius, new Vector2D (building.Pos.X, building.Pos.Y)))
					.OfType<thrts.IBuilding> ()
						.Where (__ => selfFaction == __.Faction && (__.Parameter is HomeBaseParam) && __.GetProgress != 0.0f);
			ret = homes.Count () > 0;
				
			if (!ret && selfFaction.Parameter.GetType () == typeof(Faction.Green)) {
				var minoriko = world.GetObjectFromCircle (new Circle2D (searchRadius, new Vector2D (building.Pos.X, building.Pos.Y)))
						.OfType<IUnit> ()
							.Where (unit => null != unit && unit.Exists () && unit.Faction == selfFaction && unit.Parameter.GetType () == typeof(Unit.Green.Unique.Minoriko));
				if (minoriko.Any ()) {
					if (EnableBuildingOnResourceKindSequence.Contains (0))
						ret = minoriko.All (u => u.GetSpellWithName (Spell.Green.Owotoshi.SpellName).Level > 2);
						
				}
			}

			if (ret)
				base.OnDecidePlan (building);
			else
				building.CancelProgress ();
		}
		public override void OnCompleted(IBuilding building)
		{
			IEffect effect = null;

			int counter = 999;
			if (true)
			{
				if (effect == null)
				{
					effect = building.Faction.World.EffectManager.CreateEffect(
						typeof(CollectResource).FullName,
						new Vector3D(building.Pos.X, building.Pos.Y, building.Pos.Z + 1),
						building.Faction);
				}

			}

			// 収集者
			ITargetObject collectingCenter = null;

			// 外部に収集人数を持たす。
			building.UserValues.Add("WorkerCount", 0);
			building.UserValues.Add("PreviousWorkerCount", 0);
			building.WorkerCountRequired = building.Parameter.Capacity;

			AddWorkerPositions(building);

			var sulvageAutocast = building.GetIAutoCastWithName(AutoCast.DissolveAutoCast.AutoCastName);
			building.Updating += () =>
			{
				// 収集人数の取得
				int workerCount = (int)building.UserValues["WorkerCount"];
				workerCount = Math.Max(0, workerCount);
				workerCount += building.StayedUnitCount;

				// 人数の設定
				building.WorkerCount = workerCount;
				building.WorkerCountRequired = building.Parameter.Capacity;

				workerCount = Math.Min(workerCount, building.WorkerCountRequired);

				// とりあえず、1回5資源回収するものと仮定
				int resourcePerOccurrence = 5;

				// 回収対象が居るなら修正
				if( null != building.CollectingResource && produceResources.ContainsKey(building.CollectingResource.ResourceType))
				{
					resourcePerOccurrence = produceResources[building.CollectingResource.ResourceType];
				}

				float interval = (float)(resourcePerOccurrence * Def.FPS / (correctPerSec + incPerFarmer * workerCount));


				// 本拠地の検索
				if (EnableBuildingOnResourceKindSequence != null && (++counter) > interval)
				{
					counter = 0;
					bool ret = false;
					var selfFaction = building.Faction;
					var world = selfFaction.World;
					if (null != sulvageAutocast && !sulvageAutocast.IsEnable)
					{
						var homes = world.GetObjectFromCircle(new Circle2D(searchRadius, new Vector2D(building.Pos.X, building.Pos.Y)))
							.OfType<thrts.IBuilding>()
							.Where(__ => selfFaction == __.Faction && __.IsComplete && (__.Parameter is HomeBaseParam) );
						ret = homes.Count() > 0;

						if (ret)
						{
							collectingCenter = homes.First();
						}

						if (!ret && selfFaction.Parameter.GetType() == typeof(Faction.Green))
						{
							var minoriko = world.GetObjectFromCircle(new Circle2D(searchRadius, new Vector2D(building.Pos.X, building.Pos.Y)))
								.OfType<IUnit>()
								.Where(unit => null != unit && unit.Exists() && unit.Faction == selfFaction && unit.Parameter.GetType() == typeof(Unit.Green.Unique.Minoriko));
							if (minoriko.Any())
							{
								if (EnableBuildingOnResourceKindSequence.Contains(0))
									ret = minoriko.All(u => u.GetSpellWithName(Spell.Green.Owotoshi.SpellName).Level > 2);

							}

							if (ret)
							{
								collectingCenter = minoriko.First();
							}
						}
					}

					if (ret)
					{
						CollectResourceFromResource(building, ref effect, selfFaction, world, interval, collectingCenter);
					}
					else
					{
						counter = 0;
						if (effect != null)
						{
							effect.Stop();
							effect = null;
						}
						building.IsCollectingResources = false;
						building.SpeedToCorrectResource = 0;
						collectingCenter = null;
					}

				}

				// 人数リセット
				building.UserValues["PreviousWorkerCount"] = building.UserValues["WorkerCount"];

				if (building.CollectingResource != null && building.CollectingResource.Exists())
				{
					building.UserValues["WorkerCount"] = 0;
				}
				else
				{
					building.UserValues["WorkerCount"] = -1;
				}
			};

			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};

			base.OnCompleted(building);
		}

		private void CollectResourceFromResource(IBuilding building, ref IEffect effect, IFaction selfFaction, IWorld world, float interval, ITargetObject collectingCenter)
		{
			// 資源探索
			if (building.CollectingResource == null || !building.CollectingResource.Exists())
			{
				float radius = 0.05f * Def.MapTipLengthOnField;
				thrts.RectF rect = new thrts.RectF(building.Pos.X - radius, building.Pos.Y - radius, 2 * radius, 2 * radius);

				foreach (var r in world.GetObjectFromRectangle(rect, rect).OfType<thrts.IResource>())
				{
					if (r != null && r.Exists() && EnableBuildingOnResourceKindSequence.Any(_ => _ == r.ResourceType))
					{
						building.CollectingResource = r;
						break;
					}
				}
			}

			if (building.CollectingResource != null && building.CollectingResource.Exists())
			{
				int resType = building.CollectingResource.ResourceType;
				{
					if (EnableBuildingOnResourceKindSequence.Contains(resType))
					{

						int production = building.CollectingResource.Parameter.CollectPerOccurrence;
						/*
						if (building.HP < building.MaxHP)
						{
							// HPに応じて資源回収力を減少
							int dedcut = (int)Math.Ceiling((production * (building.MaxHP - building.HP)) / (float)building.MaxHP);
							production -= dedcut;
						}
						*/
						// 資源収集
						selfFaction.IncResource(resType, building.CollectingResource.ReduceHP(production));

						// 資源収集エフェクト
						{
							if (resType == 0 && collectingCenter != null)
							{
								Vector3D direction;
								direction.X = collectingCenter.Pos.X - building.Pos.X;
								direction.Y = collectingCenter.Pos.Y - building.Pos.Y;
								direction.Z = collectingCenter.Pos.Z - building.Pos.Z + 64.0f;

								direction.X /= 20.0f;
								direction.Y /= 20.0f;
								direction.Z /= 20.0f;

								building.Faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.AlcoholIconEffect).FullName, building.Pos, direction);
							}

							if (resType == 1 && collectingCenter != null)
							{
								Vector3D direction;
								direction.X = collectingCenter.Pos.X - building.Pos.X;
								direction.Y = collectingCenter.Pos.Y - building.Pos.Y;
								direction.Z = collectingCenter.Pos.Z - building.Pos.Z + 64.0f;

								direction.X /= 20.0f;
								direction.Y /= 20.0f;
								direction.Z /= 20.0f;

								building.Faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ReiryokuIconEffect).FullName, building.Pos, direction);
							}
						}

						// 資源収集速度計算
						building.SpeedToCorrectResource = production / ((float)interval / (float)Def.FPS);
						if (effect != null)
						{
							effect.SetScale(new Vector3D(1.0f, 1.0f, building.SpeedToCorrectResource / 4.0f));
						}
					}
				}

				if (effect == null)
				{
					effect = building.Faction.World.EffectManager.CreateEffect(
						typeof(CollectResource).FullName,
						new Vector3D(building.Pos.X, building.Pos.Y, building.Pos.Z + 1),
						building.Faction);
				}

				building.IsCollectingResources = true;
			}
			else
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}

				building.IsCollectingResources = false;
				building.SpeedToCorrectResource = 0;
			}
		}

		void AddWorkerPositions(IBuilding building)
		{
			if (building.UserValues.ContainsKey(WorkerPositionsKey)) return;
			float offset = 12.0f;
			float size = building.Size * Def.MapTipLengthOnField;

			Vector2D[] positions = new Vector2D[8];

			positions[0] = new Vector2D(building.Pos.X + (size / 2 + offset), building.Pos.Y + (size / 2 + offset));
			positions[1] = new Vector2D(building.Pos.X - (size / 2 + offset), building.Pos.Y + (size / 2 + offset));
			positions[2] = new Vector2D(building.Pos.X + (size / 2 + offset), building.Pos.Y - (size / 2 + offset));
			positions[3] = new Vector2D(building.Pos.X - (size / 2 + offset), building.Pos.Y - (size / 2 + offset));

			positions[4] = new Vector2D(building.Pos.X + (size / 2 + offset), building.Pos.Y);
			positions[5] = new Vector2D(building.Pos.X - (size / 2 + offset), building.Pos.Y);
			positions[6] = new Vector2D(building.Pos.X, building.Pos.Y + (size / 2 + offset));
			positions[7] = new Vector2D(building.Pos.X, building.Pos.Y - (size / 2 + offset));

			building.UserValues.Add(WorkerPositionsKey, positions);
		}
	}
}
