﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Unit.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Building.Green
{
	public class Cherry : General.BaseStructure
	{
		public Cherry ()
		{
			IconPath = "LotusCraft/Image/Button/Green/Building/Cherry";
			Name = "桜";
			ButtonDescription += "\n妖精(岩)の生産に必要である。";
			ShortcutKeyword = "BuildWorkshop";
			Size = 3;

			MakingTime = 35*Def.FPS;
			HP = 800;
			Armor = 2;
			Supply = 5;
			Costs = new [] {100,0,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/Green/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Cherry.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			Bullets = new string[] {  };

			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
			};
		}
		public override void OnLoad(IFaction faction)
		{
			var BoostTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Green.MetabolicBoostTech(ButtonPosition._0,ButtonPosition._0)),
			};
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new DissolveAutoCast(),
			};
			
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (BoostTechButtons[0],BoostTechButtons),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			TechnologyParams = BoostTechButtons.Select(_=>_.TechParam).ToArray();
			
		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}

		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted(IBuilding building)
		{
            BuildingUtls.ShowComplete(building);
			
			base.OnCompleted (building);
		}
	}
}
