﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Unit.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Building.Green
{
	public class Factory : General.BaseStructure
	{
		public Factory ()
		{
			IconPath = "LotusCraft/Image/Button/Green/Building/Factory";
			Name = "上級魔法陣";
            ButtonDescription += "特殊ユニットを作成する魔法陣を建築する。\n";
			Size = 3;

			MakingTime = 50*30;
			Bullets = new string[] { };
			HP = 3200;
			Armor = 2;
			Supply = 10;
			Costs = new [] {150,100,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/Green/Factory";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/MagicCircleHigh.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			
			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
			};
		}
		public override void OnLoad (IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new LotusCraft.AutoCast.DissolveAutoCast(),
			};

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};

		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}
		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted (IBuilding building)
		{
			IEffect effect = null;

			building.Updating += () =>
			{
				if (effect == null)
				{
					effect = building.Faction.World.EffectManager.CreateEffect(
							typeof(Effect.Green.Factories).FullName,
							new Vector3D(building.Pos.X, building.Pos.Y, building.Pos.Z + 16));
				}
			};

			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
				}
			};

            BuildingUtls.ShowComplete(building);
			base.OnCompleted (building);
		}
	}
}
