﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Ability.Green;
using LotusCraft.Unit.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Building.Green
{
	public class Higanbana : General.BaseStructure
	{
		public Higanbana ()
		{
			IconPath = "LotusCraft/Image/Button/Green/Building/Higanbana";
			Name = "彼岸花";
			ButtonDescription += "\n無縁塚・彼岸花から自由にユニットが出入りできる。";
            Size = 3;

			MakingTime = 20*Def.FPS;
			HP = 400;
			Armor = 2;
			Supply = 0;
			Costs = new [] {100,100,0,0};
			ViewRange = 10*Def.MapTipLengthOnField;

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/Green/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Higanbana.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			Bullets = new string[] {  };

			RequireBuildings = new string[]{
				typeof(Muenzuka).FullName,
			};

			IsBuildedInOnlyFieldOfVision = true;
		}
		public override void OnLoad(IFaction faction)
		{
			///  全ユニットが駐留可能
			StationableUnits = faction.Parameter.Units;
			Capacity = 100;

			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new DissolveAutoCast(),
			};
			
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
                new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.UnStationButton(false)),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};

		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}

		public override void OnDecidePlan (IBuilding building)
		{
			base.OnDecidePlan (building);
			//  予約が確定したら建築を開始
			building.StartConstruction();
		}

		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted(IBuilding building)
		{
			if (building.Faction.UserValues.ContainsKey(MuenzukaStayAbility.MuenzukaStayAbilityKey))
			{
				var ability = building.Faction.UserValues[MuenzukaStayAbility.MuenzukaStayAbilityKey] as MuenzukaStayAbility;
				ability.AppendBuilding(building);
			}

			BuildingUtls.ShowComplete(building);
			
			base.OnCompleted (building);
		}
	}
}
