﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Green;
using LotusCraft.AutoCast;
using LotusCraft.Unit.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Building.Green
{
	public class Muenzuka : General.BaseStructure
	{
		public Muenzuka()
		{
			IconPath = "LotusCraft/Image/Button/Green/Building/Muenduka";
			Name = "無縁塚";
			ButtonDescription += "\n妖精(水)の生産に必要である。\n無縁塚と繋がり自由にユニットが出入りできる彼岸花を作成出来る。";
			ShortcutKeyword = "BuildAtelier";
			Size = 3;

			MakingTime = 50 * Def.FPS;
			HP = 1200;
			Armor = 2;
			Supply = 10;
			Costs = new[] { 75, 50, 0, 0 };

			Animation = new TextureAnimation();
			Animation.Path = "LotusCraft/Image/Building/Green/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Muenzuka.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			Bullets = new string[] { };

			RequireBuildings = new string[]{
				typeof(Cherry).FullName,
			};
		}
		public override void OnLoad(IFaction faction)
		{
			///  全ユニットが駐留可能
			StationableUnits = faction.Parameter.Units;
			Capacity = 100;

			var LongBowTechButtons = new[]{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Green.LongBow(ButtonPosition._0,ButtonPosition._0)),
			};
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new DissolveAutoCast(),
			};

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new thrts.ButtonParameterHolder<thrts.IBuilding>(new LotusCraft.Button.CreateBuildingByBuildingButton(
					faction.GetBuildingParameter(typeof(Higanbana).FullName),
					ButtonPosition._0,
					ButtonPosition._2,
					shortcutKeyword : "BuildSpecial1"
					)),
				new ButtonParameterHolder<IBuilding> (LongBowTechButtons[0],LongBowTechButtons),
                new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.UnStationButton(false)),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			TechnologyParams = LongBowTechButtons.Select(_ => _.TechParam).ToArray();

		}
		public override void OnVanish(IBuilding building)
		{
			building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
			var faction = building.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
			var scale = new Vector3D(25, 25, 25);
			effect.SetScale(scale);
		}

		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted(IBuilding building)
		{

			if (!building.Faction.UserValues.ContainsKey(MuenzukaStayAbility.MuenzukaStayAbilityKey))
			{
				var a = new MuenzukaStayAbility();
				if (building.AbilityManager.AppendAbility(a))
					building.Faction.UserValues.Add(MuenzukaStayAbility.MuenzukaStayAbilityKey, a);
			}

			if (building.Faction.UserValues.ContainsKey(MuenzukaStayAbility.MuenzukaStayAbilityKey))
			{
				var ability = building.Faction.UserValues[MuenzukaStayAbility.MuenzukaStayAbilityKey] as MuenzukaStayAbility;
				ability.AppendBuilding(building);
			}


            BuildingUtls.ShowComplete(building);

			base.OnCompleted(building);
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class MuenzukaStayAbility : thrts.AbilityTemplate
	{
		public const string MuenzukaStayAbilityKey = "mouenzuka_key";
		public LinkedList<IUnit> StayUnits;
		public LinkedList<IBuilding> Buildings;
		public const string AbilityName = "無縁塚";
		public MuenzukaStayAbility()
		{
			this.Name = AbilityName;
			this.DurationTime = 1;
			StayUnits = new LinkedList<IUnit>();
			Buildings = new LinkedList<IBuilding>();

		}

		public override bool OnVanish(IUnit unit)
		{
			/// round robin ability
			if (null != Buildings && Buildings.Any())
			{
				foreach (var b in Buildings)
					if (b != null && b.Exists() && b.AbilityManager.AppendAbility(this))
						return base.OnVanish(unit);
			}

			/// destroy if all vanished

			/// destroy units
			if (StayUnits != null)
			{
				foreach (var u in StayUnits)
				{
					if (u.Exists())
						u.HP = -1;
				}
			}
			/// destroy building
			if (Buildings != null)
			{
				foreach (var b in Buildings)
				{
					if (b.Exists())
						b.HP = -1;
				}
			}
			return base.OnVanish(unit);
		}
		public override bool OnVanish(IBuilding building)
		{
			/// round robin ability
			if (null != Buildings && Buildings.Any())
			{
				foreach (var b in Buildings)
				{
					if (b != null && b.Exists() && b.AbilityManager.AppendAbility(this))
					{
						DurationTime = 1;
						return base.OnVanish(building);
					}
				}
			}

			/// destroy if all vanished

			/// destroy units
			if (StayUnits != null)
			{
				foreach (var u in StayUnits)
				{
					if (u.Exists())
						u.HP = -1;
				}
			}
			/// destroy sukima
			if (Buildings != null)
			{
				foreach (var b in Buildings)
				{
					if (b.Exists())
						b.HP = -1;
				}
			}
			return base.OnVanish(building);
		}

		// inf ability
		public override void DecDurationTime()
		{
			return;
		}

		public void AppendBuilding(IBuilding building)
		{
			if (building == null || !building.Exists())
				return;

			if (null != building && !Buildings.Contains(building))
			{
				Buildings.AddLast(building);
				building.UnitStayed += OnStayUnit;
				building.UnitReleased += OnReleaseUnit;
				building.Destoroyed += () => OnDestroyBuilding(building);
				foreach (var u in StayUnits)
				{
					building.StayUnit(u);
				}
			}
		}
		public void OnStayUnit(IBuilding building, IUnit unit)
		{
			if (building == null || !building.Exists() || !Buildings.Contains(building))
				return;

			if (null != unit && unit.Exists() && !StayUnits.Contains(unit))
			{
				StayUnits.AddLast(unit);
				foreach (var b in Buildings)
				{
					if (b != null && b.Exists())
						if (building.UniqueID != b.UniqueID)
							b.StayUnit(unit);
				}
			}
		}
		public void OnReleaseUnit(IBuilding building, IUnit unit)
		{
			if (building == null || !building.Exists() || !Buildings.Contains(building))
				return;

			if (null != unit && StayUnits.Contains(unit))
			{
				StayUnits.Remove(unit);
				foreach (var b in Buildings)
				{
					if (b != null && b.Exists())
						if (building.UniqueID != b.UniqueID)
							b.RemoveStayedUnit(unit);
				}
			}
		}
		public void OnDestroyBuilding(IBuilding building)
		{
			if (building == null || !Buildings.Contains(building))
				return;

			/// roundrobin stayed units
			var unitList = StayUnits.ToArray();

			/// remove sukima list
			if (Buildings.Contains(building))
				Buildings.Remove(building);

			/// seach roundrobin units
			var targets = Buildings.Where(_ => null != _ && _.Exists() && _.UniqueID != building.UniqueID);

			if (targets.Any())
			{
				var nextBuilding = targets.First();
				foreach (var u in unitList)
					building.MigrateStayedUnit(nextBuilding, u);
			}
			else
			{
				// kill all units
				foreach (var u in unitList)
				{
					building.RemoveStayedUnit(u);
					u.HP = 0;
					if (building.Faction.UserValues.ContainsKey(MuenzukaStayAbility.MuenzukaStayAbilityKey))
					{
						building.Faction.UserValues.Remove(MuenzukaStayAbility.MuenzukaStayAbilityKey);
					}
				}
			}
		}
	}

}
