﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Unit.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Building.Green
{
	public class Tower : General.BaseStructure
	{
		public Tower ()
		{
			IconPath = "LotusCraft/Image/Button/Green/Building/Tower";
			Name = "塔";
            ButtonDescription += "広い視界を持ち、敵ユニットを自動的に攻撃する塔を建築する。\n";
			ShortcutKeyword = "BuildBulwark";
			Size = 2;
			ViewRange = 14*Def.MapTipLengthOnField;


			MakingTime = 25*30;
			Bullets = new string[] { };
			HP = 500;
			Armor = 0;
			Supply = 0;
			Costs = new [] {125,25,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/Green/Tower";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Tower.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned2.mdl";

			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
				typeof(BlackSmith).FullName,
			};

			Bullets = new string[] { typeof(TowerBullet).FullName };

			Crenelle = 1;

		}
		public override void OnLoad (IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new LotusCraft.AutoCast.DissolveAutoCast(),
			};

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			
		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}
		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted (IBuilding building)
		{
            BuildingUtls.ShowComplete(building);
			base.OnCompleted (building);
		}
	}
}

namespace LotusCraft.Bullet.Green
{
	public class TowerBullet :General.LongRangeBullet
	{
		public TowerBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Sky;
			Attack = 5;
			Speed = 10.0f * BasisSpeed;
			Gravity = 0;
			
			MaxRange = 7 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackRigor = (int)(0.0f * Def.FPS);
			AttackSpeed = (int)(0.4f * Def.FPS);

			OnHit += OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/uroko";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
            LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target);
		}
	}
}
