﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.AutoCast;
using LotusCraft.Unit.White;
using LotusCraft.Unit.White.Unique;
using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;

namespace LotusCraft.Building.White
{
	public class HomeBase : General.BaseStructure
	{
		static string defenceUnitKindName = typeof(Unit.Common.Erin).FullName;
		public HomeBase ()
		{
			IconPath = "LotusCraft/Image/Button/White/Building/HomeBase";
			Name = "拠点";
			ShortcutKeyword = "BuildHomeBase";
			Size = 4;
			ViewRange = 16*32;

			Height = 16.0f;

			ButtonDescription = "作業者、ヒロインの生産を行い、\n周囲の資源回収施設から資源を回収できる。";

			MakingTime = 100*Def.FPS;
			Bullets = new string[] { };
			HP = 4000;
			Armor = 2;
			Supply = 25;
			Costs = new [] {400,0,0,0};
            Capacity = 10;//この建物に何人駐留できるか

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/White/HomeBase";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			ShortcutKeyword = "Home";

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/HomeBase.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned4.mdl";

		}

		public override void OnLoad (IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new EnqueueProductionUnitAutoCast(typeof(Farmer).FullName,faction.GetUnitParameter(typeof(Farmer).FullName).MakingTime, ButtonPosition._0, ButtonPosition._0),
			};

			var ShortcutKeywords = new string[]
			{
				"CreateWorker",
				"CreateHeroine1",
				"CreateHeroine2",
				"CreateHeroine3",
				"CreateHeroine4",
				"CreateHeroine6",
				"CreateHeroine5",
			};

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[0].Name), AutocasterParams[0].ButtonPosX, AutocasterParams[0].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[0]
					)
				),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(typeof(Reimu).FullName), ButtonPosition._0, ButtonPosition._1,
					shortcutKeyword : ShortcutKeywords[1]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(typeof(Marisa).FullName), ButtonPosition._1, ButtonPosition._1,
					shortcutKeyword : ShortcutKeywords[2]
					)
					),
#if !_TRIAL
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(defenceUnitKindName), ButtonPosition._1, ButtonPosition._0,
					shortcutKeyword: "CreateDefender"
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(typeof(Eirin).FullName), ButtonPosition._2, ButtonPosition._1,
					shortcutKeyword : ShortcutKeywords[3]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(typeof(Hijiri).FullName), ButtonPosition._0, ButtonPosition._2,
					shortcutKeyword : ShortcutKeywords[4]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(typeof(Akyu).FullName), ButtonPosition._2, ButtonPosition._2,
					shortcutKeyword : ShortcutKeywords[5]
					)
					),
				//*
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(typeof(Ichirin).FullName), ButtonPosition._1, ButtonPosition._2,
					shortcutKeyword : ShortcutKeywords[6]
					)
					),
				//*/
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveHomeBaseButton()),
#endif
			};
			

		}
		public override void OnVanish(IBuilding building)
		{
			building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
			var faction = building.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
			var scale = new Vector3D(40, 40, 40);
			effect.SetScale(scale);
			/*
			var defenceUnitParam = building.Faction.GetUnitParameter(typeof(Unit.Common.Erin).FullName);
			if (null != defenceUnitParam)
			{
				int homeBaseNum = building.Faction.World.BuildingManager.GetBuildings(_ => null != _ && _.IsComplete && _.Faction == building.Faction && _.Parameter.GetType() == building.Parameter.GetType()).Count();

				if (building.IsComplete)
				{
					defenceUnitParam.MaximumExists = Math.Max(homeBaseNum - 1, 1) * 2;
				}
				else
				{
					defenceUnitParam.MaximumExists = Math.Max(homeBaseNum, 1) * 2;
				}
			}
			*/
		}

		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted (IBuilding building)
		{
			IEffect effect = null;
			/*
			var defenceUnitParam = building.Faction.GetUnitParameter (defenceUnitKindName);
			if (null != defenceUnitParam) {
				int homeBaseNum = building.Faction.World.BuildingManager.GetBuildings (_ => null != _ && _.IsComplete && _.Faction == building.Faction && _.Parameter.GetType () == building.Parameter.GetType ()).Count ();
				defenceUnitParam.MaximumExists = Math.Max (homeBaseNum,1) * 2;
			}
*/
			building.Updating += () =>
				{
					if (effect == null)
					{
						effect = building.Faction.World.EffectManager.CreateEffect(
								typeof(Effect.White.Stronghold).FullName,
								new Vector3D(building.Pos.X, building.Pos.Y, building.Pos.Z + 16));
					}
				};

			building.Destoroyed += () =>
				{
					if (effect != null)
					{
						effect.Stop();
					}
				};

			BuildingUtls.ShowComplete(building);
			
			base.OnCompleted (building);
		}
	}
}
namespace LotusCraft.Bullet.White
{
	public class HomeBaseBullet : General.LongRangeBullet
	{
		public HomeBaseBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Both;
			

			this.Attack = 6;
			DamageBonus[(int)TargetObjectType.Hero] = 3f;

			Speed = 5.0f * BasisSpeed;

			MaxRange = 9*Def.MapTipLengthOnField;

			AttackDelay = 0;
			AttackRigor = 0;
			AttackSpeed = (int)(1.5f * 30);

			OnHit = OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/arrow";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 8;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalBlue);
		}
	}
}

