﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Effect.White;

namespace LotusCraft.Building.White
{
	public class Omoikane_lv1 : OmoikaneDevice
	{
		public Omoikane_lv1()
			: base()
		{
			affectRadius = 3.0f;
			healRate = 0.01f;
			//  heal 30 times
			counter = 30;

			#region AutoCast
			AutocasterParams = new thrts.AutoCastParameter<thrts.IBuilding>[]
			{
				new LotusCraft.AutoCast.AutoCast<thrts.IBuilding>(this.Name,Def.FPS,AutoCast,true),
			};
			#endregion
		}
		public override bool AutoCast(thrts.IBuilding self)
		{
			return base.AutoCast(self);
		}

	}
	public class Omoikane_lv2 : OmoikaneDevice
	{
		public Omoikane_lv2()
			: base()
		{
			affectRadius = 3.0f;
			healRate = 0.02f;
			//  heal 30 times
			counter = 30;

			#region AutoCast
			AutocasterParams = new thrts.AutoCastParameter<thrts.IBuilding>[]
			{
				new LotusCraft.AutoCast.AutoCast<thrts.IBuilding>(this.Name,Def.FPS,AutoCast,true),
			};
			#endregion
		}
		public override bool AutoCast(thrts.IBuilding self)
		{
			return base.AutoCast(self);
		}

	}
	public class Omoikane_lv3 : OmoikaneDevice
	{
		public Omoikane_lv3()
			: base()
		{
			affectRadius = 4.0f;
			healRate = 0.03f;
			//  heal 30 times
			counter = 30;

			#region AutoCast
			AutocasterParams = new thrts.AutoCastParameter<thrts.IBuilding>[]
			{
				new LotusCraft.AutoCast.AutoCast<thrts.IBuilding>(this.Name,Def.FPS,AutoCast,true),
			};
			#endregion
		}
		public override bool AutoCast(thrts.IBuilding self)
		{
			return base.AutoCast(self);
		}
	}

	public class OmoikaneDevice : thrts.BuildingParameter
	{
		protected float affectRadius = 1;
		protected float healRate = 0.02f;
		protected int counter = 9999;
		public OmoikaneDevice()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/White/Omoikane";
			#endregion

			#region BuildingParameter
			Name = "オモイカネデバイス";
			Size = 1;
			ViewRange = 4*Def.MapTipLengthOnField;


			HP = 160;
			#endregion

			#region Animation
			Animation = new thrts.TextureAnimation();
			Animation.Path = "LotusCraft/Image/Building/White/Omoikane";
			Animation.Width = 64;
			Animation.Height = 76;
			Animation.Center.X = 0;
			Animation.Center.Y = -16;
			Animation.CenterPosition = thrts.CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = thrts.TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Omoikane.mdl";
			//BuildingObjectPath = "LotusCraft/3D/Model/BuildingUnderConstruction1.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned1.mdl";
			#endregion

			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new thrts.ButtonParameterHolder<thrts.IBuilding>(new LotusCraft.Button.AutoButton<thrts.IBuilding>("オモイカネ",thrts.ButtonPosition._0,thrts.ButtonPosition._2,"White/Spell/OmoikaneDevice/0")),
			};

			#endregion

			#region Bullets
			Bullets = new string[0];
			#endregion
		}

		public virtual bool AutoCast(thrts.IBuilding self)
		{
			/// 自勢力の取得
			var selfFaction = self.Faction;

			///  area effect
			string effectAbilityName = this.Name + "effect";

			var building = self;

			if (!building.AbilityManager.ContainsAbility(effectAbilityName))
			{
				var effect = building.Faction.World.EffectManager.CreateEffect(typeof(OmoikaneAreaEffect).FullName, building.Pos + new Vector3D(0, 0, 4), building.Faction);
				if (effect != null)
				{
					var scale = affectRadius * 5;
					effect.SetScale(new Vector3D(scale, scale, scale));
					var sAbility = new SuicideAbility(effect, counter * Def.FPS, effectAbilityName);
					sAbility.StopEffectRootOnVanish = true;
					if (!building.AbilityManager.AppendAbility(sAbility))
					{
						effect.Stop();
						effect = null;
					}
				}
			}
			/// 周囲のユニットの検索
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(self.Pos.X, self.Pos.Y)))
				.Where(_ => _ != null && selfFaction.World.IsSameTeam(selfFaction, _.Faction))
				.OfType<IUnit>();

			foreach (thrts.IUnit u in targets)
			{
				if (u.HP >= u.MaxHP)
					continue;

				// add effect
				var effect = self.Faction.World.EffectManager.CreateEffect(typeof(OmoikaneHealEffect).FullName, u.Pos, selfFaction);
				if (!u.AbilityManager.AppendAbility(new FollowEffectAbility(effect, new Vector3D(0, 0, 4), 1 * Def.FPS, this.Name + "HealEffect")))
					effect.Stop();

				// heal
				u.HP += (int)Math.Ceiling(u.MaxHP * healRate);


			}


			return true;
		}

		public override void OnCompleted(IBuilding building)
		{
			IEffect effect;

			//			building.Destoroyed += () => {if(effect != null) effect.Stop();};

			base.OnCompleted(building);
		}

	}
}

#endif