﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Bullet.White;
using LotusCraft.Unit.White;
using LotusCraft.Effect.White;

namespace LotusCraft.Building.White
{
	public class Seirensen : General.BaseStructure
	{
		public Seirensen ()
		{
			IconPath = "LotusCraft/Image/Button/White/Building/Seirensen";
			Name = "星蓮船";
			ButtonDescription += "周囲の地上敵ユニットに攻撃を行う。\n村紗の生産に必要である。";
			ShortcutKeyword = "BuildExhibition2";
			Size = 3;

			MakingTime = 75*Def.FPS;
			HP = 1000;
			Armor = 2;
			Supply = 15;
			Costs = new [] {100,100,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/White/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Seirensen.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			Bullets = new string[] { typeof(SeirensenBullet).FullName, };

			Crenelle = 1;

			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
			};
		}
		public override void OnLoad(IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new EnqueueProductionUnitAutoCast(typeof(Murasa).FullName,faction.GetUnitParameter(typeof(Murasa).FullName).MakingTime, ButtonPosition._2, ButtonPosition._0),
				new DissolveAutoCast(),
			};
			
//			var MurasaSkillTechButtons = new []{
//				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.White.MurasaSkillTech(ButtonPosition._0,ButtonPosition._0)),
//			};

			var ShortcutKeywords = new string[]
			{
				"CreateSpecial3",
			};
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
//				new ButtonParameterHolder<IBuilding> (
//					MurasaSkillTechButtons[0],MurasaSkillTechButtons
//					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[0].Name), AutocasterParams[0].ButtonPosX, AutocasterParams[0].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[0]
					)
					),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};

//			TechnologyParams = MurasaSkillTechButtons
//					.Select (_=>_.TechParam);
		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}

		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted(IBuilding building)
		{
			BuildingUtls.ShowComplete(building);
			
			base.OnCompleted (building);
		}
	}
}
namespace LotusCraft.Bullet.White
{
	public class SeirensenBullet : thrts.BulletParameter
	{
		public SeirensenBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			Attack = 40;
			Speed = 10.0f * BasisSpeed;
			Gravity = 0;
			
			MaxRange = 6 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(1.0f * Def.FPS);
			AttackRigor = (int)(3.0f * Def.FPS);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit += OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/anchor";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			var offset = new Vector3D(-8,-8,3);
			var effect = faction.World.EffectManager.CreateEffect(typeof(Effect.Common.HitRemoteWhiteEffect).FullName, target.Pos + offset);
			var scale = new Vector3D(8,8,8);
			effect.SetScale(scale);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
		}
	}
}
