﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using LotusCraft.Spell;
using thrts;

namespace LotusCraft.Button
{
	public class SpellButton<ButtonHolder> : thrts.ButtonParameter<ButtonHolder> where ButtonHolder : thrts.IUnit
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="LotusCraft.Button.SpellButton`1"/> class.
		/// </summary>
		/// <param name='spell'>
		/// Spell.
		/// </param>
		/// <param name='spellButtonImagePath'>
		/// Spell button image path.
		/// </param>
		/// <param name='spellAction'>
		/// Spell action.
		/// </param>
		public SpellButton(
			SpellParameter<ButtonHolder> spell,
			string spellButtonImagePath,
			int lv)
		{
			Position = Tuple.Create(spell.ButtonPosX, spell.ButtonPosY);
			ButtonImagePath = "LotusCraft/Image/Button/" + spellButtonImagePath;
			ButtonName = spell.Name;
			SpellName = spell.Name;
			SpellLevel = lv;
			
			if (Position.Item2 == ButtonPosition._2) {
				if (Position.Item1 == ButtonPosition._0) {
					ShortcutKeyword = "UseSpell1";
				} else if (Position.Item1 == ButtonPosition._1) {
					ShortcutKeyword = "UseSpell2";
				} else if (Position.Item1 == ButtonPosition._2) {
					ShortcutKeyword = "UseSpell3";
				} else if (Position.Item1 == ButtonPosition._3) {
					ShortcutKeyword = "UseSpell4";
				} else if (Position.Item1 == ButtonPosition._4) {
					ShortcutKeyword = "UseSpell5";
				}
			}else if (Position.Item2 == ButtonPosition._3) {
				if (Position.Item1 == ButtonPosition._0) {
					ShortcutKeyword = "UseSpell6";
				} else if (Position.Item1 == ButtonPosition._1) {
					ShortcutKeyword = "UseSpell7";
				} else if (Position.Item1 == ButtonPosition._2) {
					ShortcutKeyword = "UseSpell8";
				} else if (Position.Item1 == ButtonPosition._3) {
					ShortcutKeyword = "UseSpell9";
				} else if (Position.Item1 == ButtonPosition._4) {
					ShortcutKeyword = "UseSpell10";
				}
			}

			string buttonDescription = string.Empty;
			
			if (spell.CoolTimes != null && spell.CoolTimes.Length < lv)
				buttonDescription += "　クールタイム: " + (spell.CoolTimes[lv] / 30).ToString() + "s" + "\n";
			
			if (lv > 0 && spell.EffectDescriptions.Length > lv - 1)
				buttonDescription += spell.EffectDescriptions[lv - 1] + "\n";
			buttonDescription += spell.Description;
			
			var text = buttonDescription.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select(_ => new ColorText(_))
					.Select(_ => new ColorTextLine(_))
					.ToList();
			MouseOverTextFunc = faction => text;
			Kind = ButtonKind.Dummy;
			CanExecute = (_, __, unit) =>
			{
				var cantExecuteMessage = new List<ColorTextLine>();
				
				var tmp = unit.GetSpellWithName(spell.Name);
				if (tmp == null)
				cantExecuteMessage.Add(new ColorTextLine(new ColorText("スペルを実行できません", new Color(1, 0, 0))));
				else if (!tmp.CanExec)
				{
					if (tmp.GetCoolTimeRate() != 0.0)
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("スペルの準備が終わっていません", new Color(1, 0, 0))));
					}
					else
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("スペルを使用できません", new Color(1, 0, 0))));
					}
				}
				return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
			};
		}

		public SpellButton (
			SpellParameter<ButtonHolder> spell,
			string spellButtonImagePath,
			int lv,
			ButtonPosition x,
			ButtonPosition y)
		{
			Position = Tuple.Create (x, y);
			ButtonImagePath = "LotusCraft/Image/Button/" + spellButtonImagePath;
			ButtonName = spell.Name;
			SpellName = spell.Name;
			SpellLevel = lv;
			
			if (Position.Item1 == ButtonPosition._2) {
				if (Position.Item1 == ButtonPosition._0) {
					ShortcutKeyword = "UseSpell1";
				} else if (Position.Item1 == ButtonPosition._1) {
					ShortcutKeyword = "UseSpell2";
				} else if (Position.Item1 == ButtonPosition._2) {
					ShortcutKeyword = "UseSpell3";
				} else if (Position.Item1 == ButtonPosition._3) {
					ShortcutKeyword = "UseSpell4";
				} else if (Position.Item1 == ButtonPosition._4) {
					ShortcutKeyword = "UseSpell5";
				}
			}else if (Position.Item1 == ButtonPosition._3) {
				if (Position.Item1 == ButtonPosition._0) {
					ShortcutKeyword = "UseSpell6";
				} else if (Position.Item1 == ButtonPosition._1) {
					ShortcutKeyword = "UseSpell7";
				} else if (Position.Item1 == ButtonPosition._2) {
					ShortcutKeyword = "UseSpell8";
				} else if (Position.Item1 == ButtonPosition._3) {
					ShortcutKeyword = "UseSpell9";
				} else if (Position.Item1 == ButtonPosition._4) {
					ShortcutKeyword = "UseSpell10";
				}
			}

			string buttonDescription = string.Empty;
			
			if (spell.CoolTimes != null && spell.CoolTimes.Length < lv)
				buttonDescription += "　クールタイム: " + (spell.CoolTimes[lv] / 30).ToString() + "s" + "\n";
			
			if (lv > 0 && spell.EffectDescriptions.Length > lv - 1)
				buttonDescription += spell.EffectDescriptions[lv - 1] + "\n";
			buttonDescription += spell.Description;
			
			var text = buttonDescription.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select(_ => new ColorText(_))
					.Select(_ => new ColorTextLine(_))
					.ToList();
			MouseOverTextFunc = faction => text;
			Kind = ButtonKind.Dummy;
			CanExecute = (_, __, unit) =>
			{
				var cantExecuteMessage = new List<ColorTextLine>();
				
				var tmp = unit.GetSpellWithName(spell.Name);
				if (tmp == null)
				cantExecuteMessage.Add(new ColorTextLine(new ColorText("スペルを実行できません", new Color(1, 0, 0))));
				else if (!tmp.CanExec)
				{
					if (tmp.GetCoolTimeRate() != 0.0)
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("スペルの準備が終わっていません", new Color(1, 0, 0))));
					}
					else
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("スペルを使用できません", new Color(1, 0, 0))));
					}
				}
				return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
			};
		}

		public SpellButton(
			SpellParameter<ButtonHolder> spell,
			string spellButtonImagePath,
			Func<ButtonHolder, IEnumerable<object>> spellAction)
			: this(spell, spellButtonImagePath, 1)
		{
			Kind = ButtonKind.Normal;
			NormalButtonEvent = (unit, isAppend) =>
			{
				unit.SetPrivilegeMode(() => spellAction(unit), isAppend);
			};
		}
		public SpellButton(SpellParameter<ButtonHolder> spell,
		                   string spellButtonImagePath,
		                   Func<ButtonHolder, int, IEnumerable<object>> spellAction,
		                   int level)
			: this(spell, spellButtonImagePath, level)
		{
			Kind = ButtonKind.Normal;
			NormalButtonEvent = (unit, isAppend) =>
			{
				unit.SetPrivilegeMode(() => spellAction(unit, level), isAppend);
			};
		}
		
		public SpellButton(SpellParameter<ButtonHolder> spell,
		                   string spellButtonImagePath,
		                   Func<ButtonHolder, int, IEnumerable<object>> spellAction,
		                   int level,
		                   ButtonPosition x,
		                   ButtonPosition y)
			: this(spell, spellButtonImagePath, level)
		{
			Kind = ButtonKind.Normal;
			NormalButtonEvent = (unit, isAppend) =>
			{
				unit.SetPrivilegeMode(() => spellAction(unit, level), isAppend);
			};
		}
		
		public SpellButton(SpellParameter<ButtonHolder> spellParam,
		                   string spellButtonImagePath,
		                   Func<ButtonHolder, int, IEnumerable<object>> spellAction,
		                   Action<ButtonHolder, int> OnFinish,
		                   int level)
			: this(spellParam, spellButtonImagePath, level)
		{
			Kind = ButtonKind.Normal;
			NormalButtonEvent = (unit, isAppend) =>
			{
				if(null == unit)
					return ;
				var spell = unit.GetSpellWithName(spellParam.Name);
				if(null == spell || !spell.CanExec)
					return;

				unit.SetPrivilegeMode(() => spellAction(unit, level), () => OnFinish(unit, level), isAppend);
			};
		}
		
		public SpellButton(SpellParameter<ButtonHolder> spellParam,
		                   string spellButtonImagePath,
		                   Func<ButtonHolder, ISpell, IEnumerable<object>> spellAction,
		                   Action<ButtonHolder, ISpell> OnFinish,
		                   int level)
			: this(spellParam, spellButtonImagePath, level)
		{
			Kind = ButtonKind.Normal;
			NormalButtonEvent = (unit, isAppend) =>
			{
				if(null == unit)
					return ;

				var spell = unit.GetSpellWithName(spellParam.Name);
				// check max range
				if(null == spell || !spell.CanExec)
					return;
				unit.SetPrivilegeMode(() => spellAction(unit, spell), () => OnFinish(unit, spell), isAppend);
			};
		}

		public SpellButton(SpellParameter<ButtonHolder> spellParam,
			string spellButtonImagePath,
			Func<ButtonHolder, thrts.IUnit, thrts.IBuilding, int, IEnumerable<object>> spellAction,
			int level,
			bool selectEnemy = true,
			bool selectFrend = false)
			: this(spellParam, spellButtonImagePath, level)
		{
			if (selectEnemy && !selectFrend)
				Kind = ButtonKind.SelectEnemy;
			if (selectFrend && !selectEnemy)
				Kind = ButtonKind.SelectFriend;
			if(selectEnemy && selectFrend)
				Kind = ButtonKind.SelectObject;

			SelectUnitBuildingButtonEvent = (self, units, buildings, isAppend) =>
			{
				if(null == self)
					return;
				/// ターゲット取得
				var uTarget = units.FirstOrDefault();
				var bTarget = buildings.FirstOrDefault();
				ITargetObject target;
				if (uTarget != null)
					target = uTarget;
				else if (bTarget != null)
					target = bTarget;
				else
					target = null;

				/// スペル射程範囲の取得のために現在のスペルインスタンスを取得
				var spell = self.GetSpellWithName(spellParam.Name);
				if(null == spell || !spell.CanExec)
					return;
				// check max range
//				if(null == spell || null == self || null == target || Utl.GetDistance(target.Pos.X,target.Pos.Y,self.Pos.X,self.Pos.Y) > spell.MaxRange )
//				return;
				
				// look at target
				self.ChangeDirection(new Vector2D(target.Pos.X,target.Pos.Y) );
				/// スペル処理の設定
				var maxRange = spell.MaxRange + self.Parameter.Radius;
//				var maxRange = self.Faction.World.Map.Count*Def.MapTipLengthOnField*0.707f;
				self.SetPrivilegeMode(() => spellAction(self, uTarget, bTarget, level), target, spell.MinRange, maxRange, isAppend);
			};

			if(null != spellParam.MaxRanges && spellParam.MaxRanges.Length > level)
			{
				var area = new ButtonEffectAroundTarget ();
				area.Type = ButtonEffectAroundTargetType.Circle;
				area.Radius = spellParam.MaxRanges[level];
				
				GetButtonEffectAroundSelfFunc = (u) =>
				{
					return area;
				};
			}
		}
		public SpellButton(SpellParameter<ButtonHolder> spellParam,
			string spellButtonImagePath,
			Func<ButtonHolder, Vector2D, int, IEnumerable<object>> spellAction,
			Action<ButtonHolder, int> onFinishSpellAction,
			int level)
			: this(spellParam, spellButtonImagePath, level)
		{
			Kind = ButtonKind.SelectPlace;
			SelectPlaceButtonEvent = (self, pos, isAppend) =>
			{
				if(null == self)
					return;
				/// スペル射程範囲の取得のために現在のスペルインスタンスを取得
				var spell = self.GetSpellWithName(spellParam.Name);
				if(null == spell || !spell.CanExec)
					return;

				// check max range
//				if(null == spell || null == self || Utl.GetDistance(pos.X,pos.Y,self.Pos.X,self.Pos.Y) > spell.MaxRange )
//				return;
				
				// look at target
				self.ChangeDirection(pos);
				/// スペル処理の設定
				var maxRange = spell.MaxRange + self.Parameter.Radius;
//				var maxRange = self.Faction.World.Map.Count*Def.MapTipLengthOnField*0.707f;
				self.SetPrivilegeMode(() => spellAction(self, pos, level), () => onFinishSpellAction(self, level), pos, spell.MinRange, maxRange, isAppend);
			};

			if(null != spellParam.MaxRanges && spellParam.MaxRanges.Length > level)
			{
				var area = new ButtonEffectAroundTarget ();
				area.Type = ButtonEffectAroundTargetType.Circle;
				area.Radius = spellParam.MaxRanges[level];
				
				GetButtonEffectAroundSelfFunc = (u) =>
				{
					return area;
				};
			}
		}

		public SpellButton(SpellParameter<ButtonHolder> spellParam,
		                   string spellButtonImagePath,
		                   Func<ButtonHolder, Vector2D, ISpell, IEnumerable<object>> spellAction,
		                   Action<ButtonHolder, ISpell> onFinishSpellAction,
		                   int level)
			: this(spellParam, spellButtonImagePath, level)
		{
			Kind = ButtonKind.SelectPlace;
			SelectPlaceButtonEvent = (self, pos, isAppend) =>
			{
				if(null == self)
					return;
				/// スペル射程範囲の取得のために現在のスペルインスタンスを取得
				var spell = self.GetSpellWithName(spellParam.Name);
				if(null == spell || !spell.CanExec)
					return;

				// check max range
//				if(null == spell || null == self || Utl.GetDistance(pos.X,pos.Y,self.Pos.X,self.Pos.Y) > spell.MaxRange )
//				return;
				
				// look at target
				self.ChangeDirection(pos);
				/// スペル処理の設定
				var maxRange = spell.MaxRange + self.Parameter.Radius;
//				var maxRange = self.Faction.World.Map.Count*Def.MapTipLengthOnField*0.707f;
				self.SetPrivilegeMode(() => spellAction(self, pos, spell), () => onFinishSpellAction(self, spell), pos, spell.MinRange, maxRange, isAppend);
			};

			if(null != spellParam.MaxRanges && spellParam.MaxRanges.Length > level)
			{
				var area = new ButtonEffectAroundTarget ();
				area.Type = ButtonEffectAroundTargetType.Circle;
				area.Radius = spellParam.MaxRanges[level];
				
				GetButtonEffectAroundSelfFunc = (u) =>
				{
					return area;
				};
			}
		}

		/// <summary>
		/// スペル用ボタンのコンストラクタ
		/// 一定距離に近づき、メソッドを実行する。
		/// 一定距離はSpellParameter.MinRanges, SpellParameter.MaxRangesにて指定する。
		/// 距離はユニットの中心からの距離である。そのため、ユニットの半径を考慮する必要がある。
		/// </summary>
		/// <param name="spellParam">スペルカードのパラメータ</param>
		/// <param name="spellButtonImagePath">ボタン画像</param>
		/// <param name="spellAction">スペルとして実行するメソッド</param>
		/// <param name="level">このスペルのレベル</param>
		public SpellButton(
			SpellParameter<ButtonHolder> spellParam,
			string spellButtonImagePath,
			Func<ButtonHolder, Vector2D, int, IEnumerable<object>> spellAction,
			int level)
			: this(spellParam, spellButtonImagePath, level)
		{
			Kind = ButtonKind.SelectPlace;
			SelectPlaceButtonEvent = (self, pos, isAppend) =>
			{
				if(null == self)
					return;
				/// スペル射程範囲の取得のために現在のスペルインスタンスを取得
				var spell = self.GetSpellWithName(spellParam.Name);
				if(null == spell || !spell.CanExec)
					return;

				// check max range
//				if(null == spell || null == self || Utl.GetDistance(pos.X,pos.Y,self.Pos.X,self.Pos.Y) > spell.MaxRange )
//				return;
				
				// look at target
				self.ChangeDirection(pos);

				/// スペル処理の設定
				var maxRange = spell.MaxRange + self.Parameter.Radius;
//				var maxRange = self.Faction.World.Map.Count*Def.MapTipLengthOnField*0.707f;
				self.SetPrivilegeMode(() => spellAction(self, pos, level), pos, spell.MinRange, maxRange, isAppend);
			};

			if(null != spellParam.MaxRanges && spellParam.MaxRanges.Length > level)
			{
				var area = new ButtonEffectAroundTarget ();
				area.Type = ButtonEffectAroundTargetType.Circle;
				area.Radius = spellParam.MaxRanges[level];

				GetButtonEffectAroundSelfFunc = (u) =>
				{
					return area;
				};
			}
		}
		public SpellButton(
			SpellParameter<ButtonHolder> spellParam,
			string spellButtonImagePath,
			Func<ButtonHolder, Vector2D, ISpell, IEnumerable<object>> spellAction,
			int level)
			: this(spellParam, spellButtonImagePath, level)
		{
			Kind = ButtonKind.SelectPlace;
			SelectPlaceButtonEvent = (self, pos, isAppend) =>
			{
				if(null == self)
					return;
				/// スペル射程範囲の取得のために現在のスペルインスタンスを取得
				var spell = self.GetSpellWithName(spellParam.Name);
				if(null == spell || !spell.CanExec)
					return;

				// check max range
//				if(null == spell || null == self || Utl.GetDistance(pos.X,pos.Y,self.Pos.X,self.Pos.Y) > spell.MaxRange )
//				return;
				
				// look at target
				self.ChangeDirection(pos);
				
				/// スペル処理の設定
				var maxRange = spell.MaxRange + self.Parameter.Radius;
//				var maxRange = self.Faction.World.Map.Count*Def.MapTipLengthOnField*0.707f;
				self.SetPrivilegeMode(() => spellAction(self, pos, spell), pos, spell.MinRange, maxRange, isAppend);
			};
			
			if(null != spellParam.MaxRanges && spellParam.MaxRanges.Length > level)
			{
				var area = new ButtonEffectAroundTarget ();
				area.Type = ButtonEffectAroundTargetType.Circle;
				area.Radius = spellParam.MaxRanges[level];
				
				GetButtonEffectAroundSelfFunc = (u) =>
				{
					return area;
				};
			}
		}

		public SpellButton(SpellParameter<ButtonHolder> spellParam,
			string spellButtonImagePath,
			Func<ButtonHolder, float, float,int, IEnumerable<object>> spellAction,
			int level)
			: this(spellParam, spellButtonImagePath, level)
		{
			Kind = ButtonKind.SelectPlace;
			SelectPlaceButtonEvent = (self, pos, isAppend) =>
			{
				if(null == self)
					return;
				/// スペル射程範囲の取得のために現在のスペルインスタンスを取得
				var spell = self.GetSpellWithName(spellParam.Name);
				if(null == spell || !spell.CanExec)
					return;

				// check max range
//				if(null == spell || null == self || Utl.GetDistance(pos.X,pos.Y,self.Pos.X,self.Pos.Y) > spell.MaxRange )
//					return;

				// look at target
				self.ChangeDirection(pos);

				/// スペル処理の設定
				var maxRange = spell.MaxRange + self.Parameter.Radius;
//				var maxRange = self.Faction.World.Map.Count*Def.MapTipLengthOnField*0.707f;
				self.SetPrivilegeMode(() => spellAction(self, pos.X, pos.Y,level), pos, spell.MinRange, maxRange, isAppend);
			};

			if(null != spellParam.MaxRanges && spellParam.MaxRanges.Length > level)
			{
				var area = new ButtonEffectAroundTarget ();
				area.Type = ButtonEffectAroundTargetType.Circle;
				area.Radius = spellParam.MaxRanges[level];
				
				GetButtonEffectAroundSelfFunc = (u) =>
				{
					return area;
				};
			}
		}

		/// <summary>
		/// 建物建設系スペルボタン
		/// </summary>
		/// <param name="spellParam"></param>
		/// <param name="spellButtonImagePath"></param>
		/// <param name="spellAction"></param>
		public SpellButton(SpellParameter<ButtonHolder> spellParam,
			string spellButtonImagePath,
			Func<ButtonHolder, bool, string> spellAction, int lv = 1)
			: this(spellParam, spellButtonImagePath, 1)
		{
			Kind = ButtonKind.CreateBuilding;
			CreateBuildingEvent = (ButtonHolder h,bool f)=>
			{
				if(null == h)
					return string.Empty;
				/// スペル射程範囲の取得のために現在のスペルインスタンスを取得
				var spell = h.GetSpellWithName(spellParam.Name);
				if(null == spell || !spell.CanExec)
					return string.Empty;

				return spellAction(h,f);
			};

			EndBuilding = (holder, isAppend) =>
			{
				if (holder as thrts.IUnit != null)
				{
					var unit = holder as IUnit;
					var spell = unit.GetSpellWithName(spellParam.Name);
					spell.ResetCoolTime();
				}
				else if (holder as thrts.IBuilding != null)
				{
					var unit = holder as IBuilding;
					var spell = unit.GetSpellWithName(spellParam.Name);
					spell.ResetCoolTime();
				}
			};
		}
	}


	public class LevelUpSpellButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public LevelUpSpellButton(
			SpellParameter<thrts.IUnit> spellParam,
			string spellButtonImagePath,
			Action<IUnit> onLevelUp = null)
		{
			Position = Tuple.Create(spellParam.LvUpButtonPosX, spellParam.LvUpButtonPosY);
			ButtonImagePath = "LotusCraft/Image/Button/" + spellButtonImagePath;

			if (spellParam.ButtonPosX == ButtonPosition._0)
			{
				ShortcutKeyword = "LvUpSpell1";
			}
			else if (spellParam.ButtonPosX == ButtonPosition._1)
			{
				ShortcutKeyword = "LvUpSpell2";
			}
			else if (spellParam.ButtonPosX == ButtonPosition._2)
			{
				ShortcutKeyword = "LvUpSpell3";
			}
			if (spellParam.ButtonPosX == ButtonPosition._3)
			{
				ShortcutKeyword = "LvUpSpell4";
			}

			ButtonName = spellParam.Name+" のレベルアップ";
			string description = string.Empty;
			if (spellParam.Description != null)
				description += spellParam.Description + "\n";
			var maxDesciptionLength = spellParam.EffectDescriptions.Max(_ => _.Length);
			for (int lv = 1; lv <= spellParam.MaxLevel; lv++)
			{
				description += lv.ToString() + " : "
					+ spellParam.EffectDescriptions[lv - 1].PadRight(maxDesciptionLength)
					+ (spellParam.RequireLevels[lv - 1] > 0 ? " [必要LV" + spellParam.RequireLevels[lv - 1] + "] " : "")
						+ "\n";
			}

			var text = description.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
					.Select(_ => new ColorText(_))
					.Select(_ => new ColorTextLine(_))
					.ToList();
			MouseOverTextFunc = faction => text;
			Kind = ButtonKind.Normal;

			NormalButtonEvent = (unit, isAppend) =>
			{
				unit.LvUpSpellWithName(spellParam.Name);
				if (onLevelUp != null)
				{
					onLevelUp(unit);
				}
			};

			CanExecute = (_, __, unit) =>
			{
				var cantExecuteMessage = new List<ColorTextLine>();
				// レベルアップ可否チェック
				var tmp = unit.GetSpellWithName(SpellParameter<thrts.IUnit>.LevelHolderName);
				if (tmp == null || tmp.Level < 1 )
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("レベルアップ不可です", new Color(1, 0, 0))));
				// 準備時間チェック
				if ( unit.Faction.World.IsInPreparatoryPeriod())
				cantExecuteMessage.Add(new ColorTextLine(new ColorText("準備時間中です", new Color(1, 0, 0))));
				// 最大レベルチェック
				tmp = unit.GetSpellWithName(spellParam.Name);
				if (tmp == null || tmp.MaxLevel <= tmp.Level)
				cantExecuteMessage.Add(new ColorTextLine(new ColorText("既に最大レベルです", new Color(1, 0, 0))));
				// 要求レベルチェック
				var spell = unit.GetSpellWithName(spellParam.Name);
				if (spell != null && spell.RequireLevel > unit.GetLevel())
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("必要LVに足りていません", new Color(1, 0, 0))));

				return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
			};
		}
	}
	public class SpellPointButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public SpellPointButton(SpellParameter<thrts.IUnit> spell,
			string buttonImagePath)
		{
			Position = Tuple.Create(spell.ButtonPosX, spell.ButtonPosY);
			ButtonImagePath = "LotusCraft/Image/Button/" + buttonImagePath;
			ButtonName = spell.Name;
			string buttonDescription = string.Empty;

			var text = buttonDescription.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select(_ => new ColorText(_))
				.Select(_ => new ColorTextLine(_))
				.ToList();
			MouseOverTextFunc = _ => text;
			Kind = ButtonKind.Dummy;
//			CanExecute = (_, __, unit) => new ExecutableInfo(false, "ほげ");
		}
	}
	public class CancelLevelUpSpellButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public CancelLevelUpSpellButton(SpellParameter<thrts.IUnit> spell,
			string buttonImagePath)
		{
			Position = Tuple.Create(spell.ButtonPosX, spell.ButtonPosY);
			ButtonImagePath = "LotusCraft/Image/Button/" + buttonImagePath;
			MouseOverTextFunc = _ =>
			(
				new[]
				{
					new ColorTextLine
						(
							new ColorText("キャンセル")
						)
				}
			);
			Kind = ButtonKind.Dummy;
		}
	}
}
