﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Black
{
	class OrinSkill:thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 5;
		const int interval = 10;
		public OrinSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			IsPassive = true;
			Name = "「死灰復燃」";
			string basePath = "Black/Spell/OrinSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = 4*30;
			CoolTimes[1] = 4*30;

			Description = "攻撃がヒットしたユニットは、死亡時に爆発する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"死亡時に爆発する効果を付与、ダメージ60(スプラッシュ有り)。",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (!selfUnit.Exists ())
				return false;

			if (!selfUnit.AbilityManager.ContainsAbility (OrinSkillAbility.AbilityName)) {
				selfUnit.AbilityManager.AppendAbility (new OrinSkillAbility ());
			} else {
				selfUnit.AbilityManager.ExtendLifetimeWithName(interval,OrinSkillAbility.AbilityName,15);
			}

			return true;
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			yield return false;
		}
	}
}


namespace LotusCraft.Ability.Black
{
	class OrinSkillAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		const int affectRadius = 2;
		public const string AbilityName = "「死灰復燃」";
		public OrinSkillAbility(IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = 15;
			this.effect = effect;
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
		public override bool ModifyBullet (IBullet bullet)
		{
			bullet.AddOnHitEvent(OnHitEffect);
			return base.ModifyBullet (bullet);
		}

		void OnHitEffect (IBullet bullet, ITargetObject target)
		{
			var faction = bullet.Generator.Faction;

			var tu = target as IUnit;
			if(tu != null && tu.Exists())
			{
				if(!bullet.Generator.Faction.World.IsSameTeam(bullet.Generator.Faction,tu.Faction))
				{
					if(tu.AbilityManager.ContainsAbility(OrinSkillBombAbility.AbilityName))
					{
						var ability = tu.AbilityManager.GetAbilityWithName(OrinSkillBombAbility.AbilityName);
						if(null != ability)
						{
							ability.DurationTime = Def.FPS/2;
						}
						else
						{
							tu.AbilityManager.RemoveAbility(OrinSkillBombAbility.AbilityName,false);
							if(!tu.AbilityManager.AppendAbility(new OrinSkillBombAbility()))
								;
						}
					}
					else if(!tu.AbilityManager.AppendAbility(new OrinSkillBombAbility(Def.FPS/2)))
					{
						;
					}
					

				}
				
			}
		}
	}
	class OrinSkillBombAbility : FollowEffectAbility
	{
		public const int BaseDamage = 60;
		public const string AbilityName ="「死灰復燃」" + "_bomb";
		public OrinSkillBombAbility (int duration = 1*Def.FPS)
			: base()
		{
			this.DurationTime = duration;
			this.Name = AbilityName;

		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool OnVanish (IUnit unit)
		{

			if (unit.HP < 1)
			{

				var affectTipRadius = 3;
				var radius100 = 0.6f * Def.MapTipLengthOnField;
				var radius80 = 1.0f * Def.MapTipLengthOnField;
				var radius50 = 2.0f * Def.MapTipLengthOnField;

				var faction = unit.Faction;
				var offset = new Vector3D(0, 0, 3);

				Unit.Utls.ShowHitEffect(unit, Unit.Utls.HitEffectType.BombRed);

				var effect = faction.World.EffectManager.CreateTextureEffect(typeof(Rin_Shikigami_HitRedTextureEffect).FullName, unit.Pos + offset);

				// bomb
				var targets = unit.Faction.World.GetObjectFromCircle(new Circle2D(affectTipRadius * Def.MapTipLengthOnField, new Vector2D(unit.Pos.X, unit.Pos.Y))).OfType<IUnit>();
				foreach (var tu in targets)
				{
//					if (unit.Faction.World.IsSameTeam(unit.Faction, tu.Faction))
//						continue;
					var distance = (tu.Pos.X - unit.Pos.X) * (tu.Pos.X - unit.Pos.X) + (tu.Pos.Y - unit.Pos.Y) * (tu.Pos.Y - unit.Pos.Y);
					if (distance < radius100 * radius100)
					{
						tu.BeDamaged(unit, BaseDamage, AttackBonusSet.None);
					}
					else if (distance < radius80 * radius80)
					{
						tu.BeDamaged(unit, (int)(BaseDamage * 0.8f), AttackBonusSet.None);
					}
					else if (distance < radius50 * radius50)
					{
						tu.BeDamaged(unit, (int)(BaseDamage * 0.5f), AttackBonusSet.None);
					}
					else
					{
						tu.BeDamaged(unit, (int)(BaseDamage * 0.25f), AttackBonusSet.None);
					}
				}
			}
			return base.OnVanish (unit);
		}
	}
}
