using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;
using LotusCraft.AutoCast;
using LotusCraft.Token;
using LotusCraft.Spell.Black;

namespace LotusCraft.Spell.Black
{
	class Shokuzai : thrts.SpellParameter<thrts.IUnit>
	{
		float[] decMoveSpeed;
		int[] damages ;

		public const int UseToken = 1;

		public Shokuzai()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "贖罪「昔時の針と痛がる怨霊」";
			string basePath = "Black/Spell/Shokuzai/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 15*Def.FPS;
			CoolTimes[2] = 15*Def.FPS;
			CoolTimes[3] = 15*Def.FPS;

			damages = new int[ MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 2;
			damages[2] = 4;
			damages[3] = 6;

			decMoveSpeed = new float[MaxLevel + 1];
			decMoveSpeed[0] = 0;
			decMoveSpeed[1] = 0.2f;
			decMoveSpeed[2] = 0.3f;
			decMoveSpeed[3] = 0.4f;

			Description = "攻撃がヒットしたユニットは移動速度が低下し徐々にHPを失う。\n効果を受けているユニットが死亡したとき、消費したトークンは返還される。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"トークン"+UseToken+"消費、移動速度-"+decMoveSpeed[1]+"、毎秒"+damages[1]+"のHPを失う、"+ShokuzaiAbility.DamageAbilityDuration/Def.FPS+"秒間",
				"トークン"+UseToken+"消費、移動速度-"+decMoveSpeed[2]+"、毎秒"+damages[2]+"のHPを失う、"+ShokuzaiAbility.DamageAbilityDuration/Def.FPS+"秒間",
				"トークン"+UseToken+"消費、移動速度-"+decMoveSpeed[3]+"、毎秒"+damages[3]+"のHPを失う、"+ShokuzaiAbility.DamageAbilityDuration/Def.FPS+"秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (spell == null || !spell.CanExec) {
				yield return false;
				yield break;
			}

			if (!selfUnit.UserValues.ContainsKey (CorpseToken.TokenName)) {
				yield return false;
				yield break;
			}

			var token = selfUnit.UserValues [CorpseToken.TokenName] as CorpseToken;

			if (token.Count < UseToken) {
				yield return false;
				yield break;
			}

			if (selfUnit.AbilityManager.ContainsAbility (ShokuzaiAbility.AbilityName)) {
				yield return false;
				yield break;
			}

			token.DecToken (UseToken);
			;

			// クールタイム再設定
			spell.ResetCoolTime ();

			var ability = selfUnit.AbilityManager.GetAbilityWithName (ShokuzaiAbility.AbilityName) as ShokuzaiAbility;

			if (null == ability || ability.IsVanished) {
				// effect offset
				Vector3D offset = new Vector3D (0.0f, 0.0f, 3.0f);

				//var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(CommonBolEffect).FullName, selfUnit.Pos);
				//effect.SetScale(new Vector3D(1, 1, 1));
				//effect.SetPosition(selfUnit.Pos + offset);

				// append ability
				{
					var effect2 = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(SyokuzaiSelfEffect).FullName, selfUnit.Pos);
					effect2.SetScale (new Vector3D (8.0f, 8.0f, 8.0f));
					effect2.SetPosition (selfUnit.Pos + offset);
					selfUnit.AbilityManager.AppendAbility (new ShokuzaiAbility (damages [level], decMoveSpeed [level], effect2));
				}

			} else {
				ability.Damage = damages[level];
				ability.DecMove = decMoveSpeed[level];
			}
			yield return false;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class ShokuzaiAbility : FollowEffectAbility
	{
		public const string AbilityName = "贖罪「昔時の針と痛がる怨霊」_speller";
		public int Damage = 0;
		public float DecMove = 0.0f;
		public const int DamageAbilityDuration = 8*Def.FPS;
		public ShokuzaiAbility(int damage,float decMove,IEffect effect=null)
			:base(effect,new Vector3D(),2,AbilityName)
		{
			this.Damage = damage;
			this.DecMove = decMove;
		}
		public ShokuzaiAbility(int incAtk,float decMove,Vector3D offset,IEffect effect)
			:base(effect,offset,2,AbilityName)
		{
			this.Damage = incAtk;
			this.DecMove = decMove;
		}
		public override bool ModifyBullet (IBullet bullet)
		{
			bullet.AddOnHitEvent(OnHitShokuzai);
			return base.ModifyBullet (bullet);
		}
		public void OnHitShokuzai (IBullet bullet, ITargetObject target)
		{
			var u = target as IUnit;
			var speller = bullet.Generator as IUnit;

			if (null == u || !u.Exists ())
				return;
			if (null == speller || !speller.Exists ())
				return;

			var ability = u.AbilityManager.GetAbilityWithName (ShokuzaiDamageAbility.AbilityName) as ShokuzaiDamageAbility;
			if (null != ability) {
				ability.Damage = this.Damage;
				ability.DecMove = this.DecMove;
				ability.DurationTime = DamageAbilityDuration;
			} else {
				// create effect

				IEffect effect = bullet.Generator.Faction.World.EffectManager.CreateEffect(typeof(SyokuzaiEffect).FullName, u.Pos + new Vector3D(0, 0, 5.0f));
				effect.SetScale(new Vector3D(10.0f, 10.0f, 10.0f));
				effect.SetPosition(u.Pos + new Vector3D(0,0,5.0f));
				u.AbilityManager.AppendAbility(new ShokuzaiDamageAbility(speller,Damage,DecMove,DamageAbilityDuration,effect));
			}

			speller.AbilityManager.RemoveAbility(ShokuzaiAbility.AbilityName);
		}
		public override void DecDurationTime ()
		{
			// no dec -> inf ability
			;
		}
	}

	class ShokuzaiDamageAbility : FollowEffectAbility
	{
		public const string AbilityName = "贖罪「昔時の針と痛がる怨霊」";
		public int Damage = 0;
		public float DecMove = 0.0f;
		IUnit speller;
		int counter = 0;
		const int interval = 1*Def.FPS;
		public ShokuzaiDamageAbility(IUnit speller,int damage,float decMove,int duration,IEffect effect=null)
			:base(effect,new Vector3D(),duration,AbilityName)
		{
			this.Damage = damage;
			this.DecMove = decMove;
			this.speller = speller;
		}
		public ShokuzaiDamageAbility(IUnit speller,int damage,float decMove,int duration,Vector3D offset,IEffect effect)
			:base(effect,offset,duration,AbilityName)
		{
			this.Damage = damage;
			this.DecMove = decMove;
			this.speller = speller;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case AbilityOrder.ConstDecParam:
				unit.Speed -= DecMove;
				if(counter++ > interval)
				{
					unit.HP -= Damage;
					counter = 0;
				}
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool OnAdd (IUnit unit)
		{
			unit.AddOnVanishEvent(OnVanishHolder);
			return base.OnAdd (unit);
		}

		public void OnVanishHolder (IUnit unit)
		{
			if (null != speller) {
				// トークンを返還
				var token = speller.UserValues[CorpseToken.TokenName] as CorpseToken;
				if(null != token)
				{
					token.IncToken(Shokuzai.UseToken);
				}
			}
		}
	}
}