﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.AutoCast;
using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;

namespace LotusCraft.Spell.Black
{
	class SatoriSkill : thrts.SpellParameter<thrts.IUnit>
	{
		/// <summary>
		/// The mode change time[s].
		/// </summary>
		const float modeChangeTime = 4.0f;
		const float affectRadius = 5;
		const int duration = 4 * Def.FPS;
		const int interval = 10;
		public SatoriSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "想起「恐怖催眠術」";
			string basePath = "Black/Spell/SatoriSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = 15 * Def.FPS;
			CoolTimes[1] = 15 * Def.FPS;

			Description = "周囲の敵が受けるダメージを増加させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"半径" + affectRadius + "、ダメージ+20%",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new SpellAutoCast<IUnit>(this,interval,null),
				new SpellAutoCast<IUnit>(this,interval,(unit)=>Spelling (unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			if (!selfUnit.AbilityManager.ContainsAbility(this.Name + "areaEffect"))
			{
				var offset = new Vector3D(2, 2, 1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(CommonBolAreaEffect).FullName, selfUnit.Faction, selfUnit.Pos);
				var scale = affectRadius;
				effect.SetScale(new Vector3D(scale, scale, scale));
				effect.SetPosition(selfUnit.Pos + offset);
				if (!selfUnit.AbilityManager.AppendAbility(new FollowEffectAbility(effect, offset, interval * 2, this.Name + "areaEffect")))
					effect.Stop();
			}
			else
			{
				selfUnit.AbilityManager.ExtendLifetimeWithName(interval, this.Name + "areaEffect", interval * 2);
			}



			var selfFaction = selfUnit.Faction;

			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_ => !selfFaction.World.IsSameTeam(selfFaction, _.Faction)))
			{
				if (u.AbilityManager.ContainsAbility(SatoriSkillAbility.AbilityName))
				{
					var ability = u.AbilityManager.GetAbilityWithName(SatoriSkillAbility.AbilityName);
					ability.DurationTime = interval * 2;
				}
				else
				{
					var offset = new Vector3D(2, 2, 1);
					var effect = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(SatoriTargetTextureEffect).FullName, u.Pos + offset);
					if (!u.AbilityManager.AppendAbility(new SatoriSkillAbility(interval * 2, effect)))
					{
						effect.Stop();
					}
				}

			}

			return true;
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;


			/// reset cooltime
			selfUnit.GetSpellWithName(Name).ResetCoolTime();

			if (!selfUnit.AbilityManager.ContainsAbility(this.Name + "areaEffect"))
			{
				var offset = new Vector3D(2, 2, 1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(CommonBolAreaEffect).FullName, selfUnit.Pos);
				var scale = affectRadius;
				effect.SetScale(new Vector3D(scale, scale, scale));
				effect.SetPosition(selfUnit.Pos + offset);
				if (!selfUnit.AbilityManager.AppendAbility(new FollowEffectAbility(effect, offset, 1 * Def.FPS, this.Name + "areaEffect")))
					effect.Stop();
			}

			var selfFaction = selfUnit.Faction;

			// 対象を探索
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_ => !selfFaction.World.IsSameTeam(selfFaction, _.Faction)))
			{
				if (u.AbilityManager.ContainsAbility(SatoriSkillAbility.AbilityName))
				{
					var ability = u.AbilityManager.GetAbilityWithName(SatoriSkillAbility.AbilityName);
					ability.DurationTime = duration;
				}
				else
				{
					var offset = new Vector3D(0, 0, 2);
					var effect = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(SatoriTargetTextureEffect).FullName, u.Pos + offset);
					if (!u.AbilityManager.AppendAbility(new SatoriSkillAbility(4 * Def.FPS, effect, offset)))
					{
						effect.Stop();
					}
				}

			}


			yield return false;
		}

	}
}

namespace LotusCraft.Ability.Black
{
	class SatoriSkillAbility : thrts.AbilityTemplate
	{
		IEffectCommon effect;
		Vector3D offset;
		public static float IncDamageRatio = 0.2f;
		public const string AbilityName = "想起「恐怖催眠術」";
		public SatoriSkillAbility(int duration, IEffectCommon effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;

		}
		public SatoriSkillAbility(int duration, IEffectCommon effect, Vector3D offset)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			this.offset = offset;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case thrts.AbilityOrder.ConstDecParam:
					break;
				case thrts.AbilityOrder.RateDecParam:
					break;
				case thrts.AbilityOrder.ConstIncParam:
					unit.DamageRatio += IncDamageRatio;
					break;
				case thrts.AbilityOrder.RateIncParam:
					break;
				case AbilityOrder.OtherExecs:
					if (effect != null)
						effect.SetPosition(unit.Pos + offset);
					break;
				default:
					break;
			}

			return true;
		}
		public override bool OnVanish(IUnit unit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}

			return base.OnVanish(unit);
		}
	}
}
