using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;
using LotusCraft.AutoCast;
using LotusCraft.Token;

namespace LotusCraft.Spell.Black
{
	class Jafu : thrts.SpellParameter<thrts.IUnit>
	{
		int[] healHP;
		int[] incAtk;

		const int duration = 12*Def.FPS;

		const int useToken = 1;

		public Jafu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "邪符「グーフンイエグイ」";
			string basePath = "Black/Spell/Jafu/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 12 * 30;
			CoolTimes[2] = 12 * 30;
			CoolTimes[3] = 12 * 30;

			incAtk = new int[ MaxLevel + 1];
			incAtk[0] = 0;
			incAtk[1] = 3;
			incAtk[2] = 5;
			incAtk[3] = 7;

			healHP = new int[MaxLevel + 1];
			healHP[0] = 0;
			healHP[1] = 80;
			healHP[2] = 120;
			healHP[3] = 160;

			Description = "HPが回復し、一定時間攻撃力が上昇する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"トークン"+useToken+"消費、HP"+healHP[1]+"回復、攻撃力+"+incAtk[1],
				"トークン"+useToken+"消費、HP"+healHP[2]+"回復、攻撃力+"+incAtk[2],
				"トークン"+useToken+"消費、HP"+healHP[3]+"回復、攻撃力+"+incAtk[3],
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (spell == null || !spell.CanExec) {
				yield return false;
				yield break;
			}

			if (!selfUnit.UserValues.ContainsKey (CorpseToken.TokenName)) {
				yield return false;
				yield break;
			}

			var token = selfUnit.UserValues [CorpseToken.TokenName] as CorpseToken;

			if (token.Count < useToken) {
				yield return false;
				yield break;
			}

			token.DecToken( useToken);;

			// クールタイム再設定
			spell.ResetCoolTime();

			{
				// heal
				selfUnit.HP += healHP[level];

				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 5.0f);

				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(JafuHealEffect).FullName, selfUnit.Pos);
				effect.SetScale(new Vector3D(15, 15, 15));
				effect.SetPosition(selfUnit.Pos + offset);

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);

				// append ability
				{
					var ability = selfUnit.AbilityManager.GetAbilityWithName(JafuAbility.AbilityName) as JafuAbility;
					if(null != ability)
					{
						ability.DurationTime = duration;
					}
					else
					{
						var effect2 = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(JafuSelfEffect).FullName, selfUnit.Pos);
						effect2.SetScale(new Vector3D(15, 15, 15));
						effect2.SetPosition(selfUnit.Pos + offset);
						selfUnit.AbilityManager.AppendAbility(new JafuAbility(incAtk[level],duration, effect2));
					}
				}

			}
			yield return false;
		}

		public int DisableSelectBullet(IUnit unit,ITargetObject target,float distance)
		{
			return -1;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class JafuAbility : FollowEffectAbility
	{
		public const string AbilityName = "邪符「グーフンイエグイ」";
		public int incAtk = 0;
		public JafuAbility(int incAtk,int duration,IEffect effect=null)
			:base(effect,new Vector3D(),duration,AbilityName)
		{
			this.incAtk = incAtk;
		}
		public JafuAbility(int incAtk,int duration,Vector3D offset,IEffect effect)
			:base(effect,offset,duration,AbilityName)
		{
			this.incAtk = incAtk;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case AbilityOrder.ConstIncParam:
				foreach(var b in unit.ModifiedBulletParameters)
					b.Attack += incAtk;
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
}