using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;
using LotusCraft.AutoCast;
using LotusCraft.Spell.Black;
using LotusCraft.Token;

namespace LotusCraft.Spell.Black
{
	class Kourei : thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadiuses;

		float[] durationTime;

		const int interval = 10;
		const int useToken = 4;

		public Kourei()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "降霊「死人タンキー」";
			string basePath = "Black/Spell/Kourei/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 10 * 30;
			CoolTimes[2] = 10 * 30;
			CoolTimes[3] = 10 * 30;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 4;
			affectRadiuses[2] = 5;
			affectRadiuses[3] = 5;

			durationTime = new float[MaxLevel + 1];
			durationTime[0] = 0;
			durationTime[1] = 4;
			durationTime[2] = 6;
			durationTime[3] = 8;

			Description = "範囲内で死亡したユニットを12秒の間、味方として蘇らせる。\n作業者とヒロイン、召喚ユニットは蘇らない。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"トークン"+useToken+"消費"+"、範囲"+ affectRadiuses[1]+"、" + durationTime[1] + "秒間",
				"トークン"+useToken+"消費"+"、範囲"+ affectRadiuses[2]+"、" + durationTime[2] + "秒間",
				"トークン"+useToken+"消費"+"、範囲"+ affectRadiuses[3]+"、" + durationTime[3] + "秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName(Name);
			if (spell == null || !spell.CanExec)
			{
				yield return false;
				yield break;
			}

			if (!selfUnit.UserValues.ContainsKey(CorpseToken.TokenName))
			{
				yield return false;
				yield break;
			}

			var token = selfUnit.UserValues[CorpseToken.TokenName] as CorpseToken;

			if (token.Count < useToken)
			{
				yield return false;
				yield break;
			}

			token.DecToken(useToken);

			// クールタイム再設定
			spell.ResetCoolTime();

			{
				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);


				// add area effect
				IEffect effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(KoureiAreaEffect).FullName, selfUnit.Pos + offset);
				var scale = affectRadiuses[level] * 8;
				effect.SetScale(new Vector3D(scale, scale, scale));

				selfUnit.AbilityManager.AppendAbility(
					new SpellContinuousAbility((unit) => 
						Spelling(unit, level), 
						effect, 
						offset, 
						(int)(durationTime[level] * Def.FPS), 
						this.Name + "spelling", interval)
						);
			}
			yield return false;
		}

		public bool Spelling(IUnit selfUnit, int level)
		{
			var affectRadius = affectRadiuses[level];
			var selfFaction = selfUnit.Faction;
			
			// ターゲット検索
			var targets = selfFaction.World.GetObjectFromCircle(
				new Circle2D(affectRadius * Def.MapTipLengthOnField, 
					new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>().Where(_ => _ != null);

			foreach (var u in targets)
			{
				if (null == u || !u.Exists())
					continue;

				if (u.Parameter.IsFarmer || u.Parameter.IsUnique || u.IsSummon)
					continue;

				// 復活対象に降霊アビリティを付ける
				{
					if (u.AbilityManager.ContainsAbility(KoureiReservAbility.AbilityName))
					{
						u.AbilityManager.ExtendLifetimeWithName(interval, KoureiReservAbility.AbilityName, interval * 2);
					}
					else
					{
						IEffect effect = null;
						u.AbilityManager.AppendAbility(new KoureiReservAbility(selfUnit, interval * 2, effect));
					}
				}
			}
			return true;
		}

		public override void OnUpdate(IUnit holder)
		{
			if (holder.AbilityManager.ContainsAbility(KoureiAbility.AbilityName))
			{
				holder.AbilityManager.ExtendLifetimeWithName(1, KoureiAbility.AbilityName, 2);
			}
			else
			{
				holder.AbilityManager.AppendAbility(new KoureiAbility(2));
			}
			base.OnUpdate(holder);
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class KoureiAbility : AbilityTemplate
	{
		public const string AbilityName = "降霊「死人タンキー」";

		List<IUnit> summons = new List<IUnit>();
		public const int summonDuration = 12 * Def.FPS;

		public KoureiAbility(int duration)
		{
			Name = AbilityName;
			DurationTime = duration;
		}

		public override bool OnVanish(IUnit unit)
		{
			foreach (var u in summons)
			{
				if (u != null && u.Exists())
				{
					u.HP = 0;
				}
			}
			summons.Clear();
			return base.OnVanish(unit);
		}

		public override bool OnVanish(IBullet bullet)
		{
			var units = summons.ToArray();
			foreach (var u in units)
			{
				if (u != null && u.Exists())
				{
					u.HP = 0;
				}
			}
			summons.Clear();
			return base.OnVanish(bullet);
		}

		public IUnit Summon(IUnit unit, IUnit summoner)
		{
			if (null == unit || unit.IsSummon)
				return null;

			if (null == summoner)
				return null;

			if (unit.HP > 0)
				return null;

			var u = unit;

			if (u != null)
			{
				u.HP = u.MaxHP;
				u.AbilityManager.AppendAbility(new SummonUnitAbility(null, summonDuration));
				u.ChangeFaction(summoner.Faction);

				summons.Add(u);

				if (!summoner.UserValues.ContainsKey(Taofu.SeigaSummonUnitsKey))
					summoner.UserValues.Add(Taofu.SeigaSummonUnitsKey, new List<IUnit>());

				List<IUnit> summonUnits = summoner.UserValues[Taofu.SeigaSummonUnitsKey] as List<IUnit>;
				if (null != summonUnits)
					summonUnits.Add(u);
			}

			return u;
		}
	}

	/// <summary>
	/// 復活対象につけるアビリティ
	/// </summary>
	class KoureiReservAbility : AbilityTemplate
	{
		public const string AbilityName = "降霊「死人タンキー」" + "reserv";
		IUnit speller = null;

		public KoureiReservAbility(IUnit speller, int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.speller = speller;
		}

		public override bool OnAdd(IUnit unit)
		{
			if (null != unit && unit.Exists())
				unit.Vanishing += OnVanishHolder;
			return base.OnAdd(unit);
		}

		public override bool OnVanish(IUnit unit)
		{
			if (null != unit)
				unit.Vanishing -= OnVanishHolder;
			return base.OnVanish(unit);
		}

		public void OnVanishHolder(IUnit unit)
		{
			if (speller == null) return;
			if (!speller.Exists()) return;

			// 存在しててもHP0のケースがあるため
			if (speller.HP == 0) return;

			var ability = speller.AbilityManager.GetAbilityWithName(KoureiAbility.AbilityName) as KoureiAbility;
			if (ability == null) return;

			// 復活させてみる
			var u = ability.Summon(unit, speller);
			if (u != null)
			{
				u.AbilityManager.RemoveAbility(KoureiReservAbility.AbilityName);


				var faction = speller.Faction;
				var effect = faction.World.EffectManager.CreateEffect(typeof(KoureiReviveEffect).FullName, u.Pos + new Vector3D(1.0f, 1.0f, 10.0f));
				effect.SetScale(new Vector3D(5, 5, 5));
				u.HP /= 2;
			}
		}
	}
}