using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;
using LotusCraft.AutoCast;
using LotusCraft.Unit.Black;
using LotusCraft.Token;

namespace LotusCraft.Spell.Black
{
	class Nyuma : thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadiuses;

		int[] summons;

		const int useToken = 2;

		const int summonDuration = 4*Def.FPS;

		public Nyuma()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "入魔「ゾウフォルゥモォ」";   
			string basePath = "Black/Spell/Nyuma/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 4 * 30;
			CoolTimes[2] = 2 * 30;
			CoolTimes[3] = 1 * 30;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 3;
			affectRadiuses[2] = 4;
			affectRadiuses[3] = 5;

			summons = new int[MaxLevel + 1];
			summons[0] = 0;
			summons[1] = 1;
			summons[2] = 1;
			summons[3] = 1;

			Description = "攻撃力80(対農民半減)で自爆攻撃する霊魂を召喚する。\n"+"霊魂は" + summonDuration/Def.FPS + "秒で消滅する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"トークン"+useToken+"消費で自爆攻撃する霊魂を1体召喚、クールタイム4秒",
				"トークン"+useToken+"消費で自爆攻撃する霊魂を1体召喚、クールタイム2秒",
				"トークン"+useToken+"消費で自爆攻撃する霊魂を1体召喚、クールタイム1秒",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (spell == null || !spell.CanExec) {
				yield return false;
				yield break;
			}

			if (!selfUnit.UserValues.ContainsKey (CorpseToken.TokenName)) {
				yield return false;
				yield break;
			}
			
			var token = selfUnit.UserValues [CorpseToken.TokenName] as CorpseToken;
			
			if (token.Count < useToken) {
				yield return false;
				yield break;
			}
			int summonCount = 0;

			for (int i=0; i<summons[level] && token.Count >= useToken; ++i) {
				token.DecToken (useToken);
				summonCount++;
			}

			// クールタイム再設定
			spell.ResetCoolTime();

			{
				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

				for(int i=0;i<summonCount;++i)
				{
					var unit = Utl.PutUnitAroundPoint( selfUnit.Faction.World,selfUnit.Faction, typeof(BombSpirit).FullName, new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y));
					if(null != unit)
					{
						var size = 5;
						var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(NyumaEffect).FullName, unit.Pos + offset);
						effect.SetScale(new Vector3D(size, size, size));
//                        var ability = new FollowEffectAbility(effect, offset, summonDuration + Def.FPS, this.Name + "Effect");
						unit.AbilityManager.AppendAbility(new SummonUnitAbility(effect,summonDuration));
//                        unit.AbilityManager.AppendAbility(ability);

						if(!selfUnit.UserValues.ContainsKey(Taofu.SeigaSummonUnitsKey))
							selfUnit.UserValues.Add(Taofu.SeigaSummonUnitsKey,new List<IUnit>());

						List<IUnit> summonUnits = selfUnit.UserValues[Taofu.SeigaSummonUnitsKey] as List<IUnit>;
						if(null != summonUnits)
							summonUnits.Add(unit);
                    }
				}

			}
			yield return false;
		}

		public int DisableSelectBullet(IUnit unit,ITargetObject target,float distance)
		{
			return -1;
		}
	}
}
