﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Black
{
	class Jigoku : thrts.SpellParameter<thrts.IUnit>
	{
		const float affectRadius = 7 * Def.MapTipLengthOnField;
		const int duration = 20 * Def.FPS;
		int[] chargeTime;
		int[] damages;
		float[] damageBonus;
		public Jigoku()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "「地獄の人口太陽」";
			string basePath = "Black/Spell/Jigoku/";

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 24;

			damageBonus = new float[AttackBonusSet.None.Length];
			for(int i=0;i<this.damageBonus.Length;++i)
				damageBonus[i] = AttackBonusSet.None[i];

			damageBonus[(int)TargetObjectType.Hero] = 0.50f;
			damageBonus[(int)TargetObjectType.Structure] = 2.0f;

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180 * Def.FPS;

			chargeTime = new int[MaxLevel + 1];
			chargeTime[0] = 0;
			chargeTime[1] = 3 * Def.FPS;


			Description = "範囲内のユニットを敵味方関係無く引き寄せ、ダメージを与える。\n" + "0.2秒毎に" + (damages[1]) + "ダメージ (防御力無視)\n対ヒロインボーナス-50%、対鋼ボーナス+100%";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName(Name);
				if (null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}

				spell.ResetCoolTime();

				var scale = (affectRadius) / 5.0f;

				// チャージエフェクト
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(
					typeof(UtsuhoJigokuChangeEffect).FullName,
					selfUnit.Pos + new Vector3D(0.0f, 0.0f, 5.0f));

				effect.SetScale(new Vector3D(scale, scale, scale));

				if (!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect, chargeTime[level], this.Name + "chargeEffect")))
					effect.Stop();

				// 警告音
				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Alarm00).FullName, selfUnit.Pos);

				for (int i = 0; i < chargeTime[level]; ++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}

				// stay stop
				selfUnit.Speed = 0.0f;

				// カットイン
				selfUnit.ShowCutin();

				// 攻撃エフェクト
				Vector3D offset = new Vector3D(0.0f, 0.0f, 15.0f);
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(UtsuhoJigokuAreaEffect).FullName, selfUnit.Pos + offset);
				areaEffect.SetScale(new Vector3D(scale, scale, scale));

				var ability = new SpellContinuousAbility(
					(unit) => Spelling(unit, level), 
					areaEffect,
					offset,
					duration, 
					this.Name + "areaEffect");
				ability.StopEffectRootOnVanish = true;

				if (!selfUnit.AbilityManager.AppendAbility(ability))
					areaEffect.Stop();

				// 開始音
				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Cleaner01).FullName, selfUnit.Pos);
			}
		}


		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists())
				return false;

			var selfFaction = selfUnit.Faction;

			selfUnit.SelectBullet = (unit, target, distance) => -1;

			// search targets
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).Where(_ => _ != null);

			// attract units
			foreach (var u in targets.OfType<IUnit>())
			{
				if (null == u || !u.Exists() || u.UniqueID == selfUnit.UniqueID)
					continue;

				var d = Utl.GetDistance(selfUnit.Pos.X, selfUnit.Pos.Y, u.Pos.X, u.Pos.Y);
				var r = new Vector2D((u.Pos.X - selfUnit.Pos.X) / d, (u.Pos.Y - selfUnit.Pos.Y) / d);
				if (d > 1 && !u.TryMove(-r.X, -r.Y, true))
				{ 
				
				}
			}

			if (!selfUnit.AbilityManager.ContainsAbility(JigokuSkillAbility.AbilityName))
			{
				foreach (var t in targets)
				{
					if (null != t && t.Exists())
					{
						var u = t as IUnit;
						var b = t as IBuilding;
						if (null != u || null != b)
						{
							var faction = selfUnit.Faction;

							// 自分以外に攻撃エフェクト
							if (u != selfUnit)
							{
								var effect = faction.World.EffectManager.CreateEffect(typeof(UtsuhoSunJigokuHitEffect).FullName, t.Pos);
								effect.SetScale(new Vector3D(10, 10, 10));
								faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitOther).FullName, t.Pos);
							}
						}
						if (null != u)
							u.BeDamaged(selfUnit, damages[level], damageBonus, true);
						if (null != b)
							b.BeDamaged(selfUnit, damages[level], damageBonus, true);
					}
				}

				selfUnit.AbilityManager.AppendAbility(new JigokuSkillAbility());

			}

			return true;
		}

	}
}

namespace LotusCraft.Ability.Black
{
	class JigokuSkillAbility : thrts.AbilityTemplate
	{
		public static string AbilityName = "「地獄の人口太陽」";
		public JigokuSkillAbility()
		{
			this.Name = AbilityName;
			this.DurationTime = Def.FPS/5;
		}

		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case thrts.AbilityOrder.ConstDecParam:
					break;
				case thrts.AbilityOrder.RateDecParam:
					break;
				case thrts.AbilityOrder.ConstIncParam:
					break;
				case thrts.AbilityOrder.RateIncParam:
					break;
				case AbilityOrder.OtherExecs:
					break;
				default:
					break;
			}

			return true;
		}
	}
}
#endif