#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;

namespace LotusCraft.Spell.Black
{
	class Kakufu : thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime;
		float[] affectRadiuses;

		int[] damages;

		public Kakufu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "核符「クリーピングサン」";
			string basePath = "Black/Spell/Kakufu/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 60 * 30;
			CoolTimes[2] = 45 * 30;
			CoolTimes[3] = 30 * 30;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6 * Def.MapTipLengthOnField;
			MaxRanges[2] = 7 * Def.MapTipLengthOnField;
			MaxRanges[3] = 8 * Def.MapTipLengthOnField;

			chargeTime = new int[MaxLevel + 1];
			chargeTime[0] = 0;
			chargeTime[1] = 5 * Def.FPS;
			chargeTime[2] = 5 * Def.FPS;
			chargeTime[3] = 5 * Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 4;
			affectRadiuses[2] = 5;
			affectRadiuses[3] = 6;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 250;
			damages[2] = 300;
			damages[3] = 350;

			Description = "指定地点を中心とした広範囲にダメージを与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"射程" + MaxRanges[1]/Def.MapTipLengthOnField + "、範囲"+ affectRadiuses[1]+"、" + damages[1] + "ダメージ",
				"射程" + MaxRanges[2]/Def.MapTipLengthOnField + "、範囲"+ affectRadiuses[2]+"、" + damages[2] + "ダメージ",
				"射程" + MaxRanges[3]/Def.MapTipLengthOnField + "、範囲"+ affectRadiuses[3]+"、" + damages[3] + "ダメージ",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, OnFinishPrivilegeMode, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, OnFinishPrivilegeMode, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, OnFinishPrivilegeMode, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName(Name);
			if (spell == null || !spell.CanExec)
			{
				yield return false;
				yield break;
			}

			// クールタイム再設定
			spell.ResetCoolTime();

			{

				float affectRadius = affectRadiuses[level];

				var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

				// チャージエフェクト
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(UtsuhoSunChangeEffect).FullName, targetPos + offset);
				var scale = (affectRadius) * 32 / 5.0f;
				effect.SetScale(new Vector3D(scale, scale, scale));

				if (!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect, chargeTime[level], this.Name + "chargeEffect")))
				{
					effect.Stop();
				}

				IEffect attackEffect = null;

				for (int i = 0; i < chargeTime[level]; ++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					// disable attack
					selfUnit.SelectBullet = DisableSelectBullet;

					// この時点で攻撃エフェクト発動開始
					if (i == chargeTime[level] - 60)
					{
						attackEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(UtsuhoSunAreaEffect).FullName, targetPos + offset);
						attackEffect.SetScale(new Vector3D(scale, scale, scale));
						selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Cleaner02).FullName, targetPos);

						Action<IUnit> vanished = null;
						vanished = (u) =>
							{
								attackEffect.StopRoot();
								selfUnit.Vanished -= vanished;
							};
						selfUnit.Vanished += vanished;

					}

					yield return true;
				}

				// stay stop
				selfUnit.Speed = 0.0f;


				// 攻撃開始
				var selfFaction = selfUnit.Faction;
				/// search targets
				var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(targetPos.X, targetPos.Y))).Where(_ => _ != null);
				foreach (ITargetObject t in targets)
				{
					if (null != t && t.Exists())
					{

						var u = t as IUnit;
						var b = t as IBuilding;

						// onHit
						if (null != u || null != b)
						{
							//var faction = selfUnit.Faction;
							//effect = faction.World.EffectManager.CreateEffect(typeof(UtsuhoSunJigokuHitEffect).FullName, t.Pos + offset);
							//effect.SetScale(new Vector3D(10, 10, 10));
							selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitOther).FullName, t.Pos);
						}
						if (u != null)
							u.BeDamaged(selfUnit, damages[level], AttackBonusSet.None);
						if (b != null)
							b.BeDamaged(selfUnit, damages[level], AttackBonusSet.None);
					}
				}

			}
			yield return false;
		}
		public void OnFinishPrivilegeMode(IUnit selfUnit, int level)
		{
			;
		}

		public int DisableSelectBullet(IUnit unit,ITargetObject target,float distance)
		{
			return -1;
		}
	}
}
#endif