﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Black
{
	class Kakunetsu : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		int[] incAtks;
		Dictionary<float, float>[] splashs;

		const int interval = 10;

		public Kakunetsu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "核熱「核反応制御不能」";
			string basePath = "Black/Spell/Kakunetsu/";

			incAtks = new int[MaxLevel + 1];
			incAtks[0] = 0;
			incAtks[1] = 3;
			incAtks[2] = 6;
			incAtks[3] = 9;
			splashs = new Dictionary<float, float>[]
			{
				new Dictionary<float, float>(),
				new Dictionary<float, float>(){{0.5f*Def.MapTipLengthOnField,0.5f},{1.0f*Def.MapTipLengthOnField,0.25f},},
				new Dictionary<float, float>(){{0.6f*Def.MapTipLengthOnField,0.8f},{0.9f*Def.MapTipLengthOnField,0.5f},{1.5f*Def.MapTipLengthOnField,0.25f},},
				new Dictionary<float, float>(){{0.7f*Def.MapTipLengthOnField,1.0f},{1.0f*Def.MapTipLengthOnField,0.8f},{1.4f*Def.MapTipLengthOnField,0.5f},{2.0f*Def.MapTipLengthOnField,0.25f},},
			};

			Description = "攻撃力が上昇し、攻撃にスプラッシュを付加する。\nただし、味方もダメージを受けるようになる。";

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃力+" + incAtks[1] + "、スプラッシュ範囲1.0",
				"攻撃力+" + incAtks[2] + "、スプラッシュ範囲1.5",
				"攻撃力+" + incAtks[3] + "、スプラッシュ範囲2.0",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists())
				return false;

			var ability = selfUnit.AbilityManager.GetAbilityWithName(KakunetsuAbility.AbilityName) as KakunetsuAbility;
			if (null == ability)
			{
				var offset = new Vector3D(0, 0, 1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(UtsuhoKakuAreaEffect).FullName, selfUnit.Pos);
				var scale = 8;
				effect.SetScale(new Vector3D(scale, scale, scale));
				effect.SetPosition(selfUnit.Pos + offset);
				if (!selfUnit.AbilityManager.AppendAbility(new KakunetsuAbility(incAtks[level],splashs[level],effect, offset, interval*2)))
					effect.Stop();
			}
			else
			{
				ability.ExtendLifetime(interval,interval*2);
				ability.IncAtk = incAtks[level];
				ability.Splash = splashs[level];
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class KakunetsuAbility : FollowEffectAbility
	{
		public static string AbilityName = "核熱「核反応制御不能」";
		public int IncAtk = 0;
		public Dictionary<float, float> Splash;
		public KakunetsuAbility(int incAtk,Dictionary<float, float> splash,IEffect effect,Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
			this.IncAtk = incAtk;
			this.Splash = splash;
		}
		
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				foreach (var b in unit.ModifiedBulletParameters)
				{
					b.Attack += IncAtk;
					if(null != Splash)
					{
						b.HasSplash = true;
						b.SplashDamageRates = Splash;
						b.HitAlly = true;
						b.DoesHitSplashMyself = false;
					}
				}
				
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
}
#endif
