#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;

namespace LotusCraft.Spell.Black
{
	class CaptureWeb:thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime;
		float[] affectRadiuses;
		int[] durationTimes;

		public CaptureWeb()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "罠符「キャプチャーウェブ」";
			string basePath = "Black/Spell/CaptureWeb/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 45*30;
			CoolTimes[2] = 30*30;
			CoolTimes[3] = 15*30;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 3*32;
			MaxRanges[2] = 4*32;
			MaxRanges[3] = 5*32;

			chargeTime = new int[MaxLevel+1];
			chargeTime[0] = 0;
			chargeTime[1] = 2*Def.FPS;
			chargeTime[2] = 2*Def.FPS;
			chargeTime[3] = 2*Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 3;
			affectRadiuses[2] = 3;
			affectRadiuses[3] = 4;

			durationTimes = new int[MaxLevel + 1];
			durationTimes[0] = 0;
			durationTimes[1] = 120;
			durationTimes[2] = 180;
			durationTimes[3] = 210;


			Description = "指定地点の周囲に移動速度を低下させる罠を設置します。\n飛行ユニットが掛かった場合は一時的に地上ユニットになります、";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲"+ affectRadiuses[1]+"、敵ユニットが引っかかるか" + durationTimes[1] + "秒経つと消滅。",
				"範囲"+ affectRadiuses[2]+"、敵ユニットが引っかかるか" + durationTimes[2] + "秒経つと消滅。",
				"範囲"+ affectRadiuses[3]+"、敵ユニットが引っかかるか" + durationTimes[3] + "秒経つと消滅。",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, OnFinishPrivilegeMode, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, OnFinishPrivilegeMode, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, OnFinishPrivilegeMode, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName (Name);
				if(null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}

				spell.ResetCoolTime ();

				float affectRadius = affectRadiuses[level];

				var targetPos = selfUnit.Faction.World.GetGroundAtPos (targetPosition);

				// effect offset
				Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);

				/// change effect
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(CaptureChargeEffect).FullName, targetPos + offset, selfUnit.Faction);
				var scale = Def.MapTipLengthOnField * (affectRadius) / 5.0f;
				effect.SetScale (new Vector3D ( scale, scale, scale ) );
				
				if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect,chargeTime[level],this.Name + "chargeEffect")))
					effect.Stop();
				
				for(int i=0;i<chargeTime[level];++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}
				/// change effect
				var completeEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(RopeEffect).FullName, targetPos + offset, selfUnit.Faction);
				completeEffect.SetScale (new Vector3D ( scale, scale, scale ) );
				
				if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(completeEffect,Def.FPS,this.Name + "chargeCompleteEffect")))
					completeEffect.Stop();

				// stay stop
				selfUnit.Speed = 0.0f;

				// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(CaptureAreaEffect).FullName, targetPos + offset, selfUnit.Faction);
				areaEffect.SetScale (new Vector3D (scale, scale, scale));
				areaEffect.IsShownToOnlyAlly = true;

				if(selfUnit.AbilityManager.ContainsAbility(CaptureWebAbility.AbilityName))
				{
					selfUnit.AbilityManager.RemoveAbility(CaptureWebAbility.AbilityName,true);
				}

				// nop
				yield return true;

				// stay stop
				selfUnit.Speed = 0.0f;

				if(!selfUnit.AbilityManager.AppendAbility(new CaptureWebAbility(selfUnit,targetPosition,affectRadiuses[level],durationTimes[level]*Def.FPS,areaEffect)))
					areaEffect.Stop();

			}
			yield return false;
		}
		public void OnFinishPrivilegeMode (IUnit selfUnit,int level)
		{
			;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class CaptureWebAbility : thrts.AbilityTemplate
	{
		IUnit speller;
		IEffect effect;
		float affectRadius;
		Vector2D targetPosition;
		bool captured;
		int counter;
		public const string AbilityName = "罠符「キャプチャーウェブ」";
		public CaptureWebAbility (IUnit speller,Vector2D targetPosition,float affectRadius,int duration = 120*Def.FPS,IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.speller = speller;
			this.effect = effect;
			this.affectRadius = affectRadius;
			this.targetPosition =targetPosition;
			counter = 0;
			captured = false;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(++counter > Def.FPS && DurationTime % Def.FPS == 0)
				{
					var selfFaction = speller.Faction;
					/// search targets
					var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, targetPosition)).OfType<IUnit>().Where (_ => _ != null);
					foreach (var u in targets) {
						if ( null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction,selfFaction) && u.UniqueID != speller.UniqueID ) {
							if(u.AbilityManager.ContainsAbility(CapturedAbility.AbilityName))
							{
								var ability = u.AbilityManager.GetAbilityWithName(CapturedAbility.AbilityName);
								ability.ExtendLifetime(Def.FPS,Def.FPS*2);
							}
							else
							{
								Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);
								var capturedEffect = unit.Faction.World.EffectManager.CreateEffect(typeof(CaptureEffect).FullName, u.Pos + offset, unit.Faction);
								capturedEffect.SetScale(new Vector3D(5, 5, 5));
								
								if (u.AbilityManager.AppendAbility(new CapturedAbility(4 * Def.FPS, capturedEffect)))
								{
									// onHit unit
									if(!captured)
									{
										captured = true;
										// 誰かが引っかかったら全員に見えるように
										if(null != effect)
										{
//											effect.Stop();
//											effect = null;
//											var targetPos = selfFaction.World.GetGroundAtPos(targetPosition);
//											var scale = Def.MapTipLengthOnField * (affectRadius) / 5.0f;
//											effect = selfFaction.World.EffectManager.CreateEffect(typeof(CaptureAreaEffect).FullName, targetPos + offset, selfFaction);
//											effect.SetScale (new Vector3D (scale, scale, scale));
											effect.IsShownToOnlyAlly = false;
										}
										// 2 kai tuika hantei
										DurationTime = 2*Def.FPS;
										u.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, u.Pos);
									}
								}
								else if(null != capturedEffect)
								{
									capturedEffect.Stop();
									capturedEffect = null;
								}
							}
						}
					}
				}

				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}


	}	class CapturedAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		const float DecMoveRatio = 0.75f;
		bool isCaught;
		public const string AbilityName = "罠符「キャプチャーウェブ」";
		public CapturedAbility (int duration = 4*Def.FPS,IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			// fly -> ground
			this.isCaught = false;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				// to touch ground
				
				break;
			case thrts.AbilityOrder.RateDecParam:
				// dec move ratio
				unit.Speed -= DecMoveRatio*unit.Speed;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnAdd (IUnit unit)
		{
			if (unit.FlyingState == FlyingState.Fly || unit.FlyingState == FlyingState.ToFly) {
				isCaught = true;
				unit.TouchGround();
			}
			return base.OnAdd (unit);
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists () && isCaught && (unit.FlyingState == FlyingState.TouchGround || unit.FlyingState == FlyingState.ToTouchGround)) {
				unit.Fly();
			}

			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}

#endif