﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Black
{
	class Miasma : thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadiuses;

		const int duration = 8*Def.FPS;
		
		public Miasma()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "瘴符「フィルドミアズマ」";
			string basePath = "Black/Spell/Miasma/";

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] =  6*Def.MapTipLengthOnField;
			MaxRanges[2] =  7*Def.MapTipLengthOnField;
			MaxRanges[3] =  8*Def.MapTipLengthOnField;

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 20*30;
			CoolTimes[2] = 20*30;
			CoolTimes[3] = 18*30;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 2;
			affectRadiuses[2] = 3;
			affectRadiuses[3] = 4;


			Description = "触れたユニットを毒状態にする瘴気を発生させます。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"12秒持続する毒BS(2秒毎に5%のHPを失う)付与、範囲" + affectRadiuses[1],
				"12秒持続する毒BS(2秒毎に5%のHPを失う)付与、範囲" + affectRadiuses[2],
				"12秒持続する毒BS(2秒毎に5%のHPを失う)付与、範囲" + affectRadiuses[3],
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}

			spell.ResetCoolTime ();
			
			float affectRadius = affectRadiuses[level];
			Vector2D targetPosition = new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y);
			
			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(FilledEffect).FullName, selfUnit.Faction.World.GetGroundAtPos(targetPosition) + offset, selfUnit.Faction);
			var scale = 8.0f*(affectRadius);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));

			var ability = new SpellContinuousAbility((unit)=>Spelling(unit,targetPosition,level),areaEffect,offset,duration,this.Name + "areaEffect");
			ability.FollowEffect = false;
			if(!selfUnit.AbilityManager.AppendAbility(ability))
				areaEffect.Stop();
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);
			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit,Vector2D targetPos, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			float affectRadius = affectRadiuses[level];

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			var selfFaction = selfUnit.Faction;
			// damage 
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, targetPos)).OfType<IUnit>().Where (_ => _ != null);
			foreach (var u in targets) {
				if ( null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction,selfFaction) && u.UniqueID != selfUnit.UniqueID ) {

					if(u.AbilityManager.ContainsAbility(PoisonAbility.AbilityName))
					{
						var ability = u.AbilityManager.GetAbilityWithName(PoisonAbility.AbilityName);
						ability.DurationTime = 12*Def.FPS;;
						continue;
					}

					// onHit unit
					{
						var ability = new PoisonAbility(12*Def.FPS);
						if(u.AbilityManager.AppendAbility(ability))
						{
							;
						}
					}
				}
			}
			
			return true;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class PoisonAbility : thrts.AbilityTemplate
	{
		IEffectCommon effect;
		int counter;
		const int interval = 2 * Def.FPS;
		public const string AbilityName = "瘴符「フィルドミアズマ」";
		public float DecHPRatio = 0.05f;
		public PoisonAbility(int duration = 12*Def.FPS, IEffectCommon effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			counter = interval;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case thrts.AbilityOrder.ConstDecParam:
					break;
				case thrts.AbilityOrder.RateDecParam:
					if (++counter > interval)
					{
						unit.HP -= (int)Math.Round(DecHPRatio * unit.MaxHP, 1);
						counter = 0;
					}
					break;
				case thrts.AbilityOrder.ConstIncParam:
					break;
				case thrts.AbilityOrder.RateIncParam:
					break;
				case AbilityOrder.OtherExecs:
					if (effect != null)
						effect.SetPosition(unit.Pos + new Vector3D(2, 2, 1));
					break;
				default:
					break;
			}

			return true;
		}
		public override bool OnVanish(IUnit unit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}

			return base.OnVanish(unit);
		}
	}
}


#endif