#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;

namespace LotusCraft.Spell.Black
{
	class Rope:thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime;
		float[] affectRadiuses;

		public Rope()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "細綱「カンダタロープ」";
			string basePath = "Black/Spell/Rope/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30*30;
			CoolTimes[2] = 20*30;
			CoolTimes[3] = 10*30;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 8*32;
			MaxRanges[2] = 12*32;
			MaxRanges[3] = 16*32;

			chargeTime = new int[MaxLevel+1];
			chargeTime[0] = 0;
			chargeTime[1] = 1*Def.FPS;
			chargeTime[2] = 1*Def.FPS;
			chargeTime[3] = 1*Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 2;
			affectRadiuses[2] = 3;
			affectRadiuses[3] = 4;


			Description = " 指定地点を中心とした範囲に居る味方ユニットを自分の近くに引き寄せる。\n味方ユニットが居なかった場合、指定地点まで移動する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"射程" + (MaxRanges[1]/Def.MapTipLengthOnField) + "範囲"+ affectRadiuses[1],
				"射程" + (MaxRanges[2]/Def.MapTipLengthOnField) + "範囲"+ affectRadiuses[2],
				"射程" + (MaxRanges[3]/Def.MapTipLengthOnField) + "範囲"+ affectRadiuses[3],
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, OnFinishPrivilegeMode, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, OnFinishPrivilegeMode, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, OnFinishPrivilegeMode, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName (Name);
				if(null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}

				spell.ResetCoolTime ();

				float affectRadius = affectRadiuses[level];

				var targetPos = selfUnit.Faction.World.GetGroundAtPos (targetPosition);

				// effect offset
				Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);

				/// change effect
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(CaptureChargeEffect).FullName, targetPos + offset);
				var scale = 8.0f*(affectRadius);
				effect.SetScale (new Vector3D ( scale, scale, scale ) );
				
				if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect,chargeTime[level],this.Name + "chargeEffect")))
					effect.Stop();
				
				for(int i=0;i<chargeTime[level];++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}

				// stay stop
				selfUnit.Speed = 0.0f;

				/// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(RopeEffect).FullName, targetPos + offset);
				areaEffect.SetScale (new Vector3D (scale, scale, scale));
				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin02).FullName, selfUnit.Pos);

				var selfFaction = selfUnit.Faction;
				/// search targets
				var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, targetPosition)).OfType<IUnit>().Where (_ => _ != null && _.UniqueID != selfUnit.UniqueID);
				bool flag = false;
				foreach (var u in targets) {
					if(null == u || !u.Exists() || !selfFaction.World.IsSameTeam(u.Faction,selfFaction))
					{
						continue;
					}

					u.Warp(new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y),true);

					flag = true;
				}
				if(!flag)
				{
					selfUnit.Warp (targetPosition,true);
				}
			}
			yield return false;
		}
		public void OnFinishPrivilegeMode (IUnit selfUnit,int level)
		{
			;
		}
	}
}

#endif