﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Black
{
	class Fucho : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		public Fucho()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "符牒「死蝶の舞」";
			string basePath = "Black/Spell/Fuchou/";

			Description = "蝶弾のヒットしたユニットからHPを吸収する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"HP吸収20%",
				"HP吸収40%",
				"HP吸収60%",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				var offset = new Vector3D (2, 2, 1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(FuchouEffect).FullName,selfUnit.Pos);
				var scale = affectRadius * 10;
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);
				if (! selfUnit.AbilityManager.AppendAbility (new FollowEffectAbility (effect, offset, 15, this.Name + "areaEffect")))
					effect.Stop ();
			} else {
				selfUnit.AbilityManager.ExtendLifetimeWithName (1, this.Name + "areaEffect",15);
			}

			foreach (var b in selfUnit.ModifiedBulletParameters) {
				switch(level)
				{
				case 1:
					b.OnHit += OnHit_lv1;
					break;
				case 2:
					b.OnHit += OnHit_lv2;
					break;
				case 3:
					b.OnHit += OnHit_lv3;
					break;
				default:
					break;
				}
			}

			return true;
		}
		
		public void OnHit_lv1 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,0.2f);
		}
		public void OnHit_lv2 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,0.4f);
		}
		public void OnHit_lv3 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,0.6f);
		}
		public void OnHit (IBullet bullet, ITargetObject target,float absRatio)
		{
			var selfUnit = bullet.Generator as IUnit;

			if(null == selfUnit)
				return;

			var u = target as IUnit;


			if (null != u) {
				var damage = bullet.Attack;

				var bounsRatio = 1.0f;

				foreach (var type in u.Parameter.UnitType) {
					if (bullet.Parameter.DamageBonus.Length > (int)type)
						bounsRatio *= bullet.Parameter.DamageBonus [(int)type];
				}

				damage = (int)Math.Max((bullet.Attack - (bullet.Parameter.IgnoreArmor?0:u.Armor)) * bounsRatio,1);

				// abs hp
				selfUnit.HP += (int)Math.Ceiling(damage*absRatio);
			}
		}
	}
}
#endif