﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Black
{
	class Hangon : thrts.SpellParameter<thrts.IUnit>
	{
		const float affectRadius = 12;
		
		const int interval = Def.FPS/2;
		const int chargeTime = 8 *Def.FPS;

		const float absRatio = 0.01f;
		const float damageRatio = 0.6f;

		const string totalAbsHpKey = "total_abs_hp_key";

		public Hangon()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "「反魂蝶」";
			string basePath = "Black/Spell/Hangon/";
			
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180*Def.FPS;

			Description = "詠唱中、周囲に居る全てのユニットからHPを収集する。\n詠唱が中断するか完了したとき、収集したHPに比例するダメージを周囲の全ユニットに与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"0.5秒毎にHP"+absRatio.ToString("###.%")+"を収集、収集HPの60%をダメージ",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();
			selfUnit.ShowCutin();
			
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			/// create spell Effect;
			var areaEffect = world.EffectManager.CreateEffect (typeof(HangonAreaEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 6.0f*(affectRadius);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin01).FullName, selfUnit.Pos);


			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility(Spelling,areaEffect,offset,chargeTime,this.Name + "areaEffect",interval)))
				areaEffect.Stop();
			selfUnit.AddOnVanishEvent(ExplodeDamage);

			for(int i=0;i<chargeTime;++i)
				yield return true;
			selfUnit.RemoveOnVanishEvent(ExplodeDamage);

			ExplodeDamage(selfUnit);

			yield return false;
		}

		public bool Spelling(IUnit selfUnit)
		{
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			if(!selfUnit.UserValues.ContainsKey(totalAbsHpKey))
				selfUnit.UserValues.Add (totalAbsHpKey,0);
			int totAbsHP = (int)selfUnit.UserValues[totalAbsHpKey];

			/// search targets
			var targets = world.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>();
			foreach (var u in targets) {
				if ( null == u || !u.Exists() )
					continue;

				var abs = (int)Math.Ceiling(u.MaxHP * absRatio);
				totAbsHP += abs;

				// only other unit
				if (u.UniqueID == selfUnit.UniqueID)
					continue;

				u.HP -= abs;

				var offset = new Vector3D(2, 2, 1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(HangonEffect).FullName, u.Pos + offset);
				var scale = 10.0f;
				effect.SetScale(new Vector3D(scale, scale, scale));
				effect.SetPosition(u.Pos + offset);
			}

			selfUnit.UserValues[totalAbsHpKey] = totAbsHP;

			return true;
		}
		public void ExplodeDamage (IUnit unit)
		{
			if (unit.UserValues.ContainsKey (totalAbsHpKey)) {

				var offset = new Vector3D(2, 2, 1);
				var effect = unit.Faction.World.EffectManager.CreateEffect(typeof(HangonEmitEffect).FullName, unit.Pos);
				var scale = affectRadius * 8;
				effect.SetScale(new Vector3D(scale, scale, scale));
				effect.SetPosition(unit.Pos + offset);
				if (!unit.AbilityManager.AppendAbility(new FollowEffectAbility(effect, offset, 20, this.Name + "emitEffect")))
					effect.Stop();

				unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Burst).FullName, unit.Pos);


				int totAbsHP = (int)unit.UserValues[totalAbsHpKey];
				int damage = (int)(totAbsHP*damageRatio);
				var targets = unit.Faction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (unit.Pos.X, unit.Pos.Y))).OfType<IUnit>();
				foreach (var u in targets) {
					if ( null == u || !u.Exists() || u.UniqueID == unit.UniqueID )
						continue;
					u.BeDamaged(unit,damage,AttackBonusSet.None,true);
				}

				if(unit.HP > 0)
					unit.HP += totAbsHP-damage;

				unit.UserValues[totalAbsHpKey] = 0;
			}

			return;
		}
	}
}
#endif