using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Ability.Blue;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Blue
{
	class PerfectFreeze : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		float[] affectRadiuses;

		const int duration = 8*Def.FPS;
		const int interval = 10;
		const int chargeTime = 2*Def.FPS;
		public PerfectFreeze()
		{
			DefaultLevel = 0;
			RequireLevels[0] = 6;
			MaxLevel = 1;
			Name = "凍符「パーフェクトフリーズ」";
			string basePath = "Blue/Spell/PerfectFreeze/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180*30;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 10;


			Description = "周囲の敵ユニットは攻撃とスペルができなくなる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃不可、スペル不可。凍傷時はさらに移動不可。",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3 ;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();
			
			float affectRadius = affectRadiuses[level];


			// charge effect
			var chargeEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.ChargeEffect).FullName,selfUnit.Pos);
			var scale = affectRadius / 4.0f;
			chargeEffect.SetScale(new Vector3D(scale,scale,scale));
			if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(chargeEffect,chargeTime,this.Name+"chargeEffect")))
			{
				chargeEffect.Stop();
			}

			// charge
			for(int i=0;i<chargeTime;++i)
				yield return true;

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);

			selfUnit.ShowCutin();

			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(PerfectEffect).FullName, selfUnit.Pos + offset);
			scale = affectRadius * Def.MapTipLengthOnField / 8.0f;
			areaEffect.SetScale (new Vector3D (scale, scale, scale));
			
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),areaEffect,offset,duration,this.Name + "areaEffect",interval)))
				areaEffect.Stop();
			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			float affectRadius = affectRadiuses[level];

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			var selfFaction = selfUnit.Faction;
			// damage 
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>().Where (_ => _ != null);
			foreach (var u in targets) {
				if ( null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction,selfFaction) ) {

					var ability = u.AbilityManager.GetAbilityWithName(PerfectFreezeAbility.AbilityName);

					// onHit unit
					if(null == ability)
					{
						if(u.AbilityManager.AppendAbility(new PerfectFreezeAbility(interval*2)))
							;
					}
					else
					{
						ability.ExtendLifetime (interval,interval*2);
					}
				}
			}
			
			return true;		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class PerfectFreezeAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		const int affectRadius = 2;
		public const string AbilityName = "凍符「パーフェクトフリーズ」";
		public PerfectFreezeAbility (int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				// 凍傷なら移動不可
				if(unit.AbilityManager.ContainsAbility(ChilblainAbility.AbilityName))
					unit.Speed = 0.0f;
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.IsForbiddenAttacking = true;
				unit.IsForbiddenSpelling = true;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnAdd (IUnit unit)
		{

			return base.OnAdd (unit);
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists ()) {
				unit.IsForbiddenAttacking = false;
				unit.IsForbiddenSpelling = false;
			}
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}
