﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Ability.Blue;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Blue
{
	class IkuSkill:thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 5;
		const int interval = 10;
		public IkuSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			IsPassive = true;
			Name = "雷符「エレキテルの龍宮」";
			string basePath = "Blue/Spell/IkuSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = 4*30;
			CoolTimes[1] = 4*30;

			Description = "攻撃が即時必中になり、着弾地点近くに居るヒロイン以外の敵ユニットに移動速度低下を与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"即時必中, 移動速度-"+IkuParalyzeSkillAbility.DecAttackSpeedRatio.ToString("###.%"),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (!selfUnit.Exists ())
				return false;

			if (selfUnit.AbilityManager.ContainsAbility (IkuSkillAbility.AbilityName)) {
				selfUnit.AbilityManager.ExtendLifetimeWithName(interval,IkuSkillAbility.AbilityName,interval*2);
			}
			else
			{
				selfUnit.AbilityManager.AppendAbility(new IkuSkillAbility(interval*2));
			}

			return true;
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			yield return false;
		}
	}
}


namespace LotusCraft.Ability.Blue
{
	class IkuSkillAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		const int affectRadius = 2;
		public const string AbilityName = "雷符「エレキテルの龍宮」";
		const int paralyzeDuration = 4 * Def.FPS;
		public IkuSkillAbility(int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;

		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case thrts.AbilityOrder.ConstDecParam:
					break;
				case thrts.AbilityOrder.RateDecParam:
					break;
				case thrts.AbilityOrder.ConstIncParam:
					break;
				case thrts.AbilityOrder.RateIncParam:
					break;
				case AbilityOrder.OtherExecs:
					var b = unit.GetModifiedBulletParameter(typeof(IkuBullet).FullName);
					if (null != b)
					{
						b.Type = BulletType.ShortRange;
					}
					if (effect != null)
						effect.SetPosition(unit.Pos + new Vector3D(2, 2, 1));
					break;
				default:
					break;
			}

			return true;
		}
		public override bool OnVanish(IUnit unit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}

			return base.OnVanish(unit);
		}
		public override bool ModifyBullet(IBullet bullet)
		{
			bullet.AddOnHitEvent(OnHitEffect);
			return base.ModifyBullet(bullet);
		}

		void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			var faction = bullet.Generator.Faction;

			var offset = new Vector3D(0, 0, 1);
			var effect = faction.World.EffectManager.CreateEffect(typeof(IkuThunderEffect).FullName, target.Pos + offset, faction);
			var scale = new Vector3D(10, 10, 10);
			effect.SetScale(scale);

			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Thunder1).FullName, target.Pos);

			var targets = bullet.Generator.Faction.World.GetObjectFromCircle(new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(target.Pos.X, target.Pos.Y))).OfType<IUnit>();
			foreach (var tu in targets)
			{
				// 味方とヒロインには当たらない
				if (bullet.Generator.Faction.World.IsSameTeam(bullet.Generator.Faction, tu.Faction) || tu.Parameter.IsUnique)
					continue;

				if (tu != null && tu.Exists())
				{
					var ability = tu.AbilityManager.GetAbilityWithName(IkuParalyzeSkillAbility.AbilityName) as IkuParalyzeSkillAbility;
					if (null == ability)
					{
						if (!tu.AbilityManager.AppendAbility(new IkuParalyzeSkillAbility(paralyzeDuration)))
							;
					}
					else
					{
						ability.DurationTime = paralyzeDuration;
					}
				}

			}
		}
	}

	class IkuParalyzeSkillAbility : FollowEffectAbility
	{
		public const float DecAttackSpeedRatio = 0.25f;
		public const string AbilityName ="雷符「エレキテルの龍宮」" + "_paralyze";
		public IkuParalyzeSkillAbility (int duration = 4*Def.FPS)
			: base()
		{
			this.DurationTime = duration;
			this.Name = AbilityName;

		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Speed -= DecAttackSpeedRatio*unit.Speed;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
}
