﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;

namespace LotusCraft.Spell.Blue
{

	class Kioku:thrts.SpellParameter<thrts.IUnit>
	{
		int[] durations;
		float[] affectRadiuses;
		float[] decArmorRatio;

		public Kioku()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "記憶「ＤＮＡの瑕」";
			string basePath = "Blue/Spell/Kioku/";

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] =  3*Def.MapTipLengthOnField;
			MaxRanges[2] =  3*Def.MapTipLengthOnField;
			MaxRanges[3] =  4*Def.MapTipLengthOnField;

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 60*30;
			CoolTimes[1] = 20*30;
			CoolTimes[2] = 20*30;
			CoolTimes[3] = 20*30;

			durations = new int[MaxLevel + 1];
			durations[0] = 0;
			durations[1] = 8*Def.FPS;
			durations[2] = 12*Def.FPS;
			durations[3] = 14*Def.FPS;

			decArmorRatio = new float[MaxLevel+1];
			decArmorRatio[0] = 0;
			decArmorRatio[1] = 0.50f;
			decArmorRatio[2] = 1.0f;
			decArmorRatio[3] = 1.0f;

			affectRadiuses = new float[MaxLevel+1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 2;
			affectRadiuses[2] = 3;
			affectRadiuses[3] = 3;


			Description = "範囲内にいる敵ユニットの防御力を低下させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲"+affectRadiuses[1]+"、防御力-"+decArmorRatio[1].ToString("###.%")+"、"+durations[1]/Def.FPS+"秒間",
				"範囲"+affectRadiuses[2]+"、防御力-"+decArmorRatio[2].ToString("###.%")+"、"+durations[2]/Def.FPS+"秒間",
				"範囲"+affectRadiuses[3]+"、防御力-"+decArmorRatio[3].ToString("###.%")+"、"+durations[3]/Def.FPS+"秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			spell.ResetCoolTime ();

			var affectRadius = affectRadiuses [level];
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var targetPos = world.GetGroundAtPos (targetPosition);

			bool ret = false;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, targetPosition)).OfType<IUnit> ();
			foreach (var u in targets) {
				if (null == u || !u.Exists () || selfFaction.World.IsSameTeam (u.Faction, selfFaction))
					continue;


				if (u.AbilityManager.ContainsAbility (KiokuAbility.AbilityName))
					u.AbilityManager.ExtendLifetimeWithName (durations [level], KiokuAbility.AbilityName, durations [level]);
				else {
					Vector3D offset = new Vector3D (1, 1, 2);
					// hit effect 
					var hitEffect = world.EffectManager.CreateEffect (typeof(DnaAreaEffect).FullName, u.Pos + offset);
					var scale = 6;
					hitEffect.SetScale (new Vector3D (scale, scale, scale));
					if (!u.AbilityManager.AppendAbility (new FollowEffectAbility (hitEffect, offset, 2 * Def.FPS, this.Name + "HitEffect")) && null != hitEffect)
						hitEffect.Stop ();

					IEffectCommon effect = world.EffectManager.CreateTextureEffect (typeof(DnaEffect).FullName, u.Pos + offset);

					if (!u.AbilityManager.AppendAbility (new KiokuAbility (effect, offset, durations [level], decArmorRatio [level])) && null != effect) {
						effect.Stop ();
						effect = null;
					}
					
				}
				ret = true;
			}

			if (!ret) {
				// miss effect 
				var hitEffect = world.EffectManager.CreateEffect (typeof(DnaAreaEffect).FullName, targetPos);
				var scale = 4*affectRadius   ;
				hitEffect.SetScale (new Vector3D (scale, scale, scale));
				if (!selfUnit.AbilityManager.AppendAbility (new EffectContinuousAbility (hitEffect, 2 * Def.FPS, this.Name + "HitEffect")) && null != hitEffect)
					hitEffect.Stop ();
			}

			yield return false;
		}
		public bool CastAction (thrts.IUnit selfUnit, int level)
		{

			return true;
		}

	}
}

namespace LotusCraft.Ability.Blue
{
	class KiokuAbility : thrts.AbilityTemplate
	{
		IEffectCommon effect;
		Vector3D offset;
		public float decArmorRatio;
		public static string AbilityName = "記憶「ＤＮＡの瑕」";
		public KiokuAbility(IEffectCommon effect,Vector3D offset,int duration,float decArmorRatio)
		{
			this.effect = effect;
			this.offset = offset;
			this.DurationTime = duration;
			this.decArmorRatio = decArmorRatio;

		}
		
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Armor -= (int)Math.Ceiling(unit.Armor*decArmorRatio);
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(null != effect)
					effect.SetPosition(unit.Pos+offset);
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != effect) {
				effect.Stop();
				effect = null;
			}
			return base.OnVanish (unit);
		}
	}
}



#endif
