﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;

namespace LotusCraft.Spell.Blue
{

	class Philosophy:thrts.SpellParameter<thrts.IUnit>
	{
		float[] durations;
		float[] affectRadiuses;
		const int interval = 5;
		public Philosophy()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "「嫌われ者のフィロソフィ」";
			string basePath = "Blue/Spell/Philosophy/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 60*30;
			CoolTimes[1] = 20*30;
			CoolTimes[2] = 20*30;
			CoolTimes[3] = 20*30;

			durations = new float[MaxLevel + 1];
			durations[0] = 0;
			durations[1] = 8;
			durations[2] = 10;
			durations[3] = 12;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 4;
			affectRadiuses[2] = 4;
			affectRadiuses[3] = 5;

			Description = "周囲の敵ユニットはこいし以外を目標とする攻撃に参加出来ない";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲"+affectRadiuses[1]+"、"+durations[1]+"秒間",
				"範囲"+affectRadiuses[2]+"、"+durations[2]+"秒間",
				"範囲"+affectRadiuses[3]+"、"+durations[3]+"秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			if (selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				yield return false;
				yield break;
			}

			spell.ResetCoolTime ();

			Vector3D offset = new Vector3D (1, 1, 2);
			IEffect effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(PhilosophyAreaEffect).FullName, selfUnit.Pos + offset,selfUnit.Faction);
			var scale = 8 * affectRadiuses [level];
			if (null != effect) {
				effect.IsShownToOnlyAlly = true;
				effect.SetScale (new Vector3D (scale, scale, scale));
			}
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),effect,offset,(int)(durations[level]*Def.FPS),this.Name+"areaEffect",interval)) && null != effect)
			{
				effect.Stop();
				effect = null;
			}

			yield return false;
		}
		public bool Spelling (IUnit selfUnit, int level)
		{
			if(null == selfUnit || !selfUnit.Exists())
				return false;

			var affectRadius = affectRadiuses[level];
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit> ();
			foreach (var u in targets) {
				if (null == u || !u.Exists () || selfFaction.World.IsSameTeam (u.Faction, selfFaction))
					continue;

				if(u.AbilityManager.ContainsAbility(PhilosophyAbility.AbilityName))
					u.AbilityManager.ExtendLifetimeWithName(interval,PhilosophyAbility.AbilityName,interval*2);
				else
				{
					Vector3D offset = new Vector3D(1,1,2);
					IEffect effect = world.EffectManager.CreateEffect(typeof(PhilosophyAreaEffect).FullName,u.Pos+offset);
					var scale = 8;
					if(null != effect)
						effect.SetScale(new Vector3D(scale,scale,scale));

					if(!u.AbilityManager.AppendAbility(new PhilosophyAbility(effect,offset,interval*2,selfUnit)) && null != effect)
					{
						effect.Stop();
						effect = null;
					}

				}
			}

			return true;
		}
		public bool CastAction (thrts.IUnit selfUnit, int level)
		{

			return true;
		}

	}
}

namespace LotusCraft.Ability.Blue
{
	class PhilosophyAbility : FollowEffectAbility
	{
		public static string AbilityName = "「嫌われ者のフィロソフィ」";
		Func<IUnit,ITargetObject,float,int> selectBullet;
		public IUnit speller = null;
		public PhilosophyAbility(IEffect effect,Vector3D offset,int duration,IUnit speller)
			:base(effect,offset,duration,AbilityName)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.speller = speller;
		}
		
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				selectBullet = unit.SelectBullet;
				unit.SelectBullet = this.OverrideSelectBullet;
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public int OverrideSelectBullet (IUnit unit, ITargetObject target, float distance)
		{
			var u = target as IUnit;

			if (null != u && u != speller) {
				return -1;
			} else if (null != selectBullet) {
				return selectBullet (unit, target, distance);
			} else {
				return 0;
			}
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists ()) {
				// 自動索敵を再開
				unit.DoFindEnemyAutomatically = true;
				// reset
				unit.IsTransparent = false;
				unit.IsPreventedFromFindingEnemy = false;
			}
			return base.OnVanish (unit);
		}
	}
}


#endif