﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;

namespace LotusCraft.Spell.Blue
{

	class Rose:thrts.SpellParameter<thrts.IUnit>
	{
		float[] durations;
		float[] affectRadiuses;
		const int interval = 5;
		const int chargeTime = 1*Def.FPS;
		public Rose()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "「サブタレイニアンローズ」";
			string basePath = "Blue/Spell/Rose/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999*30;
			CoolTimes[1] = 180*30;

			durations = new float[MaxLevel + 1];
			durations[0] = 0;
			durations[1] = 12;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 8;

			Description = "周囲の敵ユニットは攻撃と移動が出来なくなります。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"移動不可、攻撃不可。" + durations[1] + "秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}
			
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			if(!spell.CanExec){
				yield return false;
				yield break;
			}

			
			if(selfUnit.AbilityManager.ContainsAbility(this.Name+"areaEffect"))
			{
				yield return false;
				yield break;
			}

			spell.ResetCoolTime();

			// charge effect
			var chargeEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.ChargeEffect).FullName,selfUnit.Pos);
			var scale = 1.0f*affectRadiuses[level];
			chargeEffect.SetScale(new Vector3D(scale,scale,scale));
			if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(chargeEffect,chargeTime,this.Name+"chargeEffect")))
			{
				chargeEffect.Stop();
			}
			
			// charge
			for(int i=0;i<chargeTime;++i)
				yield return true;

			selfUnit.ShowCutin();

			Vector3D offset = new Vector3D(1,1,1);
			IEffect effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(RoseAreaEffect).FullName,selfUnit.Pos+offset);
			scale = 2.6f * affectRadiuses[level];
			if(null != effect)
				effect.SetScale(new Vector3D(scale,scale,0.1f));

			var targetPos = new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y);
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,targetPos,level),effect,offset,(int)(durations[level]*Def.FPS),this.Name+"areaEffect",interval)) && null != effect)
			{
				effect.Stop();
				effect = null;
			}
			
			yield return false;
		}
		public bool Spelling (IUnit selfUnit,Vector2D targetPos, int level)
		{
			if(null == selfUnit || !selfUnit.Exists())
				return false;
			
			var affectRadius = affectRadiuses[level];
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField,targetPos)).OfType<IUnit> ();
			foreach (var u in targets) {
				if (null == u || !u.Exists () || selfFaction.World.IsSameTeam (u.Faction, selfFaction))
					continue;
				
				if(u.AbilityManager.ContainsAbility(RoseAbility.AbilityName))
					u.AbilityManager.ExtendLifetimeWithName(interval,RoseAbility.AbilityName,interval*2);
				else
				{
					Vector3D offset = new Vector3D(1,1,2);
					IEffect effect = world.EffectManager.CreateEffect(typeof(RoseEffect).FullName,u.Pos+offset);
					var scale = 4;
					if(null != effect)
						effect.SetScale(new Vector3D(scale,scale,scale));
					
					if(!u.AbilityManager.AppendAbility(new RoseAbility(effect,offset,interval*2)) && null != effect)
					{
						effect.Stop();
						effect = null;
					}
					
				}
			}
			
			return true;
		}

	}
}

namespace LotusCraft.Ability.Blue
{
	class RoseAbility : FollowEffectAbility
	{
		public static string AbilityName = "「サブタレイニアンローズ」";
		public RoseAbility(IEffect effect,Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
		}

		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				// 自動索敵を停止
				unit.DoFindEnemyAutomatically = false;
				// 不可視系の設定は定数増加で行う
				unit.IsSemitransparent = true;
				//
				unit.IsForbiddenAttacking = true;
				//
				unit.Speed= 0;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists ()) {
				// 自動索敵を再開
				unit.DoFindEnemyAutomatically = true;
				// reset
				unit.IsTransparent = false;
				unit.IsPreventedFromFindingEnemy = false;
				unit.IsForbiddenAttacking = false;
			}
			return base.OnVanish (unit);
		}
	}
}

#endif