﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Building.Blue;

namespace LotusCraft.Spell.Blue
{
	class LunarSkill : thrts.SpellParameter<thrts.IUnit>
	{
		public LunarSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "障光「ムーンライトウォール」";
			string basePath = "Blue/Spell/Lunar/";

			CoolTimes = new int[2];
			CoolTimes[0] = 30 * Def.FPS;
			CoolTimes[1] = 30 * Def.FPS;

			MaxRanges = new float[2];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6 * Def.MapTipLengthOnField;

			Description = "味方のみ通行可能な壁を出現させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				(15)+ "秒持続",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			yield return Spelling(selfUnit, targetPosition, level);
		}

		public bool Spelling(thrts.IUnit selfUnit, Vector2D targetPos, int level)
		{

			var selfFaction = selfUnit.Faction;
			var world = selfUnit.Faction.World;

			var spell = selfUnit.GetSpellWithName(this.Name);

			if (null == spell)
				return false;
			if (!spell.CanExec)
			{
				return false;
			}


			var building = world.BuildingManager.Create(typeof(Shade).FullName, selfUnit.Parameter.Faction, (int)(targetPos.X / thrts.Def.MapTipLengthOnField), (int)(targetPos.Y / thrts.Def.MapTipLengthOnField), 100.0f, selfFaction);
			if (building != null)
			{
				building.AbilityManager.AppendAbility(new SuicideAbility(null, 15 * Def.FPS, this.Name + "suicide"));

				// 建物の中に何かあったら外に出す
				var width = building.Size * Def.MapTipLengthOnField;
				var height = building.Size * Def.MapTipLengthOnField;
				var targets = world.GetObjectFromRectangle(new RectF(), new RectF(building.Pos.X - width / 2, building.Pos.Y - height / 2, width, height)).OfType<IUnit>();

				foreach (var tu in targets)
				{
					tu.Warp(new Vector2D(tu.Pos.X, tu.Pos.Y), true);
				}

				spell.ResetCoolTime();
			}
			return true;
		}
	}
}
