using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect.Blue;

namespace LotusCraft.Spell.Blue
{
	class HumanCage:thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime;
		float[] affectRadiuses;
		float[] durationTime;

		const int interval = 10;

		public HumanCage()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "鳥符「ヒューマンケージ」";
			string basePath = "Blue/Spell/HumanCage/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 20*30;
			CoolTimes[2] = 20*30;
			CoolTimes[3] = 20*30;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6*32;
			MaxRanges[2] = 7*32;
			MaxRanges[3] = 8*32;

			chargeTime = new int[MaxLevel+1];
			chargeTime[0] = 0;
			chargeTime[1] = 2*Def.FPS;
			chargeTime[2] = 2*Def.FPS;
			chargeTime[3] = 2*Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 3;
			affectRadiuses[2] = 4;
			affectRadiuses[3] = 5;

			durationTime = new float[MaxLevel + 1];
			durationTime[0] = 0;
			durationTime[1] = 6;
			durationTime[2] = 8;
			durationTime[3] = 10;

			Description = "効果範囲内の敵ユニットは攻撃ができなくなる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"持続" + durationTime[1]+"秒、範囲"+ affectRadiuses[1]+"、攻撃不可",
				"持続" + durationTime[2]+"秒、範囲"+ affectRadiuses[2]+"、攻撃不可",
				"持続" + durationTime[3]+"秒、範囲"+ affectRadiuses[3]+"、攻撃不可、スペル使用不可",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName (Name);
				if(null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}

				spell.ResetCoolTime ();

				float affectRadius = affectRadiuses[level];

				var targetPos = selfUnit.Faction.World.GetGroundAtPos (targetPosition);

				// effect offset
				Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);

				/// change effect
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(Effect.Common.ChargeEffect).FullName, targetPos + offset, selfUnit.Faction);
				var scale = 1.0f*(affectRadius);
				effect.SetScale (new Vector3D ( scale, scale, scale ) );

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, targetPos);
				

				if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect,chargeTime[level],this.Name + "chargeEffect")))
					effect.Stop();
				
				for(int i=0;i<chargeTime[level];++i)
				{
					yield return true;
				}

				/// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(CageAreaEffect).FullName, targetPos + offset, selfUnit.Faction);
				scale = 5.0f*affectRadius;
				areaEffect.SetScale (new Vector3D (scale, scale, scale));

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, targetPos);
				

				var ability = new SpellContinuousAbility((unit)=>Spelling(unit,targetPosition,level),areaEffect,offset,(int)(durationTime[level]*Def.FPS),this.Name + "areaEffect",interval);
				ability.FollowEffect = false;
				if(!selfUnit.AbilityManager.AppendAbility(ability))
					areaEffect.Stop();

			}
			yield return false;
		}

		public bool Spelling(IUnit selfUnit,Vector2D targetPosition,int level)
		{
			float affectRadius = affectRadiuses[level];

			var targetPos = selfUnit.Faction.World.GetGroundAtPos (targetPosition);

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);

			var selfFaction = selfUnit.Faction;
			// damage 
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (targetPos.X, targetPos.Y))).OfType<IUnit>().Where (_ => _ != null);
			foreach (var u in targets) {
				if ( null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction,selfFaction) && u.UniqueID != selfUnit.UniqueID ) {
					var ability = u.AbilityManager.GetAbilityWithName(HumanCageAbility.AbilityName);
					if(null != ability)
					{
						ability.ExtendLifetime(interval,interval*2);
					}
					else
					{
						IEffect effect = null;
						
						// エフェクトを描画
						{
							var e = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(CageEffect).FullName, u.Pos + offset);
							e.SetScale(new Vector3D(8, 8, 8));
							effect = e;
						}
						if(!u.AbilityManager.AppendAbility(new HumanCageAbility(level,interval*2,effect)))
						{
							if(null != effect)
								effect.Stop();
						}
					}
				}
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class HumanCageAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		const int affectRadius = 2;
		public const string AbilityName = "鳥符「ヒューマンケージ」";
		public float DecMoveRatio = 0.2f;
		public bool forbiddenSpelling = false;
		public HumanCageAbility (int level,int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			
			switch (level) {
			case 1:
				forbiddenSpelling = false;
				break;
			case 2:
				forbiddenSpelling = false;
				break;
			case 3:
				forbiddenSpelling = true;
				break;
			default:
				break;
			}
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.IsForbiddenAttacking = true;
				unit.IsForbiddenSpelling = forbiddenSpelling;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			unit.IsForbiddenAttacking = false;
			unit.IsForbiddenSpelling = false;
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}