﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;

namespace LotusCraft.Spell.Blue
{
	class StarSkill : thrts.SpellParameter<thrts.IUnit>
	{
		/// <summary>
		/// The mode change time[s].
		/// </summary>
		const float modeChangeTime = 4.0f;
		const float affectRadius = 30;
		const int interval = 15;
		public StarSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "星符「スターライトレイン」";
			string basePath = "Blue/Spell/Star/";

			CoolTimes = new int[2];
			CoolTimes[0] = 4 * Def.FPS;
			CoolTimes[1] = 4 * Def.FPS;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"移動と攻撃の能力を失う代わり、\n広い視界と視界外の敵にも気付く探知能力を得る。",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				null,
				null,
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>AutoCastAction(unit,1)),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",(unit)=>PrivilegeModeSpelling(unit,1)),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public void Spelling(thrts.IUnit selfUnit, int level)
		{
			thrts.AbilityTemplate abiity = null;
			selfUnit.AbilityManager.AppendAbility(abiity);
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;

			/// reset cooltime
			selfUnit.GetSpellWithName(Name).ResetCoolTime();

			int modeCTime = (int)(modeChangeTime * Def.FPS);

			if (!selfUnit.GetIAutoCastWithName(this.Name).IsEnable)
			{
				/// change effect
				var offset = new Vector3D(0, 0, 1 - Def.FlyingHeightOffset);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(CommonBolEffect).FullName, selfUnit.Pos + offset);
				var scale = 1.0f * (affectRadius);
				effect.SetScale(new Vector3D(scale, scale, scale));
				// 視界外でも範囲表示が見える
				effect.IsShownInFog = true;
				// 敵には範囲表示が見えないように
				effect.IsShownToOnlyAlly = true;

				for (int i = 0; i < modeCTime; ++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}
				// stay stop
				selfUnit.Speed = 0.0f;

				// clear charge effect
				effect.Stop();

				/// append Sensor mode ability
				if (!selfUnit.AbilityManager.ContainsAbility(Ability.Blue.StarSensorAbility.AbilityName))
				{
					/// set area effect
					effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(CommonBolAreaEffect).FullName, selfUnit.Pos + offset);
					effect.SetScale(new Vector3D(scale, scale, scale));
					// 視界外でも範囲表示が見える
					effect.IsShownInFog = true;
					// 敵には範囲表示が見えないように
					effect.IsShownToOnlyAlly = true;
					if (!selfUnit.AbilityManager.AppendAbility(new Ability.Blue.StarSensorAbility(level, effect, offset)))
						effect.Stop();
				}

				/// toggle autocast
				selfUnit.GetIAutoCastWithName(this.Name).ToggleAutoCast();
			}
			else
			{

				/// toggle autocast
				selfUnit.GetIAutoCastWithName(this.Name).ToggleAutoCast();

				/// remove ability
				if (selfUnit.AbilityManager.ContainsAbility(Ability.Blue.StarSensorAbility.AbilityName))
				{
					selfUnit.AbilityManager.RemoveAbility(Ability.Blue.StarSensorAbility.AbilityName);
				}

				for (int i = 0; i < modeCTime; ++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}

			}


			yield return false;
		}

		public bool AutoCastAction(thrts.IUnit unit, int level)
		{
			/// 
			if (unit.GetIAutoCastWithName(this.Name).IsEnable)
			{
				/// append Cannon mode ability
				if (!unit.AbilityManager.ContainsAbility(Ability.Blue.StarSensorAbility.AbilityName))
				{
					// set area effect
					var offset = new Vector3D(0, 0, 1 - Def.FlyingHeightOffset);
					var scale = 1.0f * (affectRadius);

					var effect = unit.Faction.World.EffectManager.CreateEffect(typeof(CommonBolAreaEffect).FullName, unit.Pos + offset, unit.Faction);
					effect.SetScale(new Vector3D(scale, scale, scale));
					// 視界外でも範囲表示が見える
					effect.IsShownInFog = true;
					// 敵には範囲表示が見えないように
					effect.IsShownToOnlyAlly = true;

					unit.AbilityManager.AppendAbility(new Ability.Blue.StarSensorAbility(level, effect, offset));
				}

				/// search targets
				var targets = unit.Faction.World.GetObjectFromCircle(new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(unit.Pos.X, unit.Pos.Y))).OfType<IUnit>();
				/// show unit marker
				foreach (var tu in targets)
				{
					if (null == tu || tu.Faction.World.IsSameTeam(tu.Faction, unit.Faction))
						continue;

					if (!tu.IsPreventedFromFindingEnemy)
					{
						if (!tu.AbilityManager.ContainsAbility(this.Name + "marker"))
						{
							/// set marker effect
							var offset = new Vector3D(0, 0, 1);
							var scale = unit.Radius / Def.MapTipLengthOnField;

							var effect = unit.Faction.World.EffectManager.CreateEffect(typeof(CommonBolAreaEffect).FullName, tu.Pos + offset, unit.Faction);
							effect.SetScale(new Vector3D(scale, scale, scale));
							// 視界外でもマーカーが見える
							effect.IsShownInFog = true;
							// 敵にはマーカーが見えないように
							effect.IsShownToOnlyAlly = true;
							if (!tu.AbilityManager.AppendAbility(new FollowEffectAbility(effect, offset, interval + 2, this.Name + "marker")))
								effect.Stop();

						}
						else
						{
							tu.AbilityManager.ExtendLifetimeWithName(interval, this.Name + "marker",interval+2);
						}
					}
				}

			}
			else
			{
				/// remove ability
				if (unit.AbilityManager.ContainsAbility(Ability.Blue.StarSensorAbility.AbilityName))
				{
					unit.AbilityManager.RemoveAbility( Ability.Blue.StarSensorAbility.AbilityName);
				}
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class StarSensorAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		Vector3D offset;
		public const string AbilityName = "星符「スターライトレイン」";
		public StarSensorAbility(int level,IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = 1;
			this.effect = effect;
			
		}
		public StarSensorAbility(int level,IEffect effect,Vector3D offset)
		{
			this.Name = AbilityName;
			this.DurationTime = 1;
			this.effect = effect;
			this.offset = offset;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				/// set speed=0;
				unit.Speed = 0.0f;
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.ViewRange = 20 * Def.MapTipLengthOnField;
				// 索敵を停止し攻撃能力を削除
				unit.DoFindEnemyAutomatically = false;
				unit.SelectBullet = DisableBullet;

				// 不可視系の設定は定数増加で行う
				unit.IsPreventedFromFindingEnemy = true;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+offset);
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			unit.DoFindEnemyAutomatically = true;
			unit.IsPreventedFromFindingEnemy = false;
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
		// inf ability
		public override void DecDurationTime ()
		{
			return ;
		}
		static int DisableBullet(IUnit unit,ITargetObject target,float distance)
		{
			/// return invalid bullet
			return -1;
		}
	}
	
}
