﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;

namespace LotusCraft.Spell.Blue
{

	class SunnySkill:thrts.SpellParameter<thrts.IUnit>
	{
		float affectRadius = 2;
		public SunnySkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "光精「クロスディフュージョン」";
			string basePath = "Blue/Spell/Sunny/";

			CoolTimes = new int[2];
			CoolTimes[0] = 15*30;
			CoolTimes[1] = 15*30;

			Description = "周囲の味方ユニットが、敵AIに見つからないようになる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"周囲の味方ユニットが敵AIに見つからない",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,(unit)=>Spelling(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling (thrts.IUnit selfUnit, int level)
		{

			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				var offset = new Vector3D(2,2,1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(CommonBolAreaEffect).FullName, selfUnit.Faction, selfUnit.Pos);
				var scale = affectRadius;
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);
				if (! selfUnit.AbilityManager.AppendAbility(new FollowEffectAbility(effect,offset,1*Def.FPS,this.Name+"areaEffect")))
					effect.Stop();
			}
			else
			{
				selfUnit.AbilityManager.ExtendLifetimeWithName (15,this.Name+"areaEffect",1*Def.FPS);
			}

			var selfFaction = selfUnit.Faction;

			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).Where (_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>selfFaction.World.IsSameTeam(selfFaction,_.Faction))) {
				if(u.AbilityManager.ContainsAbility(SunnySkillAbility.AbilityName))
				{
					u.AbilityManager.ExtendLifetimeWithName(15, SunnySkillAbility.AbilityName,Def.FPS/2);
				}
				else
				{
					if(!u.AbilityManager.AppendAbility(new SunnySkillAbility(Def.FPS/2)))
					{
						;
					}
				}

			}

			return true;
		}

	}
}

namespace LotusCraft.Ability.Blue
{
	class SunnySkillAbility : thrts.AbilityTemplate
	{
		public static string AbilityName = "光精「クロスディフュージョン」";
		public SunnySkillAbility(int duration = Def.FPS/2)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
		}

		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				// 不可視系の設定は定数増加で行う
				// 近くにDetectorが居たら透明になれない1
				if(unit.Faction.World.GetObjectFromCircle(new Circle2D(16*Def.MapTipLengthOnField,new Vector2D(unit.Pos.X,unit.Pos.Y)))
				   .OfType<IUnit>()
				   .Any(_=>!unit.Faction.World.IsSameTeam(_.Faction,unit.Faction) && _.IsDetector && Utl.InCircle(unit.Pos,_.Pos,_.ViewRange) ) )
				{
					unit.IsPreventedFromFindingEnemy = false;
				}
				else
				{
					unit.IsPreventedFromFindingEnemy = true;
				}
				unit.IsSemitransparent = true;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return true;
		}

		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists ()) {
				unit.IsPreventedFromFindingEnemy = false;
				unit.IsSemitransparent = false;
			}
			return base.OnVanish (unit);
		}
	}
	
}