﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;

namespace LotusCraft.Spell.Blue
{
	class Hisouten:thrts.SpellParameter<thrts.IUnit>
	{
		const int damage = 30;
		const int bulidingBonus = 10;
		const int chargeTime = 3*Def.FPS;

		const int duration = 8*Def.FPS;
		const int interval = Def.FPS/2;

		readonly Vector2D sparkSize = new Vector2D(4*Def.MapTipLengthOnField,18*Def.MapTipLengthOnField);

		public Hisouten()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "「全人類の緋想天」";
			string basePath = "Blue/Spell/Hisouten/";

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = sparkSize.Y * 0.7f;

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 180*Def.FPS;

			Description = "指定方向に向けて範囲攻撃を行う。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"" + damage + "ダメージx"+(2*duration/Def.FPS)+"、建物に"+bulidingBonus+"倍",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			spell.ResetCoolTime ();

			// look at target
			selfUnit.ChangeDirection (targetPosition);

			// show cutin
			selfUnit.ShowCutin ();
			var world = selfUnit.Faction.World;

			// charge
			Vector3D offset = new Vector3D (2,2,2);
			var chargeEffect = world.EffectManager.CreateEffect (typeof(HisoutenChargeEffect).FullName, selfUnit.Pos + offset);
			var scale = 10;
			chargeEffect.SetScale (new Vector3D (scale,scale,scale));
			if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(chargeEffect,chargeTime,this.Name + "chargeEffect")))
				chargeEffect.Stop();

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);
			for (int i=0; i<chargeTime; ++i) {
				yield return true;
			}

			// create area effect
			var rot = (float)(Math.Atan2 (targetPosition.X - selfUnit.Pos.X, targetPosition.Y - selfUnit.Pos.Y));
			offset = new Vector3D (24*(float)Math.Sin (rot),24*(float)Math.Cos (rot),32);
			rot *= 180 / (float)Math.PI;
			var areaEffect = world.EffectManager.CreateEffect (typeof(HisoutenAreaEffect).FullName, selfUnit.Pos + offset);
			areaEffect.SetScale (new Vector3D (Def.MapTipLengthOnField, Def.MapTipLengthOnField, Def.MapTipLengthOnField));
			areaEffect.SetRotation (new Vector3D (0, 0, rot));

			var diff = world.GetGroundAtPos(targetPosition) - selfUnit.Pos;
			diff.Z=0;
			diff.Normalize();
			var center = new Vector2D(selfUnit.Pos.X + diff.X*sparkSize.Y/2.0f,selfUnit.Pos.Y + diff.Y*sparkSize.Y/2.0f);
			if (!selfUnit.AbilityManager.AppendAbility (new SpellContinuousAbility ((unit) => Spelling (unit, center, rot), areaEffect, offset, duration, this.Name + "areaEffect", interval))) {
				areaEffect.Stop();
			}

			// ラストスペル中の効果音
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Burst).FullName, selfUnit.Pos);

			for (int i=0; i<duration; ++i) {
				yield return true;
			}

			yield return false;
		}
		public bool Spelling (IUnit selfUnit, Vector2D center, float rot)
		{
			// search targets;
			var world = selfUnit.Faction.World;

			// 後まで少し範囲を広げる
			var offsetY = 2*Def.MapTipLengthOnField;
			var targets = world.GetObjectOnRectangle (center.X, center.Y + offsetY/2.0f, sparkSize.X, sparkSize.Y + offsetY, rot, true, true);
			/*
			selfUnit.Faction.World.DrawDebugRectangle(new Vector3D(center.X, center.Y - offsetY/2.0f,selfUnit.Pos.Z + 10.0f),
			                                          sparkSize.X, sparkSize.Y + offsetY, rot,
			                                          new Color(1.0f, 1.0f, 1.0f, 1.0f));
			//*/
			foreach (var t in targets) {
				if(null == t || !t.Exists() || world.IsSameTeam(t.Faction,selfUnit.Faction))
					continue;

				var u = t as IUnit;
				var b = t as IBuilding;

				if( null != u || null != b)
				{
					// show hit effect
					var offset = new Vector3D(-2,-2,8);
					var effect = world.EffectManager.CreateEffect(typeof(Effect.Common.HitRemoteRedEffect).FullName, t.Pos + offset);
					var scale = new Vector3D(10,10,10);
					effect.SetScale(scale);
					world.SoundManager.PlaySoundEffect (typeof(SoundEffect.HitOther).FullName,t.Pos);
				}

				if(null != u)
				{
					u.BeDamaged(selfUnit,damage,AttackBonusSet.Hero);
				}
				if(null != b)
				{
					b.BeDamaged(selfUnit,damage*bulidingBonus,AttackBonusSet.Hero);
				}
			}

			return true;
		}
	}
}
#endif