﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Ability.Blue;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Blue
{
	class Kifu : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		float[] decDamageRatio;

		float[] durationTime;

		const int interval = 10;
		public Kifu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "気符「無念無想の境地」";
			string basePath = "Blue/Spell/Kifu/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30*Def.FPS;
			CoolTimes[2] = 30*Def.FPS;
			CoolTimes[3] = 30*Def.FPS;

			durationTime = new float[MaxLevel + 1];
			durationTime[0] = 0;
			durationTime[1] = 10;
			durationTime[2] = 10;
			durationTime[3] = 10;

			decDamageRatio = new float[MaxLevel + 1];
			decDamageRatio[0] = 0;
			decDamageRatio[1] = 0.25f;
			decDamageRatio[2] = 0.50f;
			decDamageRatio[3] = 0.75f;

			Description = "一定時間、天子が受けるダメージが減少する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				durationTime[1] + "秒間、受けるダメージ-" + decDamageRatio[1].ToString("###.%"),
				durationTime[2] + "秒間、受けるダメージ-" + decDamageRatio[2].ToString("###.%"),
				durationTime[3] + "秒間、受けるダメージ-" + decDamageRatio[3].ToString("###.%"),
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);
			
			// onHit unit
			var ability = selfUnit.AbilityManager.GetAbilityWithName (KifuAbility.AbilityName) as KifuAbility;
			if (null != ability) {
				ability.DurationTime = (int)(durationTime[level]*Def.FPS);;
				ability.DamageRatio = 1-decDamageRatio[level];
			} else {
				// effect offset
				Vector3D offset = new Vector3D (0.0f, 0.0f, 0.0f);
			
				/// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(KifuEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
				var scale = 8.0f * (affectRadius);
				areaEffect.SetScale (new Vector3D (scale, scale, scale));

				if (!selfUnit.AbilityManager.AppendAbility (new KifuAbility((1-decDamageRatio[level]),(int)(durationTime[level]*Def.FPS),areaEffect)))
					areaEffect.Stop ();
			}
			yield return false;
		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class KifuAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		public const string AbilityName = "気符「無念無想の境地」";
		public float DamageRatio = 1.0f;
		public Vector3D Offset = new Vector3D();
		public KifuAbility (float damageRatio,int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			this.DamageRatio = damageRatio;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.DamageRatio = DamageRatio;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+Offset);
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}

#endif