﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect.Blue;

namespace LotusCraft.Spell.Blue
{
	class DanmakuKekkai:thrts.SpellParameter<thrts.IUnit>
	{
		const int chargeTime = 1*Def.FPS;
		const int affectRange = 8;
		MusoTenseiAbility[] spellAbilities = new []
		{
			new MusoTenseiAbility(0),
			new MusoTenseiAbility(1),
		};
		public DanmakuKekkai()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;

			Name = "紫奥義「弾幕結界」";
			string basePath = "Blue/Spell/DanmakuKekkai/";
			Description = "敵ユニットを結界の中に綴じ込める。";

			 CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 60*Def.FPS;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (level < 1) {
				yield return false;
				yield break;
			}

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			
			selfUnit.ShowCutin();
			
			// reset cooltime
			selfUnit.GetSpellWithName(Name).ResetCoolTime();
			
			// chage effect
			Vector3D offset = new Vector3D(0.0f,0.0f,1.0f);
			var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(YukariKekkaiChargeEffect).FullName, selfUnit.Pos + offset);
			float effectScale = 2 *affectRange*1.5f;
			effect.SetScale (new Vector3D (effectScale, effectScale, effectScale));
			
			for(int i=0;i<chargeTime;++i)
				yield return true;
			
			//effect.Stop();
			
			// create call Effect;
			effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(YukariKekkaiAreaEffect).FullName, selfUnit.Pos + offset);
			effectScale = Def.MapTipLengthOnField * affectRange / 2.0f / 8.0f;
			effect.SetScale (new Vector3D (effectScale, effectScale, effectScale)); 
			effect.SetRotation(new Vector3D( 0, 0, 45 ));
			var pos = selfUnit.Pos;
			var ability = new Ability.SpellContinuousAbility((unit)=>Spelling(unit,pos,level),effect,offset, 20*Def.FPS, this.Name+"AreaEffect");
			if(null != ability)
				ability.FollowEffect = false;

			/// add spell continuous ability
			bool flag = selfUnit.AbilityManager.AppendAbility(ability);
			if(!flag)
				effect.Stop();

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin02).FullName, selfUnit.Pos);
			
			yield return true;
		}

		public bool Spelling (IUnit selfUnit,Vector3D targetPos, int level)
		{
			if(level < 0 || level>MaxLevel)
				return false;
			
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var outerRect = new RectF(targetPos.X - affectRange * Def.MapTipLengthOnField/2 , targetPos.Y - affectRange * Def.MapTipLengthOnField/2, affectRange * Def.MapTipLengthOnField, affectRange * Def.MapTipLengthOnField);
			var innerRect = new RectF(outerRect.X + outerRect.Width/4, outerRect.Y + outerRect.Height/4, outerRect.Width/2, outerRect.Height/2);
			var targets = selfFaction.World.GetObjectFromRectangle(outerRect,outerRect).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>!selfFaction.World.IsSameTeam(selfFaction,_.Faction)))
			{
				if ( u.Pos.X > innerRect.X && u.Pos.X < innerRect.X + innerRect.Width 
				  && u.Pos.Y > innerRect.Y && u.Pos.Y < innerRect.Y + innerRect.Height)
				{
					continue;
				}

				var ability = u.AbilityManager.GetAbilityWithName(spellAbilities[level].Name);
				if(null == ability)
				{
					/// エフェクトを描画
					var faction = u.Faction;
					Vector3D offset = new Vector3D(2.0f,2.0f,1.0f);
					var effect = world.EffectManager.CreateEffect(typeof(YukariKekkaiEffect).FullName, u.Pos + offset);
					effect.SetScale(new Vector3D(12,12,12));
					if(!u.AbilityManager.AppendAbility(new MusoTenseiAbility(level,innerRect,outerRect,effect)))
						effect.Stop();
				}
				else
				{
					ability.ExtendLifetime(1,spellAbilities[level].DurationTime);
				}
			}
			
			return true;
		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class MusoTenseiAbility : thrts.AbilityTemplate
	{
		public float AtkRate{get; private set;}
		public int IncArmor{get;private set;}
		public RectF InnerRect{ get; private set; }
		public RectF OuterRect{ get; private set; }
		thrts.IEffect effect;
		public MusoTenseiAbility(int level,thrts.IEffect effect = null)
		{
			this.Name = "大結界「博麗弾幕結界」";
			this.DurationTime = Def.FPS/2;
			this.effect = effect;
			switch (level)
			{
			case 1:
				AtkRate = 1.2f;
				IncArmor = 4;
				break;
			default:
				AtkRate = 1.0f;
				IncArmor = 0;
				break;
			}
			
		}
		public MusoTenseiAbility (int level,RectF inner,RectF outer, thrts.IEffect effect = null)
			:this(level,effect)
		{
			this.InnerRect = inner;
			this.OuterRect = outer;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				/// inc armor
				unit.Armor += IncArmor;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case thrts.AbilityOrder.OtherExecs:
				Vector2D pos = new Vector2D(unit.Pos.X,unit.Pos.Y);
				if( unit.Pos.X < OuterRect.X )
					pos.X = OuterRect.X + unit.Radius +4;
				else if( unit.Pos.X > OuterRect.X + OuterRect.Width )
					pos.X = OuterRect.X + OuterRect.Width - unit.Radius;
				if( unit.Pos.Y < OuterRect.Y )
					pos.Y = OuterRect.Y + unit.Radius +4;
				else if( unit.Pos.Y > OuterRect.Y + OuterRect.Height )
					pos.Y = OuterRect.Y + OuterRect.Height - unit.Radius;
				/*
				if( unit.Pos.Y >= InnerRect.Y && unit.Pos.Y <= InnerRect.Y + InnerRect.Height)
				{
					if( unit.Pos.X > InnerRect.X )
						pos.X = InnerRect.X - unit.Radius;
					if( unit.Pos.X < InnerRect.X + InnerRect.Width )
						pos.X = InnerRect.X + InnerRect.Width + unit.Radius + 4;
				}
				/*
				if( unit.Pos.X >= InnerRect.X && unit.Pos.X <= InnerRect.X + InnerRect.Width)
				{
					if( unit.Pos.Y > InnerRect.Y )
						pos.Y = InnerRect.Y - unit.Radius;
					if( unit.Pos.Y < InnerRect.Y + InnerRect.Height )
						pos.Y = InnerRect.Y + InnerRect.Height + unit.Radius + 4;
				}
//*/
				if(pos.X != unit.Pos.X || pos.Y != unit.Pos.Y)
				{
					unit.Escape();
					unit.Enter(pos);
				}

				if(effect != null)
					effect.SetPosition(unit.Pos+new thrts.Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (thrts.IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			return base.OnVanish (unit);
		}
		public override bool ModifyBullet(thrts.IBullet bullet)
		{
			/// inc damage
			bullet.Attack = (int)(bullet.Attack*AtkRate);
			return base.ModifyBullet(bullet);
		}
	}
	
}
