﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


using thrts;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Building.Blue;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;

namespace LotusCraft.Spell.Blue
{
	class Haisen:thrts.SpellParameter<thrts.IUnit>
	{
		const string TXKey = "TXBullets";

		float[] reachRadius;
		float[] bulletCounts;
		public Haisen()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "外力「無限の超高速飛行体」";
			string basePath = "Blue/Spell/Haisen/";

			reachRadius = new float[MaxLevel+1];
			reachRadius[0] = 0;
			reachRadius[1] = 15*Def.MapTipLengthOnField;
			reachRadius[2] = 30*Def.MapTipLengthOnField;
			reachRadius[3] = 45*Def.MapTipLengthOnField;

			bulletCounts = new float[MaxLevel + 1];
			bulletCounts[0] = 0;
			bulletCounts[1] = 1;
			bulletCounts[2] = 2;
			bulletCounts[3] = 3;

			CoolTimes[0] = 30*Def.FPS;
			CoolTimes[1] = 30*Def.FPS;
			CoolTimes[2] = 30*Def.FPS;
			CoolTimes[3] = 30*Def.FPS;

			MaxRanges = new float[MaxLevel+1];
			MaxRanges[0] = 0*Def.MapTipLengthOnField;
			MaxRanges[1] = 5*Def.MapTipLengthOnField;
			MaxRanges[2] = 5*Def.MapTipLengthOnField;
			MaxRanges[3] = 5*Def.MapTipLengthOnField;

			Description = "指定地点に向けて、近くのスキマから弾が射出される。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃力40、最大"+bulletCounts[1] + "発",
				"攻撃力40、最大"+bulletCounts[2] + "発",
				"攻撃力40、最大"+bulletCounts[3] + "発",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(IUnit selfUnit, Vector2D targetPosition, int level)
		{
			var spell = selfUnit.GetSpellWithName(this.Name);

			if (spell == null)
			{
				yield return false;
				yield break;
			}

			if(!spell.CanExec){
				yield return false;
				yield break;
			}

			if (!selfUnit.UserValues.ContainsKey(TXKey))
			{
				selfUnit.UserValues.Add(TXKey, new List<Tuple<IBullet,IModelEffect, IEffect>>());
			}
			List<Tuple<IBullet, IModelEffect,IEffect>> bullets = selfUnit.UserValues[TXKey] as List<Tuple<IBullet, IModelEffect,IEffect>>;

			var world = selfUnit.Faction.World;
			var faction = selfUnit.Faction;

			// 近辺のスキマすべて取得
			var buildings = world.BuildingManager
				.GetBuildings(_ => _.Faction == faction && _.Parameter.GetType() == typeof(SukimaBuilding) && ReachPosition(_, targetPosition, reachRadius[level]))
					.OrderBy(_ => (_.Pos.X - targetPosition.X) * (_.Pos.X - targetPosition.X) + (_.Pos.Y - targetPosition.Y) * (_.Pos.Y - targetPosition.Y));

			var pos = world.GetGroundAtPos(targetPosition);
			selfUnit.GetModifiedBulletParameter(typeof(HaisenBullet).FullName).MaxRange = 1.5f * reachRadius[level];

			// 弾を生成
			int count = 0;
			foreach (var b in buildings)
			{
				var bullet = selfUnit.CreateBullet(typeof(HaisenBullet).FullName, pos, b.Pos);
				var modelEffect = selfUnit.Faction.World.EffectManager.CreateModelEffect(typeof(Effect.Blue.YukariTXModelEffect).FullName, bullet.Position);

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Train1).FullName);

				var angle = (float)Math.Atan2(pos.X - b.Pos.X, pos.Y - b.Pos.Y) / 3.1415f * 180f;
				modelEffect.SetRotation(new Vector3D(0, 0, angle));

				var effect = faction.World.EffectManager.CreateEffect(typeof(YukariTXEffect).FullName,bullet.Position + new Vector3D(0,0,16));
				effect.SetRotation(new Vector3D(0, 0, angle));
				effect.SetScale(new Vector3D(15, 15, 15));

				bullets.Add(new Tuple<IBullet, IModelEffect,IEffect>(bullet, modelEffect, effect));

				if (++count >= bulletCounts[level])
				{
					break;
				}
			}

			if (count > 0)
			{
				spell.ResetCoolTime();
			}

			// 削除されたときのイベント

			Action<IUnit> vanished = null;
			vanished = (u) =>
			{
				u.Vanished -= vanished;

				foreach (var bullet in bullets)
				{
					bullet.Item1.Vanish();
					bullet.Item2.Stop();
					bullet.Item3.StopRoot();
				}
			};

			selfUnit.Vanished += vanished;

			yield return true;
		}

		public override void OnUpdate(IUnit holder)
		{
			var selfUnit = holder;

			if (!selfUnit.UserValues.ContainsKey(TXKey))
			{
				selfUnit.UserValues.Add(TXKey, new List<Tuple<IBullet, IModelEffect, IEffect>>());
			}
			List<Tuple<IBullet, IModelEffect, IEffect>> bullets = selfUnit.UserValues[TXKey] as List<Tuple<IBullet, IModelEffect, IEffect>>;

			// 位置更新
			bool removed = false;
			foreach (var bullet in bullets)
			{
				if (bullet.Item1.Exists)
				{
					bullet.Item2.SetPosition(bullet.Item1.Position);
					bullet.Item3.SetPosition(bullet.Item1.Position + new Vector3D(0, 0, 16));

					var direction = bullet.Item1.Vector;

					var angle = (float)Math.Atan2(direction.X, direction.Y) / 3.1415f * 180f;
					bullet.Item2.SetRotation(new Vector3D(0, 0, angle));
					bullet.Item3.SetRotation(new Vector3D(0, 0, angle));
				}
				else
				{
					bullet.Item2.Stop();
					bullet.Item3.StopRoot();
					removed = true;
				}
			}

			if (removed)
			{
				bullets.RemoveAll(_ => !_.Item1.Exists);
			}
		}

		static bool ReachTarget (IBuilding building, ITargetObject target, float radius)
		{
			var distance2 = (building.Pos.X - target.Pos.X)*(building.Pos.X - target.Pos.X) + (building.Pos.Y - target.Pos.Y)*(building.Pos.Y - target.Pos.Y);
			return distance2 < radius*radius;
		}
		static bool ReachPosition (IBuilding building, Vector2D targetPos, float radius)
		{
			var distance2 = (building.Pos.X - targetPos.X)*(building.Pos.X - targetPos.X) + (building.Pos.Y - targetPos.Y)*(building.Pos.Y - targetPos.Y);
			return distance2 < radius*radius;
		}
	}
}


namespace LotusCraft.Bullet.Blue
{
	public class HaisenBullet : thrts.BulletParameter
	{
		public HaisenBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;

			HitType = BulletHitType.Pierce;

			this.Attack = 40;
            Speed = 30.0f;
			Gravity = 0f;

			Radius = 30;

			MaxRange = 20 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.5f * 30);
			AttackRigor = (int)(0.5f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor = 0;
			
			OnHit = OnHitEffect;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			Unit.Utls.ShowHitEffect(target, Unit.Utls.HitEffectType.Bomb);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
		}
	}
}
