﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect.Blue;

namespace LotusCraft.Spell.Blue
{
	class KyakkanKekkai : thrts.SpellParameter<thrts.IUnit>
	{
		KibakujinAbility[] spellAbilities = new []
		{
			new KibakujinAbility(0),
			new KibakujinAbility(1),
			new KibakujinAbility(2),
			new KibakujinAbility(3),
		};
		const int spellDuration = 15*Def.FPS;
		public KyakkanKekkai()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			CoolTimes = new int[4];
			CoolTimes[0] = 30*Def.FPS;
			CoolTimes[1] = 30*Def.FPS;
			CoolTimes[2] = 30*Def.FPS;
			CoolTimes[3] = 30*Def.FPS;
			Name = "結界「生と死の境界」 ";
			Description = "敵ユニットの移動速度と攻撃力を下げる結界を設置する。";
			string basePath = "Blue/Spell/KyakkanKekkai/";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"移動速度-" + (Math.Round(100 - spellAbilities[1].SpeedRate*100)).ToString() + "%, 攻撃力-" + (Math.Round(100 - spellAbilities[1].AtkRate*100)).ToString() + "%",
				"移動速度-" + (Math.Round(100 - spellAbilities[2].SpeedRate*100)).ToString() + "%, 攻撃力-" + (Math.Round(100 - spellAbilities[2].AtkRate*100)).ToString() + "%",
				"移動速度-" + (Math.Round(100 - spellAbilities[3].SpeedRate*100)).ToString() + "%, 攻撃力-" + (Math.Round(100 - spellAbilities[3].AtkRate*100)).ToString() + "%",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;

			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (IUnit selfUnit, int level)
		{
			if (null == selfUnit) {
				yield return false;
				yield break;
			}
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			// reset cooltime
			spell.ResetCoolTime();

			float affectRadius = 0;
			switch (level)
			{
				case 1:
					affectRadius = 3;
					break;
				case 2:
					affectRadius = 4;
					break;
				case 3:
					affectRadius = 5;
					break;
				default:
					affectRadius = 0;
					break;
			}
			/// get call Effect;
			var offset = new Vector3D(0,0,2);
			IEffect effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(YukariSeitoshinoAreaEffect).FullName, selfUnit.Pos + offset);
			effect.SetScale(new Vector3D(
				affectRadius * Def.MapTipLengthOnField / 5.0f,
				affectRadius * Def.MapTipLengthOnField / 5.0f, 
				affectRadius * Def.MapTipLengthOnField / 5.0f));

			effect.SetRotation(new Vector3D(0,0,45));
			/// add spell continuous ability
			var targetPosition = new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y);
			var ability = new Ability.SpellContinuousAbility((unit)=>Spelling(unit,targetPosition,level),effect,offset,spellDuration,this.Name+"AreaEffect",15);
			if(ability != null)
				ability.FollowEffect = false;

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);


			bool flag = selfUnit.AbilityManager.AppendAbility(ability);
			if(!flag && null != effect)
				effect.Stop();

			yield return true;
		}

		public bool Spelling(thrts.IUnit selfUnit,Vector2D targetPosition, int level)
		{
			float affectRadius = 0;
			switch (level)
			{
				case 1:
					affectRadius = 3;
					break;
				case 2:
					affectRadius = 4;
					break;
				case 3:
					affectRadius = 5;
					break;
				default:
					affectRadius = 0;
					break;
			}

			/// 地点(x,y)を中心にユニットを選択
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var targets = world.GetObjectFromCircle(new Circle2D(affectRadius*Def.MapTipLengthOnField, targetPosition)).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>!world.IsSameTeam(selfFaction,_.Faction)))
			{
				if(!u.AbilityManager.ContainsAbility(spellAbilities[level].Name))
				{
					IEffect effect = null;
					if(u != selfUnit)
					{
						/// エフェクトを描画
						var faction = u.Faction;
						Vector3D offset = new Vector3D(2.0f,2.0f,1.0f);
						effect = world.EffectManager.CreateEffect(typeof(Effect.White.KibakujinEffect).FullName, u.Pos + offset);
						effect.SetScale(new Vector3D(12,12,12));
					}
					if(u.AbilityManager.AppendAbility(new KibakujinAbility(level,effect)))
					{
						;
					}
					else
					{
						effect.Stop();
						effect = null;
					}
				}
			}

			return true;
		}
	}
}


namespace LotusCraft.Ability.Blue
{
	class KibakujinAbility : thrts.AbilityTemplate
	{
		public float SpeedRate{ get; private set; }
		public float AtkRate{ get; private set; }
		IEffect effect;
		public KibakujinAbility (int level,IEffect effect=null)
		{
			this.Name = "結界「生と死の境界」  lv"+level.ToString();
			this.DurationTime = 4*30;
			this.effect = effect;
			
			switch (level)
			{
			case 1:
				SpeedRate = 0.6f;
				AtkRate = 0.9f;
				break;
			case 2:
				SpeedRate = 0.55f;
				AtkRate = 0.8f;
				break;
			case 3:
				SpeedRate = 0.5f;
				AtkRate = 0.7f;
				break;
			default:
				SpeedRate = 1.0f;
				AtkRate = 1.0f;
				break;
			}
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				foreach ( var b in unit.ModifiedBulletParameters)
				{
					b.Attack = (int)(AtkRate*b.Attack);
				}
				///移動速度をspeedRate倍
				unit.Speed *= SpeedRate;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			return base.OnVanish (unit);
		}
	}
	
}
