﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Unit.Green;
using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Spell.Green
{
	class Autumn:thrts.SpellParameter<thrts.IUnit>
	{
		float[] changeRatio;
		public Autumn()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "秋符「オータムスカイ」";
			string basePath = "Green/Spell/Autumn/";

			changeRatio = new float[MaxLevel + 1];
			changeRatio[0] = 0;
			changeRatio[1] = 0.10f;
			changeRatio[2] = 0.15f;
			changeRatio[3] = 0.20f;

			Description = "ユニット生産に必要な石の一部が酒になる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"必要な石の" + changeRatio[1].ToString("###.%") + "が酒になる",
				"必要な石の" + changeRatio[2].ToString("###.%") + "が酒になる",
				"必要な石の" + changeRatio[3].ToString("###.%") + "が酒になる",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,60,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,60,(unit) => AutoCastAction(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,60,(unit) => AutoCastAction(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,60,(unit) => AutoCastAction(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool AutoCastAction (thrts.IUnit unit, int level)
		{
			if (unit.AbilityManager.ContainsAbility (AutumnAbility.AbilityName)) {
				var ability = unit.AbilityManager.GetAbilityWithName (AutumnAbility.AbilityName) as AutumnAbility;
				if(null == ability || ability.ChangeRatio != changeRatio[level])
					unit.AbilityManager.RemoveAbility(AutumnAbility.AbilityName);
			}

			if (!unit.AbilityManager.ContainsAbility (AutumnAbility.AbilityName)) {
				unit.AbilityManager.AppendAbility (new AutumnAbility (unit.Faction, changeRatio [level]));
			}
			return true;
		}
	}

}

namespace LotusCraft.Ability.Green
{
	class AutumnAbility : thrts.AbilityTemplate
	{
		public static string AbilityName = "秋符「オータムスカイ」";
		public float ChangeRatio { get; private set; }
		Dictionary<UnitParameter,int[]> origCosts;
		public AutumnAbility (IFaction faction,float changeRatio)
		{
			this.Name = AbilityName;
			this.DurationTime = 1 * Def.FPS;
			this.ChangeRatio = changeRatio;

			if (null != faction && null != faction) {
				origCosts = new Dictionary<UnitParameter, int[]>();
				UnitParameter unitParam;
				foreach(var name in faction.Parameter.Units)
				{
					unitParam= faction.GetUnitParameter(name);
					if(unitParam.IsUnique || unitParam.Costs[1] == 0)
						continue;

					origCosts.Add(unitParam,unitParam.Costs.ToArray());
					var mig = (int)(unitParam.Costs[1] * ChangeRatio);
					unitParam.Costs[0] += mig;
					unitParam.Costs[1] -= mig;
				}
			}
		}
		public override bool OnVanish (IUnit unit)
		{
			foreach (var up in origCosts) {
				up.Key.Costs[0] = up.Value[0];
				up.Key.Costs[1] = up.Value[1];
			}
			return base.OnVanish (unit);
		}
		public override void DecDurationTime ()
		{
			// non-dec => inf. ability
			return ;
		}
	}
}