﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Green
{
	class Boesendorfer : thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadiuses;

		float[] durationTime;

		const int interval = 10;

		IEffect effect = null;

		public Boesendorfer()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "鍵霊「ベーゼンドルファー神奏」";
			string basePath = "Green/Spell/Boesendorfer/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30*30;
			CoolTimes[2] = 25*30;
			CoolTimes[3] = 20*30;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 6;
			affectRadiuses[2] = 7;
			affectRadiuses[3] = 8;

			durationTime = new float[MaxLevel + 1];
			durationTime[0] = 0;
			durationTime[1] = 4;
			durationTime[2] = 8;
			durationTime[3] = 12;
			
			
			Description = "範囲内に居る敵ユニットのスキル・スペルを全て停止させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				durationTime[1] + "秒間、範囲" + affectRadiuses[1] ,
				durationTime[2] + "秒間、範囲" + affectRadiuses[2] ,
				durationTime[3] + "秒間、範囲" + affectRadiuses[3] ,
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();
			
			float affectRadius = affectRadiuses[level];
			
			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(ShinsouAreaEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 8.0f*(affectRadius);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);
			
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),areaEffect,offset,(int)(durationTime[level]*Def.FPS),this.Name + "areaEffect",interval)))
				areaEffect.Stop();
			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;
			
			float affectRadius = affectRadiuses[level];
			
			var selfFaction = selfUnit.Faction;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>().Where (_ => _ != null);
			foreach (var u in targets) {
				if ( null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction,selfFaction) ) {
					
					if(u.AbilityManager.ContainsAbility(BoesendorferAbility.AbilityName))
					{
						var ability = u.AbilityManager.GetAbilityWithName(BoesendorferAbility.AbilityName);
						
						
						if(null != ability)
						{
							ability.ExtendLifetime(interval,Def.FPS/2);
						}
					}
					else
					{
						/// エフェクトを描画
						Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
						effect = selfFaction.World.EffectManager.CreateEffect(typeof(ShinsouEffect).FullName, u.Pos + offset);
						effect.SetScale(new Vector3D(8, 8, 8));

						selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);

						var ability = new BoesendorferAbility(Def.FPS/2, effect);
						if(!u.AbilityManager.AppendAbility(ability))
						{
							if (effect != null)
							{
								effect.Stop();
								effect = null;
							}
						}
					}
				}
			}
			
			return true;
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class BoesendorferAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		public const string AbilityName = "鍵霊「ベーゼンドルファー神奏」";
		public BoesendorferAbility (int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				// スペルの完全停止を設定
				unit.IsForbiddenSpelling = true;
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			unit.IsForbiddenSpelling = false;
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}

#endif