﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Unit.Green;
using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Spell.Green
{
	class Fazioli:thrts.SpellParameter<thrts.IUnit>
	{
		public static string SpellName = "冥鍵「ファツィオーリ冥奏」";
		const float affectRadius = 7;
		float[] decBonusRatio;
		public Fazioli()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = SpellName;
			string basePath = "Green/Spell/Fazioli/";

			IsPassive= true;

			decBonusRatio = new float[MaxLevel + 1];
			decBonusRatio[0] = 0;
			decBonusRatio[1] = 0.25f;
			decBonusRatio[2] = 0.50f;
			decBonusRatio[3] = 1.00f;


			Description = "周囲に居るヒロイン以外のユニットが持つ属性ボーナスを減少させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"属性ボーナス-" + decBonusRatio[1].ToString("###.%"),
				"属性ボーナス-" + decBonusRatio[2].ToString("###.%"),
				"属性ボーナス-" + decBonusRatio[3].ToString("###.%"),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			var offset = new Vector3D(2,2,1);
			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(MeisouAreaEffect).FullName, selfUnit.Pos, selfUnit.Faction);
				var scale = affectRadius * 8;
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);
				if (! selfUnit.AbilityManager.AppendAbility(new FollowEffectAbility(effect,offset,1*Def.FPS,this.Name+"areaEffect")))
					effect.Stop();
			}
			else
			{
				selfUnit.AbilityManager.ExtendLifetimeWithName (15,this.Name+"areaEffect",1*Def.FPS);
			}
			
			var selfFaction = selfUnit.Faction;
			
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * 32, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).Where (_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>!_.Parameter.IsUnique && _.UniqueID != selfUnit.UniqueID)) {
				if(u.AbilityManager.ContainsAbility(FazioliAbility.AbilityName))
				{
					var ability = u.AbilityManager.GetAbilityWithName(FazioliAbility.AbilityName) as FazioliAbility;
					if(null != ability)
					{
						ability.ExtendLifetime(15,Def.FPS);
						ability.DecBonusRatio = decBonusRatio[level];
					}

				}
				else
				{
					var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(MeisouEffect).FullName, selfUnit.Pos, selfUnit.Faction);
					var scale = 16;
					effect.SetScale (new Vector3D (scale, scale, scale));
					effect.SetPosition (selfUnit.Pos + offset);

					if(!u.AbilityManager.AppendAbility(new FazioliAbility(decBonusRatio[level],effect,offset,Def.FPS)))
					{
						;
					}
				}
				
			}
			
			return true;
		}
	}
}
namespace LotusCraft.Ability.Green
{
	class FazioliAbility : FollowEffectAbility
	{
		public float DecBonusRatio = 0;
		public const string AbilityName = "冥鍵「ファツィオーリ冥奏」";
		public FazioliAbility(float decBonusRatio,IEffect effect,Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
			this.DecBonusRatio = decBonusRatio;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case AbilityOrder.RateIncParam:
					foreach(var b in unit.ModifiedBulletParameters)
					{
						for(int i=0;i<b.DamageBonus.Length;++i)
						{
							if(b.DamageBonus[i] > 1)
								b.DamageBonus[i] -= (b.DamageBonus[i] - 1)*DecBonusRatio;
						}
					}
					break;
				default:
					break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
}

#endif