﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Green
{
	class Genso : thrts.SpellParameter<thrts.IUnit>
	{
		float affectRadius = 10;
		const int duration = 16*Def.FPS;
		const int interval = 10;

		const int chargeTime = Def.FPS;

		public Genso()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "幻想「花鳥風月、嘯風弄月」";
			string basePath = "Green/Spell/Genso/";
			
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 180*30;
			
			Description = "花畑を踏んでいる周囲の敵ユニットは移動が制限される。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲" + affectRadius+"、"+duration/Def.FPS+"秒間。"+"ヒロインは移動速度-50%、それ以外は移動不可。",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();
			// charge
			
			for(int i=0;i<chargeTime;++i)
				yield return true;
			
			selfUnit.ShowCutin();
			
			// 効果範囲は視界
			affectRadius = selfUnit.ViewRange/Def.MapTipLengthOnField;
			
			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(GensoEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 3.0f*(affectRadius);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));
			
			
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),areaEffect,offset,duration,this.Name + "areaEffect",interval)))
				areaEffect.Stop();
			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;
			
			// 効果範囲は視界
			affectRadius = selfUnit.ViewRange/Def.MapTipLengthOnField;
			
			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			var selfFaction = selfUnit.Faction;
			// damage 
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>().Where (_ => _ != null);
			foreach (var u in targets) {
				if ( null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction,selfFaction)) {
					
					var ability = u.AbilityManager.GetAbilityWithName(GensoAbility.AbilityName);
					if(null == ability)
					{
						IEffectCommon effect = null;
						offset = new Vector3D(2.0f, 2.0f, 4.0f);
						
						// エフェクトを描画
						{
							var e = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(GensoEffect).FullName, u.Pos + offset);
							e.SetScale(new Vector3D(3,3,3));
							effect = e;
						}
						
						if(u.AbilityManager.AppendAbility(new GensoAbility(interval*2,effect)))
						{
						}
					}
					else
					{
						ability.ExtendLifetime(interval,interval*2);
					}
				}
			}
			
			return true;		}
	}
}

namespace LotusCraft.Ability.Green
{
	class GensoAbility : thrts.AbilityTemplate
	{
		IEffectCommon effect;
		const int affectRadius = 2;
		public const string AbilityName = "幻想「花鳥風月、嘯風弄月」";
		public GensoAbility (int duration, IEffectCommon effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime =  duration;
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				if(unit.Parameter.IsUnique)
					unit.Speed /= 2.0f;
				else
					unit.Speed = 0;

				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(0,0,4));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}
