using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Effect;
using LotusCraft.Effect.Green;
using LotusCraft.Bullet.Green;

namespace LotusCraft.Spell.Green
{

	class KomachiSkill:thrts.SpellParameter<thrts.IUnit>
	{
		float affectRadius = 5;
		const int interval = 5;
		 public KomachiSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "舟符「河の流れのように」";
			string basePath = "Green/Spell/KomachiSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = 15*30;
			CoolTimes[1] = 15*30;

			Description = "周囲の自軍ユニットの射程が上昇し、\n地上と空中の両方を攻撃できるようになる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"射程+1、対地・対空付与",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling (thrts.IUnit selfUnit, int level)
		{

			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				//var offset = new Vector3D(2,2,1);
				//var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(KomachiSkillEffect).FullName, selfUnit.Faction, selfUnit.Pos);
				//var scale = affectRadius;
				//effect.SetScale (new Vector3D (scale, scale, scale));
				//effect.SetPosition (selfUnit.Pos + offset);
				//if (! selfUnit.AbilityManager.AppendAbility(new FollowEffectAbility(effect,offset,1*Def.FPS,this.Name+"areaEffect")))
				//	effect.Stop();
			}
			else
			{
				selfUnit.AbilityManager.ExtendLifetimeWithName (interval,this.Name+"areaEffect",interval*2);
			}

			var selfFaction = selfUnit.Faction;

			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * 32, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).Where (_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>selfFaction==_.Faction)) {
				if(u.AbilityManager.ContainsAbility(KomachiSkillAbility.AbilityName))
				{
					u.AbilityManager.ExtendLifetimeWithName(interval, KomachiSkillAbility.AbilityName,interval*2);
				}
				else
				{
                    var offset = new Vector3D(2, 2, 1);
                    var effect = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(KomachiTextureEffect).FullName, u.Pos);
                    //var scale = 10;
                    //effect.SetScale(new Vector3D(scale, scale, scale));
					if(!u.AbilityManager.AppendAbility(new KomachiSkillAbility(interval*2,effect)))
					{
                        if (effect != null)
                        {
                            effect.Stop();
                            effect = null;
                        }
					}
				}

			}

			return true;
		}

	}
}

namespace LotusCraft.Ability.Green
{
	class KomachiSkillAbility : thrts.AbilityTemplate
	{
		public static float  incAtkRange = 1;
		public static string AbilityName = "死歌「八重霧の渡し」";
        IEffectCommon effect;
        public KomachiSkillAbility(int duration,IEffectCommon effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
            this.effect = effect;
		}

		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				foreach(var b in unit.ModifiedBulletParameters)
				{
					if(b.MinRange > Def.MapTipLengthOnField)
						b.MinRange += incAtkRange*Def.MapTipLengthOnField;
					if(b.MaxRange > 2*Def.MapTipLengthOnField)
						b.MaxRange += incAtkRange*Def.MapTipLengthOnField;
				}
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				foreach(var b in unit.ModifiedBulletParameters)
				{
					b.SpaceType = BulletSpaceType.Both;
				}
                if (effect != null)
                    effect.SetPosition(unit.Pos + new Vector3D(2, 2, 1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool ModifyBullet(thrts.IBullet bullet)
		{
			
			return base.ModifyBullet(bullet);
		}

        public override bool OnVanish(IUnit unit)
        {
            if (effect != null)
            {
                effect.Stop();
                effect = null;
            }
            return base.OnVanish(unit);
        }
	}
	
}