﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Effect.Green;
using LotusCraft.Bullet.Green;

namespace LotusCraft.Spell.Green
{
	class MedicineSkill:thrts.SpellParameter<thrts.IUnit>
	{
		public MedicineSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "毒符「神経の毒」";
			string basePath = "Green/Spell/MedicineSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = 15*30;
			CoolTimes[1] = 15*30;

			Description = "弾が命中した時、周囲の敵ユニットの移動速度を低下させ\n継続ダメージを与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				MedicineSkillAbility.DurationSec + "秒持続、移動速度-" + (int)(MedicineSkillAbility.DecSpeedRate*100) +"%、合計" +MedicineSkillAbility.Damages + "ダメージ",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			var selfFaction = selfUnit.Faction;
			var world = selfUnit.Faction.World;
			if (!selfUnit.AbilityManager.ContainsAbility (MedicineSkillSelfAbility.AbilityName)) {
                IEffect effect = null;
				if (! selfUnit.AbilityManager.AppendAbility (new MedicineSkillSelfAbility(selfUnit, 1 * Def.FPS)))
					effect.Stop ();
			} else {
				selfUnit.AbilityManager.ExtendLifetimeWithName (1, MedicineSkillSelfAbility.AbilityName,Def.FPS);
			}

			return true;
		}

	}
}

namespace LotusCraft.Ability.Green
{
	class MedicineSkillAbility : FollowEffectAbility
	{
		public static int Damages = 60;
		public static float DecSpeedRate = 0.2f;
		public static float DurationSec = 4;
		public static string AbilityName = "毒符「神経の毒」";
		public IUnit speller;
		int counter = 0;
		int interval = Def.FPS/2;
		int damage = 0;
		public MedicineSkillAbility(IUnit speller, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = (int)Math.Ceiling(DurationSec*Def.FPS);
			this.Effect = effect;
			damage = Damages*interval/DurationTime;
			counter = 0;
		}
		
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{

			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				if(++counter == interval)
				{
					counter = 0;
					unit.BeDamaged(speller,damage,AttackBonusSet.None,true);
				}
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Speed -= unit.Speed*DecSpeedRate;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility (unit,order);
		}
	}
	class MedicineSkillSelfAbility : FollowEffectAbility
	{
		public static string AbilityName = "毒符「神経の毒」";
		public IUnit speller;
		float affectRadius = 1;

		public MedicineSkillSelfAbility(IUnit speller,int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.Effect = effect;
			this.speller = speller;
		}

		public override bool ModifyBullet (IBullet bullet)
		{
			var selfUnit = speller;
			var selfFaction = speller.Faction;
			var world = selfFaction.World;
			Action<IBullet,ITargetObject> onHit =  (b,t) => 
			{
				//var acidEffect = world.EffectManager.CreateEffect(typeof(MedicineAcidCloudEffect).FullName, b.Position);
				//acidEffect.SetScale(new Vector3D(affectRadius * 8, affectRadius * 8, affectRadius * 8));
				IEffectCommon acidEffect = null;
				acidEffect = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(MediTextureEffect).FullName, b.Position);
				acidEffect.SetPosition(b.Position);

				var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (b.Position.X,b.Position.Y))).Where (_ => _ != null);
				foreach (var u in targets.OfType<IUnit>().Where(_=>!selfFaction.World.IsSameTeam(selfFaction,_.Faction))) 
				{
					if(!u.AbilityManager.ContainsAbility(MedicineSkillAbility.AbilityName))
					{
						Vector3D offset = new Vector3D(2.0f,2.0f,1.0f);
						var effect = world.EffectManager.CreateEffect(typeof(MedicineAcidEffect).FullName, u.Pos + offset);
						effect.SetScale(new Vector3D(12,12,12));
						if(!u.AbilityManager.AppendAbility (new MedicineSkillAbility(selfUnit,effect)))
							effect.Stop();
					}
				}
			};
			bullet.AddOnHitEvent(onHit);
			return base.ModifyBullet (bullet);
		}
	}
}
