﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Green
{
	class Migite : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		float[] incDamages;
		 public Migite()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "魔を掴む右手";
			string basePath = "Green/Spell/Migite/";

			incDamages = new float[MaxLevel + 1];
			incDamages[0] = 0;
			incDamages[1] = 0.5f;
			incDamages[2] = 1.0f;
			incDamages[3] = 1.5f;
			Description = "中立・召喚ユニットと下級ユニットへのダメージが増加する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"中立・召喚に+"+incDamages[1].ToString("###.%"),
				"中立・召喚に+"+incDamages[2].ToString("###.%") +"、下級に"+incDamages[1].ToString("###.%"),
				"中立・召喚に+"+incDamages[3].ToString("###.%") +"、下級に"+incDamages[2].ToString("###.%"),
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				var offset = new Vector3D (0, 0, 1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(MigiteEffect).FullName, selfUnit.Pos);
				var scale = affectRadius * 5;
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);
				if (! selfUnit.AbilityManager.AppendAbility (new FollowEffectAbility (effect, offset, 15, this.Name + "areaEffect")))
					effect.Stop ();
			} else {
				selfUnit.AbilityManager.ExtendLifetimeWithName (1, this.Name + "areaEffect",15);
			}

			foreach (var b in selfUnit.ModifiedBulletParameters) {
				switch(level)
				{
				case 1:
					b.OnHit += OnHit_lv1;
					break;
				case 2:
					b.OnHit += OnHit_lv2;
					break;
				case 3:
					b.OnHit += OnHit_lv3;
					break;
				default:
					break;
				}
			}

			return true;
		}
		
		public void OnHit_lv1 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,1);
		}
		public void OnHit_lv2 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,2);
		}
		public void OnHit_lv3 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,3);
		}
		public void OnHit (IBullet bullet, ITargetObject target, int level)
		{
			var selfUnit = bullet.Generator as IUnit;
			var u = target as IUnit;

			if (null == selfUnit || null == u || !u.Exists () || u.Parameter.IsUnique || level < 1)
				return;


			if (target.Faction.Parameter.GetType () == typeof(Faction.Neutral) || u.IsSummon) {
				bullet.Attack += (int)(bullet.Attack * incDamages [level]);
			} else if (!u.Parameter.IsUnique && !(u.Parameter is Unit.General.Special || u.Parameter is Unit.General.Fly)) {
				bullet.Attack += (int)(bullet.Attack * incDamages [level - 1]);
			}

			return;

		}
	}
}

#endif