﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Green
{
	class Noiseflow:thrts.SpellParameter<thrts.IUnit>
	{
		float[] decArmorRatio;
		float[] decMove;
		float[] affectRadiuses;
		const int duration = 8*Def.FPS;

		public Noiseflow()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "騒符「ソウルノイズフロー」";
			string basePath = "Green/Spell/Noiseflow/";
			Description += "範囲内に居る敵の移動速度と防御力を低下させる。";

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 5*Def.MapTipLengthOnField;
			MaxRanges[2] = 6*Def.MapTipLengthOnField;
			MaxRanges[3] = 7*Def.MapTipLengthOnField;

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 25*30;
			CoolTimes[2] = 25*30;
			CoolTimes[3] = 25*30;
			  
			decArmorRatio = new float[MaxLevel + 1];
			decArmorRatio[0] = 0;
			decArmorRatio[1] = 0.2f;
			decArmorRatio[2] = 0.4f;
			decArmorRatio[3] = 0.6f;

			decMove = new float[MaxLevel+1];
			decMove[0] = 0;
			decMove[1] = 0.2f;
			decMove[2] = 0.3f;
			decMove[3] = 0.4f;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 2;
			affectRadiuses[2] = 3;
			affectRadiuses[3] = 3;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"移動速度-"+decMove[1]+"、防御力-"+decArmorRatio[1].ToString("###.%"),
				"移動速度-"+decMove[2]+"、防御力-"+decArmorRatio[2].ToString("###.%"),
				"移動速度-"+decMove[3]+"、防御力-"+decArmorRatio[3].ToString("###.%"),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (IUnit selfUnit, Vector2D targetPosition, ISpell spell)
		{
			if (null == spell || !spell.CanExec || null == selfUnit || !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}

			spell.ResetCoolTime();

			var level = spell.Level;
			var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

			float affectRadius = affectRadiuses [level];

			// target pos effect
			var posEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(SoulAreaEffect).FullName, targetPos);
			var scale = affectRadius * 3;
			posEffect.SetScale (new Vector3D (scale, scale, scale));
			
			if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(posEffect, Def.FPS/2 ,this.Name + "posEffect")))
				posEffect.Stop();

			
			var selfFaction = selfUnit.Faction;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, targetPosition)).OfType<IUnit> ().Where (_ => _ != null);
			foreach (var u in targets) {
				if (null == u || !u.Exists () || u.UniqueID == selfUnit.UniqueID)
					continue;

				if (!selfFaction.World.IsSameTeam (u.Faction, selfFaction)) {
					
					var ability = selfUnit.AbilityManager.GetAbilityWithName(NoiseFlowAbility.AbilityName) as NoiseFlowAbility;
					if (null == ability) {
						/// エフェクトを描画
						Vector3D offset = new Vector3D (2.0f, 2.0f, 1.0f);
						var effect = selfFaction.World.EffectManager.CreateEffect (typeof(SoloEffect).FullName, u.Pos + offset);
						effect.SetScale (new Vector3D (8, 8, 8));
						
						//						selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);
						
						ability = new NoiseFlowAbility (decMove[level],decArmorRatio[level],effect, offset, duration);
						if (!u.AbilityManager.AppendAbility (ability)) {
							if (effect != null) {
								effect.Stop ();
								effect = null;
							}
						}
					}
					else
					{
						ability.DecMove = decMove[level];
						ability.DecArmorRatio = decArmorRatio[level];
						ability.DurationTime = duration;
					}
				}
			}
			yield return false;
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class NoiseFlowAbility : FollowEffectAbility
	{
		public const string AbilityName = "騒符「ソウルノイズフロー」";
		public float DecMove = 0;
		public float DecArmorRatio = 0;
		public NoiseFlowAbility (float decMove,float decArmorRatio,IEffect effect, Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
			DecMove = decMove;
			DecArmorRatio = decArmorRatio;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				unit.Speed -= DecMove*UnitParameter.BasisSpeed;
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Armor -= (int)Math.Ceiling(unit.Armor*DecArmorRatio);
				if(unit.Armor < 0)
					unit.Armor = 0;

				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
}

#endif