﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Unit.Green;
using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Spell.Green
{
	class Owotoshi : thrts.SpellParameter<thrts.IUnit>
	{
		public const string SpellName = "豊穣「オヲトシハーベスター」";
		int[] incAlchol;

		public Owotoshi()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = SpellName;
			string basePath = "Green/Spell/Owotoshi/";

			incAlchol = new int[MaxLevel + 1];
			incAlchol[0] = 0;
			incAlchol[1] = 20;
			incAlchol[2] = 60;
			incAlchol[3] = 120;

			Description = "一定時間毎に酒を得る。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"10秒毎に酒"+ incAlchol[1],
				"10秒毎に酒"+ incAlchol[2],
				"10秒毎に酒"+ incAlchol[3]+"、近くの酒回収施設を稼働",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,60,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,10*Def.FPS,this.AutoCastAction_lv1,true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,10*Def.FPS,this.AutoCastAction_lv2,true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,10*Def.FPS,this.AutoCastAction_lv3,true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",0),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists())
				return false;

			if (null == selfUnit.Faction)
				return false;

			// スペルエフェクト生成
			var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(OwotoshiEffect).FullName, selfUnit.Pos + new Vector3D(0.0f, 0.0f, 5.0f));
			var effectScale = 10.0f;
			effect.SetScale(new Vector3D(effectScale, effectScale, effectScale));

			// 10 * Def.FPS だと一瞬二重生成になってしまうため
			var effectAbility = new Ability.EffectFollowContinuousAbility(effect, 10 * Def.FPS - Def.FPS, this.Name + "AreaEffect", new Vector3D(0.0f, 0.0f, 5.0f));
			var flag = selfUnit.AbilityManager.AppendAbility(effectAbility);
			if (!flag)
			{
				effect.Stop();
				effect = null;
			}

			selfUnit.Faction.IncResource(0, incAlchol[level]);
			return true;
		}
		public bool AutoCastAction_lv1(thrts.IUnit unit)
		{
			return Spelling(unit, 1);
		}
		public bool AutoCastAction_lv2(thrts.IUnit unit)
		{
			return Spelling(unit, 2);
		}
		public bool AutoCastAction_lv3(thrts.IUnit unit)
		{
			return Spelling(unit, 3);
		}
	}
}