﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Unit.Green;
using LotusCraft.Effect.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Spell.Green;

namespace LotusCraft.Spell.Green
{
	class SemiAutomaton:thrts.SpellParameter<thrts.IUnit>
	{
		int[] dollsCounts;
		const int makeTime = 5;
		int makeCounter = 0;
		const string lastLvKey = "semiautomaton_lv";
		public static string SpellName = "人形「セミオートマトン」";
		public SemiAutomaton()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = SpellName;
			string basePath = "Green/Spell/SemiAutomaton/";

			dollsCounts = new int[MaxLevel+1];
			dollsCounts[0] = 2;
			dollsCounts[1] = 10;
			dollsCounts[2] = 15;
			dollsCounts[3] = 20;
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 25*Def.FPS/makeTime;
			CoolTimes[1] = 15*Def.FPS/makeTime;
			CoolTimes[2] = 10*Def.FPS/makeTime;
			CoolTimes[3] = 5*Def.FPS/makeTime;
			IsPassive= true;

			Description = "一定時間ごとに人形ユニットを1体作成する。\n基本は25秒毎に作成、最大" + dollsCounts[0]+ "体。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"人形の生産時間 -10秒、人形の最大数 +"+(dollsCounts[1] - dollsCounts[0])+"体",
				"人形の生産時間 -15秒、人形の最大数 +"+(dollsCounts[2] - dollsCounts[0])+"体",
				"人形の生産時間 -20秒、人形の最大数 +"+(dollsCounts[3] - dollsCounts[0])+"体",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,0),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			var spell = selfUnit.GetSpellWithName (this.Name);

			if (spell == null)
				return false;

			/// get farction and world
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			
			if (selfUnit.AbilityManager.ContainsAbility (ParadeAliceAbility.AbilityName)) {
				spell.BeDecCooltime = true;
			} else if ((makeCounter % (makeTime)) == 0) {
				if (selfUnit.ActualAction == UnitAction.Free)
					spell.BeDecCooltime = true;
				else if (selfUnit.ActualAction == UnitAction.Move && makeCounter % 2 == 0)
					spell.BeDecCooltime = true;
				else if (selfUnit.ActualAction == UnitAction.Attack && makeCounter % 4 == 0)
					spell.BeDecCooltime = true;
				else
					spell.BeDecCooltime = false;
			} else {
				spell.BeDecCooltime = false;
			}

			makeCounter++;

			int lastLevel = MaxLevel;
			if (selfUnit.UserValues.ContainsKey (lastLvKey)) {
				lastLevel = (int)selfUnit.UserValues [lastLvKey];
				selfUnit.UserValues [lastLvKey] = level;
			} else {
				selfUnit.UserValues.Add (lastLvKey,level);
			}

			if (lastLevel >= level && (!spell.CanExec || selfUnit.ActualAction == UnitAction.Move))
				return false;


			int maxDollsCount = dollsCounts [level];

			{
				var dollsWar = selfUnit.GetSpellWithName(DollsWar.SpellName);
				if(dollsWar != null)
					maxDollsCount += dollsWar.Level *2;
			}
			{
				var tripWire = selfUnit.GetSpellWithName(TripWire.SpellName);
				if(tripWire != null)
					maxDollsCount += tripWire.Level *1;
			}
			{
				var parade = selfUnit.GetSpellWithName(Parade.SpellName);
				if(parade != null)
					maxDollsCount += parade.Level *6;
			}

			/// get doll count
			var dolls = world.UnitManager.GetUnits (_ => _.Faction == selfFaction && _.Parameter.GetType () == typeof(Dolls));

			if (dolls.Count() < maxDollsCount) {
				// create doll
				var doll = thrts.Utl.PutUnitAroundPoint(world,selfFaction,typeof(Dolls).FullName,new thrts.Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y));
				if(doll != null)
				{
					// set summon flag
					doll.IsSummon = true;

					// add show string ability
					doll.AbilityManager.AppendAbility(new DollStringAbility(selfUnit));

					// show summon effect
                    var faction = selfUnit.Faction;
                    var effect = faction.World.EffectManager.CreateEffect(typeof(SemiAutoEffect).FullName, doll.Pos + new Vector3D(1.0f, 1.0f, 10.0f));
                    effect.SetScale(new Vector3D(5, 5, 5));

					// reset cooltime
					spell.ResetCoolTime();
				}
			}

			return true;
		}
	}
}
namespace LotusCraft.Ability.Green
{
	class DollStringAbility : thrts.AbilityTemplate
	{
		public static string AbilityName = "操りひも";
		IUnit performer;
		public Vector3D StartOffest = new Vector3D(0,0,16);
		public Vector3D EndOffest = new Vector3D(0,0,4);
		public Color StartColor = new Color(0.5f,0.5f,0.5f);
		public Color EndColor = new Color(0.5f,0.5f,0.5f);
		public Color TripColor = new Color(10.0f,0.5f,0.5f);
		public DollStringAbility(IUnit performer)
		{
			this.Name = AbilityName;
			this.DurationTime = 1*Def.FPS;
			this.performer = performer;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case AbilityOrder.OtherExecs:
				if(performer.Exists())
				{
					if(!unit.AbilityManager.ContainsAbility(TripWire.TripTimerAbilityName))
						unit.Faction.World.EffectManager.DrawLine(performer.Pos+StartOffest,StartColor,unit.Pos+EndOffest,EndColor,AlphaBlend.Add);
					else
						unit.Faction.World.EffectManager.DrawLine(performer.Pos+StartOffest,TripColor,unit.Pos+EndOffest,TripColor,AlphaBlend.Add);

				}
				else
				{
					// suicide if performer absent
					unit.HP = 0;
				}
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override void DecDurationTime ()
		{
			// non-dec => inf. ability
			return ;
		}
	}
}