﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Green
{
	class Yukaku : thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadiuses;

		float[] durationTime;

		const int interval = 10;

		public Yukaku()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "片腕有角の仙人";
			string basePath = "Green/Spell/Yukaku/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180*30;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 10;

			durationTime = new float[MaxLevel + 1];
			durationTime[0] = 0;
			durationTime[1] = 12;

			
			Description = "周囲に居る味方ユニットの防御力と移動速度を上昇させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				durationTime[1] + "秒間、防御力+"+YukakuAbility.IncArmor+"、移動速度+"+YukakuAbility.IncMove,
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();

			selfUnit.ShowCutin();
			
			float affectRadius = affectRadiuses[level];
			
			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(YukakuEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 5.0f*(affectRadius);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);
			
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),areaEffect,offset,(int)(durationTime[level]*Def.FPS),this.Name + "areaEffect",interval)))
				areaEffect.Stop();
			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;
			
			float affectRadius = affectRadiuses[level];
			
			var selfFaction = selfUnit.Faction;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>().Where (_ => _ != null);
			foreach (var u in targets) {
				if ( null != u && u.Exists() && selfFaction.World.IsSameTeam(u.Faction,selfFaction) && u.UniqueID != selfUnit.UniqueID ) {
					
					if(u.AbilityManager.ContainsAbility(YukakuAbility.AbilityName))
					{
						var ability = u.AbilityManager.GetAbilityWithName(YukakuAbility.AbilityName);
						
						
						if(null != ability)
						{
							ability.ExtendLifetime(interval,Def.FPS/2);
						}
					}
					else
					{
						/// エフェクトを描画
						Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
						var effect = selfFaction.World.EffectManager.CreateTextureEffect(typeof(YukakuTextureEffect).FullName, u.Pos + offset);

//						selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);

						var ability = new YukakuAbility(Def.FPS/2, effect);
						if(!u.AbilityManager.AppendAbility(ability))
						{
							if (effect != null)
							{
								effect.Stop();
								effect = null;
							}
						}
					}
				}
			}
			
			return true;
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class YukakuAbility : thrts.AbilityTemplate
	{
		IEffectCommon effect;
		public const string AbilityName = "片腕有角の仙人";
		public const float IncMove = 0.2f;
		public const int IncArmor = 4;
		public YukakuAbility (int duration, IEffectCommon effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.Armor += IncArmor;
				unit.Speed += IncMove*UnitParameter.BasisSpeed;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(0,0,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}

#endif