﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;

namespace LotusCraft.Spell.Red
{
	class Sarutahiko:thrts.SpellParameter<thrts.IUnit>
	{
		SarutahikoAbility[] spellAbilities = new[]
		{
			new SarutahikoAbility(0),
			new SarutahikoAbility(1),
			new SarutahikoAbility(2),
			new SarutahikoAbility(3),
		};

		int[] damages;
		float[] speeds;
		const int duration = 1*Def.FPS;

		public Sarutahiko()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "突風「猿田彦の先導」";
			Description = "使用時に指定地点まで高速移動する。\n移動経路上と周辺の敵を押しのけ、ダメージを与える。\n作業員には効果無し。";
			string basePath = "Red/Spell/Sarutahiko/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 30*Def.FPS;
			CoolTimes[2] = 30*Def.FPS;
			CoolTimes[3] = 30*Def.FPS;

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 8*Def.MapTipLengthOnField;
			MaxRanges[2] = 9*Def.MapTipLengthOnField;
			MaxRanges[3] = 10*Def.MapTipLengthOnField ;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 4;
			damages[2] = 8;
			damages[3] = 12 ;

			speeds = new float[MaxLevel + 1];
			speeds[0] = 0;
			speeds[1] = 5*UnitParameter.BasisSpeed;
			speeds[2] = 6*UnitParameter.BasisSpeed;
			speeds[3] = 7*UnitParameter.BasisSpeed;
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲3、0.2秒毎に" + damages[1] + "ダメージ",
				"範囲3、0.2秒毎に" + damages[2] + "ダメージ",
				"範囲3、0.2秒毎に" + damages[3] + "ダメージ",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = thrts.ButtonPosition._0;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}

			var spell = selfUnit.GetSpellWithName (this.Name);

			if (null == spell) {
				yield return false;
				yield break;
			}
			spell.ResetCoolTime ();

			var world = selfUnit.Faction.World;

			// charge ?

			var tick = world.GetGroundAtPos (targetPosition) - selfUnit.Pos;

			{
				var offset = new Vector3D (0, 0, 32);
				var areaEffect = world.EffectManager.CreateEffect (typeof(SarutahikoEffect).FullName, selfUnit.Pos + offset);
				areaEffect.SetScale (new Vector3D (16, 16, 16));
				areaEffect.SetRotation (new Vector3D (0, 0, (float)(Math.Atan2 (tick.X, tick.Y) * 180 / Math.PI)));

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Jet).FullName, selfUnit.Pos);

				// エフェクトを生成
				var ability = new FollowEffectAbility (areaEffect, offset, duration, this.Name + "areaEffect");
				
				if (!selfUnit.AbilityManager.AppendAbility (ability))
					areaEffect.Stop ();
			}
			// 移動速度3で飛行
			var count = (int)Math.Ceiling(tick.Length/(speeds[level]));
			if(count > duration)
				count = duration;

			tick *= 1/(float)count;
			

			for(int i=0;i<duration;++i)
			{
				if(i<=count)
					selfUnit.TryMove(tick.X,tick.Y,true);

				var radius = 4 * Def.MapTipLengthOnField;
				
				var targets = world.GetObjectFromCircle(new Circle2D(radius, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y)));

				// push unit
				foreach (var u in targets.OfType<IUnit>())
				{
					if (null == u || !u.Exists() || u.Parameter.IsFarmer || world.IsSameTeam(u.Faction, selfUnit.Faction))
						continue;

					// push unit
					var direction = u.Pos - selfUnit.Pos;
					direction.Normalize();
					u.TryMove(direction.X*UnitParameter.BasisSpeed/2,direction.Y*UnitParameter.BasisSpeed/2,true);
					
				}

				if(i%(Def.FPS/5) == 0)
				{
					var areaeffect = world.EffectManager.CreateEffect(typeof(SarutahikoAreaEffect).FullName, new Vector3D(selfUnit.Pos.X, selfUnit.Pos.Y, world.Map.GetHeightOnPoint(selfUnit.Pos.X, selfUnit.Pos.Y) + 2));
					var scale = new Vector3D(15, 15, 15);
					areaeffect.SetScale(scale);
					
					// be damage
					foreach (var t in targets)
					{
						if (null == t || !t.Exists() || world.IsSameTeam(t.Faction, selfUnit.Faction))
							continue;

						var u = t as IUnit;
						var b = t as IBuilding;

						if (null != u)
						{
							if(u.Parameter.IsFarmer)
								continue;

							// damage
							u.BeDamaged(selfUnit, damages[level], AttackBonusSet.Hero);
						}
						if (null != b)
						{
							b.BeDamaged(selfUnit, damages[level], AttackBonusSet.Hero);
						}

						if (null != u || null != b)
						{
							// show hit effect
							var offset = new Vector3D(-2, -2, 8);
							var effect = world.EffectManager.CreateEffect(typeof(KouanHitEffect).FullName, t.Pos + offset);
							var scale2 = new Vector3D(8, 8, 8);
							effect.SetScale(scale2);
							
							world.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitZangeki).FullName, t.Pos);
						}
						
					}
				}

				yield return true;
			}

			yield return false;
		}
	}

	class SarutahikoAbility : thrts.AbilityTemplate
	{
		static string AbilityName = "「猿田彦の先導」";
		public float SpeedRate { get; private set; }
		IEffect effect;
		public SarutahikoAbility(int level, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = 2 * 30;
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case thrts.AbilityOrder.ConstDecParam:
					break;
				case thrts.AbilityOrder.RateDecParam:
					///移動速度をspeedRate倍
//					unit.Speed *= SpeedRate;
					break;
				case thrts.AbilityOrder.ConstIncParam:
					break;
				case thrts.AbilityOrder.RateIncParam:
					break;
				case AbilityOrder.OtherExecs:
					if (effect != null)
						effect.SetPosition(unit.Pos + new Vector3D(1, 1, 1));
					break;
				default:
					break;
			}

			return true;
		}
		public override bool OnVanish(IUnit unit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish(unit);
		}
	}
}

#endif