﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Ability.Red;
using LotusCraft.Ability;
using LotusCraft.Unit.Red;

namespace LotusCraft.Spell.Red
{
	class FlandreSkill:thrts.SpellParameter<thrts.IUnit>
	{
		const int duration = 30*Def.FPS;
		const int imageCount = 3;
		public FlandreSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "禁忌「フォーオブアカインド」";
			string basePath = "Red/Spell/FlandreSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = int.MaxValue;
			CoolTimes[1] = int.MaxValue;

			Description = "攻撃能力の無い分身を作り出す。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃ダメージ0、被ダメージ2倍、3体",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",(unit)=>PrivilegeModeSpelling(unit,1)),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public void Spelling(thrts.IUnit selfUnit, int level)
		{
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
			{
				yield return false;
				yield break;
			}

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName(Name);
			if (null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			/// reset cooltime
			spell.ResetCoolTime();
			spell.BeDecCooltime = false;


			var affectRadius = 3;

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			/// show effect
			var offset = new Vector3D(2, 2, 2);
			var effect = world.EffectManager.CreateEffect(typeof(FlandreDeadEffect).FullName, selfUnit.Pos + offset);
			var scale = affectRadius * 4;
			effect.SetScale(new Vector3D(scale, scale, scale));

			var images = new IUnit[imageCount];

			if (!selfUnit.AbilityManager.AppendAbility(new ImageControllAbility(effect, duration, images)) && null != effect)
				effect.Stop();

			for (int i = 0; i < Def.FPS / 2; ++i)
				yield return true;

			Vector2D popPos = new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y);
			for (int i = 0; i < images.Length; ++i)
				images[i] = Utl.PutUnitAroundPoint(world, selfUnit.Parameter.Faction, typeof(FlandreImage).FullName, popPos, selfUnit.Faction);


			yield return false;
		}

		class ImageControllAbility : EffectContinuousAbility
		{
			const string AbilityName = "FlandreImageAbility";
			IUnit[] imageUnits;
			public ImageControllAbility(IEffect effect,int duration,IUnit[] imageUnits)
				:base(effect,duration,AbilityName)
			{
				this.imageUnits = imageUnits;
			}
			public override bool OnVanish (IUnit unit)
			{
				if(null != imageUnits)
				{
					for(int i=0;i<imageUnits.Length;++i)
					{
						if(null != imageUnits[i] && imageUnits[i].Exists())
							imageUnits[i].HP = 0;
					}
				}
				return base.OnVanish (unit);
			}
			public override bool OnVanish (IBuilding building)
			{
				if(null != imageUnits)
				{
					for(int i=0;i<imageUnits.Length;++i)
					{
						if(null != imageUnits[i] && imageUnits[i].Exists())
							imageUnits[i].HP = 0;
					}
				}

				return base.OnVanish (building);
			}
			public override bool OnVanish (IBullet bullet)
			{
				if(null != imageUnits)
				{
					for(int i=0;i<imageUnits.Length;++i)
					{
						if(null != imageUnits[i] && imageUnits[i].Exists())
							imageUnits[i].HP = 0;
					}
				}
				return base.OnVanish (bullet);
			}
		}
	}
}
