﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;

namespace LotusCraft.Spell.Red
{
	class NamazuSkill:thrts.SpellParameter<thrts.IUnit>
	{
		public NamazuSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "ナマズ「ほらほら世界が震えるぞ？」";
			string basePath = "Red/Spell/NamazuSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = 15*30;
			CoolTimes[1] = 15*30;

			Description = "周囲の敵にダメージを与え、さらに移動速度を低下させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"30ダメージ, 移動速度-80%",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",(unit)=>PrivilegeModeSpelling(unit,1)),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public void Spelling(thrts.IUnit selfUnit, int level)
		{
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;

			var affectRadius = 4;

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			/// reset cooltime
			spell.ResetCoolTime ();

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			/// show effect
			var offset = new Vector3D(2,2,2);
			var effect = world.EffectManager.CreateEffect(typeof(NamazuEarthquakeEffect).FullName,selfUnit.Pos+offset);
			var scale = affectRadius * 18;
			effect.SetScale(new Vector3D(scale,scale,scale));
			if(!selfUnit.AbilityManager.AppendAbility(new Ability.EffectContinuousAbility(effect,1*Def.FPS,this.Name+"Effect")))
				effect.Stop();

			world.SoundManager.PlaySoundEffect(typeof(SoundEffect.Jishin).FullName, selfUnit.Pos);

			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).Where (_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>!selfFaction.World.IsSameTeam(selfFaction,_.Faction))) {
				if(u.AbilityManager.AppendAbility(new NamazuJisinAbility(level)))
				{
					u.BeDamaged(selfUnit,30,AttackBonusSet.None);
				}
			}
			foreach (IBuilding b in targets.OfType<IBuilding>().Where(_=>!selfFaction.World.IsSameTeam(selfFaction,_.Faction))) {
				b.BeDamaged(selfUnit,30,AttackBonusSet.None);
			}
			/// 硬直1s
			for(int i=0;i<1*Def.FPS;++i)
				yield return true;


			yield return false;
		}
	}
}
