﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Ability.Red;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Red
{
	class Agni : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		const int interval = 1;
		float[] durationTime;
		public Agni()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "火符「アグニレイディアンス」";
			string basePath = "Red/Spell/Agni/";

			durationTime = new float[MaxLevel + 1];
			durationTime[0] = 0;
			durationTime[1] = 5;
			durationTime[2] = 10;
			durationTime[3] = 15;

			Description = "パチュリーの攻撃が命中した相手が継続ダメージを受けるようにする。";

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"毎秒3ダメージ、防御力無視、"+durationTime[1]+"秒間",
				"毎秒3ダメージ、防御力無視、"+durationTime[2]+"秒間",
				"毎秒3ダメージ、防御力無視、"+durationTime[3]+"秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists())
				return false;

			if (!selfUnit.AbilityManager.ContainsAbility(this.Name + "areaEffect"))
			{
				var offset = new Vector3D(0, 0, 1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(AgniEffect).FullName, selfUnit.Pos);
				var scale = affectRadius * 12;
				effect.SetScale(new Vector3D(scale, scale, scale));
				effect.SetPosition(selfUnit.Pos + offset);
				if (!selfUnit.AbilityManager.AppendAbility(new FollowEffectAbility(effect, offset, interval+2, this.Name + "areaEffect")))
					effect.Stop();
			}
			else
			{
				selfUnit.AbilityManager.ExtendLifetimeWithName(interval, this.Name + "areaEffect",interval*2);
			}

			foreach(var b in selfUnit.ModifiedBulletParameters)
			{
				switch(level)
				{
				case 1:
					b.OnHit += OnHit_lv1;
					b.OnHitSplash += OnHitSplash_lv1;
					break;
				case 2:
					b.OnHit += OnHit_lv2;
					b.OnHitSplash += OnHitSplash_lv2;
					break;
				case 3:
					b.OnHit += OnHit_lv3;
					b.OnHitSplash += OnHitSplash_lv3;
					break;
				default:
					break;
				}
			}

			return true;
		}

		void OnHit_lv1 (IBullet bullet, ITargetObject target)
		{
			AppendAgniAbility(bullet.Generator as IUnit,target as IUnit,1);
		}
		void OnHit_lv2 (IBullet bullet, ITargetObject target)
		{
			AppendAgniAbility(bullet.Generator as IUnit,target as IUnit,2);
		}
		void OnHit_lv3 (IBullet bullet, ITargetObject target)
		{
			AppendAgniAbility(bullet.Generator as IUnit,target as IUnit,3);
		}
		int OnHitSplash_lv1(IBullet bullet, ITargetObject target,int damage)
		{
			AppendAgniAbility(bullet.Generator as IUnit,target as IUnit,1);
			return damage;
		}
		int OnHitSplash_lv2(IBullet bullet, ITargetObject target,int damage)
		{
			AppendAgniAbility(bullet.Generator as IUnit,target as IUnit,2);
			return damage;
		}
		int OnHitSplash_lv3(IBullet bullet, ITargetObject target,int damage)
		{
			AppendAgniAbility(bullet.Generator as IUnit,target as IUnit,3);
			return damage;
		}

		void AppendAgniAbility (IUnit speller, IUnit u, int level)
		{
			if (null == speller || !speller.Exists ())
				return;
			if (null == u || !u.Exists ())
				return;

			var ability = u.AbilityManager.GetAbilityWithName (AgniAbility.AbilityName);
			if (null != ability) {
				ability.DurationTime = (int)(durationTime [level] * Def.FPS);
			} else {
				var effect = speller.Faction.World.EffectManager.CreateEffect(typeof(AgniEffect).FullName, u.Pos);
				var scale = (u.Radius/Def.MapTipLengthOnField) * 12;
				effect.SetScale(new Vector3D(scale, scale, scale));
				if (!u.AbilityManager.AppendAbility(new AgniAbility(speller,(int)(durationTime[level]*Def.FPS),effect)))
					effect.Stop();
			}

		}
	}
}

namespace LotusCraft.Ability.Red
{
	class AgniAbility : thrts.AbilityTemplate
	{
		const int interval = 1*Def.FPS;
		const int damage = 3;
		IEffect effect;
		int counter;
		IUnit speller;
		public const string AbilityName = "火符「アグニレイディアンス」";
		public AgniAbility (IUnit speller,int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			this.speller = speller;
			counter = 0;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(++counter>interval)
				{
					unit.BeDamaged(speller,damage,AttackBonusSet.None,true);
					counter = 0;
				}
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}

#endif