﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Ability.Red;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Red
{
	class Kenja : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 7;
		int[] intervals;
		public Kenja()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "賢者の石";
			string basePath = "Red/Spell/Kenja/";

			intervals = new int[MaxLevel + 1];
			intervals[0] = 0;
			intervals[1] = 9;
			intervals[2] = 3;
			intervals[3] = 1;

			Description = "近くに居る味方ヒロインのスペル準備時間を短縮する。";

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"スペル準備時間-"+(1/(float)(intervals[1]+1)).ToString("###.%"),
				"スペル準備時間-"+(1/(float)(intervals[2]+1)).ToString("###.%"),
				"スペル準備時間-"+(1/(float)(intervals[3]+1)).ToString("###.%"),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,intervals[1],(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,intervals[2],(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,intervals[3],(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				var offset = new Vector3D(2,2,1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(KenjaAreaEffect).FullName, selfUnit.Pos, selfUnit.Faction);
				var scale = affectRadius * 1.7f;
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);
				if (! selfUnit.AbilityManager.AppendAbility(new FollowEffectAbility(effect,offset,intervals[level]*2,this.Name+"areaEffect")))
					effect.Stop();
				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin01).FullName, selfUnit.Pos);
			}
			else
			{
				selfUnit.AbilityManager.ExtendLifetimeWithName (intervals[level] ,this.Name+"areaEffect",intervals[level]*2);
			}

			// search heros
			var selfFaction = selfUnit.Faction;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y)))
				.OfType<IUnit> ();
			foreach (var u in targets) {
				if(null == u || !u.Exists() || !u.Parameter.IsUnique || !selfFaction.World.IsSameTeam(u.Faction,selfFaction))
					continue;

				u.DecSpellCoolTime(1);
			}

			return true;
		}

	}
}

#endif