﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Effect.Red;

namespace LotusCraft.Spell.Red
{
	class Pillar:thrts.SpellParameter<thrts.IUnit>
	{
		int[] damages;
		Dictionary<float,float>[] splashs;
		public Pillar()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "火金符「セントエルモピラー」";
			string basePath = "Red/Spell/Pillar/";
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 20*30;
			CoolTimes[2] = 15*30;
			CoolTimes[3] = 10*30;

			MaxRanges= new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 8*Def.MapTipLengthOnField;
			MaxRanges[2] = 12*Def.MapTipLengthOnField;
			MaxRanges[3] = 16*Def.MapTipLengthOnField;

			damages = new int[MaxLevel +1];
			damages[0] = 0;
			damages[1] = 60;
			damages[2] = 120;
			damages[3] = 180;

			splashs = new Dictionary<float, float>[]
			{
				new Dictionary<float, float>(),
				new Dictionary<float, float>(){{0.7f*Def.MapTipLengthOnField,1.0f},{1.0f*Def.MapTipLengthOnField,0.8f},{1.4f*Def.MapTipLengthOnField,0.5f},{2.0f*Def.MapTipLengthOnField,0.25f},},
				new Dictionary<float, float>(){{0.9f*Def.MapTipLengthOnField,1.0f},{1.3f*Def.MapTipLengthOnField,0.8f},{1.8f*Def.MapTipLengthOnField,0.5f},{2.5f*Def.MapTipLengthOnField,0.25f},},
				new Dictionary<float, float>(){{1.2f*Def.MapTipLengthOnField,1.0f},{1.6f*Def.MapTipLengthOnField,0.8f},{2.2f*Def.MapTipLengthOnField,0.5f},{3.0f*Def.MapTipLengthOnField,0.25f},},
			};


			Description = "指定地点を中心に範囲ダメージを与える。\n作業員へのダメージ-50%";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃力" + damages[1] + "、スプラッシュ範囲2.0" ,
				"攻撃力" + damages[2] + "、スプラッシュ範囲2.5" ,
				"攻撃力" + damages[3] + "、スプラッシュ範囲3.0" ,
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit,Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName (Name);
				if(null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}
				
				spell.ResetCoolTime ();

				var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);
				var initOffest = new Vector3D(0,0,64);


				var b = selfUnit.GetModifiedBulletParameter(typeof(PillarBullet).FullName);
				IBullet bullet = null;
				if(null != b)
				{
					switch(level)
					{
					case 1:
						bullet = selfUnit.CreateBullet(typeof(PillarLv1Bullet).FullName,targetPos,selfUnit.Pos + initOffest);
						break;
					case 2:
						bullet = selfUnit.CreateBullet(typeof(PillarLv2Bullet).FullName,targetPos,selfUnit.Pos + initOffest);
						break;
					case 3:
						bullet = selfUnit.CreateBullet(typeof(PillarLv3Bullet).FullName,targetPos,selfUnit.Pos + initOffest);
						break;
					default:
						break;
					}
				}
				if(null != bullet)
				{
					bullet.Attack = damages[level];
					var posEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(CommonBolAreaEffect).FullName, targetPos);
					var scale = bullet.Parameter.SplashDamageRates.Max(_=>_.Key)/Def.MapTipLengthOnField;
					posEffect.SetScale (new Vector3D (scale, scale, scale));
					var diff = targetPos-selfUnit.Pos;

					if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(posEffect, (int)(diff.Length/bullet.Parameter.Speed)+2*Def.FPS ,this.Name + "posEffect")))
						posEffect.Stop();
				}
			}
			yield return false;
		}
	}
}

namespace LotusCraft.Bullet.Red
{
	public class PillarLv1Bullet : PillarBullet
	{
		public PillarLv1Bullet()
		{
			HasSplash = true;
			IsSplashMissHit = true;
			SplashDamageRates = new Dictionary<float, float>(){{0.7f*Def.MapTipLengthOnField,1.0f},{1.0f*Def.MapTipLengthOnField,0.8f},{1.4f*Def.MapTipLengthOnField,0.5f},{2.0f*Def.MapTipLengthOnField,0.25f},};
		}
		
	}
	public class PillarLv2Bullet : PillarBullet
	{
		public PillarLv2Bullet()
		{
			HasSplash = true;
			IsSplashMissHit = true;
			SplashDamageRates = new Dictionary<float, float>(){{0.9f*Def.MapTipLengthOnField,1.0f},{1.3f*Def.MapTipLengthOnField,0.8f},{1.8f*Def.MapTipLengthOnField,0.5f},{2.5f*Def.MapTipLengthOnField,0.25f},};
		}
		
	}
	public class PillarLv3Bullet : PillarBullet
	{
		public PillarLv3Bullet()
		{
			HasSplash = true;
			IsSplashMissHit = true;
			SplashDamageRates = new Dictionary<float, float>(){{1.2f*Def.MapTipLengthOnField,1.0f},{1.6f*Def.MapTipLengthOnField,0.8f},{2.2f*Def.MapTipLengthOnField,0.5f},{3.0f*Def.MapTipLengthOnField,0.25f},};
		}
	}

	public class PillarBullet : General.LongRangeBullet
	{
		public PillarBullet()
		{
			Type = BulletType.LongRange;
			this.Attack = 0;
			FarmerDamageRatio = 0.5f;
            Speed = 22.0f;
//			Speed = 1;
			Gravity = 0;
			MaxRange = 18.0f * Def.MapTipLengthOnField;

			IsSplashMissHit = true;
			HasSplash = true;
			
			AttackDelay = (int)(1.0f * Def.FPS);
			AttackRigor = (int)(3.0f * Def.FPS);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor = 0;
			
			OnHit = OnHitEffect;
			OnMiss = OnMissEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/ball_yellow";
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
			TextureAnimation.Width = 32;
			TextureAnimation.Height = 32;
			TextureAnimation.Center.X = 16;
			TextureAnimation.Center.Y = 16;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 4;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 4;
		}
		
		void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			float scale = (bullet.Parameter.SplashDamageRates.Max(_ => _.Key) / Def.MapTipLengthOnField) * 5;
			CreateEffect(faction, target.Pos, scale);
		}
		
		void OnMissEffect(IBullet bullet)
		{
			var faction = bullet.Generator.Faction;
			float scale = (bullet.Parameter.SplashDamageRates.Max(_ => _.Key) / Def.MapTipLengthOnField) * 5;
			CreateEffect(faction, bullet.Position, scale);
		}
		
		public static void CreateEffect(IFaction faction, Vector3D pos, float scale)
		{
			var offset = new Vector3D(0, 0, 4);
			var effect = faction.World.EffectManager.CreateEffect(typeof(PillarAreaEffect).FullName, pos + offset);
			effect.SetScale(new Vector3D(scale,scale,scale));
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom01).FullName, pos);
		}
	}
}

#endif