#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;

namespace LotusCraft.Spell.Red
{
    class HeartBreak : thrts.SpellParameter<thrts.IUnit>
    {
        int[] chargeTime = new int[4];
        int[] bulletCounts = new int[4];
        int[] damage = new int[4];
        IBullet spellbullet = null;
		IEffect effect = null;

		const int rigor = Def.FPS/2;
        public HeartBreak()
        {
            DefaultLevel = 0;
            MaxLevel = 3;
            Name = "必殺「ハートブレイク」";
			Description = "指定地点に向けて槍を投げ、直線上の敵ユニットにダメージを与える。\n作業員へのダメージ-50%";
            string basePath = "Red/Spell/HeartBreak/";

            CoolTimes = new int[MaxLevel + 1];
            CoolTimes[0] = 0;
            CoolTimes[1] = 12 * 30;
            CoolTimes[2] = 12 * 30;
            CoolTimes[3] = 12 * 30;

            // 最長範囲の指定
            MaxRanges = new float[4];
            MaxRanges[0] = 0;
            MaxRanges[1] = 6 * thrts.Def.MapTipLengthOnField;
            MaxRanges[2] = 8 * thrts.Def.MapTipLengthOnField;
            MaxRanges[3] = 10 * thrts.Def.MapTipLengthOnField;

            chargeTime[0] = 0;
            chargeTime[1] = Def.FPS/2;
			chargeTime[2] = Def.FPS/2;
			chargeTime[3] = Def.FPS/2;

            damage[0] = 0;
            damage[1] = 50;
            damage[2] = 70;
            damage[3] = 90;

            /// レベル別の能力説明
            EffectDescriptions = new string[]
			{
				damage[1] + "ダメージ",
				damage[2] + "ダメージ",
				damage[3] + "ダメージ",
			};

            /// ボタンの位置
            ButtonPosX = thrts.ButtonPosition._0;
            ButtonPosY = thrts.ButtonPosition._2;
            LvUpButtonPosX = ButtonPosX;
            LvUpButtonPosY = thrts.ButtonPosition._1;

            /// オートキャスト
            SpellAutoCasts = null;
            /// スペルのボタンを生成
            SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
            LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
        }

        public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPos, int level)
        {
            // reset cooltime
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			spell.ResetCoolTime();

			// エフェクト消去
			if (effect != null) effect.Stop();

            var world = selfUnit.Faction.World;
            var target = world.GetGroundAtPos(targetPos);
            Vector3D offset = new Vector3D(0, 0, 0f);
            spellbullet = selfUnit.CreateBullet(typeof(HeartBullet).FullName, target, selfUnit.Pos + offset);
            Vector3D v = spellbullet.Vector;
            spellbullet.SetVector(new Vector3D(0, 0, 0f));
            spellbullet.Attack = damage[level];

			effect = world.EffectManager.CreateEffect(typeof(Effect.Red.HeartBulletEffect).FullName, spellbullet.Position + new Vector3D(0,0,16));
			if (effect != null)
			{
				effect.SetScale(new Vector3D(8, 8, 8));
				var direction = targetPos - new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y);

				var angle = (float)Math.Atan2(direction.X, direction.Y) / 3.1415f * 180f;
				effect.SetRotation(new Vector3D(0, 0, angle));
				effect.SetPosition(spellbullet.Position);
			}

			selfUnit.Vanished += OnVanishUnit;

            yield return true;

            for (int i = 0; i < chargeTime[level]; ++i)
            {
                spellbullet.SetVector(new Vector3D(0, 0, 0f));
                yield return true;
            }

            spellbullet.SetVector(v);

            for (int i = 0; i < 8; ++i) { yield return true; }

			// rigor
			for (int i = 0; i < rigor; ++i) { yield return true; }


            yield return true;
        }

        public void OnVanishUnit(thrts.IUnit selfUnit)
        {
			if (spellbullet != null && spellbullet.Vector.Length <= 5)
            {
                spellbullet.Vanish();
                spellbullet = null;
            }
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			selfUnit.Vanished -= OnVanishUnit;
		}

		public override void OnUpdate(IUnit holder)
		{
			if (spellbullet != null && !spellbullet.Exists)
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
					holder.Vanished -= OnVanishUnit;
				}
			}

			if (spellbullet != null && spellbullet.Exists && spellbullet.Vector.Length > 0)
			{
				if (effect != null)
				{ 
					var direction = spellbullet.Vector;

					var angle = (float)Math.Atan2(direction.X, direction.Y) / 3.1415f * 180f;
					effect.SetRotation(new Vector3D(0, 0, angle));
					effect.SetPosition(spellbullet.Position + new Vector3D(0, 0, 16));
				}
			}
		}
    }
}

namespace LotusCraft.Bullet.Red
{
    public class HeartBullet : thrts.BulletParameter
    {
        public HeartBullet()
        {
            Type = BulletType.LongRange;
            SpaceType = BulletSpaceType.Both;
			FarmerDamageRatio = 0.5f;
            Speed = 30f;
            Gravity = 0;
            Radius = 1.0f * 32;
            this.HitType = BulletHitType.Pierce;
            this.AttackType = BulletAttackType.Absolute;
            this.Attack = 0;

            MaxRange = 12 * 32;

            OnHit = OnHitEffect;
        }
        protected static void OnHitEffect(IBullet bullet, ITargetObject target)
        {
            var faction = bullet.Generator.Faction;
            var offset = new Vector3D(0, 0, 0);
            var effect = faction.World.EffectManager.CreateEffect(typeof(HeartHitEffect).FullName, target.Pos + offset);
            var scale = new Vector3D(16, 16, 16);
            effect.SetScale(scale);
            faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitZangeki).FullName, target.Pos);
        }
    }
}

#endif