#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;

namespace LotusCraft.Spell.Red
{
	class NightMare : thrts.SpellParameter<thrts.IUnit>
	{
		int[] damage = new int[4];
		float[] selfdamageRatio = new float[4];
		float[] affectRadius;

		const int interval = 10;
		const int autocastInterval = 30;

		const string spellAbilityName = "night_mare_spell_ability";

		const string SpellName = "魔符「全世界ナイトメア」";
		public NightMare()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = SpellName;
			Description = "トグルONの間、自分と周囲の敵ユニットのHPが減少し続ける。\nトグルをONにするのにHPを200消費する。\n作業員へのダメージ-50%";
			string basePath = "Red/Spell/Nightmare/";
			
			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 1 * 30;
			CoolTimes[2] = 1 * 30;
			CoolTimes[3] = 1 * 30;
			
			// 最長範囲の指定
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 10 * thrts.Def.MapTipLengthOnField;
			MaxRanges[2] = 10 * thrts.Def.MapTipLengthOnField;
			MaxRanges[3] = 10 * thrts.Def.MapTipLengthOnField;
			
			damage[0] = 0;
			damage[1] = 4;
			damage[2] = 6;
			damage[3] = 8;
			selfdamageRatio[0] = 0;
			selfdamageRatio[1] = 0.6f;
			selfdamageRatio[2] = 0.4f;
			selfdamageRatio[3] = 0.2f;

			affectRadius = new float[MaxLevel + 1];
			affectRadius[0] = 0;
			affectRadius[1] = 4;
			affectRadius[2] = 5;
			affectRadius[3] = 6;
			
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"1秒ごとに敵に"+damage[1]+"、自分に与えたダメージの"+(selfdamageRatio[1]*100)+"%ダメージ",
				"1秒ごとに敵に"+damage[2]+"、自分に与えたダメージの"+(selfdamageRatio[2]*100)+"%ダメージ",
				"1秒ごとに敵に"+damage[3]+"、自分に与えたダメージの"+(selfdamageRatio[3]*100)+"%ダメージ",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>AutocastAction(unit,1)),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>AutocastAction(unit,2)),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>AutocastAction(unit,3)),
			};

			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			// reset cooltime
			spell.ResetCoolTime();
			
			if (selfUnit.GetIAutoCastWithName(Name).IsEnable)
			{

				// disable autocast
				selfUnit.GetIAutoCastWithName(Name).ToggleAutoCast();
				selfUnit.AddOnVanishEvent(OnVanishUnit);
			}
			else
			{
				if(!selfUnit.AbilityManager.ContainsAbility(spellAbilityName))
				{
					if(selfUnit.HP < 200)
					{
						yield return false;
						yield break;
					}
					
					selfUnit.HP -= 200;
				}

				for (int i = 0; i < 30; ++i)
				{
					yield return true;
				}

				/// toggle autocast
				selfUnit.GetIAutoCastWithName(this.Name).ToggleAutoCast();
				selfUnit.RemoveOnVanishEvent(OnVanishUnit);
			}
		}

		public bool AutocastAction (IUnit selfUnit, int level)
		{
			var ability = selfUnit.AbilityManager.GetAbilityWithName (spellAbilityName) as SpellContinuousAbility;
			var scale = 2.7f * affectRadius[level] * 2;

			if (null == ability) {
				Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(NightmareAreaEffect).FullName, selfUnit.Pos + offset);
				areaEffect.SetScale(new Vector3D(scale,scale,scale));

				var scAbility = new SpellContinuousAbility(SpellAbility, areaEffect, offset, interval * 2, spellAbilityName, autocastInterval);
				scAbility.StopEffectRootOnVanish = true;
				if(!selfUnit.AbilityManager.AppendAbility(scAbility))
					areaEffect.Stop();
			} else {
				if(null == ability.Effect)
					ability.Effect.SetScale(new Vector3D(scale,scale,scale));
				ability.ExtendLifetime(interval,interval*2);
			}

			return true;
		}

		public bool SpellAbility(IUnit selfUnit)
		{
			var spell = selfUnit.GetSpellWithName(Name);
			if(spell == null)
				return false;

			var level = spell.Level;

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius[level] * 32, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).Where(_ => _ != null);
			int totalDamage = 0;
			foreach (IUnit u in targets.OfType<IUnit>().Where(_ => !world.IsSameTeam(selfFaction, _.Faction)))
			{
				if (!u.AbilityManager.ContainsAbility(this.Name + "HitEffect"))
				{
					/// エフェクトを描画
					var faction = u.Faction;
					Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);
					var effect = world.EffectManager.CreateEffect(typeof(NightmareEffect).FullName, u.Pos);
					var scale = new Vector3D(12, 12, 12);
					effect.SetScale(scale);
					
					var hitAbility = new Ability.EffectContinuousAbility(effect, 20, this.Name + "HitEffect");
					hitAbility.StopEffectRootOnVanish = true;

					var flag = u.AbilityManager.AppendAbility(hitAbility);
					if (!flag)
						effect.Stop();
					else
					{
						if(u.Parameter.IsFarmer)
							u.BeDamaged(selfUnit, (int)Math.Ceiling(0.5f*damage[level]), AttackBonusSet.None, true);
						else
							u.BeDamaged(selfUnit, (int)(damage[level]), AttackBonusSet.None, true);
						selfFaction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, u.Pos);
						totalDamage += damage[level];
					}
				}
			}
			
			selfUnit.BeDamaged(selfUnit, (int)(selfdamageRatio[level] * totalDamage), AttackBonusSet.None, true);
			
			return true;
		}

		static public void OnVanishUnit (IUnit unit)
		{
			if (unit.GetIAutoCastWithName(NightMare.SpellName).IsEnable)
			{
				// toggle autocast
				unit.GetIAutoCastWithName(NightMare.SpellName).ToggleAutoCast();
			}
			unit.RemoveOnVanishEvent(OnVanishUnit);
		}
	}
}

#endif