﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;

namespace LotusCraft.Spell.Red
{
	class Hakuchu:thrts.SpellParameter<thrts.IUnit>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="LotusCraft.Spell.Red.NijuKekkai"/> class.
		/// </summary>
		public Hakuchu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "奇跡「白昼の客星」";
			Description = "指定した地点の視界を一時的に得る。";
			string basePath = "Red/Spell/Hakuchu/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 60 * 30;
			CoolTimes[2] = 45 * 30;
			CoolTimes[3] = 30 * 30;

			// 最小範囲の指定
			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;

			// 最長範囲の指定
			MaxRanges = new float[4];
			MaxRanges[0] = thrts.Def.MapTipLengthOnField * 512;
			MaxRanges[1] = thrts.Def.MapTipLengthOnField * 512;
			MaxRanges[2] = thrts.Def.MapTipLengthOnField * 512;
			MaxRanges[3] = thrts.Def.MapTipLengthOnField * 512;

			Description = "指定した地点の視界を一時的に得る。";

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲4. 8秒",
				"範囲8. 8秒",
				"範囲16. 8秒",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = null;

			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			// クールダウンを再設定する
			spell.ResetCoolTime();

			// レベルごとに半径を設定
			float affectRadius = 0;
			switch (level)
			{
				case 1:
					affectRadius = 6;
					break;
				case 2:
					affectRadius = 12;
					break;
				case 3:
					affectRadius = 24;
					break;
				default:
					break;
			}

			// 高さを含めた座標を取得
			var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			// ユニットの位置にエフェクトの生成
			{
				Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(HakuchuSelfEffect).FullName, selfUnit.Pos + offset);
				var scale = 2.5f * 2;
				effect.SetScale(new Vector3D(scale, scale, scale));
				effect.SetPosition(selfUnit.Pos + offset);
				selfUnit.AbilityManager.AppendAbility(new Ability.EffectContinuousAbility(effect, 8 * Def.FPS, this.Name + "SelfEffect"));

			}

			// 視界を得る
			selfFaction.GetSite(affectRadius * Def.MapTipLengthOnField, targetPosition.X, targetPosition.Y, 8 * Def.FPS);

			// 視界を得た位置にエフェクトの生成
			{
				Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(HakuchuShotEffect).FullName, targetPos + new Vector3D(2.0f, 20.0f, 1.0f));
				var scale = 2.5f * affectRadius;
				effect.SetScale(new Vector3D(scale, scale, scale));
				selfUnit.AbilityManager.AppendAbility(new Ability.EffectContinuousAbility(effect, 8 * Def.FPS, this.Name + "AreaEffect"));

                world.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, targetPos);
            }

			world.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, targetPos);

			// 5フレーム硬直
			yield return true;
			yield return true;
			yield return true;
			yield return true;
			yield return true;

			yield return false;
		}
	}
}
