﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;

namespace LotusCraft.Spell.Red
{
	class KamiNoKaze : thrts.SpellParameter<thrts.IUnit>
	{
		KamiNoKazeAbility[] spellAbilities = new[]
		{
			new KamiNoKazeAbility(0),
			new KamiNoKazeAbility(1),
			new KamiNoKazeAbility(2),
			new KamiNoKazeAbility(3),
		};
		IEffect areaEffect = null;
		// effect time=30f -> 1/2 
		int autocastInterval = 15;
		int counter = 0;

		public KamiNoKaze()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "奇跡「神の風」";
			Description = "近くにいる味方兵ユニットの攻撃力と移動速度を上昇させる。";
			string basePath = "Red/Spell/KamiNoKaze/";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃力+"+(spellAbilities[1].IncAtkRate-1.0f).ToString("###.%")+" , "+
				"移動速度+"+(spellAbilities[1].IncSpeedRatio-1.0f).ToString("###.%")+"",
				"攻撃力+"+(spellAbilities[2].IncAtkRate-1.0f).ToString("###.%")+" , "+
				"移動速度+"+(spellAbilities[2].IncSpeedRatio-1.0f).ToString("###.%")+"",
				"攻撃力+"+(spellAbilities[3].IncAtkRate-1.0f).ToString("###.%")+" , "+
				"移動速度+"+(spellAbilities[3].IncSpeedRatio-1.0f).ToString("###.%")+"",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};

			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling(IUnit selfUnit, int level)
		{

			float affectRadius = 0;
			switch (level)
			{
				case 1:
					affectRadius = 4;
					break;
				case 2:
					affectRadius = 4;
					break;
				case 3:
					affectRadius = 4;
					break;
				default:
					break;
			}


			{
				Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
				if (areaEffect == null)
				{
					areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(KamiNoKazeAreaEffect).FullName, selfUnit.Pos + offset);
					areaEffect.SetScale(new Vector3D(2.5f * affectRadius * 2, 2.5f * affectRadius * 2, 2.5f * affectRadius * 2));
					selfUnit.Vanished += OnUnitIsVanished;
					
				}
				areaEffect.SetPosition(selfUnit.Pos + offset);
			}

			if (++counter < autocastInterval)
			{
				return true;
			}

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius * 32, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_ => world.IsSameTeam(selfFaction, _.Faction) && !_.Parameter.IsFarmer ))
			{
				if (u.AbilityManager.ContainsAbility(spellAbilities[level].Name))
				{
					/// 既に掛かっていたら時間を延長
					u.AbilityManager.ExtendLifetimeWithName(autocastInterval, spellAbilities[level].Name,spellAbilities[level].DurationTime);
				}
				else
				{
					//// 前のが残ってたら削除
					if(level > 1 && u.AbilityManager.ContainsAbility(spellAbilities[level-1].Name))
					{
						u.AbilityManager.RemoveAbility(spellAbilities[level-1].Name,true);
					}

					IEffectCommon effect = null;
					if (u != selfUnit)
					{
						/// エフェクトを描画
						var faction = u.Faction;
						//Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
						//effect = world.EffectManager.CreateEffect(typeof(KamiNoKazeEffect).FullName, selfFaction, u.Pos + offset);
						//effect.SetScale(new Vector3D(12, 12, 12));

                        {
                            effect = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(KaminokazeTextureEffect).FullName, u.Pos);
                        }
					}
					if (!u.AbilityManager.AppendAbility(new KamiNoKazeAbility(level, effect)))
					{
						effect.Stop();
					}
				}
			}

			counter = 0;

			return true;
		}

		public void OnUnitIsVanished(IUnit selfUnit)
		{
			// 範囲エフェクト停止
			if (areaEffect != null)
			{
				areaEffect.Stop();
				areaEffect = null;
			}

			// イベントを取り除く
			selfUnit.Vanished -= OnUnitIsVanished;
		}
	}
}

namespace LotusCraft.Ability.Red
{
	class KamiNoKazeAbility : thrts.AbilityTemplate
	{
		public float IncSpeedRatio;
		public float IncAtkRate;
		IEffectCommon effect;
		public KamiNoKazeAbility(int level,IEffectCommon effect = null)
		{
			this.Name = "奇跡「神の風」 lv"+level.ToString();
			this.DurationTime = 1*Def.FPS;
			this.effect = effect;
			switch (level)
			{
			case 1:
				IncSpeedRatio = 1.10f;
				IncAtkRate = 1.05f;
				break;
			case 2:
				IncSpeedRatio = 1.15f;
				IncAtkRate = 1.10f;
				break;
			case 3:
				IncSpeedRatio = 1.2f;
				IncAtkRate = 1.15f;
				break;
			default:
				IncSpeedRatio = 1.0f;
				IncAtkRate = 1.0f;
				break;
			}
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				foreach(var b in unit.ModifiedBulletParameters)
				{
					/// inc damage
					b.Attack = (int)(b.Attack*IncAtkRate);
				}
				
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				unit.Speed *= IncSpeedRatio;
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			return base.OnVanish (unit);
		}
		public override bool ModifyBullet(thrts.IBullet bullet)
		{
			return base.ModifyBullet(bullet);
		}
	}
	
}
