using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;

namespace LotusCraft.Spell.Red
{
	class KoanNoKamikaze : thrts.SpellParameter<thrts.IUnit>
	{
		KoanAbility[] spellAbilities = new[]
		{
			new KoanAbility(0),
			new KoanAbility(1),
		};
		int[] chargeTime = new int[2];
		IEffect areaEffect = null;
		IEffect selfEffect = null;
		const float dps = 70;
		const int duration = 18;
		const int interval = 1 * Def.FPS;
		int changeStateCount = 0;
		public KoanNoKamikaze()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "奇跡「弘安の神風」";
			string basePath = "Red/Spell/KoanNoKamikaze/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 120 * 30;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 10 * 32;
			MaxRanges[2] = 10 * 32;
			MaxRanges[3] = 10 * 32;

			chargeTime[0] = 0;
			chargeTime[1] = 2 * 30;

			Description = "選択した場所の周辺に継続してダメージを与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				dps.ToString("#0.#") + "ダメージ/秒, " + duration + "秒",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				null,
				null,
				null,
				null,
				new AutoCast.AutoCast<IUnit>(this.Name+"bulletFollower",1,null,false),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, OnFinishPrivilegeMode, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		const string AttackEffectName = "SanaeSelfEffect";

		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			// 使用中の場合呼ばない
			if (selfUnit.AbilityManager.ContainsAbility(AttackEffectName))
			{
				yield return false;
			}

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			// クールタイムリセット
			spell.ResetCoolTime();

			int affectRange = 0;
			switch (level)
			{
				case 1:
					affectRange = 12;
					break;
				default:
					break;
			}

			// 攻撃位置を取得
			var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

			// 自動で索敵しないように変更
			selfUnit.DoFindEnemyAutomatically = false;

			Vector3D effectOffset = new Vector3D(0.0f, 0.0f, 1.0f);

			// 呼び出し中エフェクト作成
			selfEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(KouanSelfEffect).FullName, selfUnit.Pos + effectOffset);
			float effectScale = 1.7f * 2 * 3;
			selfEffect.SetScale(new Vector3D(effectScale, effectScale, effectScale));

			// チャージエフェクト表示
			var chargeEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(KouanChargeEffect).FullName, targetPos + effectOffset);
			effectScale = affectRange;
			chargeEffect.SetScale(new Vector3D(effectScale, effectScale, effectScale));
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);

			for (int i = 0; i < chargeTime[level]; ++i)
				yield return true;

			chargeEffect.Stop();

			// カットイン表示
			selfUnit.ShowCutin();
			for (int i = 0; i < 1 * Def.FPS; ++i)
				yield return true;

			// 範囲エフェクト生成
			areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(KouanAreaEffect).FullName, targetPos + effectOffset);
			effectScale = 1.7f * affectRange;
			areaEffect.SetScale(new Vector3D(effectScale, effectScale, effectScale));

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Jet).FullName, targetPos);

			// スペル攻撃&エフェクト継続アビリティ追加
			var ability = new Ability.SpellContinuousAbility(
				(unit) => Spelling(unit, targetPos, level, affectRange),
				areaEffect,
				effectOffset,
				duration * Def.FPS,
				AttackEffectName,
				interval);

			ability.StopEffectRootOnVanish = true;
			ability.FollowEffect = false;

			bool flag = selfUnit.AbilityManager.AppendAbility(ability);

			// 自分を中心とした範囲エフェクト生成
			var areaAbility = new Ability.EffectContinuousAbility(selfEffect, duration * Def.FPS, this.Name + "AreaEffect");
			flag = selfUnit.AbilityManager.AppendAbility(areaAbility) && flag;

			// 失敗してしまった
			if (!flag)
			{
				areaEffect.Stop();
				selfEffect.Stop();
				areaEffect = null;
				selfEffect = null;
				
				selfUnit.DoFindEnemyAutomatically = true;
			}

			// 硬直
			for (int i = 0; i < duration * Def.FPS; ++i)
				yield return true;


			yield return false;
		}

		public void OnFinishPrivilegeMode(IUnit selfUnit, int level)
		{
			if (selfUnit.AbilityManager.ContainsAbility(AttackEffectName))
			{
				// 行動を起こした場合、ラストスペルを停止するように。
				selfUnit.ChangedActualAction += OnMoveUnit;
			}
		}

		public bool Spelling(IUnit selfUnit, Vector3D targetPos, int level, int affectRange = 0)
		{
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRange * Def.MapTipLengthOnField / 2, new Vector2D(targetPos.X, targetPos.Y))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_ => !selfFaction.World.IsSameTeam(selfFaction, _.Faction)))
			{
				if (!u.AbilityManager.ContainsAbility(this.Name))
				{
					// ヒットエフェクトを描画
					var faction = u.Faction;
					Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
					// 50dmg/1s => 50dps
					var hitEffect = world.EffectManager.CreateEffect(typeof(KouanHitEffect).FullName, u.Pos);
					float effectScale = 1.7f * 5;
					hitEffect.SetScale(new Vector3D(effectScale, effectScale, effectScale));
					if (!u.AbilityManager.AppendAbility(new Ability.FollowEffectAbility(hitEffect, offset, 2 * Def.FPS, this.Name)))
						hitEffect.Stop();
					else
						u.BeDamaged(selfUnit, (int)(dps * interval / Def.FPS), AttackBonusSet.Hero);
					selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitZangeki).FullName, targetPos);

				}
				else
				{
					u.AbilityManager.ExtendLifetimeWithName(interval, this.Name,interval*2*Def.FPS);
					u.BeDamaged(selfUnit, (int)(dps * interval / Def.FPS), AttackBonusSet.Hero);
				}
			}

			return true;
		}

		public void OnMoveUnit(IUnit selfUnit)
		{
			if (changeStateCount++ > 0)
			{
				selfUnit.AbilityManager.RemoveAbility(AttackEffectName);

				// stop spell if move
				selfUnit.ChangedActualAction -= OnMoveUnit;
				// don't search unit
				selfUnit.DoFindEnemyAutomatically = true;

				if (areaEffect != null)
				{
					areaEffect.Stop();
					areaEffect = null;
				}
				if (selfEffect != null)
				{
					selfEffect.Stop();
					selfEffect = null;
				}
				changeStateCount = 0;
			}
		}

		public void OnVanishUnit(IUnit selfUnit)
		{
			if (areaEffect != null)
			{
				areaEffect.Stop();
				areaEffect = null;
			}
			if (selfEffect != null)
			{
				selfEffect.Stop();
				selfEffect = null;
			}
		}
	}
}
