﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;

namespace LotusCraft.Spell.Red
{
	class Omamori : thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime = new int[4];
		int[] bulletCounts = new int[4];

		public Omamori()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "祈願「商売繁盛守り」";
			string basePath = "Red/Spell/Omamori/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 20 * 30;
			CoolTimes[2] = 20 * 30;
			CoolTimes[3] = 20 * 30;

			// 最小範囲の指定
			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;

			// 最長範囲の指定
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6 * thrts.Def.MapTipLengthOnField;
			MaxRanges[2] = 6 * thrts.Def.MapTipLengthOnField;
			MaxRanges[3] = 6 * thrts.Def.MapTipLengthOnField;

			chargeTime[0] = 0;
			chargeTime[1] = 1 * 30;
			chargeTime[2] = 1 * 30;
			chargeTime[3] = 1 * 30;
			bulletCounts[0] = 0;
			bulletCounts[1] = 10;
			bulletCounts[2] = 20;
			bulletCounts[3] = 35;

			Description = "対象方向に向かって複数の弾を発射する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"1発8ダメージ, "+bulletCounts[1]+"発",
				"1発8ダメージ, "+bulletCounts[2]+"発",
				"1発8ダメージ, "+bulletCounts[3]+"発",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPos, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			// reset cooltime
			spell.ResetCoolTime();
			/*
			// chage effect
			Vector3D offset = new Vector3D(0.0f,0.0f,1.0f);
			var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(HanjoThrowEffect).FullName, selfUnit.Faction, selfUnit.Pos + offset);
			float effectScale = 12;
			effect.SetScale (new Vector3D (effectScale, effectScale, effectScale));

			for(int i=0;i<chargeTime[level];++i)
				yield return true;

			effect.Stop ();
*/
			Spelling(selfUnit, targetPos, level);

			yield return true;
		}

		public bool Spelling(IUnit speller, Vector2D targetPos, int level)
		{
			var selfFaction = speller.Faction;
			var world = selfFaction.World;
			Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

			// 投げエフェクト生成
			{
				var throwEffect = world.EffectManager.CreateEffect(typeof(HanjoThrowEffect).FullName, speller.Pos + offset);
				float effectScale = 15;
				throwEffect.SetScale(new Vector3D(effectScale, effectScale, effectScale));
				var dx = speller.Pos.X - targetPos.X;
				var dy = speller.Pos.Y - targetPos.Y;
				float rot = (float)Math.Atan2(dx, dy);
				rot += (float)Math.PI;

				rot *= 180 / (float)Math.PI;
				throwEffect.SetRotation(new Vector3D(0, 0, rot));
			}


			world.SoundManager.PlaySoundEffect(typeof(SoundEffect.Burst).FullName, speller.Pos);

			var slugRange = 4f * Def.MapTipLengthOnField;

			// 攻撃対象を探索する
			var targets = world.GetObjectFromCircle(new Circle2D(slugRange, targetPos))
					.OfType<IUnit>()
					.Where(__ => !world.IsSameTeam(selfFaction, __.Faction))
					.OrderBy(_ => (_.Pos.X - targetPos.X) * (_.Pos.X - targetPos.X) + (_.Pos.Y - targetPos.Y) * (_.Pos.Y - targetPos.Y));

			// 攻撃対象が存在する場合
			for (int i = 0; i < Math.Min(bulletCounts[level], targets.Count()); ++i)
			{
				speller.CreateBullet(typeof(OmamoriBullet).FullName, targets.ElementAt(i), speller.Pos);
			}

			// 攻撃対象が存在しない場合
			for (int i = 0; i < Math.Abs(bulletCounts[level] - targets.Count()); ++i)
			{
				var target = world.GetGroundAtPos(targetPos);
				target.X += (float)(1.0f * slugRange * world.NextRandDouble() * (world.NextRand(2) == 0 ? -1 : 1));
				target.Y += (float)(1.0f * slugRange * world.NextRandDouble() * (world.NextRand(2) == 0 ? -1 : 1));
				//				target.Z += (float)(0.4f*slugRange*world.NextRandDouble());
				speller.CreateBullet(typeof(OmamoriBullet).FullName, target, speller.Pos);
			}

			return true;
		}
	}
}


namespace LotusCraft.Bullet.Red
{
	public class OmamoriBullet : General.LongRangeBullet
	{
		public OmamoriBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Both;
			
			Attack = 8;
			Speed = 10.0f * BasisSpeed;
			Gravity = 0;
			
			MaxRange = 7 * Def.MapTipLengthOnField;
			
			OnHit = OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/ofuda";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			var offset = new Vector3D(-2,-2,8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(HanjoHitEffect).FullName,target.Pos+offset);
			var scale = new Vector3D(8,8,8);
			effect.SetScale(scale);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
		}
	}
}

