﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.White;
using LotusCraft.Ability;
using LotusCraft.Effect.White;

namespace LotusCraft.Spell.White
{
	class Miare:thrts.SpellParameter<thrts.IUnit>
	{
		float[] incDamageRatio;
		float[] decArmorRatio;
		const int interval = 10;
		public Miare()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "御阿礼の子";
			Description = "範囲内の敵ユニットにより多くのダメージを与える。\nこの効果は阿求以外によるダメージも増強させる。";
			string basePath = "White/Spell/Miare/";
			IsPassive = true;

			incDamageRatio = new float[MaxLevel + 1];
			incDamageRatio[0] = 0;
			incDamageRatio[1] = 0.05f;
			incDamageRatio[2] = 0.10f;
			incDamageRatio[3] = 0.15f;

			decArmorRatio = new float[MaxLevel + 1];
			decArmorRatio[0] = 0;
			decArmorRatio[1] = 0.25f;
			decArmorRatio[2] = 0.5f;
			decArmorRatio[3] = 1.0f;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"ダメージ+"+incDamageRatio[1].ToString("###.%")+"、防御力-"+decArmorRatio[1].ToString("###.%"),
				"ダメージ+"+incDamageRatio[2].ToString("###.%")+"、防御力-"+decArmorRatio[2].ToString("###.%"),
				"ダメージ+"+incDamageRatio[3].ToString("###.%")+"、防御力-"+decArmorRatio[3].ToString("###.%"),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			if (selfUnit == null || !selfUnit.Exists())
				return false;

			var affectRadius = selfUnit.ViewRange;
			var areaScale = affectRadius / 6.0f;
			
			var ability = selfUnit.AbilityManager.GetAbilityWithName("MiareAreaEffect") as FollowEffectAbility;
			if (null != ability)
			{
				ability.ExtendLifetime(interval,interval*2);
				ability.Effect.SetScale(new Vector3D(areaScale, areaScale, areaScale));
			}
			else
			{
				// 範囲表示を追加
				var offset = new Vector3D(0, 0, 0);
				var areaeffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(MiareAreaEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
				areaeffect.SetScale(new Vector3D(areaScale, areaScale, areaScale));

				if (!selfUnit.AbilityManager.AppendAbility(new FollowEffectAbility(areaeffect, offset, interval * 2, "MiareAreaEffect")))
				{
					areaeffect.Stop();
				}
			}

			/// 自分を中心にユニットを選択
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var target = world.GetObjectFromCircle(new thrts.Circle2D(affectRadius, new thrts.Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>();
			foreach (var u in target)
			{
				/// ユニットを識別
				if (u == null || !u.Exists() || world.IsSameTeam(selfFaction, u.Faction))
					continue;

				var abiity = u.AbilityManager.GetAbilityWithName(MiareAbility.AbilityName) as MiareAbility;
				if (null == abiity)
				{
					/// エフェクトを描画
					var faction = u.Faction;
					var offset = new Vector3D(0, 0, 0);
					var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(MiareEffect).FullName, u.Pos + offset, selfUnit.Faction);
					var scale = 20;
					effect.SetScale(new Vector3D(scale, scale, scale));


					/// uTargetに対するボーナスアビリティを付与
					abiity = new MiareAbility(incDamageRatio[level], decArmorRatio[level], effect, offset, interval * 2);
					abiity.StopEffectOnVanish = true;
		
					if (!u.AbilityManager.AppendAbility(abiity))
						;
				}
				else
				{
					ability.ExtendLifetime(interval,interval*2);
					abiity.IncDamageRatio = incDamageRatio[level];
					abiity.DecArmorRatio = decArmorRatio[level];
				}
			}
			return true;
		}

		class MiareAbility : FollowEffectAbility
		{
			public static string AbilityName = "御阿礼の子";
			public float IncDamageRatio = 0;
			public float DecArmorRatio = 0;
			public MiareAbility(float incDamageRatio,float decArmorRatio,IEffect effect,Vector3D offset,int duration)
				:base(effect,offset,duration,AbilityName)
			{
				this.IncDamageRatio = incDamageRatio;
				this.DecArmorRatio = decArmorRatio;
			}
			public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
			{
				switch (order)
				{
				case thrts.AbilityOrder.ConstDecParam:
					break;
				case thrts.AbilityOrder.RateDecParam:
					unit.Armor -= (int)Math.Ceiling(DecArmorRatio*unit.Armor);
					break;
				case thrts.AbilityOrder.ConstIncParam:
					unit.DamageRatio += IncDamageRatio;
					break;
				case thrts.AbilityOrder.RateIncParam:
					break;
					case thrts.AbilityOrder.OtherExecs:
						Effect.SetPosition(unit.Pos + Offset);
					break;
				default:
					break;
				}
				
				return true;
			}
			public override bool ModifyBullet(thrts.IBullet bullet)
			{
				
				return base.ModifyBullet(bullet);
			}
		}
	}
}
