﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;
using LotusCraft.Ability.White;
using LotusCraft.Ability;

namespace LotusCraft.Spell.White
{
	class Nyorai:thrts.SpellParameter<thrts.IUnit>
	{
		const int duration = 8*Def.FPS;
		NyoraiAbility[] spellAbilities = new[]
		{
			new NyoraiAbility(0),
			new NyoraiAbility(1),
			new NyoraiAbility(2),
			new NyoraiAbility(3),
		};
		public Nyorai()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "習合「垂迹大日如来」";
			Description = duration/Def.FPS+"秒間、攻撃速度と防御力を上昇させる。";
			string basePath = "White/Spell/Nyorai/";

			CoolTimes = new int[4];
			CoolTimes[0] = 0;
			CoolTimes[1] = 16 * 30;
			CoolTimes[2] = 16 * 30;
			CoolTimes[3] = 16 * 30;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃速度+"+(spellAbilities[1].AtkSpeedRatio - 1.0f).ToString("###.%")+"/防御力+"+spellAbilities[1].IncArmor,
				"攻撃速度+"+(spellAbilities[2].AtkSpeedRatio - 1.0f).ToString("###.%")+"/防御力+"+spellAbilities[2].IncArmor,
				"攻撃速度+"+(spellAbilities[3].AtkSpeedRatio - 1.0f).ToString("###.%")+"/防御力+"+spellAbilities[3].IncArmor,
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
            
            {
            }
			return true;
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists () || selfUnit.AbilityManager.ContainsAbility(NyoraiAbility.AbilityName)) {
				yield return false;
				yield break;
			}
			var spell = selfUnit.GetSpellWithName (this.Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime();

			{
				Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(NyoraiEffect).FullName, selfUnit.Pos + offset);
				var scale = 3 * 3.5f;
				areaEffect.SetScale(new Vector3D(scale, scale, scale));
				areaEffect.SetPosition(selfUnit.Pos + offset);
				selfUnit.AbilityManager.AppendAbility(new NyoraiAbility(level,duration,offset,areaEffect));
			}

		}
		public void OnVanishUnit(thrts.IUnit selfUnit)
        {
        }
	}
}


namespace LotusCraft.Ability.White
{
	class NyoraiAbility : thrts.AbilityTemplate
	{
		public float AtkSpeedRatio;
		public int IncArmor;
		Vector3D offset;
		IEffect effect = null;
		public const string AbilityName = "習合「垂迹大日如来」";
		public NyoraiAbility (int level, int duration, Vector3D offset, IEffect effect)
			:this(level,effect)
		{
			this.offset = offset;
			DurationTime = duration;
		}
		public NyoraiAbility (int level,IEffect effect = null)
		{
			this.Name = "習合「垂迹大日如来」";
			this.DurationTime = 8*Def.FPS;
			this.effect = effect;
			this.offset  = new Vector3D(1,1,2);
			switch (level)
			{
			case 1:
				AtkSpeedRatio = 1.3f;
				IncArmor = 3;
				break;
			case 2:
				AtkSpeedRatio = 1.6f;
				IncArmor = 5;
				break;
			case 3:
				AtkSpeedRatio = 2.0f;
				IncArmor = 7;
				break;
			default:
				AtkSpeedRatio = 1.0f;
				IncArmor = 0;
				break;
			}
			
		}
		public override bool ExecUnitAbility (thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order) {
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.Armor += IncArmor;
				break;
			case thrts.AbilityOrder.RateIncParam:
				///攻撃速度をspeedRate倍
				var bullet = unit.GetModifiedBulletParameter(typeof(HijiriBullet).FullName);
				if(null != bullet)
				{
					bullet.AttackDelay  = (int)Math.Ceiling(bullet.AttackDelay /AtkSpeedRatio);
					bullet.AttackRigor = (int)Math.Ceiling(bullet.AttackRigor /AtkSpeedRatio);
					bullet.AttackSpeed = (int)Math.Floor(bullet.AttackSpeed /AtkSpeedRatio);
				}
				break;
			case AbilityOrder.OtherExecs:
				if(null != effect)
				{
					effect.SetPosition(unit.Pos+offset);
				}
				break;
			default: 
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != effect) {
				effect.Stop ();
				effect = null;
			}
			return base.OnVanish (unit);
		}
		public override bool ModifyBullet(thrts.IBullet bullet)
		{
			
			return base.ModifyBullet(bullet);
		}
	}
	
}

#endif