﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.White;
using LotusCraft.Ability;
using LotusCraft.Ability.White;


namespace LotusCraft.Spell.White
{
	class TenmonMissou:thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadius;
		public TenmonMissou()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "秘術「天文密葬法」";
			Description = "指定地点の周囲に居る敵ユニットを一定時間の間追放する。";
			string basePath = "White/Spell/TenmonMissou/";
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 600*Def.FPS;
			CoolTimes[1] = 75*Def.FPS;
			CoolTimes[2] = 75*Def.FPS;
			CoolTimes[3] = 75*Def.FPS;

			MaxRanges = new float[MaxLevel+1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 4*Def.MapTipLengthOnField;
			MaxRanges[2] = 6*Def.MapTipLengthOnField;
			MaxRanges[3] = 8*Def.MapTipLengthOnField;


			affectRadius = new float[MaxLevel+1];
			affectRadius[0] = 0;
			affectRadius[1] = 3;
			affectRadius[2] = 4;
			affectRadius[3] = 5;



			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"直径" + (affectRadius[1] * 2).ToString(),
				"直径" + (affectRadius[2] * 2).ToString(),
				"直径" + (affectRadius[3] * 2).ToString(),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, float x, float y, int level)
		{


			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			spell.ResetCoolTime ();

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			var targetPos = world.GetGroundAtPos(new Vector2D(x, y)) + new Vector3D(0, 0, 4);

 			// wait delay
			int waitTime = 4*Def.FPS;

			var areaEffect = world.EffectManager.CreateEffect(typeof(MissouChargeEffect).FullName,targetPos);
			var scale = 5.0f*(affectRadius[level]);
			areaEffect.SetScale (new Vector3D ( scale, scale, scale ) );

			var waitAbility = new EffectContinuousAbility(areaEffect, waitTime, this.Name+"ChargeEffect");
			if(!selfUnit.AbilityManager.AppendAbility(waitAbility))
				if(areaEffect != null)
					areaEffect.Stop();

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin01).FullName, selfUnit.Pos);

			for (int i=0; i<waitTime; ++i)
				yield return true;

			var effect = world.EffectManager.CreateEffect(typeof(MissouAreaEffect).FullName, targetPos);
			effect.SetScale(new Vector3D(scale, scale, scale));

			var areaAbility = new EffectContinuousAbility(effect, 2 * Def.FPS, this.Name + "AreaEffect");
			if (!selfUnit.AbilityManager.AppendAbility(areaAbility))
				if (effect != null)
					effect.Stop();

			// get targets
			/// 地点(x,y)を中心にユニットを選択
			var targets = world.GetObjectFromCircle (new Circle2D (affectRadius [level] * 32, new Vector2D (x, y)))
				.Where (_ => _ != null && !world.IsSameTeam (selfFaction, _.Faction))
					.OfType<IUnit> ();
			
			// add force escaping untis ability
			var missouAbility = new MissouAbility(spell, targets, 20*Def.FPS);

			selfUnit.AbilityManager.AppendAbility (missouAbility);

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin02).FullName, selfUnit.Pos);

			// wait 3 frames
			yield return true;
			yield return true;
			yield return true;


			yield return false;
		}
	}
}

#endif
