﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.White;
using LotusCraft.Effect.White;

namespace LotusCraft.Spell.White
{
	class TaidenNyudo:thrts.SpellParameter<thrts.IUnit>
	{
		IEffect areaEffect = null;
		int nowlevel = 0;
		float[] damage = new float[4];
		const float reflectRadius = 2*Def.MapTipLengthOnField;

		public TaidenNyudo()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "稲妻「帯電入道」";
			Description = "周囲の敵ユニットから攻撃を受けたとき、ダメージを反射する。";
			string basePath = "White/Spell/TaidenNyudo/";

            damage[0] = 0;
            damage[1] = 0.60f;
            damage[2] = 1.00f;
            damage[3] = 1.20f;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲2、受けたダメージの"+((int)(damage[1] * 100)).ToString() + "%",
				"範囲2、受けたダメージの"+((int)(damage[2] * 100)).ToString() + "%",
				"範囲2、受けたダメージの"+((int)(damage[3] * 100)).ToString() + "%",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling(IUnit selfUnit, int level)
		{
			{
				var affectRadius = 0.8f;
				Vector3D offset = new Vector3D(2.0f, 2.0f, 10.0f);
				if (areaEffect == null || nowlevel != level)
				{
					if (areaEffect != null)
					{
						areaEffect.Stop();
						areaEffect = null;
					}
					areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(TaidenEffect).FullName, selfUnit.Pos + offset);
					areaEffect.SetScale(new Vector3D(5 * affectRadius * 2, 5 * affectRadius * 2, 5 * affectRadius * 2));
					selfUnit.AddOnVanishEvent(OnVanishUnit);
				}
				areaEffect.SetPosition(selfUnit.Pos + offset);
			}

			return true;
		}

		public override void OnLvUp (IFaction faction, int lv)
		{
			var unitParam = faction.GetUnitParameter (typeof(Unit.White.Unique.Ichirin).FullName);

			if (null != unitParam) {
				switch (nowlevel) {
				case 1:
					unitParam.TakeBullet -= ReflectDamageLv1;
					break;
				case 2:
					unitParam.TakeBullet -= ReflectDamageLv2;
					break;
				case 3:
					unitParam.TakeBullet -= ReflectDamageLv3;
					break;
				default:
					break;
				}
				switch (lv) {
				case 1:
					unitParam.TakeBullet += ReflectDamageLv1;
					break;
				case 2:
					unitParam.TakeBullet += ReflectDamageLv2;
					break;
				case 3:
					unitParam.TakeBullet += ReflectDamageLv3;
					break;
				default:
					break;
				}
			}

			nowlevel = lv;

			base.OnLvUp (faction, lv);
		}

		public void OnVanishUnit(IUnit selfUnit)
		{
			if (areaEffect != null)
			{
				areaEffect.Stop();
				areaEffect = null;
			}
		}

		public void ReflectDamage(int level,IBullet src,IUnit dist,int dmg,int atk)
		{
			if(null ==src)
				return;
			
			var u = src.Generator as IUnit;
			if(null == u)
				return;

			if( u.UniqueID == dist.UniqueID)
				return;

			if(Utl.GetDistance(u.Pos.X,u.Pos.Y,dist.Pos.X,dist.Pos.Y) <= reflectRadius)
			{
				
				/// エフェクトを描画
				var faction = u.Faction;
				var world = faction.World;
				Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
				var effect = world.EffectManager.CreateEffect(typeof(HitSplashRedEffect).FullName, u.Pos + offset);
				var scale = new Vector3D(8, 8, 8);
				effect.SetScale(scale);
				
				var hitAbility = new Ability.EffectContinuousAbility(effect, 13, this.Name + "HitEffect");
				var flag = u.AbilityManager.AppendAbility(hitAbility);
				if (!flag)
					effect.Stop();
				else
				{
					//反射ダメージ
					u.BeDamaged(dist, (int)Math.Ceiling(dmg*damage[level]),AttackBonusSet.None);
					
					world.SoundManager.PlaySoundEffect(typeof(SoundEffect.BiriBiri).FullName, u.Pos);
				}
			}
			
		}
		public void ReflectDamageLv1(IBullet src,IUnit dist,int dmg,int atk)
		{
			ReflectDamage(1,src,dist,dmg,atk);
		}
		public void ReflectDamageLv2(IBullet src,IUnit dist,int dmg,int atk)
		{
			ReflectDamage(2,src,dist,dmg,atk);
		}
		public void ReflectDamageLv3(IBullet src,IUnit dist,int dmg,int atk)
		{
			ReflectDamage(3,src,dist,dmg,atk);
		}
	}
}

#endif
