﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.White;
using LotusCraft.Effect.White;

namespace LotusCraft.Spell.White
{
	class NijuKekkai:thrts.SpellParameter<thrts.IUnit>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="LotusCraft.Spell.White.NijuKekkai"/> class.
		/// </summary>
		NijuKekkaiAbility[] spellAbilities = new []
		{
			new NijuKekkaiAbility(0),
			new NijuKekkaiAbility(1),
			new NijuKekkaiAbility(2),
			new NijuKekkaiAbility(3),
		};
		IEffect areaEffect = null;
		int autocastInterval = 15;
		int counter = 0;
		int[] affectRadius;
		public NijuKekkai()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			IsPassive = true;
			Name = "夢符「二重結界」";

			affectRadius = new int[MaxLevel + 1];
			affectRadius[0] = 0;
			affectRadius[1] = 5;
			affectRadius[2] = 6;
			affectRadius[3] = 7;

			Description = "近くにいる味方ユニットの防御力を上昇させる。";
			string basePath = "White/Spell/NijuKekkai/";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲"+affectRadius[1]+"、"+"防御+"+spellAbilities[1].IncArmor.ToString(),
				"範囲"+affectRadius[2]+"、"+"防御+"+spellAbilities[2].IncArmor.ToString(),
				"範囲"+affectRadius[3]+"、"+"防御+"+spellAbilities[3].IncArmor.ToString(),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling (IUnit selfUnit, int level)
		{
			if(level > MaxLevel || level >= affectRadius.Length)
				return false;

			float radius = affectRadius[level];
			

			{
				Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
				if (areaEffect == null)
				{
					areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(NijuKekkaiAreaEffect).FullName, selfUnit.Pos + offset);
					areaEffect.SetScale(new Vector3D(8 * radius, 8 * radius, 8 * radius));
					selfUnit.AddOnVanishEvent(OnVanishUnit);
				}
				areaEffect.SetPosition(selfUnit.Pos + offset);
			}

			if (++counter < autocastInterval) {
				return true;
			}

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D( radius*Def.MapTipLengthOnField,new Vector2D( selfUnit.Pos.X, selfUnit.Pos.Y ))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>selfFaction.World.IsSameTeam(selfFaction,_.Faction)))
			{
				// 自分には効果無し
				if(u.UniqueID == selfUnit.UniqueID)
					continue;

				if(!u.AbilityManager.ContainsAbility(spellAbilities[level].Name))
				{
					IEffectCommon effect = null;

					if(u != selfUnit)
					{
						// エフェクトを描画
						/*
						{
							Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
							var e = world.EffectManager.CreateEffect(typeof(NijuKekkaiEffect).FullName, u.Pos + offset);
							e.SetScale(new Vector3D(12, 12, 12));
							effect = e;
						}
						*/

						{
							effect = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(NijuKekkaiTextureEffect).FullName, u.Pos);
						}
					}
					if(!u.AbilityManager.AppendAbility(new NijuKekkaiAbility(level,effect)))
					{
						if(effect != null)
						{
							effect.Stop();
							effect = null;
						}
					}
				}
				else
				{
					u.AbilityManager.ExtendLifetimeWithName(autocastInterval,spellAbilities[level].Name,spellAbilities[level].DurationTime);
				}
			}

			counter = 0;

			return true;
		}
		public void OnVanishUnit (IUnit selfUnit)
		{
			if (areaEffect != null) {
				areaEffect.Stop();
				areaEffect = null;
			}
		}
	}
}


namespace LotusCraft.Ability.White
{
	class NijuKekkaiAbility : thrts.AbilityTemplate
	{
		public int IncArmor;
		IEffectCommon effect;
		public NijuKekkaiAbility(int level, IEffectCommon effect = null)
		{
			this.Name = "夢符「二重結界」 lv" + level.ToString();
			this.DurationTime = 1 * 30;
			this.effect = effect;
			switch (level)
			{
			case 1:
				IncArmor = 1;
				break;
			case 2:
				IncArmor = 2;
				break;
			case 3:
				IncArmor = 3;
				break;
			default:
				IncArmor = 0;
				break;
			}
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.Armor += IncArmor;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if (effect != null)
					effect.SetPosition(unit.Pos + new Vector3D(2, 2, 1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish(IUnit unit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish(unit);
		}
	}
	
}
