﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.White;
using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;
using LotusCraft.Spell.White;

namespace LotusCraft.Spell.White
{
	class MusoHuin:thrts.SpellParameter<thrts.IUnit>
	{
		IBullet[] bullets = new IBullet[3];
		IEffect[] bulletEffects = new IEffect[3];

		
		int[] chargeTime = new int[4];
		const int searchRadius = 8;

		int[] damages;
		float[] durationTime;
		public readonly static Dictionary<float,float>[] Splashs = new []
		{
			new Dictionary<float, float>(),
			new Dictionary<float, float>(),
			new Dictionary<float, float>(){{0.4f*Def.MapTipLengthOnField,0.8f},{0.6f*Def.MapTipLengthOnField,0.5f},{1.0f*Def.MapTipLengthOnField,0.25f},},
			new Dictionary<float, float>(){{0.5f*Def.MapTipLengthOnField,1.0f},{0.7f*Def.MapTipLengthOnField,0.8f},{1.0f*Def.MapTipLengthOnField,0.5f},{1.5f*Def.MapTipLengthOnField,0.25f},},
		};
		
		public readonly static string[] BulletEffectName = new string[]
		{
			typeof(MusoHuinRedBulletEffect).FullName,
			typeof(MusoHuinBlueBulletEffect).FullName,
			typeof(MusoHuinGreenBulletEffect).FullName,
			typeof(MusoHuinRedBulletEffect).FullName,
			typeof(MusoHuinBlueBulletEffect).FullName,
			typeof(MusoHuinGreenBulletEffect).FullName,
		};
		public readonly static string[] BulletName = new string[]
		{
			typeof(MusoHuinRedBullet).FullName,
			typeof(MusoHuinBlueBullet).FullName,
			typeof(MusoHuinGreenBullet).FullName,
			typeof(MusoHuinRedBullet).FullName,
			typeof(MusoHuinBlueBullet).FullName,
			typeof(MusoHuinGreenBullet).FullName,
		};

		public MusoHuin()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "霊符「夢想封印」";
			Description = "自分の周辺にいる敵ユニットに向けて、追尾弾を発射する。";
			string basePath = "White/Spell/MusoHuin/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 18*30;
			CoolTimes[2] = 20*30;
			CoolTimes[3] = 22*30;

			RequireLevels[0] = 1;
			RequireLevels[1] = 3;
			RequireLevels[2] = 5;

			chargeTime[0] = 0;
			chargeTime[1] = 1*30;
			chargeTime[2] = 1*30;
			chargeTime[3] = 1 *30;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 30;
			damages[2] = 50;
			damages[3] = 70;

			durationTime = new float[MaxLevel + 1];
			durationTime[0] = 0;
			durationTime[1] = 4;
			durationTime[2] = 6;
			durationTime[3] = 8;

			
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				damages[1] + "ダメージ",
				damages[2] + "ダメージ, スプラッシュ小",
				damages[3] + "ダメージ, スプラッシュ大",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				null,
				null,
				null,
				null,
				new AutoCast.AutoCast<IUnit>(this.Name+"searchTarget",Def.FPS,SeachTarget,true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			// reset cooltime
			spell.ResetCoolTime();

			// chage effect
			Vector3D offset = new Vector3D(0.0f,0.0f,1.0f);
			var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(MusoHuinChargeEffect).FullName, selfUnit.Pos + offset);
			float effectScale = 8*searchRadius;
			effect.SetScale (new Vector3D (effectScale, effectScale, effectScale));
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);

			if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect,chargeTime[level],this.Name+"chargeEffect")))
				effect.Stop();

			for(int i=0;i<chargeTime[level];++i)
				yield return true;


			Spelling(selfUnit,level);

			yield return true;
		}
		public bool Spelling (IUnit speller, int level)
		{
			if (!speller.Exists ())
				return false;

			var selfFaction = speller.Faction;
			var world = selfFaction.World;
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);

			// clear bullets
			for (int i=0; i<bullets.Count(); ++i)
			{
				if(bullets[i]!= null)
				{
					if(bullets[i].Exists)
						bullets[i].Vanish();
					bullets [i] = null;
				}
			}
			// create ball effect
			for (int i=0; i<bulletEffects.Count(); ++i) {
				if (bulletEffects [i] == null) {
					bulletEffects [i] = world.EffectManager.CreateEffect (BulletEffectName [i], speller.Pos + offset);
					float effectScale = 20;
					bulletEffects [i].SetScale (new Vector3D (effectScale, effectScale, effectScale));
				}
			}
			
			var ability = speller.AbilityManager.GetAbilityWithName(MusoHuinBulletControllAbility.AbilityName) as MusoHuinBulletControllAbility;
			if (null == ability) {
				ability = new MusoHuinBulletControllAbility (speller, damages [level], Splashs[level], bullets, bulletEffects, (int)(durationTime [level] * Def.FPS));

				if (!speller.AbilityManager.AppendAbility (ability))
					ability.OnVanish (speller);

			} else {
				ability.Damage = damages[level];
				ability.Splash = Splashs[level];
			}

			return true;
		}

		public bool SeachTarget (IUnit speller)
		{
			var ability = speller.AbilityManager.GetAbilityWithName(MusoHuinBulletControllAbility.AbilityName) as MusoHuinBulletControllAbility;

			if(null == ability)
				return false;

			var spell = speller.GetSpellWithName (this.Name);
			if (null == spell)
				return false;

			var center = new Vector2D (speller.Pos.X, speller.Pos.Y);
			var world = speller.Faction.World;
			/// search targets
			var targets = world.GetObjectFromCircle (new Circle2D (searchRadius * Def.MapTipLengthOnField, center))
				.OfType<IUnit> ()
					.Where (__ => !world.IsSameTeam (speller.Faction, __.Faction))
					.OrderBy (_ => (_.Pos.X - center.X) * (_.Pos.X - center.X) + (_.Pos.Y - center.Y) * (_.Pos.Y - center.Y));
			
			
			foreach(var target in targets)
			{
				if(null == target || !target.Exists())
					continue;

				int i = ability.AddMusouHuinTarget(target);

				if(i < 0 || i >= bullets.Count())
					return false;

				var bullet = bullets[i];
				// 更新タイミングの関係で弾に効果がならない可能性があるため弾の攻撃力を再指定
				bullet.Attack = damages[spell.Level];
			}
			return true;
		}
	}
}

namespace LotusCraft.Ability.White
{
	class MusoHuinBulletControllAbility : thrts.AbilityTemplate
	{
		IUnit speller;
		public IBullet[] Bullets;
		public IEffect[] BulletEffects;
		public int Damage;
		public Dictionary<float,float> Splash;
		float[] theta = new float[9];
		float[] alpha = new float[9];
		const float radius = 1.5f*Def.MapTipLengthOnField;

		public const string AbilityName =  "霊符「夢想封印」";

		public MusoHuinBulletControllAbility(IUnit speller,int damage,Dictionary<float,float> splash,IBullet[] bullets,IEffect[] bulletEffects,int duration)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.Bullets = bullets;
			this.BulletEffects = bulletEffects;

			this.Damage = damage;
			this.Splash = splash;

			this.speller = speller;

			for(int i=0;i<theta.Length;++i)
				theta[i] = (float)(2*Math.PI * speller.Faction.World.NextRandDouble());
			// 角速度 1~4秒に1周 = 2PI/30 ~ 2PI/120 = PI/60~4PI/60
			for(int i=0;i<alpha.Length;++i)
				alpha[i] = (float)(Math.PI * (1 + 3*speller.Faction.World.NextRandDouble()))/60.0f;
		}

		public int AddMusouHuinTarget (ITargetObject target)
		{
			if(null == target || !target.Exists())
				return -1;

			int i = 0;
			while(i < Bullets.Count() && (null == BulletEffects[i] || null != Bullets[i] ) )
				i++;

			if(i >= Bullets.Count())
				return -1;

			var relPos = new Vector3D(radius*(float)Math.Sin(theta[i]),radius*(float)Math.Cos(theta[i]),32);
			Bullets [i] = speller.CreateBullet (MusoHuin.BulletName[i], target, relPos + speller.Pos);

			return i;
		}

		public void Clear ()
		{
			for(int i=0;i<Bullets.Length;++i)
			{
				if(BulletEffects[i]!=null)
				{
					BulletEffects[i].Stop();
					BulletEffects[i] = null;
					if(null != Bullets[i] && Bullets[i].Exists)
						Bullets[i].Vanish();
					
					Bullets[i] = null;
				}
			}
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				// 効果中は通常攻撃不可
				unit.IsForbiddenAttacking = true;
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case thrts.AbilityOrder.OtherExecs:
				foreach(var bn in MusoHuin.BulletName)
				{
					var b = unit.GetModifiedBulletParameter (bn);
					if (null != b) {
						b.Attack = Damage;
						if (Splash != null && Splash.Any()) {
							b.HasSplash = true;
							b.IsSplashMissHit = true;
							b.SplashDamageRates = Splash;
						}
					}
				}

				if(null != BulletEffects && BulletEffects.Any(_=>null != _))
				{
					for (int i=0; i<BulletEffects.Length; ++i) {
						if(null == BulletEffects[i])
							continue;

						if (null != Bullets && Bullets [i] != null) {
							if (Bullets[i].Exists && (Bullets[i].Target == null || Bullets[i].Target.Exists()) ) {

								var bullet = Bullets[i];
								BulletEffects [i].SetPosition (bullet.Position);

								// change bullet direction
								var diff = bullet.Target.Pos - bullet.Position;
								if(diff.LengthSquared > 0)
								{
									var r = bullet.Vector.Length / diff.Length;
									bullet.SetVector(diff * r);
								}

							} else {
								BulletEffects [i].Stop ();
								BulletEffects [i] = null;
								Bullets [i] = null;
							}
						}
						else
						{
							var relPos = new Vector3D(radius*(float)Math.Sin(theta[i]),radius*(float)Math.Cos(theta[i]),32);
							// move ball effect
							BulletEffects [i].SetPosition (relPos + speller.Pos);
							theta[i] += alpha[i];
						}
					}
				}
				else
				{
					// 全ての弾を打ち切ったら削除
					this.DurationTime = 0;
				}
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnAdd (IUnit unit)
		{
			unit.IsForbiddenAttacking = true;
			return base.OnAdd (unit);
		}
		public override bool OnVanish (thrts.IUnit unit)
		{
			Clear();
			unit.IsForbiddenAttacking = false;
			return base.OnVanish (unit);
		}
		public override bool ModifyBullet(thrts.IBullet bullet)
		{
			return base.ModifyBullet(bullet);
		}
	}
	
}


namespace LotusCraft.Bullet.White
{
	public class MusoHuinBullet : thrts.BulletParameter
	{
		public MusoHuinBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Both;
			
			this.Attack = 10;
			Speed = 8.0f;
			// 放物線を描かない
			Gravity = 0;
			Radius = Def.MapTipLengthOnField;
			
			MaxRange = 16*Def.MapTipLengthOnField;
			
		}
	}
	public class MusoHuinBlueBullet : MusoHuinBullet
	{
		public MusoHuinBlueBullet()
		{
			OnHit = OnHitEffect;
		}
		public void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			/// create effect
			var effect = bullet.Generator.Faction.World.EffectManager.CreateEffect(typeof(MusoHuinBlueHitEffect).FullName, bullet.Position);
			float effectScale = 20;
			effect.SetScale (new Vector3D (effectScale, effectScale, effectScale));
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom01).FullName, bullet.Position);
		}
	}
	public class MusoHuinRedBullet : MusoHuinBullet
	{
		public MusoHuinRedBullet()
		{
			OnHit = OnHitEffect;
		}
		public void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			/// create effect
			var effect = bullet.Generator.Faction.World.EffectManager.CreateEffect(typeof(MusoHuinRedHitEffect).FullName, bullet.Position);
			float effectScale = 20;
			effect.SetScale (new Vector3D (effectScale, effectScale, effectScale));
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom01).FullName, bullet.Position);
		}
	}
	public class MusoHuinGreenBullet : MusoHuinBullet
	{
		public MusoHuinGreenBullet()
		{
			OnHit = OnHitEffect;
		}
		public void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			/// create effect
			var effect = bullet.Generator.Faction.World.EffectManager.CreateEffect(typeof(MusoHuinGreenHitEffect).FullName, bullet.Position);
			float effectScale = 20;
			effect.SetScale (new Vector3D (effectScale, effectScale, effectScale));
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom01).FullName, bullet.Position);
		}

	}
}
