﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;
using LotusCraft.Ability.White;
using LotusCraft.Ability;

namespace LotusCraft.Spell.White
{
	class ReisenSkill:thrts.SpellParameter<thrts.IUnit>
	{
		/// <summary>
		/// The mode change time[s].
		/// </summary>
		const float modeChangeTime = 4.0f;
		const int affectRadius = 5;
		const int healHP = 10;
		const int interval = (int)(2*Def.FPS);

		public ReisenSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			IsPassive = true;
			Name = "生薬「国士無双の薬」";
			string basePath = "White/Spell/Reisen/";

			CoolTimes = new int[2];
			CoolTimes[0] = 4*30;
			CoolTimes[1] = 4*30;

			Description = "自分の周囲にいる味方ユニットのHPを回復させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"+"+(healHP*Def.FPS/interval).ToString()+" HP / s",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>AutoCastAction(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public void Spelling(thrts.IUnit selfUnit, int level)
		{
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			yield return false;
		}

		public bool AutoCastAction (thrts.IUnit unit, int level)
		{
			var selfFaction = unit.Faction;
			// get target from world
			var target = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius*Def.MapTipLengthOnField, new Vector2D(unit.Pos.X,unit.Pos.Y)))
					.OfType<IUnit> ()
					.Where(_ => _!=unit && _.HP < _.MaxHP &&  selfFaction.World.IsSameTeam(_.Faction,selfFaction))
					.OrderBy(_=>_.HP/_.MaxHP);

			// heal team unit
			foreach(var u in target)
			{
				if(u != null && u.Exists() && !u.AbilityManager.ContainsAbility(this.Name + "HealEffect"))
				{
					// add effect
					var effect  = unit.Faction.World.EffectManager.CreateEffect(typeof(HealEffect).FullName, u.Pos);
					if(!u.AbilityManager.AppendAbility(new FollowEffectAbility(effect, new Vector3D(0,0,1), Def.FPS/2, this.Name+"HealEffect")))
					{
						effect.Stop();
					}
					else
					{
						// HPが最も減っている自分以外ユニット1体を選択
						u.HP += healHP;
						if(u.HP > u.MaxHP)
							u.HP = u.MaxHP;
	
						break;
					}
				}
			}
			return true;
		}
	}
}
