﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;
using LotusCraft.Ability.White;
using LotusCraft.Ability;

namespace LotusCraft.Spell.White
{
	class ToramaruSkill:thrts.SpellParameter<thrts.IUnit>
	{
		/// <summary>
		/// The mode change time[s].
		/// </summary>
		const int interval = 10;

		public ToramaruSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			IsPassive = true;
			Name = "光符「正義の威光」";
			string basePath = "White/Spell/Toramaru/";

			CoolTimes = new int[2];
			CoolTimes[0] = 4*30;
			CoolTimes[1] = 4*30;

			Description = "防御力と攻撃力が上昇する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"防御力+"+ToramaruSkillAbility.IncArmor+", 攻撃力+"+ToramaruSkillAbility.IncAtk,
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			{
				var unitRadius = selfUnit.Parameter.Radius;
				Vector3D offset = new Vector3D (0.0f, 0.0f, unitRadius/2);

				if (selfUnit.AbilityManager.ContainsAbility(ToramaruSkillAbility.AbilityName))
				{
					selfUnit.AbilityManager.ExtendLifetimeWithName(interval,ToramaruSkillAbility.AbilityName,interval*2);
				}
				else
				{
					//var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(BackLightEffect).FullName, selfUnit.Faction, selfUnit.Pos + offset);
					//var scale = unitRadius/Def.MapTipLengthOnField * 2;
					//effect.SetScale (new Vector3D (scale,scale,scale));
                    
                    var effect = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(AuraTextureEffect).FullName, selfUnit.Pos);
                    
					selfUnit.AbilityManager.AppendAbility(new ToramaruSkillAbility(effect,offset,interval*2));

				}
			}
			return true;
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			yield return false;
		}
	}
}

namespace LotusCraft.Ability.White
{
	class ToramaruSkillAbility : AbilityTemplate
	{
		public const string AbilityName = "光符「正義の威光」";

		public const int IncArmor = 2;
		public const int IncAtk = 2;
        public IEffectCommon effect = null;
		public ToramaruSkillAbility (IEffectCommon effect,Vector3D offset,int duration)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
            this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.Armor += IncArmor;
				foreach(var b in unit.ModifiedBulletParameters)
					b.Attack += IncAtk;

				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
            case AbilityOrder.OtherExecs:
                if (effect != null)
                    effect.SetPosition(unit.Pos + new Vector3D(2, 2, 1));
                break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool OnVanish (IUnit unit)
		{
            if (effect != null)
            {
                effect.Stop();
                effect = null;
            }
			return base.OnVanish (unit);
		}
	}
}
