﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.Story
{
	public class Story02 : thrts.Mission
	{
		/// <summary>
		/// 自軍の勢力
		/// </summary>
		/// <returns></returns>
		public override string[] GetSelectableFactions()
		{
			return new string[] { 
				typeof(LotusCraft.Faction.White).FullName 
			};
		}

		/// <summary>
		/// 登場する勢力
		/// </summary>
		/// <returns></returns>
		public override string[] GetEnableFactions()
		{
			return new string[] { 
				typeof(LotusCraft.Faction.White).FullName,
				typeof(LotusCraft.Faction.Blue).FullName, 
			};
		}

		/// <summary>
		/// 使用するマップ
		/// </summary>
		/// <returns></returns>
		public override string[] GetSelectableMaps()
		{
			return new string[] {
				"LotusCraft/Story/Story02.map",
			};
		}

		public Story02()
			: base()
		{
			Name = "ストーリー02";
			Description = "2話のマップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		/// <summary>
		/// マップ上に設定した領域
		/// </summary>
		Area[] purposeAreas = null;

		public override void OnInitialize(thrts.IWorld world)
		{
			world.Factions[1].PlayerName = "青勢力";

			// 最初から負けないように
			world.Factions.ForEach(f => f.DoesNotLose = true);

			// 初期視点位置
			world.Factions[0].Parameter.DefaultScreenPosition
				= new Point2D((int)(1175), (int)(697));

			// 初期資源を配布
			world.Factions[0].IncResource(0, 800);
			world.Factions[0].IncResource(1, 200);

			// 難易度調整
			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Chirno).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Mystia).FullName).HP /= 2;

				world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP /= 2;
				world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP /= 2;
				world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Chirno).FullName).HP /= 2;
				world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Mystia).FullName).HP /= 2;

				world.UnitManager.GetUnits(u => u.Faction.Index != 0).Do(u => u.MaxHP = u.Parameter.HP).ForEach(u => u.HP = u.Parameter.HP);
			}
			else if (world.StoryUtilities.GetDifficulty() == 2)
			{
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP += (int)(world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP * 0.2);
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP += (int)(world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP * 0.2);
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Chirno).FullName).HP += (int)(world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Chirno).FullName).HP * 0.2);
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Mystia).FullName).HP += (int)(world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Mystia).FullName).HP * 0.2);

				world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP += (int)(world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP * 0.2);
				world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP += (int)(world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Low02).FullName).HP * 0.2);
				world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Chirno).FullName).HP += (int)(world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Chirno).FullName).HP * 0.2);
				world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Mystia).FullName).HP += (int)(world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Unique.Mystia).FullName).HP * 0.2);

				world.UnitManager.GetUnits(u => u.Faction.Index != 0).Do(u => u.MaxHP = u.Parameter.HP).ForEach(u => u.HP = u.Parameter.HP);
			}

			var chirno = GetUnits(world, world.Factions[2], typeof(LotusCraft.Unit.Blue.Unique.Chirno)).Single();
			var mystia = GetUnits(world, world.Factions[1], typeof(LotusCraft.Unit.Blue.Unique.Mystia)).Single();

			chirno.AddExperience(1000);
			mystia.AddExperience(500);

			// ボタン仕様禁止
			var home = world.BuildingManager.GetBuildings(_ => _.Faction == world.Factions[0] && _.Parameter.GetType() == typeof(LotusCraft.Building.White.HomeBase)).First();
			var ui = world.StoryUI;
			ui.HideButton(home, ButtonPosition._0, ButtonPosition._1);
			ui.HideButton(home, ButtonPosition._1, ButtonPosition._1);
			ui.HideButton(home, ButtonPosition._2, ButtonPosition._1);
			ui.HideButton(home, ButtonPosition._0, ButtonPosition._2);
			ui.HideButton(home, ButtonPosition._1, ButtonPosition._2);


			base.OnInitialize(world);
		}

		IEnumerable<IUnit> GetUnits(IWorld world, IFaction faction, Type paramType)
		{
			return world.UnitManager.GetUnits(_ => _.Faction == faction && _.Parameter.GetType() == paramType);
		}

		int[] Reimu;
		int[] Marisa;
		int[] Akyu;
		int[] Chilno;
		int[] Mystia;
		int[] Yukari;

		public override System.Collections.IEnumerable Update(thrts.IWorld world)
		{
			world.ChangeBGM(BGMList.Boushuen_Story1);

			var ui = world.StoryUI;
			// 立ち絵
			Reimu = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Reimu_N"), ui.LoadImage("LotusCraft/Image/Tachie/Reimu_S"), ui.LoadImage("LotusCraft/Image/Tachie/Reimu_W") };
			Marisa = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Marisa_N"), ui.LoadImage("LotusCraft/Image/Tachie/Marisa_S"), ui.LoadImage("LotusCraft/Image/Tachie/Marisa_W") };
			Akyu = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Akyu_N"), ui.LoadImage("LotusCraft/Image/Tachie/Akyu_S"), ui.LoadImage("LotusCraft/Image/Tachie/Akyu_W") };
			Chilno = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Chilno_N"), ui.LoadImage("LotusCraft/Image/Tachie/Chilno_S"), ui.LoadImage("LotusCraft/Image/Tachie/Chilno_W") };
			Mystia = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Mystia_N"), ui.LoadImage("LotusCraft/Image/Tachie/Mystia_S"), ui.LoadImage("LotusCraft/Image/Tachie/Mystia_W") };
			Yukari = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Yukari_N"), ui.LoadImage("LotusCraft/Image/Tachie/Yukari_S"), ui.LoadImage("LotusCraft/Image/Tachie/Yukari_W") };


			#region 1
			{
				ui.PauseGame();
				ui.Show();
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【霊夢】\n偶然にも人里に訪れた私たちは");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【魔理沙】\nいやお前が行こうって言ったんだろうが");
				ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【霊夢】\n細かいことは気にしちゃダメよ。私たちはここで頼もしい仲間を見つけないといけないんだから");
				ui.ShowText("【魔理沙】\n頼もしい仲間な。その理由をまだ聞いてないんだが、どうして最初に人里なんだ？");
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【霊夢】\nいい？　魔理沙。大体こういうイベントで、しかも大人数で行動することを許可されている場合、紅魔館みたいな連中はさっさと身内で固まるわ");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\nだろうな。あそこは自分たちだけで揃っちまう。他所と組む必要性はないわけだ");
				ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【霊夢】\n永遠亭の連中は行くのも骨だし、戦力を借りられたとしてもせいぜい兎の一匹二匹。選択肢からは除外ね");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\n早苗のところはー……。あそこも私たちと協調路線を取りそうにはないだろうしなぁ・・・");
				ui.ShowText("【霊夢】\nええ、神社にお燐が居れば話は別だったけど、ここ数日姿を見てない。地霊殿に行って何も収穫無しってなったら寒いを通り越すわ");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\n仙人様たちも普段どこに居るかわからないし、冥界なんてもってのほか。最後に残る選択肢が人里ってわけか");
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【霊夢】\nそうよ。正確には人里の近所の妖怪寺の連中だったら、手っ取り早く戦力として計算できるわ");
				ui.ShowText("【魔理沙】\nなるほど。アリスとかの在野の妖怪は後回しでもいいってわけだ");
				ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【霊夢】\nアリスも二つ返事で仲間になってくれるほど篭絡しやすい相手でもないしね");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\nまぁな");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				var talkEnd = ui.ShowText("", false);

				while (!ui.IsCompleted(talkEnd))
				{
					yield return null;
				}

				ui.Hide();
				ui.ResumeGame();

				var aq = GetUnits(world, world.Factions[0], typeof(LotusCraft.Unit.White.Unique.Akyu)).Single();
				ui.JumpScreenPosition(aq.Pos);

				for (int i = 0; i < 50; i++)
					yield return null;

				ui.PauseGame();
				ui.Show();

				ui.ShowImage(Akyu[0], StoryUIImagePosition.Front, true, false);
				ui.ShowText("【阿求】\nいまの声……もしや我が友、李徴氏ではないか？");
				ui.ShowImage(-1, StoryUIImagePosition.Front, true);
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【魔理沙】\n別人だぜ。李徴氏は男だし、そもそも私は虎じゃない。魔法使いだ");
				ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【霊夢】\n友達であることは認めるのね");
				ui.ShowImage(Akyu[1], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【阿求】\n嬉しいことです。レッツマブタチです。ところで面白そうな言葉が聞こえたのですが");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【魔理沙】\nあー？　これから妖怪寺に訪問することがそんなに楽しいことか？");
				ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【霊夢】\nどちらかというと憂鬱な部類に入りそうなものだけど");
				ui.ShowImage(Akyu[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【阿求】\nいえ、仲間を求めてのくだりですよ");
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【霊夢】\nまさかあんたが？");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(Akyu[1], StoryUIImagePosition.Front, false, false);
				ui.ShowText("【阿求】\nわたしは そうりょひえだ たたかいはできませんが ちりょうのつえがつかえます");
				ui.ShowImage(-1, StoryUIImagePosition.Front, true);
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【霊夢】\n却下");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【魔理沙】\nナシだな");
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(Akyu[1], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【阿求】\n冗談ですよ。知っていますよ。今度の戦いは複数人で戦う変形スペルカードルールなんですよね？　私のような参謀役が居れば役立つに違いありません");
				ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【霊夢】\n本音のところは");
				ui.ShowImage(Akyu[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【阿求】\n調子に乗って妖精をからかってたら氷精と夜雀に決闘を挑まれて一人じゃどうしようもないので助けて博麗の巫女様");
				ui.ShowImage(Reimu[1], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【霊夢】\n自業自得じゃないの");
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【霊夢】\nでも妖怪退治は博麗の巫女の本分。ついでにあんたがどれぐらい戦えるのか見てあげるわ");
				ui.ShowImage(Akyu[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【阿求】\nできれば私は戦いたくないんですが");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【魔理沙】\nそんな美味い話は世の中ないぜ");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(Chilno[0], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【チルノ】\nよーやく見つけた！");
				ui.ShowImage(Mystia[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【ミスティア】\nずっとこないものだからこっちから来てあげたわ！さぁ尋常に勝負！");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(Akyu[1], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【阿求】\nふふふ、もう勝ったような顔ですね。私には頼もしい助っ人が居るんですよ？　泣かせますよ？");
				ui.ShowImage(Mystia[2], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【ミスティア】\nげっ、巫女に黒白");
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【霊夢】\nドーモ、チルノ＝サン、ミスティア＝サン。ヨウカイスレイヤーです。ヨウカイ、殺すべし");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【魔理沙】\n殺すな殺すな。まぁ私たちは立会い人みたいなもんだぜ。多少は手伝うが、基本的には阿求一人にやらせる");
				ui.ShowImage(Akyu[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【阿求】\nえっ、聞いてないですよ");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\n言ってないからな");
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【霊夢】\nでも阿求がどれぐらい戦えるのかの確認としては最適そうだから");
				ui.ShowImage(Akyu[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【阿求】\n(な……ハメられたというのか……この私が……)");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(Chilno[2], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【チルノ】\nえーと…なんかよくわかんないけど、紅白も白黒も手伝ったりしないのよね？");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【魔理沙】\nあぁ、三人で仲良く戦争ごっこしててくれ");
				ui.ShowImage(Chilno[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【チルノ】\nそうと決まれば！");
				ui.ShowImage(Mystia[1], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【ミスティア】\n勝負よ！　日頃の恨み晴らさでおくべきか");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				talkEnd = ui.ShowText("", false);

				while (!ui.IsCompleted(talkEnd))
				{
					yield return null;
				}

				ui.Hide();
				ui.ResumeGame();
			}
			#endregion

			world.ChangeBGM(BGMList.Battle_Hitozato);
			for (int i = 0; i < 50; i++)
			{
				yield return null;
			}

			#region 2
			{
				var akyu = GetUnits(world, world.Factions[0], typeof(LotusCraft.Unit.White.Unique.Akyu)).Single();

				ui.PauseGame();
				ui.Show();
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【魔理沙】\nと意気込んでいた割に、いきなり下がっていったな、逃げていったわけじゃなさそうだが");
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【霊夢】\n大方紫の式を忘れてきたんでしょう？");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\n霊夢の言うとおりワリと危ない花火だからなぁ\n紫のやつ、妖精にまで渡すのはちょっとやりすぎじゃないか？");
				ui.ShowImage(Akyu[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【阿求】\nなに、目には目を、こちらも兵隊で対抗してやりましょう");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\nなんだ、結局やる気満々じゃないか");
				ui.ShowImage(Akyu[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【阿求】\n半分やけっぱちです、多少手伝うって言ったからには少しは働いてくださいよ？");
				ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【霊夢】\n手伝うって言ったのは魔理沙だし、私は傍観させてもらうわ");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\nむ…まぁ言い出したのは私だしな、そこはしょうがない。\nそれで、私はなにをすればいいんだ？");
				ui.ShowImage(Akyu[1], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【阿求】\nそうですね、こっちも兵を準備しますからそれまで向こうの様子を見てきてくれますか？");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\nいわゆる斥候ってやつだな、まぁ適任か、まかされよう");
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【霊夢】\n隠密判定は盗賊のスキルじゃなかった？");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\n古いぜ霊夢、時代は戦争だ");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Front, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(ui.LoadImage("LotusCraft/Image/Back/Black"), StoryUIImagePosition.Front, true, false, true);
				ui.ShowText("");
				ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\nよっ、と。ただいま");
				ui.ShowImage(Akyu[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【阿求】\nどうでした？");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\nあいつら里の出口を両方固めてきているな、西と東の両方に陣取ってた");
				ui.ShowImage(Akyu[0], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【阿求】\nこちらに攻めてくる気配は？");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				var talkEnd = ui.ShowText("", false);

				while (!ui.IsCompleted(talkEnd))
					yield return null;

				ui.Hide();
			}
			#endregion

			#region 3
			{
				ui.Hide();
				ui.ResumeGame();

				var chirno = GetUnits(world, world.Factions[2], typeof(LotusCraft.Unit.Blue.Unique.Chirno)).Single();

				world.Factions[0].GetSite(8 * Def.MapTipLengthOnField, chirno.Pos.X, chirno.Pos.Y, int.MaxValue);

				for (int i = 0; i < 30; i++)
					yield return null;

				ui.JumpScreenPosition(chirno.Pos);

				ui.PauseGame();

				for (int i = 0; i < 30; i++)
					yield return null;

				ui.Show();
				ui.ShowImage(Chilno[1], StoryUIImagePosition.Back, true, false);
				ui.ShowText("【チルノ】\n（ふふん、と言いたげなポーズ）");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【魔理沙】\n東は無さそうだぜ、包囲して奥で氷精が腕組んで胸はって満足気にしてやがった");
				ui.ShowImage(-1, StoryUIImagePosition.Back, true);
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				var talkEnd = ui.ShowText("", false);

				while (!ui.IsCompleted(talkEnd))
				{
					yield return null;
				}

				ui.Hide();
			}
			#endregion

			#region 4
			{
				ui.Hide();
				ui.ResumeGame();

				var mystia = GetUnits(world, world.Factions[1], typeof(LotusCraft.Unit.Blue.Unique.Mystia)).Single();
				world.Factions[0].GetSite(7 * Def.MapTipLengthOnField, mystia.Pos.X, mystia.Pos.Y, int.MaxValue);

				for (int i = 0; i < 30; i++)
					yield return null;

				ui.JumpScreenPosition(mystia.Pos);

				ui.PauseGame();

				for (int i = 0; i < 30; i++)
					yield return null;

				ui.Show();
				ui.ShowImage(Mystia[2], StoryUIImagePosition.Back, true, false);
				ui.ShowText("【ミスティア】\n(あたふた)");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【魔理沙】\n夜雀の方も似たような感じだったな、手慣れてなくて戸惑ってる感じだったが");
				ui.ShowImage(-1, StoryUIImagePosition.Back, true);
				ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【霊夢】\n囲んでるつもりかしら？");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\n袋のネズミと思われるのは癪だな");
				ui.ShowImage(Akyu[1], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【阿求】\nせっかく別れてくれてるのです、各個撃破して軍師かぶれの鼻っ面をへし折ってしまいましょう");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				var talkEnd = ui.ShowText("", false);

				while (!ui.IsCompleted(talkEnd))
				{
					yield return null;
				}

				ui.Hide();
			}
			#endregion

			for (int i = 0; i < 50; i++)
			{
				yield return null;
			}

			while (true)
			{
				foreach (var item in 農民生産ボタンを押すまで(world))
					yield return item;

				foreach (var item in 農民生産ボタン押した後(world))
					yield return item;

				foreach (var item in 農民そろえた後(world))
					yield return item;

				foreach (var item in 下級魔法陣を建築した後(world))
					yield return item;

				foreach (var item in 霊力250超えた後(world))
					yield return item;

				foreach (var item in 兵士そろえた後(world))
					yield return item;

				foreach (var item in チルノミスチー殲滅(world))
					yield return item;

				foreach (var item in 勝敗処理(world))
					yield return item;

				yield return null;
			}
		}

		bool 農民生産説明 = false;
		System.Collections.IEnumerable 農民生産ボタンを押すまで(IWorld world)
		{
			if (!農民生産説明)
			{
				var ui = world.StoryUI;

				#region 5
				{
					ui.Hide();
					ui.ResumeGame();

					var mystia = GetUnits(world, world.Factions[0], typeof(LotusCraft.Unit.White.Unique.Akyu)).Single();
					ui.JumpScreenPosition(mystia.Pos);

					ui.PauseGame();
					ui.Show();
					ui.ShowImage(Yukari[0], StoryUIImagePosition.Left, true, false);
					ui.ShowText("【紫】\n今回は本格的なゲーム進行を解説していくわ");
					ui.ShowImage(Yukari[1], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\n前回のユニットはこちらが用意したものだけれど、ここからはプレイヤーの貴方自身が生産の命令していくの");
					ui.ShowImage(Yukari[0], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\nそれと同時に、相手を迎え撃ったり兵を作るための自陣を組み立てる\"内政\"についても紹介していくわね");
					ui.ShowImage(ui.LoadImage("LotusCraft/Story/Story02/story02_homebase"), StoryUIImagePosition.Right, true);
					ui.ShowText("【紫】\nまず目の前に立っているこれが拠点");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowImage(ui.LoadImage("LotusCraft/Story/Story02/story02_mahouzin"), StoryUIImagePosition.Right, true);
					ui.ShowText("【紫】\nそして魔法陣ね");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowText("【紫】\nこれらの建造物からはユニットを生産することができるの");
					ui.ShowImage(Yukari[1], StoryUIImagePosition.Left, false, false);
					ui.ShowImage(ui.LoadImage("LotusCraft/Story/Story02/story02_homebaseUnit"), StoryUIImagePosition.Right, true);
					ui.ShowText("【紫】\n拠点からは霊夢たちヒロインユニットと、様々な建築作業を行うことのできる作業者と呼ばれるユニットを出す事ができるわ");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowImage(ui.LoadImage("LotusCraft/Story/Story02/story02_mahouzinUnit"), StoryUIImagePosition.Right, true);
					ui.ShowText("【紫】\n同じように魔法陣からも兵士を生産することができるわ");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowImage(Yukari[0], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\nさ、とりあえずユニットを増やしてみようかしら？");
					ui.ShowImage(ui.LoadImage("LotusCraft/Story/Story02/story02_farmerButton"), StoryUIImagePosition.Right, true);
					ui.ShowText("【紫】\n拠点を選択して、右下にアイコンが出てくるから対応するアイコンをクリックすれば生産が始まるわ");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowImage(-1, StoryUIImagePosition.Left, true);
					var talkEnd = ui.ShowText("", false);

					while (!ui.IsCompleted(talkEnd))
					{
						yield return null;
					}

					// 農民生産するまで待機

					ui.Hide();
					ui.ResumeGame();

					ui.ShowConditionText("目的：拠点を選択し、作業者を生産する。");

					var building = world.BuildingManager
						.GetBuildings(b => b.Parameter.Faction.Index == 0)
						.Where(b => b.Parameter.GetType() == typeof(LotusCraft.Building.White.HomeBase))
						.FirstOrDefault();

					ui.RegisterHookButtonEvent<IBuilding>(building.Parameter.ButtonParams[0].Current, () => isPushButton = true);
				}
				#endregion

				農民生産説明 = true;
			}
		}

		bool isPushButton = false;
		bool 生産説明 = false;
		System.Collections.IEnumerable 農民生産ボタン押した後(IWorld world)
		{
			if (isPushButton &&
				!生産説明)
			{
				var ui = world.StoryUI;

				#region 6
				{
					ui.ShowConditionText("");
					ui.PauseGame();
					ui.Show();
					ui.ShowImage(Yukari[1], StoryUIImagePosition.Left, true, false);
					ui.ShowImage(ui.LoadImage("LotusCraft/Story/Story02/story02_generatingUnit"), StoryUIImagePosition.Right, true);
					ui.ShowText("【紫】\n生産が始まったわね、生産中は画面下のところにこんなかんじでパーセンテージとともに表示がされるわ、100％になれば出てくるわね。");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowImage(Yukari[0], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\nほかのユニットもこんな感じで生産をしていくことが可能よ");
					ui.ShowImage(Yukari[2], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\nただし、ユニットの生産には\"資源\"を消費するわ");
					ui.ShowImage(Yukari[0], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\n上の瓢箪と陰陽玉の数字は所有している資源の量を表しているわ");
					ui.ShowText("【紫】\n資源は酒と霊力の二種類があるの");
					ui.ShowText("【紫】\n資源を集めるのには作業者が必要だから、まずは作業者が生産されるまで待ちましょう");
					ui.ShowImage(-1, StoryUIImagePosition.Left, true);
					var talkEnd = ui.ShowText("", false);

					while (!ui.IsCompleted(talkEnd))
						yield return null;

					ui.Hide();
					ui.ResumeGame();

					ui.ShowConditionText("目的：作業者を生産する。");
				}
				#endregion

				生産説明 = true;
			}
		}

		bool 魔法陣建築計画説明 = false;
		System.Collections.IEnumerable 農民そろえた後(IWorld world)
		{
			if (!魔法陣建築計画説明 &&
				GetUnits(world, world.Factions[0], typeof(LotusCraft.Unit.White.Farmer)).Any())
			{
				var ui = world.StoryUI;
				{
					ui.PauseGame();
					ui.Show();
					ui.ShowImage(Yukari[0], StoryUIImagePosition.Left, true, false);
					ui.ShowText("【紫】\n作業者が生産できたわね　このユニットは重要な役割のひとつとして、建物の建築を行うことができるわ");
					ui.ShowImage(ui.LoadImage("LotusCraft/Story/Story02/story02_buildButtons"), StoryUIImagePosition.Right, true);
					ui.ShowText("【紫】\n作業者を選択すると、右下のアイコンはこんな感じに表示されるの\nそれぞれのアイコンが建築する建物を表しているわ");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowText("【紫】\nアイコンを押してから、カーソルを建築したい地点に動かしてクリックすると建築を開始できるの");
                    ui.ShowText("【紫】\n作業者が建築を開始すると作業者の周りにエフェクトが出て、しばらく待つと建築が完了するわ");
					ui.ShowText("【紫】\nとりあえず、兵を増やすためにも、下級魔法陣を建築してみましょう。");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowImage(-1, StoryUIImagePosition.Front, true);
					var talkEnd = ui.ShowText("", false);

					while (!ui.IsCompleted(talkEnd))
						yield return null;

					ui.Hide();
					ui.ResumeGame();

					ui.ShowConditionText("目的：作業者で下級魔法陣を建築する。");
				}

				魔法陣建築計画説明 = true;
			}
		}

		bool 資源回収説明 = false;
		System.Collections.IEnumerable 下級魔法陣を建築した後(IWorld world)
		{
			if (!資源回収説明 &&
				world.BuildingManager.GetBuildings(_ => _.Parameter.GetType() == typeof(LotusCraft.Building.White.Barrack) && _.IsComplete).Count() > 2)
			{
				var ui = world.StoryUI;
				ui.ShowConditionText("");

				{
					ui.PauseGame();
					ui.Show();
					ui.ShowImage(Yukari[0], StoryUIImagePosition.Left, true, false);
					ui.ShowText("【紫】\n魔法陣が建築できたわね。でも、上の数値をみればわかるけど資源が減ってしまったわ。");
					ui.ShowText("【紫】\n作業者の重要な役割としてもう一つ、資源回収もあるの。");
					ui.ShowText("【紫】\n資源を集めるにはMAP上の酒樽や霊石に建物を立てると採取が可能よ");
					ui.ShowText("【紫】\n対応する建物は酒回収施設と霊力回収施設ね、アイコンはこれよ。\nこのふたつは酒樽や霊石に重なるようにしないと建築できないので注意が必要ね");
					ui.ShowImage(ui.LoadImage("LotusCraft/Story/Story02/story02_produceBuildings"), StoryUIImagePosition.Right, true);
					ui.ShowText("【紫】\nあと作業者は酒回収施設や霊力回収施設に向かわせると収集速度を加速させてくれるから作業者を量産して貼り付けるといいわ");
					ui.ShowText("【紫】\nそれでは、とりあえず霊力回収施設を建築して霊力を250まで増やしましょう。");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowImage(-1, StoryUIImagePosition.Front, true);
					var talkEnd = ui.ShowText("", false);

					while (!ui.IsCompleted(talkEnd))
						yield return null;

					ui.Hide();
					ui.ResumeGame();

					ui.ShowConditionText("目的：霊力回収施設を霊石の上に建築し、所持霊力250を超える。");
				}

				資源回収説明 = true;
			}
		}

		bool 兵士生産計画説明 = false;
		System.Collections.IEnumerable 霊力250超えた後(IWorld world)
		{
			if (!兵士生産計画説明 &&
				資源回収説明 &&
				world.Factions[0].GetResource(1) >= 250)
			{
				var ui = world.StoryUI;
				ui.ShowConditionText("");

				#region 7
				{
					ui.PauseGame();
					ui.Show();
					ui.ShowImage(Yukari[0], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\n霊力250を超えたわね。しっかり1つの回収施設に複数の作業者を割り振ったかしら");
					ui.ShowText("【紫】\nこの調子で、資源を回収して魔法陣を増やせば一度に生産できる数が増えるし、同時にゲーム中に存在できるユニット数も上昇するわ。");
					ui.ShowText("【紫】\nさぁ、この調子で軍隊と陣地をつくっていきましょう？");
					ui.ShowImage(-1, StoryUIImagePosition.Left, true);
					ui.ShowImage(Akyu[0], StoryUIImagePosition.Front, true, false);
					ui.ShowText("【阿求】\nさて、兵を増やしますか");
					ui.ShowImage(Akyu[1], StoryUIImagePosition.Front, false, false);
					ui.ShowText("【阿求】\nまずは10体ほど揃えましょう、霊石も消費する赤い兵のほうが性能もいいので資源量と相談しつつ出して行きましょうか。");
					ui.ShowImage(-1, StoryUIImagePosition.Front, true);
					var talkEnd = ui.ShowText("", false);

					while (!ui.IsCompleted(talkEnd))
						yield return null;

					ui.Hide();
					ui.ResumeGame();

					ui.ShowConditionText("目的：魔法陣から兵を10体生産する。");

					soldire = world.UnitManager
						.GetUnits(u => u != null && u.Exists())
						.Where(u => u.Faction.Index == 0)
						.Where(u => !u.Parameter.IsFarmer);
					soldireCount = soldire.Count();
				}
				#endregion

				兵士生産計画説明 = true;
			}
		}

		int soldireCount = 0;
		IEnumerable<IUnit> soldire = new IUnit[0];
		bool 攻める説明 = false;
		System.Collections.IEnumerable 兵士そろえた後(IWorld world)
		{
			if (!攻める説明 &&
				soldire.Skip(soldireCount + 9 - 1).Any())
			{
				var ui = world.StoryUI;
				ui.ShowConditionText("");
				ui.Hide();

				#region 8
				{
					ui.PauseGame();
					ui.Show();
					ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, true, false);
					ui.ShowText("【魔理沙】\nひい ふう みい よお いつ むう なあ やあ こお とお");
					ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false);
					ui.ShowText("【霊夢】\nとりあえず10体揃ったわね");
					ui.ShowImage(Akyu[1], StoryUIImagePosition.Right, true, false);
					ui.ShowText("【阿求】\nでは夜雀狩りといきましょう？");
					ui.ShowImage(Marisa[2], StoryUIImagePosition.Left, true, false);
					ui.ShowText("【魔理沙】\n本陣が空になるけどいいのか？");
					ui.ShowImage(Akyu[0], StoryUIImagePosition.Right, false, false);
					ui.ShowText("【阿求】\nそうですね、資源採取はずっと行なってますし、生産も自動化させておきましょう");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowImage(-1, StoryUIImagePosition.Left, true);
					ui.ShowImage(ui.LoadImage("LotusCraft/Story/Story02/story02_autoCreate"), StoryUIImagePosition.Right, true);
					ui.ShowImage(Yukari[0], StoryUIImagePosition.Left, true, false);
					ui.ShowText("【紫】\n生産アイコンは右クリックをすることで\"AUTO\"が表示されるわ");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowImage(Yukari[1], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\nこの状態だと勝手に生産をし続けてくれるの、便利だから有効活用するといいわよ？");
					ui.ShowImage(Yukari[2], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\nえ？急に資源が必要になった時とかは？");
					ui.ShowImage(Yukari[0], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\nそのときはもう一度右クリックをすれば自動化は止めれるし、\n予約されたユニットのアイコンは左クリックでキャンセルできるから大丈夫よ");
					ui.ShowImage(Yukari[1], StoryUIImagePosition.Left, false, false);
					ui.ShowText("【紫】\nでは、続きをがんばってね？");
					ui.ShowImage(-1, StoryUIImagePosition.Left, true);
					var talkEnd = ui.ShowText("", false);

					while (!ui.IsCompleted(talkEnd))
						yield return null;

					ui.Hide();
					ui.ResumeGame();

					攻める説明 = true;
				}
				#endregion

				ui.ShowConditionText("目的：戦闘準備を整え、敵を殲滅する");
			}
		}

		bool mystiaIsLooser = false;
		bool seeMystia = false;
		bool chirnoIsLooser = false;
		bool seeChirno = false;

		System.Collections.IEnumerable チルノミスチー殲滅(IWorld world)
		{
			var ui = world.StoryUI;

			if (!mystiaIsLooser &&
				!seeMystia &&
				world.UnitManager.GetUnits(u => u.Faction.Index == 0).Any(u => (world.Map.Count - 17) * Def.MapTipLengthOnField < u.Pos.X))  //CanSee(world, world.Factions[0], mystia))
			{
				#region 9
				{
					ui.PauseGame();
					ui.Show();
					ui.ShowImage(Mystia[0], StoryUIImagePosition.Right, true, false);
					ui.ShowText("【ミスティア】\n囲んで一気に袋叩きって作戦だけど…");
					ui.ShowImage(Mystia[2], StoryUIImagePosition.Right, false, false);
					ui.ShowText("【ミスティア】\nいつ攻めればいいのか決めてなかったわ");
					ui.ShowImage(Marisa[0], StoryUIImagePosition.Left, true, false);
					ui.ShowText("【魔理沙】\n簡単だ、今から戦えばいい");
					ui.ShowImage(Mystia[2], StoryUIImagePosition.Right, false, false);
					ui.ShowText("【ミスティア】\nえ？あ、えぇ？な、なんでここにいるの！？");
					ui.ShowImage(Akyu[1], StoryUIImagePosition.Left, true, false);
					ui.ShowText("【阿求】\n囲まれて縮こまってるだけと考えたのがそちらの敗因ですよ？");
					ui.ShowImage(-1, StoryUIImagePosition.Left, true);
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					var talkEnd = ui.ShowText("", false);

					while (!ui.IsCompleted(talkEnd))
						yield return null;

					ui.Hide();
					ui.ResumeGame();
				}
				#endregion

				seeMystia = true;
			}

			else if (!mystiaIsLooser && world.UnitManager.GetUnits(u => u.Faction.Index == 1).IsEmpty())
			{
				#region 10
				{
					ui.PauseGame();
					ui.Show();
					ui.ShowImage(Mystia[2], StoryUIImagePosition.Right, true, false);
					ui.ShowText("【ミスティア】\nバタンキュ～");
					ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false);
					ui.ShowText("【霊夢】\nあっさり片付いたわね");
					ui.ShowImage(Akyu[1], StoryUIImagePosition.Right, true, false);
					ui.ShowText("【阿求】\nええ、では本陣で追加生産した兵と合わせて氷精狩りに行くとしましょう");
					ui.ShowImage(-1, StoryUIImagePosition.Right, true);
					ui.ShowImage(-1, StoryUIImagePosition.Left, true);
					var talkEnd = ui.ShowText("", false);

					while (!ui.IsCompleted(talkEnd))
						yield return null;

					ui.Hide();
					ui.ResumeGame();
				}
				#endregion

				if (!chirnoIsLooser)
					for (int i = 0; i < 50; i++)
						yield return null;

				mystiaIsLooser = true;
			}
			else if (!chirnoIsLooser &&
				!seeChirno &&
				world.UnitManager.GetUnits(u => u.Faction.Index == 0).Any(u => u.Pos.X < 17 * Def.MapTipLengthOnField)) // CanSee(world, world.Factions[0], chirno))
			{
				if (mystiaIsLooser)
				{
					#region 11
					{
						ui.PauseGame();
						ui.Show();
						ui.ShowImage(Chilno[1], StoryUIImagePosition.Right, true, false);
						ui.ShowText("【チルノ】\nむ！来たな！これから攻めようとしてたけれどちょーどいいわ！");
						ui.ShowText("【チルノ】\n知らないだろうから地獄への手土産に教えてあげるわ！お前たちは今包囲されている！おとなしく降参――");
						ui.ShowImage(Marisa[0], StoryUIImagePosition.Left, true, false);
						ui.ShowText("【魔理沙】\n夜雀なら来ないぜ？");
						ui.ShowImage(Chilno[2], StoryUIImagePosition.Right, false, false);
						ui.ShowText("【チルノ】\nほえ？");
						ui.ShowImage(-1, StoryUIImagePosition.Left, true);
						ui.ShowImage(-1, StoryUIImagePosition.Right, true);
						ui.ShowImage(Akyu[0], StoryUIImagePosition.Front, true, false);
						ui.ShowText("【阿求】\n貴女がのんびりしている間に倒させていただきました");
						ui.ShowImage(Akyu[1], StoryUIImagePosition.Front, false, false);
						ui.ShowText("【阿求】\nさぁ、逆に追い詰めさせていただきましたよ、尋常に戦争ごっこといきましょうか？");
						ui.ShowImage(-1, StoryUIImagePosition.Front, true);
						ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, true, false);
						ui.ShowText("【霊夢】\n追い詰めて降伏勧告と思いきや");
						ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, true, false);
						ui.ShowText("【魔理沙】\nまさかの殲滅宣言ときたものだ");
						ui.ShowImage(-1, StoryUIImagePosition.Left, true);
						ui.ShowImage(-1, StoryUIImagePosition.Right, true);
						ui.ShowImage(Chilno[1], StoryUIImagePosition.Front, true, false);
						ui.ShowText("【チルノ】\nま、まだよ！アタイの弾幕で全部凍らせてやるんだから！");
						ui.ShowImage(-1, StoryUIImagePosition.Front, true);
						ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
						ui.ShowText("【魔理沙】\n一騎当千を宣言してるな");
						ui.ShowImage(Marisa[2], StoryUIImagePosition.Right, false, false);
						ui.ShowText("【魔理沙】\n妖精にしてはワリと強いからな\n少なくとも阿求より強い、気をつけてかかれよ？");
						ui.ShowImage(Akyu[2], StoryUIImagePosition.Left, true, false);
						ui.ShowText("【阿求】\nなら何度も言いますけど手伝ってくださってもいいじゃないですか");
						ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
						ui.ShowText("【魔理沙】\n忠告はしっかりした手伝いさ、それきたぞ");
						ui.ShowImage(-1, StoryUIImagePosition.Right, true);
						ui.ShowImage(-1, StoryUIImagePosition.Left, true);
						var talkEnd = ui.ShowText("", false);

						while (!ui.IsCompleted(talkEnd))
							yield return null;

						ui.Hide();
						ui.ResumeGame();
					}
					#endregion
				}
				else
				{
					#region 13
					{
						ui.PauseGame();
						ui.Show();
						ui.ShowImage(Chilno[1], StoryUIImagePosition.Right, true, false);
						ui.ShowText("【チルノ】\nむ！来たな！これから攻めようとしてたけれどちょーどいいわ！");
						ui.ShowText("【チルノ】\n知らないだろうから地獄への手土産に教えてあげるわ！お前たちは今包囲されている！おとなしく降参――");
						ui.ShowImage(Marisa[0], StoryUIImagePosition.Left, true, false);
						ui.ShowText("【魔理沙】\nなぁ、ちょっと聞きたいことあるんだが");
						ui.ShowText("【チルノ】\nいいわよ、強者は寛大であるべきって誰かが言ってた！");
						ui.ShowText("【魔理沙】\n包囲してても今の状況一対一だぞ？同時に戦わなきゃ意味ないだろコレ");
						ui.ShowImage(Chilno[2], StoryUIImagePosition.Right, false, false);
						ui.ShowText("【チルノ】\n……？");
						ui.ShowImage(Marisa[2], StoryUIImagePosition.Left, false, false);
						ui.ShowText("【魔理沙】\nいや、聞いた私が悪かった、打ち合わせとかしてないんだろうなコレ");
						ui.ShowImage(-1, StoryUIImagePosition.Left, true);
						ui.ShowImage(-1, StoryUIImagePosition.Right, true);
						ui.ShowImage(Akyu[0], StoryUIImagePosition.Front, true, false);
						ui.ShowText("【阿求】\n簡単に言えば貴女の作戦は失敗したんですよ");
						ui.ShowImage(Akyu[1], StoryUIImagePosition.Front, false, false);
						ui.ShowText("【阿求】\nさぁ、逆に追い詰めさせていただきましたよ、尋常に戦争ごっこといきましょうか？");
						ui.ShowImage(-1, StoryUIImagePosition.Front, true);
						ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, true, false);
						ui.ShowText("【霊夢】\n追い詰めて降伏勧告と思いきや");
						ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, true, false);
						ui.ShowText("【魔理沙】\nまさかの殲滅宣言ときたものだ");
						ui.ShowImage(-1, StoryUIImagePosition.Left, true);
						ui.ShowImage(-1, StoryUIImagePosition.Right, true);
						ui.ShowImage(Chilno[1], StoryUIImagePosition.Front, true, false);
						ui.ShowText("【チルノ】\nま、まだよ！アタイの弾幕で全部凍らせてやるんだから！");
						ui.ShowImage(-1, StoryUIImagePosition.Front, true);
						ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
						ui.ShowText("【魔理沙】\n一騎当千を宣言してるな");
						ui.ShowImage(Marisa[2], StoryUIImagePosition.Right, false, false);
						ui.ShowText("【魔理沙】\n妖精にしてはワリと強いからな、少なくとも阿求より強い、気をつけてかかれよ？");
						ui.ShowImage(Akyu[2], StoryUIImagePosition.Left, true, false);
						ui.ShowText("【阿求】\nなら何度も言いますけど手伝ってくださってもいいじゃないですか");
						ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, false, false);
						ui.ShowText("【魔理沙】\n忠告はしっかりした手伝いさ、それきたぞ");
						ui.ShowImage(-1, StoryUIImagePosition.Right, true);
						ui.ShowImage(-1, StoryUIImagePosition.Left, true);
						var talkEnd = ui.ShowText("", false);

						while (!ui.IsCompleted(talkEnd))
							yield return null;

						ui.Hide();
						ui.ResumeGame();
					}
					#endregion
				}
				seeChirno = true;
			}
			else if (!chirnoIsLooser && world.UnitManager.GetUnits(u => u.Faction.Index == 2).IsEmpty())
			{
				if (mystiaIsLooser)
					for (int i = 0; i < 50; i++)
						yield return null;

				chirnoIsLooser = true;
			}
			else if (mystiaIsLooser && chirnoIsLooser)
			{
			}
		}

		System.Collections.IEnumerable 勝敗処理(IWorld world)
		{
			if (world.UnitManager.GetUnits(u => u.Faction.Index != 0).IsEmpty())
			{
				foreach (var item in Talk勝利(world))
					yield return item;

				world.Factions.Skip(1).ForEach(f =>
				{
					f.DoesNotLose = true;
					f.Lose();
				}
				);

				world.StoryUtilities.SetDataToSaveData("LotusCraft.Prologue.02", "Clear");

				if (world.StoryUtilities.GetDifficulty() == 0)
					world.StoryUtilities.SetDataToSaveData("LotusCraft.Prologue.02.Easy", "Clear");
				else if (world.StoryUtilities.GetDifficulty() == 1)
					world.StoryUtilities.SetDataToSaveData("LotusCraft.Prologue.02.Normal", "Clear");
				else if (world.StoryUtilities.GetDifficulty() == 2)
					world.StoryUtilities.SetDataToSaveData("LotusCraft.Prologue.02.Hard", "Clear");
				
				world.ChangeBGM(BGMList.Boushuen_Win);
			}
			else if (world.UnitManager.GetUnits(u => u.Faction.Index == 0).IsEmpty())
			{
				world.Factions[0].DoesNotLose = true;
				world.Factions[0].Lose();

				world.ChangeBGM(BGMList.Boushuen_Lose);
			}
		}

		bool CanSee(IWorld world, IFaction faction, IUnit unit)
		{
			var tip = world.Map.GetMapTipIndex(unit.Pos.X, unit.Pos.Y);

			return world.Map.BeSeen(faction, tip.X, tip.Y);
		}

		private System.Collections.IEnumerable Talk勝利(IWorld world)
		{
			var ui = world.StoryUI;
			#region 12
			{
				ui.PauseGame();
				ui.Show();
				ui.ShowImage(Akyu[1], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【阿求】\nふっ、口ほどにもない");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【魔理沙】\nどうだか");
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【霊夢】\nまぁなんとか及第点といったところね。で、このボロ雑巾どもはほっといて私たちは命蓮寺に行くけど");
				ui.ShowImage(Akyu[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【阿求】\nムムム、残念ですが私はまた後で合流ということで良いですか。今日の大勝利を書にしたためたいので");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【魔理沙】\nご苦労なこったな。そしたらまたお前んちに迎えに行けばいいか？");
				ui.ShowImage(Akyu[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【阿求】\nそうしてください。できれば迎えに来ないほうがいいですが");
				ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【霊夢】\n人が足りたら呼ばないわよ");
				ui.ShowText("【阿求】\nええ、そうなると嬉しいですね。それでは私はこれにて失礼");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowText("【霊夢】\nしかし紫の奴、どんだけ暇なんだか");
				ui.ShowImage(Marisa[0], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【魔理沙】\nこれはスカウトのほうも急がないとな");
				ui.ShowImage(Chilno[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【チルノ】\nあたいたち、紫とかいうのと会ってないよ？");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【魔理沙】\nむ？");
				ui.ShowImage(Mystia[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【ミスティア】\nそうそう、青い髪のなんか偉そうな奴が、これでちょっかい出して来いって");
				ui.ShowImage(Reimu[2], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【霊夢】\n……まーためんどくさい奴が参戦したわね");
				ui.ShowImage(Marisa[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【魔理沙】\n一体どこまで巻き込むつもりだ、あいつ");
				var talkEnd = ui.ShowText("", false);

				while (!ui.IsCompleted(talkEnd))
				{
					yield return null;
				}

				ui.Hide();
				ui.ResumeGame();
			}
			#endregion

			yield return null;
		}

	}
}
