﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft.Story
{
	public class Story04 : thrts.Mission
	{
		#region State

		abstract class StoryState
		{
			#region フィールド

			protected Story04 Owner { get; set; }
			protected IWorld World { get; set; }
			protected IStoryUI ui { get; set; }

			protected int[] Reimu
			{
				get { return Owner.Reimu; }
			}
			protected int[] Marisa
			{
				get { return Owner.Marisa; }
			}
			protected int[] Akyu
			{
				get { return Owner.Akyu; }
			}
			protected int[] Eirin
			{
				get { return Owner.Eirin; }
			}
			protected int[] Ichirin
			{
				get { return Owner.Ichirin; }
			}
			protected int[] Hijiri
			{
				get { return Owner.Hijiri; }
			}
			protected int[] Chilno
			{
				get { return Owner.Chilno; }
			}
			protected int[] Nitori
			{
				get { return Owner.Nitori; }
			}
			protected int[] Aya
			{
				get { return Owner.Aya; }
			}
			protected int Back1
			{
				get { return Owner.Back1; }
			}

			#endregion

			protected IEnumerator talkFiber;
			protected IEnumerator tacticsFiber;

			public StoryState( Story04 owner, IWorld world )
			{
				this.Owner = owner;
				this.World = world;
				this.ui = world.StoryUI;

				talkFiber = Talk().GetEnumerator();
				tacticsFiber = Tactics().GetEnumerator();
			}

			public virtual void Update()
			{
				if( talkFiber != null )
				{
					if( !talkFiber.MoveNext() )
						talkFiber = null;
				}
				else
				{
					if( tacticsFiber != null )
						if( !tacticsFiber.MoveNext() )
							tacticsFiber = null;
				}
			}
			protected abstract IEnumerable Talk();
			protected abstract IEnumerable Tactics();

			#region Utility
			protected IEnumerable<IUnit> PutEnemy<TUnit>( Area area, int num ) where TUnit : UnitParameter
			{
				var rect = area as AreaRectangle;
				if( rect == null )
					return new IUnit[0];

				List<IUnit> units = new List<IUnit>();
				var type = typeof( TUnit );

				var sx = Math.Min( rect.Start.X, rect.End.X );
				var sy = Math.Min( rect.Start.Y, rect.End.Y );
				var ex = Math.Max( rect.Start.X, rect.End.X );
				var ey = Math.Max( rect.Start.Y, rect.End.Y );

				for( var y = sy; y < ey; y++ )
				{
					for( var x = sx; x < ex; x++ )
					{
						if( units.Count >= num )
							goto LoopEnd;

						var u = World.UnitManager.Create(
							type.FullName,
							World.Factions[1],
							( x + 0.5f ) * thrts.Def.MapTipLengthOnField,
							( y + 0.5f ) * thrts.Def.MapTipLengthOnField,
							World.Factions[1] );
						if( u != null )
							units.Add( u );
					}
				}
			LoopEnd:

				return units;
			}

			protected bool IsUnitOnArea( AreaRectangle region, IUnit unit )
			{
				var startx = Math.Min( region.Start.X, region.End.X ) * Def.MapTipLengthOnField;
				var starty = Math.Min( region.Start.Y, region.End.Y ) * Def.MapTipLengthOnField;
				var endx = Math.Max( region.Start.X, region.End.X ) * Def.MapTipLengthOnField;
				var endy = Math.Max( region.Start.Y, region.End.Y ) * Def.MapTipLengthOnField;
				if( unit.Pos.X < endx && startx < unit.Pos.X && unit.Pos.Y < endy && starty < unit.Pos.Y )
				{
					return true;
				}
				return false;
			}

			protected IEnumerable<IBuilding> GetBuildings<TBuilding>() where TBuilding : BuildingParameter
			{
				return World.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( TBuilding ) );
			}

			protected Vector2D To2D( Vector3D pos )
			{
				return new Vector2D( pos.X, pos.Y );
			}

			protected IEnumerable<IUnit>[] CallLunars( int left, int right )
			{
				var units1 = PutEnemy<Unit.Blue.Lunar>( Owner.areas["召喚位置１"], left );
				var units2 = PutEnemy<Unit.Blue.Lunar>( Owner.areas["召喚位置２"], right );
				return new[] { units1, units2 };
			}

			protected IEnumerable CallFiber( int timeSpan, Vector2D target1, Vector2D target2 )
			{
				var behavior = new Action<IEnumerable<IUnit>, Vector2D>[]{
					( u, p ) => World.UnitManager.SelectTarget( u, p, false ),
					( u, p ) => World.UnitManager.AttackMove( u.Select( _ => _.UniqueID ), p, false ),
				};

				for( int i = 0; ; i++ )
				{
					if( i % timeSpan == 0 )
					{
						int behaviorIndex = World.NextRand( 0, 2 );

						if( i % ( timeSpan * 2 ) == 0 )
						{
							var units = CallLunars( 0, LunarAmount );
							behavior[behaviorIndex]( units[1], target1 );
						}
						else
						{
							var units = CallLunars( LunarAmount, 0 );
							behavior[behaviorIndex]( units[0], target2 );
						}
					}
					yield return null;
				}
			}
			#endregion
		}

		class IntroductionState : StoryState
		{
			public IntroductionState( Story04 owner, IWorld world )
				: base( owner, world )
			{
			}

			protected override IEnumerable Talk()
			{
				World.ChangeBGM( BGMList.Boushuen_Story1 );
				for( int i = 0; i < 20; i++ )
				{
					yield return null;
				}

				#region Talk1
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\n難題の一つもなくあっさりと承諾されてしまった私たちは" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\n楽でいいじゃないか" );
				ui.ShowImage( Eirin[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【永琳】\n姫の気まぐれには困ったものでして……" );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\n手っ取り早くていいけどね。\nじゃあお茶でもしばらくたかってようかしら。" );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【霊夢】\nそれにしてもお手伝いさんも居て楽ちんね。\nずっと住もうかしら。稗田家に" );
				ui.ShowImage( Akyu[0], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【阿求】\n白蓮さんたちが来たら出発しましょう\n目的地はどこか知らないですが" );
				ui.ShowText( "【霊夢】\n正直戦力的には申し分ないのよね\nどうせアリスのことだし、今頃挑発されていいようにこき使われてるんじゃないかしら" );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【魔理沙】\nありえるな。アリスは案外短気だから" );
				ui.ShowImage( Akyu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【阿求】\n人外バトルになぜ私が参戦しなければいけないのか" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【魔理沙】\nまだ言ってるのか？" );
				ui.ShowImage( Eirin[0], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【永琳】\n大丈夫よ。粉々にならなければ私が直してあげるから" );
				ui.ShowText( "【阿求】\nせめて治してでお願いします" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Hijiri[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【白蓮】\nお待たせしてしまったようで" );
				ui.ShowImage( Ichirin[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【一輪】\n村紗がついて来たがって。言って聞かせるのが難儀で" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【霊夢】\nさて、これで六人揃ったわけね" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【魔理沙】\nああ。ちゃっちゃと全員はっ倒して、マヨヒガの宝物でも頂くとしようか" );
				ui.ShowImage( Hijiri[0], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【白蓮】\n自称聖人に仏の教えを説かなくては" );
				ui.ShowImage( Akyu[2], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【阿求】\n生き残りたい" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\n約一名物凄い悲痛なことを言っているけど" );
				ui.ShowImage( Ichirin[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【一輪】\nとにかく、現在の目標がいまいち定まっていないことには行動のしようがないのでは" );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【霊夢】\nそうねえ\n確実に誰かと会えそうな紅魔館にでも乗り込むか、一度神社に戻って作戦会議をするか" );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【霊夢】\nここでお茶をたかるか" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【阿求】\n神社は少し、他の場所から遠すぎますし、\n我々の拠点を人里に設定するのは悪くないと思いますが" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\n神社に居れば、寝てても全員集まってきそうだがな" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【霊夢】\nできるだけ少ない労力で皆殺しにしたいわね" );
				ui.ShowImage( Eirin[1], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【永琳】\nじゃあ毒を散布しましょうか" );
				ui.ShowImage( Hijiri[1], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【白蓮】\nもっと直接的な方法で……" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\n稗田家がテロリストの会合になってきたな。いいことだ" );
				ui.ShowImage( Akyu[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【阿求】\n良くないですよ！　もう少し真っ当で建設的な会話をですね" );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\nだってわざわざ戦いたくないしー" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\n私は茶が怖いぜ" );
				ui.ShowImage( Akyu[0], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【阿求】\nあ、そうです、手分けして偵察をしてくるというのはどうでしょう。名案です" );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\nはんたーい。私はここで寝てたい" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\n私も反対だぜ。ここらで一回昼寝するというのはどうだ" );
				ui.ShowImage( Hijiri[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【白蓮】\n確かに今日のところは一度休憩というのも" );
				ui.ShowImage( Eirin[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【永琳】\nそれなら私は一度、永遠亭に帰ろうかしら。食事の支度もしないといけなくって" );
				ui.ShowImage( Ichirin[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【一輪】\nせっかくなので酒盛りでも" );
				ui.ShowImage( Hijiri[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【白蓮】\n一輪？" );
				ui.ShowImage( Ichirin[2], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【一輪】\nはい" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Back1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "～次の日～" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\n今日も一日だらけましょう" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【魔理沙】\nそうしようそうしよう" );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, true );
				ui.ShowImage( Hijiri[2], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【白蓮】\nおはようございます。\n私はこれから寺のほうでやらなければいけないことがあるので" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\n永琳も忙しそうだし今日も何もなしね" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【魔理沙】\n一旦家に戻るかなー。たかるのにも飽きたしな" );
				ui.ShowImage( Akyu[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【阿求】\nはい" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Chilno[0], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【チルノ】\nたのもう！" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Chilno[0], StoryUIImagePosition.Right, true, false, true );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\nチルノだわ" );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\nチルノだな" );
				ui.ShowImage( Chilno[0], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【チルノ】\nリベンジにきたのよ！\n負けっぱなしだなんてあたいのプライドが許しても、お天道様が許さない！" );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【魔理沙】\nプライドのほうは許してるのか" );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\nお天道様のほうが上位なのがわかったわ" );
				ui.ShowImage( Akyu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【阿求】\nリベンジって、また同じような結果になると思いますが" );
				ui.ShowImage( Chilno[1], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【チルノ】\nふふふ。今回は頼もしい味方が居るのよ！いでよ！" );
				ui.ShowImage( Chilno[0], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【チルノ】\n河童のにとり！" );
				ui.ShowImage( Nitori[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【にとり】\nなんで私が妖精の手伝いなんてしてるんだろう……" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\nお前も参加してるのか？" );
				ui.ShowImage( Nitori[1], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【にとり】\n研究資金をくれるって言うから" );
				ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\n妖精が？何かの間違いでしょ、賽銭ですら葉っぱと小石いれる奴らよ？" );
				ui.ShowImage( Nitori[0], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【にとり】\nあぁ、もちろん雇い主は別にいて――――" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Chilno[1], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【チルノ】\nどうでもいいから勝負よ！宣戦布告したんだから馴れ合いしないで戻る！" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Nitori[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【にとり】\nやっぱり妖精にこき使われるのは釈然としない" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\nしかたないな、給料は我慢料というぜ" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				var talkEnd = ui.ShowText( "", false );
				#endregion

				while( !ui.IsCompleted( talkEnd ) )
					yield return null;

				ui.Hide();
				ui.ResumeGame();

				yield return null;
			}

			protected override IEnumerable Tactics()
			{
				Owner.state = new PreparingState( Owner, World );
				yield return null;
			}
		}

		class PreparingState : StoryState
		{
			readonly int firstAttackTime = (int)( Def.FPS * 60 * 3.5f );
			readonly int rushStartTime = Def.FPS * 60 * 5;

			public PreparingState( Story04 owner, IWorld world )
				: base( owner, world )
			{
			}

			protected override IEnumerable Talk()
			{
				World.ChangeBGM( BGMList.Battle_Hitozato );
				World.Factions[0].GetSite( 500, Owner.BlueBase.Pos.X, Owner.BlueBase.Pos.Y, 30 );
				for( int i = 0; i < 15; i++ )
				{
					yield return null;
				}
				World.Factions[0].MoveScreenPosition( To2D( Owner.BlueBase.Pos ) );

				#region Talk2,3
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【魔理沙】\nで、あのへんがあいつらの陣地か" );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【霊夢】\nまた無駄に高いところに陣取ってるわね" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【魔理沙】\n何かと煙は高いところが好きって言うのは本当なんだな" );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【魔理沙】\nしかし、あいつらあれでどうやって攻めてくるつもりなんだ？" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				var talkEnd0 = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd0 ) )
					yield return null;
				ui.Hide();
				ui.ResumeGame();

				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Chilno[1], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【チルノ】\nふふん、あたいには最強の飛行部隊がいるのよ！\nこいつで巫女も白黒もついでにもう一匹も全部焼け野原にしてやるわ！" );
				ui.ShowImage( Chilno[0], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【チルノ】\n整備士にとり！急いで準備を始めなさい！あたいはそれを眺めてるから！" );
				ui.ShowImage( Nitori[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【にとり】\nはいはい、もーちょっと時間頂戴、数がないと効果薄いからねー" );
				ui.ShowImage( Nitori[0], StoryUIImagePosition.Left, false, false );
				ui.ShowText( "【にとり】\n（まぁ、私自身が戦うわけじゃないし、本番前に式神の性能試すいい機会と見ればいいものかな）" );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, true );
				var talkEnd1 = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd1 ) )
					yield return null;
				ui.Hide();
				ui.ResumeGame();
				#endregion

				World.Factions[0].MoveScreenPosition( To2D( Owner.WhiteBase.Pos ) );

				#region Talk4
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【阿求】\n歩兵では登れそうにありませんし、使うとしたら飛べる兵を使ってくるでしょう" );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【霊夢】\nそういえばこの戦争ごっこ、飛ぶのはありなの？" );
				ui.ShowImage( Akyu[0], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【阿求】\n基本的に私たちは禁止です、一部飛ぶことがアイデンティティな人は許可されてるそうで" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【魔理沙】\n私たちは飛べない部類だな" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【阿求】\nとりあえず相手が飛んできそうですし兎(水)や置き薬の…ええと、鈴仙さんの式神を貯めておきましょうか" );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【霊夢】\nなんで？" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【阿求】\nあの兵は対空攻撃できるんですよ、撃ち落としてしまいましょう" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Front, true, false );
				ui.ShowText( "【阿求】\n対地/対空の確認はステータス画面や生産アイコンのマウスオーバーでも確認できますよ、\nあらやだメタ発言ですねこれは" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				var talkEnd2 = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd2 ) )
					yield return null;
				ui.Hide();
				ui.ResumeGame();
				#endregion

				World.Factions[0].MoveScreenPosition( Owner.BunkersCenter );

				#region Talk5
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【霊夢】\nなんか見慣れない建物あるけど、あれは？" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【阿求】\n掩体壕（えんたいごう）です、英語で言えばバンカー\n下級兵を入れておけば相手の攻撃からユニットを守りつつ攻撃してくれますよ" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【魔理沙】\n中から撃って反撃するならトーチカって言わないか？" );
				ui.ShowImage( Akyu[2], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【阿求】\n特火点ですか？あれは知名度低いのでボツです" );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【霊夢】\n名前なんてどうでもいいわ、せっかくあるんだし埃かぶらせるくらいなら使っておきましょう" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				var talkEnd3 = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd3 ) )
					yield return null;
				ui.Hide();
				ui.ResumeGame();
				#endregion

				World.Factions[0].MoveScreenPosition( To2D( Owner.WhiteBase.Pos ) );

				ui.ShowConditionText( "目的：チルノの攻撃に備える。" );

				yield return null;
			}

			protected override IEnumerable Tactics()
			{
				List<IUnit> units = new List<IUnit>();

				for( int i = 0; i < rushStartTime; i++ )
				{
					if( i % LunarSpanTime == 0 )
					{
						units.AddRange( PutEnemy<Unit.Blue.Lunar>( Owner.areas["召喚位置１"], LunarAmount / 2 ) );
						units.AddRange( PutEnemy<Unit.Blue.Lunar>( Owner.areas["召喚位置２"], LunarAmount / 2 ) );

						if( i >= firstAttackTime )
						{
							var rand = World.NextRand( 0, units.Count );
							World.UnitManager.AttackMove( units[rand].UniqueID.ToSequence(), To2D( Owner.WhiteBase.Pos ), true );
						}
					}
					yield return null;
				}

				Owner.state = new FightingState( Owner, World, units.Where( _ => _.Exists() ) );
				yield return null;
			}
		}

		class FightingState : StoryState
		{
			IEnumerable<IUnit> units { get; set; }

			public FightingState( Story04 owner, IWorld world, IEnumerable<IUnit> units )
				: base( owner, world )
			{
				this.units = units;
			}
			protected override IEnumerable Talk()
			{
				#region Talk6
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Nitori[1], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【にとり】\n全機上がったよー" );
				ui.ShowImage( Chilno[1], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【チルノ】\nよーし出撃、恐怖の爆撃であたいの凄さを知らしめてくるのよ！" );
				ui.ShowImage( Nitori[0], StoryUIImagePosition.Left, false, false );
				ui.ShowText( "【にとり】\n総員、帽振れーーーっ！！生きて帰ってこいよー！" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				var talkEnd0 = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd0 ) )
					yield return null;
				ui.Hide();
				ui.ResumeGame();
				#endregion

				ui.ShowConditionText( "目的：敵を殲滅する。" );
			}
			protected override IEnumerable Tactics()
			{
				CallLunars( LunarAmount * 2, LunarAmount * 2 );

				var u = new[]{
					units.Take( units.Count()/2 ),
					units.Skip( units.Count()/2 )
				};
				World.UnitManager.SelectTarget( u[0], To2D( Owner.WhiteBase.Pos ), false );
				World.UnitManager.AttackMove( u[1].Select( _ => _.UniqueID ), To2D( Owner.WhiteBase.Pos ), false );

				while( units.Any( _ => _.Exists() ) )
					yield return null;

				foreach( var item in Talk第一波全滅() )
					yield return null;

				if( Owner.SupplyBase.Exists() )
				{
					Owner.state = new LookingForState( Owner, World );
				}
				else
				{
					Owner.state = new SupplyBaseBrokenState( Owner, World );
				}

				yield return null;
			}

			IEnumerable Talk第一波全滅()
			{
				#region Talk7
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Nitori[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【にとり】\nあー、やっぱり対空砲火のどまんなかに式神妖精じゃ無理があったかぁ…" );
				ui.ShowText( "【にとり】\n夜雀でもいれば夜間爆撃になって効果あったかもしれないけど、まぁ言ってもしょうがないかね" );
				ui.ShowImage( Chilno[2], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【チルノ】\nむ、むむむ…にとり！次の攻撃準備よ！どうせこっちにあいつら攻めてこれないんだから！" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				var talkEnd1 = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd1 ) )
					yield return null;
				ui.Hide();
				ui.ResumeGame();
				#endregion

				#region Talk8
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\n発 魔理沙索敵部隊　宛 阿求司令本部 一○○八　テキ　バクゲキキ　ゼンメツ　ヲ　カクニン" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【阿求】\nまぁ制圧砲撃も地上戦力もなければこうなりますよね" );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\nそろそろとっちめる？" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【阿求】\nそうですね、それでは今度はこちらから乗り込んでさっさと終わらせましょう" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\n目には目を、飛んでる奴には飛んでるやつか？" );
				ui.ShowImage( Akyu[0], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【阿求】\nええ、農民に未知魔法陣を作らせましょう、" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【阿求】\nこの魔法陣からは空を飛べるUFOユニットが出せますからそれで戦うのもいいですし\n緑UFOは実はユニットを運べるんですよ、これをいくつか作って一気に乗り込んでしまいましょう" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【阿求】\n人口10の枠までしか乗り込めないのに注意してくださいね、何機か作ってしまいましょう" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				var talkEnd2 = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd2 ) )
					yield return null;
				ui.Hide();
				ui.ResumeGame();
				#endregion
			}
		}

		class LookingForState : StoryState
		{
			public LookingForState( Story04 owner, IWorld world )
				: base( owner, world )
			{
			}

			protected override IEnumerable Talk()
			{
				#region Talk9
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【魔理沙】\nそういえば、この酒とか霊石って尽きるんだよな？" );
				ui.ShowImage( Akyu[0], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【阿求】\nそうですねいずれは尽きるものですが、それがどうかしました？" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【魔理沙】\nいやな、あいつらあんな所に引きこもってるわけだ、こっちも迎撃できてるだろ？\nこのまま適当に式神撃ち落としてたら先にあっちが音を上げるんじゃないか？" );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【霊夢】\n面倒そうなので却下、無駄に長引いても面倒な事が起きるだけよそんなの" );
				ui.ShowImage( Akyu[2], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【阿求】\nまぁ、確かにそうですね、それに何処か別のところに補給拠点があるかもしれないですし…\n魔理沙さん、また斥候して適当に散歩してきますか？" );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【魔理沙】\nおいおい、それこそ式神にやらせとけよ" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, false, false );
				ui.ShowText( "【魔理沙】\n式神は小間使いで、指揮官は一日8時間昼寝できるのが正しいスタイルなんだぞ、紫が言ってた" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				var talkEnd0 = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd0 ) )
					yield return null;
				ui.Hide();
				ui.ResumeGame();
				#endregion

				ui.ShowConditionText( "目的：チルノを倒し、チルノの拠点を破壊する。\nサブ目的：チルノの補給拠点を探す。" );

				yield return null;
			}
			protected override IEnumerable Tactics()
			{
				var areaRect = Owner.areas["補給拠点イベント"] as AreaRectangle;
				IEnumerator fiber = CallFiber( LunarSpanTime, Owner.BunkersCenter, To2D( Owner.WhiteBase.Pos ) ).GetEnumerator();

				while( true )
				{
					if( fiber != null )
						if( !fiber.MoveNext() )
							fiber = null;

					var whiteUnits = World.UnitManager.GetUnits( _ => _.Faction.Index == World.Factions[0].Index );
					if( whiteUnits.Any( _ => IsUnitOnArea( areaRect, _ ) ) )
					{
						Owner.state = new FindState( Owner, World );
						break;
					}
					yield return null;
				}
				yield return null;
			}
		}

		class FindState : StoryState
		{
			public FindState( Story04 owner, IWorld world )
				: base( owner, world )
			{
			}

			protected override IEnumerable Talk()
			{
				World.Factions[0].MoveScreenPosition( To2D( Owner.SupplyBase.Pos ) );

				#region Talk10
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【魔理沙】\n御阿礼のカンも意外と当たるものだな、巫女だけでなく" );
				ui.ShowImage( Akyu[0], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【阿求】\nありましたか？下手に殴らないでくださいね、気づかれてしまいますから" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, false, false );
				ui.ShowText( "【魔理沙】\n殴らないでどうするんだ？こっそり並んで酒でもちょろまかせと？" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【阿求】\nそれもできれば魅力的ですが、盗賊は時代遅れなのでしょう？" );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Left, false, false );
				ui.ShowText( "【魔理沙】\nそれもそうだな、それでどうする？" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【阿求】\nまずは気付かれない程度の距離に軍を置きます" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【阿求】\nそのあとで少しの量の軍で殴ります、相手は迎撃しないといけません" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【阿求】\nそこを大群で返り討ちもいいですし、軍を相手の本陣近くに待機させれば奇襲もできます" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, false, false );
				ui.ShowText( "【魔理沙】\nなるほど、軍師殿はあくどいと見える" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【阿求】\nいかに楽して勝つかを考えるのが軍師の仕事ですから" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				var talkEnd1 = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd1 ) )
					yield return null;
				ui.Hide();
				ui.ResumeGame();
				#endregion

				ui.ShowConditionText( "目的：チルノを倒し、チルノの拠点を破壊する。\nサブ目的：チルノの補給拠点を破壊する。" );
			}
			protected override IEnumerable Tactics()
			{
				IEnumerator fiber = CallFiber( LunarSpanTime, Owner.BunkersCenter, To2D( Owner.WhiteBase.Pos ) ).GetEnumerator();

				while( true )
				{
					if( fiber != null )
						if( !fiber.MoveNext() )
							fiber = null;

					if( Owner.SupplyBase.HP < Owner.SupplyBase.MaxHP - 500 )
					{
						Owner.state = new AssaultState( Owner, World );
						break;
					}
					yield return null;
				}
			}
		}

		class AssaultState : StoryState
		{
			public AssaultState( Story04 owner, IWorld world )
				: base( owner, world )
			{
			}

			protected override IEnumerable Talk()
			{
				#region Talk11
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Nitori[0], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【にとり】\nん…？" );
				ui.ShowImage( Nitori[2], StoryUIImagePosition.Left, false, false );
				ui.ShowText( "【にとり】\nうわ、見つかったかぁ…光学迷彩しとくべきだったかな？\nチルノー、後ろの補給陣地攻撃されてるよー" );
				ui.ShowImage( Chilno[2], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【チルノ】\nえ？ええと、とりあえず迎撃！返り討ちにしてやるわ！" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Nitori[0], StoryUIImagePosition.Front, true, false );
				ui.ShowText( "【にとり】\n………" );
				ui.ShowImage( Nitori[2], StoryUIImagePosition.Front, false, false );
				ui.ShowText( "【にとり】\n……なんとなーく、嫌な予感するなぁ…" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				var talkEnd2 = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd2 ) )
					yield return null;
				ui.Hide();
				ui.ResumeGame();
				#endregion

				yield return null;
			}
			protected override IEnumerable Tactics()
			{
				IEnumerator fiber = CallFiber( LunarSpanTime, To2D( Owner.SupplyBase.Pos ), To2D( Owner.SupplyBase.Pos ) ).GetEnumerator();

				{
					var units = CallLunars( LunarAmount / 2, LunarAmount / 2 );
					World.UnitManager.AttackMove( units.SelectMany( _ => _ ).Select( _ => _.UniqueID ), Owner.SupplyBase, false );
				}

				while( true )
				{
					if( fiber != null )
						if( !fiber.MoveNext() )
							fiber = null;

					if( !Owner.SupplyBase.Exists() )
					{
						Owner.state = new SupplyBaseBrokenState( Owner, World );
						break;
					}
					yield return null;
				}
			}
		}

		class SupplyBaseBrokenState : StoryState
		{
			public SupplyBaseBrokenState( Story04 owner, IWorld world )
				: base( owner, world )
			{
			}

			public override void Update()
			{
				if( talkFiber != null )
					if( !talkFiber.MoveNext() )
						talkFiber = null;
				if( tacticsFiber != null )
					if( !tacticsFiber.MoveNext() )
						tacticsFiber = null;
			}
			protected override IEnumerable Talk()
			{
				ui.AppendChatLogMessage(
					"LotusCraft/Image/DefaultChatIcon/阿求",
					"阿求",
					new Color( 0.95f, 0.9f, 1 ),
					"敵の補給拠点を破壊しましたね！",
					new Color( 1, 1, 1 ) );

				for( int i = 0; i < 60; i++ )
					yield return null;

				ui.AppendChatLogMessage(
					"LotusCraft/Image/DefaultChatIcon/阿求",
					"阿求",
					new Color( 0.95f, 0.9f, 1 ),
					"これで相手の戦力も半減でしょう",
					new Color( 1, 1, 1 ) );
				yield return null;
			}
			protected override IEnumerable Tactics()
			{
				foreach( var item in CallFiber( LunarSpanTime * 2, To2D( Owner.BlueBase.Pos ), To2D( Owner.WhiteBase.Pos ) ) )
				{
					yield return null;
				}
			}
		}

		#endregion

		#region フィールド
		static readonly int LunarSpanTime = (int)( Def.FPS * 60 * 1.2f );
		static readonly int LunarAmount = 2;
		static readonly int[] FirstResoure = new List<int>{ 500, 200, 200 }.ToArray();
		static readonly float[] HPRate = new List<float>{ 0.5f, 1, 1.2f }.ToArray();

		IBuilding WhiteBase { get; set; }
		IBuilding BlueBase { get; set; }
		IBuilding SupplyBase { get; set; }
		Vector2D BunkersCenter;
		Dictionary<string, Area> areas = new Dictionary<string, Area>();
		StoryState state { get; set; }

		int[] Reimu = null;
		int[] Marisa = null;
		int[] Akyu = null;
		int[] Eirin = null;
		int[] Ichirin = null;
		int[] Hijiri = null;
		int[] Chilno = null;
		int[] Nitori = null;
		int[] Aya = null;
		int Back1;
		#endregion

		public Story04()
			: base()
		{
			Name = "白1話";
			Description = "白1話のマップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		public override string[] GetSelectableFactions()
		{
			return new string[] {
				typeof( Faction.White ).FullName
			};
		}

		public override string[] GetEnableFactions()
		{
			return new string[]	{
				typeof( Faction.White ).FullName,
				typeof( Faction.Blue ).FullName
			};
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] {
				"LotusCraft/Story/Story04.map",
			};
		}

		public override void OnInitialize( thrts.IWorld world )
		{
			world.Factions[1].PlayerName = "青勢力";

			var ui = world.StoryUI;

			// 最初から負けないように
			world.Factions[0].DoesNotLose = true;
			world.Factions[1].DoesNotLose = true;

			var blueBases = GetBuildings<Building.Blue.HomeBase>( world );
			var bunkers = GetBuildings<Building.White.Bunker>( world );
			WhiteBase = GetBuildings<Building.White.HomeBase>( world ).First();
			BlueBase = blueBases.ElementAt( 0 );
			if( blueBases.Count() >= 2 )
				SupplyBase = blueBases.ElementAt( 1 );
			BunkersCenter = new Vector2D( bunkers.Average( _ => _.Pos.X ) + 2, bunkers.Average( _ => _.Pos.Y ) + 2 );

			areas = new Dictionary<string, Area>();
			world.AllPurposeArea.ForEach( _ => areas[_.Name] = _ );

			// 初期視点位置
			world.Factions[0].Parameter.DefaultScreenPosition = new Point2D( (int)WhiteBase.Pos.X, (int)WhiteBase.Pos.Y );

			// ボタン仕様禁止
			var home = world.BuildingManager.GetBuildings(_ => _.Faction == world.Factions[0] && _.Parameter.GetType() == typeof(LotusCraft.Building.White.HomeBase)).Single();
			ui.HideButton(home, ButtonPosition._0, ButtonPosition._2);
			ui.HideButton(home, ButtonPosition._1, ButtonPosition._2);
			ui.HideButton(home, ButtonPosition._2, ButtonPosition._2);

			Reimu = LoadTachies( ui, "LotusCraft/Image/Tachie/Reimu_N", "LotusCraft/Image/Tachie/Reimu_S", "LotusCraft/Image/Tachie/Reimu_W" );
			Marisa = LoadTachies( ui, "LotusCraft/Image/Tachie/Marisa_N", "LotusCraft/Image/Tachie/Marisa_S", "LotusCraft/Image/Tachie/Marisa_W" );
			Akyu = LoadTachies( ui, "LotusCraft/Image/Tachie/Akyu_N", "LotusCraft/Image/Tachie/Akyu_S", "LotusCraft/Image/Tachie/Akyu_W" );
			Eirin = LoadTachies( ui, "LotusCraft/Image/Tachie/Eirin_N", "LotusCraft/Image/Tachie/Eirin_S", "LotusCraft/Image/Tachie/Eirin_W" );
			Hijiri = LoadTachies( ui, "LotusCraft/Image/Tachie/Hijiri_N", "LotusCraft/Image/Tachie/Hijiri_S", "LotusCraft/Image/Tachie/Hijiri_W" );
			Ichirin = LoadTachies( ui, "LotusCraft/Image/Tachie/Ichirin_N", "LotusCraft/Image/Tachie/Ichirin_S", "LotusCraft/Image/Tachie/Ichirin_W" );
			Chilno = LoadTachies( ui, "LotusCraft/Image/Tachie/Chilno_N", "LotusCraft/Image/Tachie/Chilno_S", "LotusCraft/Image/Tachie/Chilno_W" );
			Nitori = LoadTachies( ui, "LotusCraft/Image/Tachie/Nitori_N", "LotusCraft/Image/Tachie/Nitori_S", "LotusCraft/Image/Tachie/Nitori_W" );
			Aya = LoadTachies( ui, "LotusCraft/Image/Tachie/Aya_N", "LotusCraft/Image/Tachie/Aya_S", "LotusCraft/Image/Tachie/Aya_W" );
			Back1 = ui.LoadImage( "LotusCraft/Image/Back/Black" );

			SetDifficulty( world );

			base.OnInitialize( world );
		}

		public override System.Collections.IEnumerable Update( IWorld world )
		{
			var chirnoUnit = world.UnitManager.GetUnits( _ => _.Parameter.GetType() == typeof( Unit.Blue.Unique.Chirno ) ).First();
			IBuilding whiteBase = GetBuildings<Building.White.HomeBase>( world ).First();
			IBuilding blueBase;
			IBuilding supplyBase;

			{
				var blueBases = GetBuildings<Building.Blue.HomeBase>( world ).ToArray();
				blueBase = blueBases[0];
				supplyBase = blueBases[1];
			}
			state = new IntroductionState( this, world );
			chirnoUnit.AddExperience( 3000 );

			while( true )
			{
				yield return null;

				state.Update();

				if( !chirnoUnit.Exists() && !blueBase.Exists() )
				{
					foreach( var item2 in Talk勝利( world ) )
					{
						yield return null;
					}
					world.Factions[1].DoesNotLose = false;
					world.Factions[1].Lose();
					world.StoryUtilities.SetDataToSaveData( GetClearFlagName( world ), "Clear" );
					break;
				}
				else if( !whiteBase.Exists() )
				{
					foreach( var item2 in Talk敗北( world ) )
					{
						yield return null;
					}
					world.Factions[0].DoesNotLose = false;
					world.Factions[0].Lose();
					break;
				}
			}

			yield return null;
		}

		private void SetDifficulty( IWorld world )
		{
			int difficulty = world.StoryUtilities.GetDifficulty();

			// 初期資源を配布
			world.Factions[0].IncResource( 0, FirstResoure[difficulty] );
			world.Factions[0].IncResource( 1, FirstResoure[difficulty] );

			var blueTypes = new[]{
				typeof( Unit.Blue.Lunar ),
				typeof( Unit.Blue.Unique.Chirno ),
			};
			var neutralTypes = new[]{
				typeof( Unit.Neutral.Low01 ),
				typeof( Unit.Neutral.Low02 ),
				typeof( Unit.Neutral.Low11 ),
				typeof( Unit.Neutral.Low12 ),
				typeof( Unit.Neutral.High01 ),
				typeof( Unit.Neutral.High02 ),
			};
			blueTypes.Select( _ => world.Factions[1].GetUnitParameter( _.FullName ) )
				.Concat( neutralTypes.Select( _ => world.Factions[2].GetUnitParameter( _.FullName ) ) )
				.ForEach( _ => _.HP = (int)( _.HP * HPRate[difficulty] ) );
		}

		private string GetClearFlagName( IWorld world )
		{
			switch( world.StoryUtilities.GetDifficulty() )
			{
			case 0: return "LotusCraft.White.01.Easy";
			case 1: return "LotusCraft.White.01.Normal";
			case 2: return "LotusCraft.White.01.Hard";
			default: throw new InvalidOperationException( "不正な難易度が設定されています。" );
			}
		}


		private System.Collections.IEnumerable Talk勝利( IWorld world )
		{
			var ui = world.StoryUI;

			#region Talk12
			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Chilno[2], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【チルノ】\nぎぇぴー" );
			ui.ShowImage( Nitori[0], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【にとり】\nじゃあ私は帰るよ。ばいばい盟友たち" );
			ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【魔理沙】\nああ。またな" );
			ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【霊夢】\n結局にとりの雇用主って誰だったんでしょうね" );
			ui.ShowImage( Akyu[2], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【阿求】\nということはチルノも仲間に引き入れているってことですよね、多分" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			ui.ShowImage( Back1, StoryUIImagePosition.Front, true, false, true );
			ui.ShowText( "～次の日～" );
			ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
			ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【霊夢】\nそろそろ紫辺りが何かしてくると思ったけど、結局チルノが来たぐらいだったわね" );
			ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【魔理沙】\nあいつはあいつで忙しいんじゃないか？" );
			ui.ShowImage( Akyu[2], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【阿求】\nというか私たちがやる気なさすぎるんだと思いますよ" );
			ui.ShowImage( Reimu[2], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【霊夢】\n勝手に潰しあってもらって横から美味しいところを頂きましょうそうしましょう" );
			ui.ShowImage( Akyu[2], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【阿求】\nロクでもない思想ですね" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【霊夢】\n私の座右の銘は果報は寝て待てだから" );
			ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【魔理沙】\nこうして待っていればいずれ何か起きるさ" );
			ui.ShowImage( Akyu[2], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【阿求】\nま、まぁ戦いに出なくて良いのは嬉しいですけど" );
			ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【魔理沙】\n後ろでふんぞり返ってる分には楽しそうに見えてるけどな" );
			ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【阿求】\nええ、ヒロイン抜きでしたら魔理沙さんにも遅れ取らないでしょうし" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【霊夢】\nはいはいそこまで、\n話戻すけれど、こういうときに私を狙ってくるってのは大抵レミリアとかその辺だから、外に出ないで焦らしてやればいいのよ" );
			ui.ShowImage( Akyu[0], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【阿求】\nあー" );
			ui.ShowImage( Marisa[2], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【魔理沙】\nあぁ、そうだ、どうも今回は幽香がアリスを連れているらしいぜ。\n張り紙がしてあった" );
			ui.ShowImage( Reimu[2], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【霊夢】\nうわー" );
			ui.ShowImage( Akyu[2], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【阿求】\nそれに白蓮さんも遺恨がある相手が居るとかなんとか" );
			ui.ShowImage( Reimu[2], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【霊夢】\nいつのまにか四面楚歌じゃない？" );
			ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【魔理沙】\nいつも通りだな" );
			ui.ShowImage( Aya[0], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【文】\nええ、まさしくいつも通り" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【霊夢】\nどこから入った" );
			ui.ShowImage( Akyu[2], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【阿求】\n私の家なので霊夢さんが殺気立つ意味がちょっと" );
			ui.ShowImage( Aya[0], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【文】\nいえ、状況をお知らせしようかと\nまぁ、要するに宣戦布告ですね" );
			ui.ShowText( "紅魔館はレミリアさんに魔法使いにメイド\nそこに私、妹紅さん、早苗さんの陣容です" );
			ui.ShowImage( Aya[1], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【文】\nまぁ、要するに宣戦布告ですね" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【霊夢】\nふむ。その言い振りだとこっちの面子もわかってるのかしら" );
			ui.ShowImage( Aya[1], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【文】\nそれは勿論。まぁ軽い気持ちでどうぞどうぞ。\n紅魔館で待っているわ、だそうです" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【霊夢】\nじゃあ夜にでも行くって伝えておいて。そっちのほうがレミリアにも嬉しいでしょ" );
			ui.ShowImage( Aya[0], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【文】\nどうもほかの方も出払っているみたいですし、そう伝えておきましょうか\nそれじゃあ待っていますよ" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, true );
			ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【霊夢】\nさーてそろそろちゃんと動きますか" );
			ui.ShowImage( Akyu[2], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【阿求】\n(あ、やっぱりやる気なかったんだ)" );
			var talkEnd0 = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd0 ) )
				yield return null;
			ui.Hide();
			ui.ResumeGame();
			#endregion

			world.StoryUtilities.SetDataToSaveData("LotusCraft.White.01", "Clear");

			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				world.StoryUtilities.SetDataToSaveData("LotusCraft.White.01.Easy", "Clear");
			}
			else if (world.StoryUtilities.GetDifficulty() == 1)
			{
				world.StoryUtilities.SetDataToSaveData("LotusCraft.White.01.Normal", "Clear");
			}
			else if (world.StoryUtilities.GetDifficulty() == 2)
			{
				world.StoryUtilities.SetDataToSaveData("LotusCraft.White.01.Hard", "Clear");
			}

			yield return null;
		}

		private System.Collections.IEnumerable Talk敗北( IWorld world )
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Chilno[1], StoryUIImagePosition.Front, true, false );
			ui.ShowText( "【チルノ】\nこれであたいが最強だー！" );
			ui.ShowImage( -1, StoryUIImagePosition.Front, true, false );
			var talkEnd = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd ) )
				yield return null;
			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}

		int[] LoadTachies( IStoryUI ui, string path_N, string path_S, string path_W )
		{
			return new int[3]{
				ui.LoadImage( path_N ),
				ui.LoadImage( path_S ),
				ui.LoadImage( path_W )
			};
		}

		IEnumerable<IBuilding> GetBuildings<T>( IWorld world ) where T : BuildingParameter
		{
			return world.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( T ) );
		}
	}
}
