﻿#if !_TRIAL

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft.Story
{
	public class StoryGreen04 : thrts.Mission
	{
		private int[] Reimu;
		private int[] Yuka;
		private int[] Suika;
		private int[] Alice;
		private int[] Minoriko;
		private int[] Lyrica;
		private int[] Kasen;

		readonly Dictionary<int, int> Experience;
		readonly Dictionary<string, Area> areas;

		int 星召喚インターバル = int.MaxValue;
		bool 霊夢ラストスペル使用 = false;

		public StoryGreen04()
		{
			Name = "緑4話";
			Description = "緑4話専用マップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
			Experience = new Dictionary<int, int>()
			{
				{0, 0},
				{1,	0}, 
				{2,	60},
				{3,	200},
				{4,	500},
				{5,	1200},
				{6,	2400},
 				{7,	4000},
				{8,	6000},
				{9, 8300},
				{10, 11000},
				{11, 14500}
			};
			areas = new Dictionary<string, Area>();
		}

		/// <summary>
		/// 自軍の勢力
		/// </summary>
		/// <returns></returns>
		public override string[] GetSelectableFactions()
		{
			return new string[] { 
				typeof(LotusCraft.Faction.Green).FullName 
			};
		}

		/// <summary>
		/// 登場する勢力
		/// </summary>
		/// <returns></returns>
		public override string[] GetEnableFactions()
		{
			return new string[] { 
				typeof(LotusCraft.Faction.Green).FullName,
				typeof(LotusCraft.Faction.White).FullName, 
			};
		}
		/// <summary>
		/// 使用するマップ
		/// </summary>
		/// <returns></returns>
		public override string[] GetSelectableMaps()
		{
			return new string[] {
				"LotusCraft/Story/StoryGreen04.map",
			};
		}

		IEnumerable<IBuilding> GetBuildings(IFaction faction, Type parameterType)
		{
			return faction.World.BuildingManager.GetBuildings(b => b.Faction == faction && b.Parameter.GetType() == parameterType);
		}
		IEnumerable<IUnit> GetUnits(IFaction faction, Type parameterType)
		{
			return faction.World.UnitManager.GetUnits(u => u.Faction == faction && u.Parameter.GetType() == parameterType);
		}

		public override void OnInitialize(thrts.IWorld world)
		{
			world.Factions[1].PlayerName = "白勢力";

			// 最初から負けないように
			world.Factions.ForEach(f => f.DoesNotLose = true);

			var home = GetBuildings(world.Factions[0], typeof(LotusCraft.Building.Green.HomeBase)).Single();
			// 初期視点位置
			world.Factions[0].Parameter.DefaultScreenPosition
				= new Point2D((int)(home.Pos.X), (int)(home.Pos.Y));

			// 初期資源を配布
			world.Factions[0].IncResource(0, 2000);
			world.Factions[0].IncResource(1, 500);

			// 難易度調整
			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Low03).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Murasa).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Toramaru).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Reisen).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Kaguya).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Unique.Reimu).FullName).HP /= 2;

				world.UnitManager.GetUnits(u => u.Faction.Index == 1).Do(u => u.MaxHP = u.Parameter.HP).ForEach(u => u.HP = u.Parameter.HP);
			}
			else if (world.StoryUtilities.GetDifficulty() == 2)
			{
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Low03).FullName).HP += (int)(world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Low03).FullName).HP * 0.2);
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Murasa).FullName).HP += (int)(world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Murasa).FullName).HP * 0.2);
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Toramaru).FullName).HP += (int)(world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Toramaru).FullName).HP * 0.2);
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Reisen).FullName).HP += (int)(world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Reisen).FullName).HP * 0.2);
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Kaguya).FullName).HP += (int)(world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Kaguya).FullName).HP * 0.2);
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Unique.Reimu).FullName).HP += (int)(world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.White.Unique.Reimu).FullName).HP * 0.2);

				world.UnitManager.GetUnits(u => u.Faction.Index == 1).Do(u => u.MaxHP = u.Parameter.HP).ForEach(u => u.HP = u.Parameter.HP);

				星召喚インターバル = 60 * 30;
				霊夢ラストスペル使用 = true;
			}

			world.AllPurposeArea.ForEach(_ => areas[_.Name] = _);

			var ui = world.StoryUI;

			{
				Reimu = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Reimu_N"), ui.LoadImage("LotusCraft/Image/Tachie/Reimu_S"), ui.LoadImage("LotusCraft/Image/Tachie/Reimu_W") };
				Yuka = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Yuka_N"), ui.LoadImage("LotusCraft/Image/Tachie/Yuka_S"), ui.LoadImage("LotusCraft/Image/Tachie/Yuka_W") };
				Suika = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Suika_N"), ui.LoadImage("LotusCraft/Image/Tachie/Suika_S"), ui.LoadImage("LotusCraft/Image/Tachie/Suika_W") };
				Alice = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Alice_N"), ui.LoadImage("LotusCraft/Image/Tachie/Alice_S"), ui.LoadImage("LotusCraft/Image/Tachie/Alice_W") };
				Minoriko = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Minoriko_N"), ui.LoadImage("LotusCraft/Image/Tachie/Minoriko_S"), ui.LoadImage("LotusCraft/Image/Tachie/Minoriko_W") };
				Lyrica = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Lyrica_N"), ui.LoadImage("LotusCraft/Image/Tachie/Lyrica_S"), ui.LoadImage("LotusCraft/Image/Tachie/Lyrica_W") };
				Kasen = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Kasen_N"), ui.LoadImage("LotusCraft/Image/Tachie/Kasen_S"), ui.LoadImage("LotusCraft/Image/Tachie/Kasen_W") };
			}

			base.OnInitialize(world);
		}
		void White初期化(IWorld world)
		{
			GetUnits(world.Factions[1], typeof(LotusCraft.Unit.White.Unique.Reimu)).Single().AddExperience(Experience[11]);

			// 水兎をバンカーへ
			{
				var bunker = GetBuildings(world.Factions[1], typeof(LotusCraft.Building.White.Bunker)).ToList();
				var low03_8 = PutEnemy<LotusCraft.Unit.White.Low03>(world.Factions[1], areas["青兎8"], 8).ToList();
				var low03_32 = PutEnemy<LotusCraft.Unit.White.Low03>(world.Factions[1], areas["青兎32"], 32).ToList();
				var low03_4 = PutEnemy<LotusCraft.Unit.White.Low03>(world.Factions[1], areas["青兎4"], 4).ToList();
				var low03_2L = PutEnemy<LotusCraft.Unit.White.Low03>(world.Factions[1], areas["青兎2_左"], 2).ToList();
				var low03_2R = PutEnemy<LotusCraft.Unit.White.Low03>(world.Factions[1], areas["青兎2_右"], 2).ToList();

				int i = 0;
				for (; i < 2; i++)
					world.UnitManager.SelectTarget(new[] { low03_8[4 * i], low03_8[4 * i + 1], low03_8[4 * i + 2], low03_8[4 * i + 3] }, bunker[i], false);
				for (; i < 10; i++)
					world.UnitManager.SelectTarget(new[] { low03_32[4 * (i - 2)], low03_32[4 * (i - 2) + 1], low03_32[4 * (i - 2) + 2], low03_32[4 * (i - 2) + 3] }, bunker[i], false);
				for (; i < 14; i++)
					world.UnitManager.SelectTarget(new[] { low03_4[(i - 10)] }, bunker[i], false);
				for (; i < 15; i++)
					world.UnitManager.SelectTarget(new[] { low03_2L[2 * (i - 14)], low03_2L[2 * (i - 14) + 1] }, bunker[i], false);
				for (; i < 16; i++)
					world.UnitManager.SelectTarget(new[] { low03_2R[2 * (i - 15)], low03_2R[2 * (i - 15) + 1] }, bunker[i], false);
			}

			// 輝夜スタンバイ
			new[] { world.Factions[1].GetUnitSpellButtonParameter("LotusCraft.Unit.White.Kaguya", new SpellButtonKey("新難題「金閣寺の一枚天井」", 1)) }
				.ForEach(p => GetUnits(world.Factions[1], typeof(LotusCraft.Unit.White.Kaguya)).ForEach(u => p.NormalButtonEvent(u, false)));

			// 星ちゃんステンバーイ
			GetUnits(world.Factions[1], typeof(LotusCraft.Unit.White.Toramaru)).ForEach(u => u.HoldPosition(false));

			GetUnits(world.Factions[1], typeof(LotusCraft.Unit.White.Murasa)).ForEach(u => u.HoldPosition(false));

			GetUnits(world.Factions[1], typeof(LotusCraft.Unit.White.Unique.Reimu))
				.Do(u => u.HoldPosition(false))
				.Do(u => u.LvUpSpellWithName("大結界「博麗弾幕結界」"))
				.Last();
		}
		public override System.Collections.IEnumerable Update(thrts.IWorld world)
		{
			world.ChangeBGM(BGMList.Boushuen_Story2);

			foreach (var item in 導入会話(world))
				yield return item;
			{
				var reimu = GetUnits(world.Factions[1], typeof(LotusCraft.Unit.White.Unique.Reimu)).Single();
				world.Factions[0].GetSite(16 * Def.MapTipLengthOnField, reimu.Pos.X, reimu.Pos.Y, int.MaxValue);
			}

			world.ChangeBGM(BGMList.Battle_Hakurei);

			int frameCount = 0;
			const int timeLimitFrame = 15 * 60 * 30;
			const int 二重結界LvUPインターバル = 7 * 60 * 30;

			White初期化(world);

			var createdUniqueUnit = new HashSet<string>();
			while (true)
			{
				// 最初に召喚されたユニークユニットのレベル底上げ
				world.UnitManager.GetUnits(u => u.Faction.Index == 0 && u.Parameter.IsUnique)
					.Where(u => createdUniqueUnit.Add(u.Parameter.GetType().FullName))
					.ForEach(u => u.AddExperience(Experience[4]));

				if (霊夢ラストスペル使用)
				{
					var 霊夢 = GetUnits(world.Factions[1], typeof(LotusCraft.Unit.White.Unique.Reimu)).SingleOrDefault();

					if (霊夢 != null && 霊夢.HP < 霊夢.MaxHP)
					{
						霊夢.Faction.GetUnitSpellButtonParameter(霊夢.Parameter.GetType().FullName, new SpellButtonKey("大結界「博麗弾幕結界」", 1)).NormalButtonEvent(霊夢, false);
						霊夢.HoldPosition(true);

						霊夢ラストスペル使用 = false;
					}
				}

				int restTime = timeLimitFrame - frameCount;
				{
					var message = "目的：博麗神社拠点の撃破。\n";
					message += string.Format("残り時間　{0:00}:{1:00}", restTime / 1800, restTime / 30 % 60);
					world.StoryUI.ShowConditionText(message);
				}
				if (frameCount % 星召喚インターバル == 0 && frameCount != 0)
				{
					var houtou = GetBuildings(world.Factions[1], typeof(LotusCraft.Building.White.Houtou));

					if (houtou.Take(1).Any())
						PutEnemy<LotusCraft.Unit.White.Toramaru>(world.Factions[1], areas["寅召喚1"], 2).ForEach(u => u.HoldPosition(false));
					if (houtou.Skip(1).Take(1).Any())
						PutEnemy<LotusCraft.Unit.White.Toramaru>(world.Factions[1], areas["寅召喚2"], 2).ForEach(u => u.HoldPosition(false));
				}
				if (frameCount % 二重結界LvUPインターバル == 0 && frameCount != 0)
				{
					GetUnits(world.Factions[1], typeof(LotusCraft.Unit.White.Unique.Reimu)).ForEach(u => u.LvUpSpellWithName("夢符「二重結界」"));
				}

				if (GetBuildings(world.Factions[1], typeof(LotusCraft.Building.White.HomeBase)).IsEmpty() && 0 < restTime)
				{
					foreach (var item in 勝利会話(world))
						yield return item;

					world.StoryUtilities.SetDataToSaveData("LotusCraft.Green.04", "Clear");

					if (world.StoryUtilities.GetDifficulty() == 0)
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Green.04.Easy", "Clear");
					else if (world.StoryUtilities.GetDifficulty() == 1)
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Green.04.Normal", "Clear");
					else if (world.StoryUtilities.GetDifficulty() == 2)
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Green.04.Hard", "Clear");

					break;
				}
				else if ((world.UnitManager.GetUnits(u => u.Faction.Index == 0).IsEmpty() && world.BuildingManager.GetBuildings(b => b.Faction.Index == 0).IsEmpty())
					|| restTime <= 0)
				{
					foreach (var item in 時間切れ敗北会話(world))
						yield return item;

					break;
				}
				else
				{
					frameCount++;

					yield return null;
				}
			}
			while (true)
				yield return null;
		}
		IEnumerable<IUnit> PutEnemy<TUnit>(IFaction faction, Area area, int num) where TUnit : UnitParameter
		{
			var rect = area as AreaRectangle;
			if (rect == null)
				return new IUnit[0];

			List<IUnit> units = new List<IUnit>();
			var type = typeof(TUnit);

			var sx = Math.Min(rect.Start.X, rect.End.X);
			var sy = Math.Min(rect.Start.Y, rect.End.Y);
			var ex = Math.Max(rect.Start.X, rect.End.X);
			var ey = Math.Max(rect.Start.Y, rect.End.Y);

			for (var y = sy; y < ey; y++)
			{
				for (var x = sx; x < ex; x++)
				{
					if (units.Count >= num)
						return units;

					var u = faction.World.UnitManager.Create(
						type.FullName,
						faction,
						(x + 0.5f) * thrts.Def.MapTipLengthOnField,
						(y + 0.5f) * thrts.Def.MapTipLengthOnField,
						faction);
					if (u != null)
						units.Add(u);
				}
			}
			return units;
		}

		IEnumerable 導入会話(IWorld world)
		{
			var ui = world.StoryUI;
			ui.PauseGame();
			ui.Show();
			ui.ShowText("～博麗神社～");
			ui.ShowImage(Yuka[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽香】\nいよいよ持って死ぬが良い");
			ui.ShowImage(Alice[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【アリス】\n唐突すぎるでしょ");
			ui.ShowImage(Yuka[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【幽香】\nはっきりといえば、\n私たちにとっては霊夢たちに勝つことが重要。そうでしょう？");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【萃香】\nまー、私は勝ったことがあるけれども");
			ui.ShowImage(Yuka[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【幽香】\n博麗の巫女が、博麗の役割でなく、\n単なる霊夢として遊んでいるときに吹っ掛けておいて後々までそれで弄るのが礼儀");
			ui.ShowImage(Alice[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【アリス】\nうわっ");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【萃香】\nまぁ闇打ちよりかはずっとマシだろう\n昨日は宴会だったし、まだ寝てるかもしらんが");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Reimu[2], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【霊夢】\n起きてるわよ\nあんたたちがおっきい声で喋るから寝れやしない");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Yuka[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽香】\n宣戦布告に来たわ");
			ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【霊夢】\nあっそう。ほかの勢力の同意がどうたらは？");
			ui.ShowImage(Yuka[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【幽香】\nいらないわ\n私たちは単に、博麗霊夢と勝負がしたいだけなんですもの");
			ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【霊夢】\nふぅーん\n別に私に勝ったところで何もないと思うけど");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Suika[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【萃香】\nあれだ\n単に遊びましょって言えないから、戦って勝つだのなんだの言ってるだけだろう");
			ui.ShowImage(Alice[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【アリス】\nあー");
			ui.ShowImage(Yuka[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽香】\nそんなことはない");
			ui.ShowImage(Suika[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【萃香】\nそれに、私たちにとって、って言ってたけど\n私らそこまで霊夢に勝ちたいなんて思っていないし、幽香だってそうだろうに");
			ui.ShowImage(Minoriko[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【穣子】\n私は―――");
			ui.ShowImage(Alice[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【アリス】\nまぁ私はね、そろそろ家に帰って寝て、普段通りの生活に戻ろうかなとか");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【萃香】\n歳を取ってくるとな、素直に遊びましょうの一言も言えないもんさ");
			ui.ShowImage(Yuka[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽香】\nぐぬぬぬぬ");
			ui.ShowImage(Lyrica[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【リリカ】\nこういうどんちゃん騒ぎ好きだから、もう少し付き合ってもいいかなー");
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(Kasen[2], StoryUIImagePosition.Right, true, false, false);
			ui.ShowText("【華仙】\n付き合ってって表現の時点で煽ってますよ、それ");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【霊夢】\nどうしてもって言うなら付き合ってあげないこともないけど\n魔理沙たちもいないし。私一人で良いかしら？");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【萃香】\nうちのわがまま大将がそれでいいならいいんじゃないか");
			ui.ShowImage(Alice[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【アリス】\nそろそろ私、帰って寝たいから手早くやってね");
			ui.ShowImage(Yuka[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽香】\n最終決戦なんだからもっと盛り上がってもいいじゃない！\nなんでこんなグダグダな雰囲気なのよ！");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Alice[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【アリス】\nなんだかんだで付き合ってるだけありがたいと思いなさいよ");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【萃香】\nだよなー");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(Kasen[2], StoryUIImagePosition.Right, true, false, false);
			ui.ShowText("【華仙】\n妖怪相手では今更かもしれないけれど\nこういうのを厚意として感謝して受け取った方がいいと思いますよ");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Yuka[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽香】\nいいわもう、霊夢なんてひと捻りにしてやるんだから！");
			ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【霊夢】\nルールは？そっちで適当に決めて構わないわよ");
			ui.ShowImage(Yuka[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【幽香】\n昨日と同じで構わないわ、制限時間は四半時キッチリ！");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Alice[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【アリス】\n……短くなってるわね");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【萃香】\n変に意地になってんじゃないかね、まぁ私としては構わないけどさ");
			ui.ShowImage(Reimu[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【霊夢】\nさっさと終わらせるわよ、ほら貴女たちも行った行った");
			ui.ShowImage(Alice[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【アリス】\nあら？幽香は？");
			ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【霊夢】\n自陣を張りにもう出て行ったわ");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【萃香】\nおやおや血気盛んなことで");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【萃香】\nでは急ぐとしようか、こっちに雷落とされちゃたまらない");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);

			var talkEnd = ui.ShowText("", false);

			while (!ui.IsCompleted(talkEnd))
			{
				yield return null;
			}

			ui.Hide();
			ui.ResumeGame();
		}

		IEnumerable 勝利会話(IWorld world)
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【霊夢】\n負けちゃったわ");
			ui.ShowImage(Alice[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【アリス】\n多勢に無勢って奴ね。大人気ない");
			ui.ShowImage(Yuka[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽香】\n勝った気がしないわ……");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【萃香】\nいやしかし、勝ったには勝ったことだし、勝者の権利があっても良いんじゃないか");
			ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【霊夢】\nあとからそう言いだすのって卑怯じゃない？\n鬼のあんたが正々堂々じゃないことを言うだなんて");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【萃香】\nいやいや、ツキものってだけで、別に何がってわけじゃない");
			ui.ShowImage(Alice[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【アリス】\n詭弁ね。何が言いたいかはわからないでもないけど");
			ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【霊夢】\n？");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Alice[1], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【アリス】\n鬼が勝ったら人を攫うでしょ？\nつまり、たまには遊びに行こうって誘ってるのよ");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("初めから霊夢のこと以外はどうでも良い風見幽香と、\nそれに付き合わされた者たちの戦いはこうして決着した。");
			ui.ShowText("長く生きてしまえば、素直になるのも苦労するもの。");
			ui.ShowImage(Alice[0], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("正々堂々の小鬼も詭弁や皮肉を言うようになるし、と、\n魔法使いの特性なのか、自分の性格なのか、\n既にひねくれている自覚のあるアリスは、前を進む三名を見ながら自嘲した。");
			ui.ShowText("そして、今のうちに、もう少し素直に生きるのも悪くはないとも思うのであった。");
			var talkEnd = ui.ShowText("", false);

			while (!ui.IsCompleted(talkEnd))
				yield return null;

			ui.Hide();
			ui.ResumeGame();

			world.Factions[0].DoesNotLose = true;
			world.Factions[0].Win();
			world.Factions[1].DoesNotLose = true;
			world.Factions[1].Lose();
		}

		IEnumerable 時間切れ敗北会話(IWorld world)
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Reimu[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【霊夢】\nはい時間切れ、私の勝ちでいいわね");
			ui.ShowImage(Alice[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【アリス】\nこれで勝てないと多勢に無勢って言葉の意味を考え直したくなるわ");
			ui.ShowImage(Reimu[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【霊夢】\nまぁ再戦くらいいつでも受けていいわよ、けれど負けたんだから…\nそうね、朝食の支度くらい手伝いなさいよ");
			ui.ShowImage(Alice[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【アリス】\nそれくらい構わないわよ、食材は何があるかしら？");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Yuka[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽香】\nく…まだよ、もう一戦\nなんだったら弾幕ごっこでも");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【萃香】\n負けず嫌いもほどほどにしときなって");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【萃香】\nそういうところ嫌いじゃないが、今日のところはこれくらいにしとき\nそら、メシだメシ、手伝い行くぞ");
			var talkEnd = ui.ShowText("", false);

			while (!ui.IsCompleted(talkEnd))
				yield return null;

			ui.Hide();
			ui.ResumeGame();

			world.Factions[0].DoesNotLose = true;
			world.Factions[0].Lose();
			world.Factions[1].DoesNotLose = true;
			world.Factions[1].Win();
		}
	}
}
#endif