﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft.Story
{
	public class StoryRed01 : thrts.Mission
	{
		/// <summary>
		/// 自軍の勢力
		/// </summary>
		/// <returns></returns>
		public override string[] GetSelectableFactions()
		{
			return new string[] { 
				typeof(LotusCraft.Faction.White).FullName 
			};
		}

		/// <summary>
		/// 登場する勢力
		/// </summary>
		/// <returns></returns>
		public override string[] GetEnableFactions()
		{
			return new string[] { 
				typeof(LotusCraft.Faction.White).FullName,
				typeof(LotusCraft.Faction.Neutral).FullName, 
			};
		}

		/// <summary>
		/// 使用するマップ
		/// </summary>
		/// <returns></returns>
		public override string[] GetSelectableMaps()
		{
			return new string[] {
				"LotusCraft/Story/StoryRed01.map",
			};
		}


		public StoryRed01()
			: base()
		{
			Name = "赤第01話";
			Description = "赤第01話専用マップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		int[] Aya = null;
		int[] Mokou = null;
		int[] Yuyuko = null;
		int[] Remilia = null;
		int[] Miko = null;
		int[] Yuka = null;
		int[] Suika = null;
		int[] Alice = null;
		int[] Sanae = null;
		int[] Patchouli = null;
		int[] Utsuho = null;
		int[] Sakuya = null;
		int[] Rin = null;
		int Back1 = -1;
		int Back2 = -1;
		int Back3 = -1;
		int Back4 = -1;

		class Patroler
		{
			public thrts.Vector2D Start;
			public thrts.Vector2D End;
			public string Type;

			public thrts.IUnit Unit;

			public float Radius = 0;

			public bool Summon = false;

			public thrts.ITextureEffect AreaEffect;
		}

		List<Patroler> Patrolers = new List<Patroler>();

		int enemyCount = 0;

		public override void OnInitialize(thrts.IWorld world)
		{
			world.Factions[1].PlayerName = "黒勢力";
			
			enemyCount = 1;

			// 見張り解析
			var patroler_areas = world.AllPurposeArea.Where(_ => _.Name.StartsWith("見張り"));

			int pCount = 0;
			foreach (var p in patroler_areas)
			{
				var strs = p.Name.Split('_');
				var num = int.Parse(strs[1]);
				if (pCount < num) pCount = num;
			}

			for (int loop = 0; loop < pCount; loop++)
			{
				Patrolers.Add(new Patroler());
			}

			foreach (var p in patroler_areas)
			{
				var p_ = p as thrts.AreaRectangle;
				var strs = p.Name.Split('_');
				var num = int.Parse(strs[1])-1;

				if(strs[2] == "s")
				{
					Patrolers[num].Start = new thrts.Vector2D(
						(p_.Start.X + p_.End.X) / 2 * 32.0f,
						(p_.Start.Y + p_.End.Y) / 2 * 32.0f);
				}

				if (strs[2] == "e")
				{
					Patrolers[num].End = new thrts.Vector2D(
						(p_.Start.X + p_.End.X) / 2 * 32.0f,
						(p_.Start.Y + p_.End.Y) / 2 * 32.0f);
				}

				Patrolers[num].Type = strs[3];
			}

			// 配置
			foreach (var p in Patrolers)
			{
				if (p.Type == "下級")
				{
					var u = world.UnitManager.Create(typeof(Unit.Black.Low02).FullName, world.Factions[1], p.Start.X, p.Start.Y, world.Factions[1]);
					p.Unit = u;
					if(p.Unit != null)
					{
						p.Radius = 3.0f * thrts.Def.MapTipLengthOnField;
						p.AreaEffect = world.EffectManager.CreateTextureEffect(typeof(Effect.Common.Area8TextureEffect).FullName, p.Unit.Pos);
					}
				}
			}

			// レベルアップ
			var moko = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Red.Unique.Moko)).FirstOrDefault();
			var aya = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Red.Unique.Aya)).FirstOrDefault();
			moko.AddExperience(500);
			aya.AddExperience(500);

			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				moko.AddExperience(1000);
				aya.AddExperience(1000);
			}

			world.Factions[1].DoesNotLose = true;

			// 初期配置
			world.Factions[0].Parameter.DefaultScreenPosition = new Point2D((int)(moko.Pos.X + aya.Pos.X) / 2 + 32 * 5, (int)(aya.Pos.Y + moko.Pos.Y) / 2 + 32 * 5);
		}

		void SearchEnemies(thrts.IWorld world)
		{
			var moko = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Red.Unique.Moko)).FirstOrDefault();
			var aya = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Red.Unique.Aya)).FirstOrDefault();

			for (var i = 0; i < Patrolers.Count; i++)
			{
				if (Patrolers[i].Unit == null) continue;
				if (!Patrolers[i].Unit.Exists())
				{
					Patrolers[i].Unit = null;
					continue;
				}

				if (Patrolers[i].Summon) continue;

				bool found = false;
				if (moko != null)
				{
					var dist = Patrolers[i].Unit.Pos - moko.Pos;
					if (dist.Length < Patrolers[i].Radius)
					{
						found = true;
					}
				}

				if (aya != null)
				{
					var dist = Patrolers[i].Unit.Pos - aya.Pos;
					if (dist.Length < Patrolers[i].Radius)
					{
						found = true;
					}
				}

				// 発見
				if (found)
				{
					if (Patrolers[i].AreaEffect != null)
					{
						Patrolers[i].AreaEffect.Stop();
						Patrolers[i].AreaEffect = null;
					}

					Patrolers[i].Summon = true;

					Patrolers[i].Unit.Stop(false);

					world.EffectManager.CreateTextureEffect(typeof(Effect.Common.ExclamationMarkTextureEffect).FullName, Patrolers[i].Unit.Pos + new thrts.Vector3D(0, 0, 64));

					for (int loop = 0; loop < enemyCount; loop++)
					{
						var u = thrts.Utl.PutUnitAroundPoint(world, world.Factions[1], typeof(Unit.Black.Low02).FullName, new thrts.Vector2D(Patrolers[i].Unit.Pos.X,Patrolers[i].Unit.Pos.Y));
						u.Stop(false);
					}
				}
			}
		}

		void UpdateEffect(thrts.IWorld world)
		{
			for (var i = 0; i < Patrolers.Count; i++)
			{
				if (Patrolers[i].Unit == null) continue;
				if (!Patrolers[i].Unit.Exists())
				{
					Patrolers[i].Unit = null;
					continue;
				}
				if (Patrolers[i].Summon) continue;

				Patrolers[i].AreaEffect.SetPosition(Patrolers[i].Unit.Pos + new thrts.Vector3D(0, 0, 5));
			}
		}

		public override System.Collections.IEnumerable Update(thrts.IWorld world)
		{
			var ui = world.StoryUI;

			Aya = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Aya_N"), ui.LoadImage("LotusCraft/Image/Tachie/Aya_S"), ui.LoadImage("LotusCraft/Image/Tachie/Aya_W") };
			Mokou = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Mokou_N"), ui.LoadImage("LotusCraft/Image/Tachie/Mokou_S"), ui.LoadImage("LotusCraft/Image/Tachie/Mokou_W") };
			Yuyuko = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Yuyuko_N"), ui.LoadImage("LotusCraft/Image/Tachie/Yuyuko_S"), ui.LoadImage("LotusCraft/Image/Tachie/Yuyuko_W") };
			Remilia = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Remilia_N"), ui.LoadImage("LotusCraft/Image/Tachie/Remilia_S"), ui.LoadImage("LotusCraft/Image/Tachie/Remilia_W") };
			Miko = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Miko_N"), ui.LoadImage("LotusCraft/Image/Tachie/Miko_S"), ui.LoadImage("LotusCraft/Image/Tachie/Miko_W") };
			Yuka = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Yuka_N"), ui.LoadImage("LotusCraft/Image/Tachie/Yuka_S"), ui.LoadImage("LotusCraft/Image/Tachie/Yuka_W") };
			Suika = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Suika_N"), ui.LoadImage("LotusCraft/Image/Tachie/Suika_S"), ui.LoadImage("LotusCraft/Image/Tachie/Suika_W") };
			Alice = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Alice_N"), ui.LoadImage("LotusCraft/Image/Tachie/Alice_S"), ui.LoadImage("LotusCraft/Image/Tachie/Alice_W") };
			Sanae = LoadTachies(ui, "LotusCraft/Image/Tachie/Sanae_N", "LotusCraft/Image/Tachie/Sanae_S", "LotusCraft/Image/Tachie/Sanae_W");
			Patchouli = LoadTachies(ui, "LotusCraft/Image/Tachie/Patchouli_N", "LotusCraft/Image/Tachie/Patchouli_S", "LotusCraft/Image/Tachie/Patchouli_W");
			Utsuho = LoadTachies(ui, "LotusCraft/Image/Tachie/Utsuho_N", "LotusCraft/Image/Tachie/Utsuho_S", "LotusCraft/Image/Tachie/Utsuho_W");
			Sakuya = LoadTachies(ui, "LotusCraft/Image/Tachie/Sakuya_N", "LotusCraft/Image/Tachie/Sakuya_S", "LotusCraft/Image/Tachie/Sakuya_W");
			Rin = LoadTachies(ui, "LotusCraft/Image/Tachie/Rin_N", "LotusCraft/Image/Tachie/Rin_S", "LotusCraft/Image/Tachie/Rin_W");

			Back1 = world.StoryUI.LoadImage("LotusCraft/Image/Back/Black");
			Back2 = world.StoryUI.LoadImage("LotusCraft/Image/Back/Black");
			Back3 = world.StoryUI.LoadImage("LotusCraft/Image/Back/Black");
			Back4 = world.StoryUI.LoadImage("LotusCraft/Image/Back/White");

			// 偵察配置
			foreach (var p in Patrolers)
			{
				if (p.Unit == null) continue;
				world.UnitManager.Patrol(new int[] { p.Unit.UniqueID }, p.End, false);
			}

			world.ChangeBGM(BGMList.Boushuen_Story1);

			var talk会話冒頭 = Talk会話冒頭(world);
			foreach (var t in talk会話冒頭)
			{
				yield return true;
			}

			world.ChangeBGM(BGMList.Battle_TaiyouNoHatake);

			int rest = Def.FPS * 60 * 10;

			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				rest = Def.FPS * 60 * 15;
			}

			if (world.StoryUtilities.GetDifficulty() == 2)
			{
				rest = Def.FPS * 60 * 6;
			}

			var goal = world.AllPurposeArea.Where(_ => _.Name == "ゴール").First() as AreaRectangle;
			var mokou = GetUnits(world, world.Factions[0], typeof(LotusCraft.Unit.Red.Unique.Moko)).First();
			while (true)
			{
				{
					var c = rest - world.CurrentTime;
					var s = c / Def.FPS;
					var m = s / 60;
					s -= m * 60;
					ui.ShowConditionText("目的：" + m.ToString() + "分 " + s.ToString() + "秒以内にマップ最上部に妹紅がたどり着く");
				}

				UpdateEffect(world);

				if (world.CurrentTime % 15 == 0)
				{
					SearchEnemies(world);
				}

				{
					while (IsUnitOnArea(goal, mokou))
					{
						var talk勝利 = Talk勝利(world);
						foreach (var t in talk勝利)
						{
							yield return true;
						}

						world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.01", "Clear");

						if (world.StoryUtilities.GetDifficulty() == 0)
						{
							world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.01.Easy", "Clear");
						}
						else if (world.StoryUtilities.GetDifficulty() == 1)
						{
							world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.01.Normal", "Clear");
						}
						else if (world.StoryUtilities.GetDifficulty() == 2)
						{
							world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.01.Hard", "Clear");
						}

						world.Factions[1].DoesNotLose = false;
						world.Factions[1].Lose();
						break;
					}
				}

				{
					var c = rest - world.CurrentTime;
					if (c < 0)
					{
						var talk敗北 = Talk敗北(world);
						foreach (var t in talk敗北)
						{
							yield return true;
						}

						world.Factions[0].Lose();
						break;
					}
				}

				yield return true;
			}
		}


		#region Story
		private System.Collections.IEnumerable Talk会話冒頭(IWorld world)
		{
			var ui = world.StoryUI;

			#region Talk1
			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Back2, StoryUIImagePosition.Back, false, false, true);
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【文】\nこうして我々記者団は迷いの竹林にやってきたのだった");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【妹紅】\n………");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nお前は何を言っているんだ");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nやー。こうしてでもないと暇で暇でしょうがなくって");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n妹紅さん、何か面白い話してください");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nいやなんで私がそんなことをしなきゃならないんだよ");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nそもそも私のところに来た理由もしょうもないし");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nあいやー。八雲紫の提案する新しい遊び！");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nしかし参加するためには最低六名を集めてこなければいけない！");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n私は考えました！　年中暇そうな妹紅さんをお誘いしようと");
			ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n言っとくけど私は暇じゃないのだけど");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n今だって竹林のパトロールがだな");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nとか言って食べられる物を探すために目線が泳いでますよ？");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n最近、道案内の仕事がめっきり減ってね");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nはーん。食に事欠いているってわけですね？　");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nなるほど。では私が妹紅さんの冬の食事情を何とかしましょう");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nたかが鴉天狗にそんなことができるの？");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nある程度階級も上がれば耕す土地が貰えるのが天狗の社会なのです。\n一人や二人増えたところで食い扶持に困る心配はありませんよ");
			ui.ShowText("【妹紅】\nそれで私を雇うってことか？");
			ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nまぁ、用心棒ということなら、吝かでもないというか。最初からそう言ってくれれば、うん");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n武士は食わねど高楊枝というわけにもいきませんか。");
			ui.ShowText("【文】\nちなみに紅魔館の方々とも共同戦線を張ってもらうことになります");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nなぜそこで紅魔館が出てくるんだ");
            ui.ShowImage(Aya[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【文】\nそもそもがですね……");
			ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Back3, StoryUIImagePosition.Back, false, false, true);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Remilia[1], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【レミリア】\n我が忠実なる下僕、射命丸よ。怪人候補を捕まえてくるのだ");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【文】\nははーっ！");
			ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Back2, StoryUIImagePosition.Back, false, false, true);
			
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);

			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【妹紅】\n下僕って何なんだ。しかも、怪人って");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【文】\nいやーモケーレ・ムベンベごっこで敗北してしまいまして");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nあ、妹紅さんはフジヤマレッドという名前で内定しています");
			ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nやめろよ！　何なんだよその名前は！");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nレミリアさんはネーミングセンスが致命的に悪くって……");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n…で、紅魔館からは四名なのか？　最低六名必要なんだろう？");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nあ、もう一人は早苗さんです");
			ui.ShowText("【妹紅】\nまた変わった人選だな");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n宴席で言質を取りました");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nそんなこったろうと思ったよ");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nでも案外ノリノリでしたよ。戦隊物といえば六人目の仲間がどうたらこうたらって");
			ui.ShowText("【妹紅】\nはあ。それでこれから私たちは紅魔館に行けばいいのか？");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nいやーその前に、我々は独自に動きます。");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n要するに、偵察ですね\n我々が紅魔館と組んでいることは言わなければまだわかりませんし\n主要な勢力がどのように手を組んでいるかを報告しなければいけません");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n(輝夜の奴はこういうのは観戦側に回りそうだな)");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n永遠亭は今回は薬師が霊夢さんたちに付くようです。強敵ですよ");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n一体どういう風の吹き回しだ？");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n大方、月のお姫様にウサギじゃ心許無いだの言われて駆り出されているんでしょう。\n不憫なことです");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n(容易にそれが想像出来るところが嫌だな)");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nさて、竹林を抜けられるようですね。");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nどうも風見幽香が人を集めているらしいので、太陽の畑へ行きましょう。\n面白いものが見られる気がします");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n私はついていくだけですよっと");
			ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Back, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Rin[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Yuyuko[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowImage(Miko[0], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【文】\nこれはこれは。片方は聖人率いる勢力のようですが、亡霊姫や地底のモノの姿も見えますねえ");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Alice[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Suika[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowImage(Yuka[0], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【妹紅】\nもう片方は魔法の森の人形遣いに、鬼だな");
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			//ui.ShowImage(Suika[1], StoryUIImagePosition.Back, true, false, true);
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【文】\n鬼ですね");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【妹紅】\n……鬼、苦手そうだな");
			ui.ShowText("【文】\nはい……");
			//ui.ShowImage(-1, StoryUIImagePosition.Back, true, false, true);
			ui.ShowText("【妹紅】\nまぁ気づかれないよう、もう少し近づこう");
			/*
			ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);
			
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			*/
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);

			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【妹紅】\nいくらなんでも近づきすぎじゃないの？");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【文】\nせっかくここまで来たのですから\nちょっといたずらしていかないかと");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n…わざわざ大回りして聖人の方の陣地に来たのは？");
			//ui.ShowImage(Suika[1], StoryUIImagePosition.Back, true, false, true);
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n藪をつついて蛇を出すわけにはいきませんから");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nやぶ蛇って分かっててやるのね");
			ui.ShowImage(-1, StoryUIImagePosition.Back, true, false, true);
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nま、雇い主様のご要望だ、無茶ぶりでなければやらねばならぬ");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n無茶ぶりしたら？");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n不死でも痛い、痛い思いは嫌なもの");
			ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n背中に気をつけるのを肝に銘じるべきじゃない？");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n名君でいること肝に銘じておきましょう\n見限られてはたまりません");
			ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【妹紅】\n具体的にはどうするわけ？");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【文】\nこの後に聖人側の補給陣地があります\nちょっとあれにちょっかいかけてみましょう");
			//ui.ShowImage(Utsuho[0], StoryUIImagePosition.Back, true, false, true);
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nいきなり後ろから殴られたら、\nそりゃ何処から沸いたと戸惑うだろうね");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nええ、私が飛んで敵陣を偵察します\n妹紅さんを誘導しますので、適当に施設を壊しちゃってくださいな");
			//ui.ShowImage(Utsuho[0], StoryUIImagePosition.Back, false, false, true);
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n地獄烏がこっちの方を担当ですし好都合\n式神が警戒してますがまぁすぐに見つかることはないかと");
			ui.ShowImage(-1, StoryUIImagePosition.Back, true, false, true);
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n危ないのは私だけ\nやれやれ不死使いの荒い天狗様だ");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nまぁまぁ、安全確認は行いますから");
			ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nあんたこそ、撃ち落とされないように気をつけなさいな");
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, true);
			ui.ShowImage(Aya[0], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【文】\nさて、今回のミッションで重要なのは妹紅さんへの命令です");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Front, false, false, true);
			ui.ShowText("【文】\n「停止」と「待機」\nそれぞれをショートカットに入れておくことをお勧めしますよ？");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Front, false, false, true);
			ui.ShowText("【文】\n待機命令をしておかないと勝手に近くの敵に戦いに行ってしまいますからね");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Front, false, false, true);
			ui.ShowText("【文】\n「停止」で立ち止まって\n「待機」で待っててもらいます");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Front, false, false, true);
			ui.ShowText("【文】\n移動したら待機命令は無くなってしまうので\n移動させるたびにやってくださいね？");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("", false);


			var talkEnd = ui.ShowText("");
			while (!ui.IsCompleted(talkEnd))
				yield return null;

			ui.Hide();
			ui.ResumeGame();
			#endregion


			yield return null;
		}

		private System.Collections.IEnumerable Talk勝利(IWorld world)
		{
			var ui = world.StoryUI;

			#region Talk12
			ui.PauseGame();
			ui.Show();
            ui.ShowImage(Aya[1], StoryUIImagePosition.Left, true, false, true);
            ui.ShowText("【文】\n無事到着ですね、では妹紅さんおもいっきりどうぞ");
            ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, true, false, true);
            ui.ShowText("【妹紅】\nおもいっきりやったら気づかれるでしょ");
            ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);

            ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
            ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
            ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);

			ui.ShowImage(Rin[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【お燐】\nなんか兵の出が悪いような…？");
			ui.ShowImage(Rin[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【お燐】\nって、なんか壊れてる！？\nちょっとお空、なに敵に入られてるのさ！？");
			ui.ShowImage(Utsuho[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【お空】\nえぇ！？入られてないよ！ここで通せんぼしてるんだから！");
			//ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【文】\n十分だと思います。巻き込まれないうちに撤退しましょう");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, true, false, true);

			var talkEnd = ui.ShowText("【妹紅】\nしかし、変な組み合わせ。普段組まないような連中が手を組んでる");
			while (!ui.IsCompleted(talkEnd))
				yield return null;

			world.ChangeBGM(BGMList.Boushuen_Win);

			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n各勢力の垣根を越えての交流イベント。果たしてそれだけなんでしょうかね。八雲紫のことですから、腹の底では何を考えているかわかりません");
			ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n私は報酬が貰えればなんでもいいのさ。さあ次に行こうか");
			ui.ShowText("", false);

			var talkEnd0 = ui.ShowText("", false);
			while (!ui.IsCompleted(talkEnd0))
				yield return null;
			ui.Hide();
			ui.ResumeGame();
			#endregion

			yield return null;
		}

		private System.Collections.IEnumerable Talk敗北(IWorld world)
		{
			var ui = world.StoryUI;

			#region Talk12
			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Utsuho[0], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【お空】\n侵入者発見！");
			ui.ShowText("【お空】\nお燐に誰も入れるないわれてるんだ！\n火力最大、発射ぁ！！");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【文】\n妹紅さん、やけに熱くないですか？");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【妹紅】\n熱いね、暑いんじゃなくて、熱い");
			ui.ShowText("【文】\nついでに向こうの方から大きくなってくる\nお天道様がもう一個見えませんか？");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n見えるね、2つ目の太陽だ");
			ui.ShowText("【文】\n時間かけすぎましたかね？");
			ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n私は一回休みになるね\n今日は烏の丸焼きか");
			ui.ShowImage(Back4, StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("", false);


			var talkEnd0 = ui.ShowText("", false);
			while (!ui.IsCompleted(talkEnd0))
				yield return null;
			ui.Hide();
			ui.ResumeGame();
			#endregion

			yield return null;
		}
		#endregion

		private int[] LoadTachies(IStoryUI ui, string path_N, string path_S, string path_W)
		{
			return new int[3]{
				ui.LoadImage( path_N ),
				ui.LoadImage( path_S ),
				ui.LoadImage( path_W )
			};
		}


		/// <summary>
		/// ユニットが領域上に重なっているか取得する。
		/// </summary>
		/// <returns></returns>
		bool IsUnitOnArea(AreaRectangle region, IUnit unit)
		{
			var startx = Math.Min(region.Start.X, region.End.X) * Def.MapTipLengthOnField;
			var starty = Math.Min(region.Start.Y, region.End.Y) * Def.MapTipLengthOnField;
			var endx = Math.Max(region.Start.X, region.End.X) * Def.MapTipLengthOnField;
			var endy = Math.Max(region.Start.Y, region.End.Y) * Def.MapTipLengthOnField;
			if (unit.Pos.X < endx && startx < unit.Pos.X && unit.Pos.Y < endy && starty < unit.Pos.Y)
			{
				return true;
			}
			return false;
		}

		/// <summary>
		/// ユニットを取得する。
		/// </summary>
		/// <param name="world"></param>
		/// <param name="faction"></param>
		/// <param name="paramType"></param>
		/// <returns></returns>
		public IEnumerable<IUnit> GetUnits(IWorld world, IFaction faction, Type paramType)
		{
			return world.UnitManager.GetUnits(_ => _.Faction == faction && _.Parameter.GetType() == paramType);
		}
	}
}

#endif