﻿#if !_TRIAL

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft.Story
{
	public class StoryWhite04 : thrts.Mission
	{
		#region State

		abstract class StoryState
		{
			#region フィールド

			protected StoryWhite04 Owner { get; set; }
			protected IWorld World { get; set; }
			protected IStoryUI ui { get; set; }
			protected IBuilding PlayerTeamsBase
			{
				get
				{
					if( Owner.whiteBase.Exists() && Owner.blueBase.Exists() )
					{
						var i = World.NextRand( 0, 2 );
						if( i == 0 )
							return Owner.whiteBase;
						else
							return Owner.blueBase;
					}
					else if( Owner.whiteBase.Exists() )
					{
						return Owner.whiteBase;
					}
					else if( Owner.blueBase.Exists() )
					{
						return Owner.blueBase;
					}
					else
					{
						return null;
					}
				}
			}

			protected int[] Reimu
			{
				get { return Owner.Reimu; }
			}
			protected int[] Marisa
			{
				get { return Owner.Marisa; }
			}
			protected int[] Akyu
			{
				get { return Owner.Akyu; }
			}
			protected int[] Hijiri
			{
				get { return Owner.Hijiri; }
			}
			protected int[] Aya
			{
				get { return Owner.Aya; }
			}
			protected int[] Sanae
			{
				get { return Owner.Sanae; }
			}
			protected int[] Remilia
			{
				get { return Owner.Remilia; }
			}
			protected int[] Mokou
			{
				get { return Owner.Mokou; }
			}
			protected int[] Sakuya
			{
				get { return Owner.Sakuya; }
			}
			protected int[] Yamame
			{
				get { return Owner.Yamame; }
			}
			protected int[] Yuyuko
			{
				get { return Owner.Yuyuko; }
			}
			protected int[] Miko
			{
				get { return Owner.Miko; }
			}
			protected int[] Suika
			{
				get { return Owner.Suika; }
			}
			protected int[] Yuka
			{
				get { return Owner.Yuka; }
			}
			protected int[] Alice
			{
				get { return Owner.Alice; }
			}
			protected int[] Tenshi
			{
				get { return Owner.Tenshi; }
			}
			protected int[] Koishi
			{
				get { return Owner.Koishi; }
			}
			protected int[] Yukari
			{
				get { return Owner.Yukari; }
			}

			protected IEnumerator talkFiber;
			protected IEnumerator tacticsFiber;

			protected Area[] SukimaProductArea { get; set; }

			#endregion

			public StoryState( StoryWhite04 owner, IWorld world )
			{
				this.Owner = owner;
				this.World = world;
				ui = world.StoryUI;
				SukimaProductArea = new[]{
					Owner.areas["スキマ生産点１"],
					Owner.areas["スキマ生産点２"],
					Owner.areas["スキマ生産点３"],
				};

				talkFiber = Talk();
				tacticsFiber = Tactics();
			}

			public virtual void Update()
			{
				if( talkFiber != null )
				{
					if( !talkFiber.MoveNext() )
						talkFiber = null;
				}
				else
				{
					tacticsFiber.MoveNext();
				}
			}
			protected abstract IEnumerator Talk();
			protected virtual IEnumerator Tactics()
			{
				var blue = TacticsBlue();
				var red = TacticsRed();
				var green = TacticsGreen();
				var black = TacticsBlack();

				while( true )
				{
					blue.MoveNext();
					red.MoveNext();
					green.MoveNext();
					black.MoveNext();
					++Owner.count;
					{
						int restTime = timeLimit - Owner.count;
						var message = "目的：２つの拠点のどちらかが残るよう防衛する。\n";
						message += string.Format( "残り時間　{0:00}:{1:00}", restTime / 1800, restTime / 30 % 60 );
						World.StoryUI.ShowConditionText( message );
					}
					yield return null;
				}
			}

			protected abstract IEnumerator TacticsBlue();
			protected abstract IEnumerator TacticsRed();
			protected abstract IEnumerator TacticsGreen();
			protected abstract IEnumerator TacticsBlack();

			#region Utility
			protected IEnumerable<IUnit> PutUnit<TUnit>( IFaction faction, Area area, int num ) where TUnit : UnitParameter
			{
				var rect = area as AreaRectangle;
				if( rect == null )
					return new IUnit[0];

				List<IUnit> units = new List<IUnit>();
				var type = typeof( TUnit );

				var sx = Math.Min( rect.Start.X, rect.End.X );
				var sy = Math.Min( rect.Start.Y, rect.End.Y );
				var ex = Math.Max( rect.Start.X, rect.End.X );
				var ey = Math.Max( rect.Start.Y, rect.End.Y );

				for( var y = sy; y < ey; y++ )
				{
					for( var x = sx; x < ex; x++ )
					{
						if( units.Count >= num )
							goto LoopEnd;

						var u = World.UnitManager.Create(
							type.FullName,
							faction,
							( x + 0.5f ) * thrts.Def.MapTipLengthOnField,
							( y + 0.5f ) * thrts.Def.MapTipLengthOnField,
							faction );

						if( u != null )
							units.Add( u );
					}
				}
			LoopEnd:

				return units;
			}

			protected IEnumerable<IUnit> PutUnit<TUnit>( IFaction faction, Vector2D pos, int num ) where TUnit : UnitParameter
			{
				List<IUnit> units = new List<IUnit>();
				var type = typeof( TUnit );

				for( int i = 0; i < num; i++ )
				{
					var u = World.UnitManager.Create(
						type.FullName,
						faction,
						pos.X + 0.4f * i * thrts.Def.MapTipLengthOnField,
						pos.Y + 0.4f * i * thrts.Def.MapTipLengthOnField,
						faction );
				}

				return units;
			}

			protected IEnumerable<IUnit> PutUnit( IFaction faction, UnitParameter parameter, Area area, int num )
			{
				var rect = area as AreaRectangle;
				if( rect == null )
					return new IUnit[0];

				List<IUnit> units = new List<IUnit>();

				var sx = Math.Min( rect.Start.X, rect.End.X );
				var sy = Math.Min( rect.Start.Y, rect.End.Y );
				var ex = Math.Max( rect.Start.X, rect.End.X );
				var ey = Math.Max( rect.Start.Y, rect.End.Y );

				for( var y = sy; y < ey; y++ )
				{
					for( var x = sx; x < ex; x++ )
					{
						if( units.Count >= num )
							goto LoopEnd;

						var u = World.UnitManager.Create(
							parameter.GetType().FullName,
							faction,
							( x + 0.5f ) * thrts.Def.MapTipLengthOnField,
							( y + 0.5f ) * thrts.Def.MapTipLengthOnField,
							faction );

						if( u != null )
							units.Add( u );
					}
				}
			LoopEnd:

				return units;
			}

			protected IEnumerable<IUnit> PutUnit( IFaction faction, UnitParameter parameter, Vector2D pos, int num )
			{
				List<IUnit> units = new List<IUnit>();

				for( int i = 0; i < num; i++ )
				{
					var u = World.UnitManager.Create(
						parameter,
						( pos.X + 0.4f * i ) * thrts.Def.MapTipLengthOnField,
						( pos.Y + 0.4f * i ) * thrts.Def.MapTipLengthOnField,
						faction );
					if( u != null )
						units.Add( u );
				}

				return units;
			}

			protected void GotoPlayersBase( IEnumerable<IUnit> units )
			{
				if( PlayerTeamsBase != null )
					World.UnitManager.AttackMove( units.Select( _ => _.UniqueID ), PlayerTeamsBase, false );
			}

			protected void GotoGuardLine( IEnumerable<IUnit> units, int index )
			{
				var lines = new[] { "防衛ライン１", "防衛ライン２", "防衛ライン３" };
				var area = Owner.areas[lines[index]] as AreaRectangle;

				var x = World.NextRand( Math.Min( area.Start.X, area.End.X ), Math.Max( area.Start.X, area.End.X ) );
				var y = World.NextRand( Math.Min( area.Start.Y, area.End.Y ), Math.Max( area.Start.Y, area.End.Y ) );
				World.UnitManager.AttackMove( units.Select( _ => _.UniqueID ), Def.MapTipLengthOnField * new Vector2D( x, y ), false );
			}

			protected Vector2D To2D( Vector3D pos )
			{
				return new Vector2D( pos.X, pos.Y );
			}


			protected IEnumerable<IUnit> PutAttakingEnemy( IFaction faction, UnitParameter parameter, int num )
			{
				Area area = null;
				if( faction.Index == Owner.redFaction.Index )
					area = Owner.areas["赤勢力スタート"];
				else if( faction.Index == Owner.greenFaction.Index )
					area = Owner.areas["緑勢力スタート"];
				else if( faction.Index == Owner.blackFaction.Index )
					area = Owner.areas["黒勢力スタート"];
				else
					throw new ArgumentException( "指定された勢力 \"{0}\" が敵勢力ではありません。", faction.Name );

				var units = PutUnit( faction, parameter, area, num );
				GotoPlayersBase( units );
				return units;
			}

			protected bool IsBluePopulationOver()
			{
				return World.UnitManager.GetUnits( _ => _.Faction.Index == Owner.blueFaction.Index ).Count() > blueUnitMax;
			}

			public IEnumerable BlueLatterTactics( int callSpan, int amount, params UnitParameter[] parameters )
			{
				int count = 0;

				for( int i = 0; ; i++ )
				{
					int sukimaIndex = i % Owner.sukimas.Length;
					if( Owner.sukimas[sukimaIndex].Exists() )
					{
						var units = new List<IUnit>();
						for( int j = 0; j < amount; j++ )
						{
							units.AddRange( PutUnit( Owner.blueFaction, parameters[count], SukimaProductArea[sukimaIndex], 1 ) );
							count = ( count + 1 ) % parameters.Length;
						}
						GotoGuardLine( units, i % 3 );
					}

					for( int j = 0; j < callSpan; j++ )
						yield return null;

					while( IsBluePopulationOver() )
						yield return null;
				}
			}

			public void BringBlueUnitsToLine()
			{
				var units = World.UnitManager.GetUnits( _ => _.Faction.Index == Owner.blueFaction.Index );
				var c = units.Count() / 3;
				for( int i = 0; i < 3; i++ )
				{
					GotoGuardLine( units.Skip( i * c ).Take( c ), i );
				}
			}
			#endregion
		}

		class IntroductionState : StoryState
		{
			public IntroductionState( StoryWhite04 owner, IWorld world )
				: base( owner, world )
			{
			}

			protected override IEnumerator Talk()
			{
				ui.PauseGame();
				ui.Show();
				ui.ShowText( "～神社にて～" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【魔理沙】\nよう。で、今日はどうするんだ？" );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\nどうしましょうね" );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【魔理沙】\n相変わらずやる気がないな" );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【霊夢】\nそうでもないわ\nむしろやる気に満ち溢れてるかも" );
				ui.ShowText( "【霊夢】\nそれに、そろそろあいつらのほうからやってくる頃じゃない？" );
				ui.ShowText( "【魔理沙】\n？" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Aya[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Sanae[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowImage( Remilia[1], StoryUIImagePosition.Front, true, false, true );
				var forBgm = ui.ShowText( "【レミリア】\nさあ、約束の再戦よ！" );
				while( !ui.IsCompleted( forBgm ) )
					yield return null;
				World.ChangeBGM( BGMList.Boushuen_Story3 );

				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Yamame[1], StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Yuyuko[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowImage( Miko[1], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【神子】\n聖白蓮と博麗の巫女ならここにいると聞いて" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Suika[1], StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Yuka[0], StoryUIImagePosition.Right, true, false, true );
				ui.ShowImage( Alice[1], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【アリス】\n悪く思わないでよね\nまずは一番面倒な霊夢たちに当たるって決まったの" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\nほら、勢ぞろい" );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【魔理沙】\nでもさすがに、三対一をやろうなんてことはないよな？" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Remilia[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【レミリア】\nさすがに、ねえ？" );
				ui.ShowImage( Alice[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【アリス】\nそこまで私は鬼じゃないつもりだけど" );
				ui.ShowImage( Miko[1], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【神子】\n連戦でやってもらうつもりではありましたけどね\n勝てれば良し。別のチームが負ければそれはそれで良し" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\n別にみんな纏めてでもいいわよ" );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【魔理沙】\n本気か？" );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【霊夢】\nだって、そのほうが面白いじゃない。気を抜いてやりましょうよ" );
				ui.ShowText( "【霊夢】\nそれで勝ち残ったら、紫になんかくれって詰め寄ればいいのよ\n財宝があろうがなかろうと、別に遊ぶのに理由はいらないんじゃないの" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Tenshi[1], StoryUIImagePosition.Left, true, false, false );
				ui.ShowImage( Koishi[1], StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( Yukari[0], StoryUIImagePosition.Front, true, false, true );
				ui.ShowText( "【紫】\nあら、私たちを仲間外れにするだなんて酷いのではなくって？" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\nこれで四対一か？" );
				ui.ShowImage( Yukari[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【紫】\n今回は霊夢たちに付こうかしら。手っ取り早く、頂上決戦まで行きましょうよ" );
				ui.ShowImage( Miko[2], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【神子】\nまぁ私は、命蓮寺と戦えればそれで良いので" );
				ui.ShowImage( Remilia[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【レミリア】\n勝てば一緒よ" );
				ui.ShowImage( Alice[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【アリス】\nまぁ私たちのチームにも不満はなさそうだし、それで構わないわ" );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【霊夢】\n別に私は三対一でも良かったけどね" );
				ui.ShowImage( Yukari[0], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【紫】\nそれでも三対二はちょっとこっちが不利だから、変則ルールにしましょう" );
				ui.ShowImage( Miko[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【神子】\nどのような内容でしょう？" );
				ui.ShowImage( Yukari[1], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【紫】\n一定時間耐え凌げばこちらの勝ち\nそれまでにこちらの本陣を潰せればそちらの勝ち\n戦力差を考えれば適正でしょう？" );
				ui.ShowImage( Remilia[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【レミリア】\nなんにせよ勝つだけだ" );
				ui.ShowImage( Alice[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【アリス】\n構わないわ" );
				ui.ShowImage( Miko[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【神子】\nほかの方が良いのであれば、私も" );
				ui.ShowImage( Yukari[0], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【紫】\n決まりね" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Left, true, false, true );
				ui.ShowText( "【魔理沙】\nいつのまにか紫が仕切ってるな" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【魔理沙】\nまぁ発案者だし、別にいいのか" );
				ui.ShowImage( Hijiri[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【白蓮】\nこの勝負、負けられませんね" );
				ui.ShowImage( Marisa[2], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【魔理沙】\nなぁ、なんであんなに目の敵にされてるんだ？" );
				ui.ShowImage( Hijiri[2], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【白蓮】\nそれはやんごとなき理由があるのですが……" );
				ui.ShowImage( Hijiri[1], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【白蓮】\nとにかく、あの者だけには負けられない理由があるのです" );
				ui.ShowImage( Akyu[1], StoryUIImagePosition.Right, true, false, true );
				ui.ShowText( "【阿求】\n色々因縁のある戦いになりそうですね" );
				ui.ShowImage( Akyu[2], StoryUIImagePosition.Right, false, false, true );
				ui.ShowText( "【阿求】\n幽香さんもやる気十分の表情ですし、\n宴会以外でここまでの数が博麗神社に集まったことが過去あったでしょうか" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Left, false, false, true );
				ui.ShowText( "【魔理沙】\n宴会以外で人が居ないってことが致命的だな" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
				var talkEnd = ui.ShowText( "", false );
				while( !ui.IsCompleted( talkEnd ) )
					yield return null;

				ui.Hide();
				ui.ResumeGame();

				yield return null;
			}
			protected override IEnumerator Tactics()
			{
				Owner.state = new FightingState( Owner, World );
				yield return null;
			}
			protected override IEnumerator TacticsBlue()
			{
				yield return null;
			}
			protected override IEnumerator TacticsRed()
			{
				yield return null;
			}
			protected override IEnumerator TacticsGreen()
			{
				yield return null;
			}
			protected override IEnumerator TacticsBlack()
			{
				yield return null;
			}
		}

		class FightingState : StoryState
		{
			const int blueSpan = 1550;
			const int redSpan = 1200;
			const int greenSpan = 1500;
			const int blackSpan = 1350;
			const int waitRate = 10;

			public FightingState( StoryWhite04 owner, IWorld world )
				: base( owner, world )
			{
			}

			protected override IEnumerator Talk()
			{
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【紫】\nそろそろ開始といこうかしら？" );
				ui.ShowImage( Miko[1], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【神子】\nこちらは完了しています\nして、制限時間は如何程を？" );
				ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【紫】\nそうね、四半時余りくらいならいいかしら？" );
				ui.ShowImage( Suika[1], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【萃香】\nわりと長く取ったね、余裕の表れかい？" );
				ui.ShowImage( Sanae[2], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【早苗】\nええと、一刻が2時間、一時が1時間だから…" );
				ui.ShowImage( Alice[1], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【アリス】\n15分強、20分といったところね" );
				ui.ShowImage( Remilia[1], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【レミリア】\nならその半分で突破してみせよう" );
				ui.ShowImage( Yuka[0], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【幽香】\nあら、見栄を張ると後で後悔するわよ？" );
				ui.ShowImage( Remilia[1], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【レミリア】\nご忠告どうも\n足を引っ張られなければ、と付け加えるべきかしら？" );
				ui.ShowImage( Suika[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【萃香】\n組むんだから少しは仲良くしなよアンタら" );
				ui.ShowImage( Yuyuko[0], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【幽々子】\nそう？仲が良い様に見えるけど？" );
				ui.ShowImage( Yuyuko[1], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【幽々子】\n紫？こっちはみんなその時間でよさそうよ？" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true );
				ui.ShowImage( Yukari[1], StoryUIImagePosition.Front, true, false );
				ui.ShowText( "【紫】\nでは始めましょう？\n幻想郷の戦争ごっこ、新しい遊びの親睦会を" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【霊夢】\nホントに置いてけぼりね" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【魔理沙】\nいや、そうでもないさ" );
				ui.ShowText( "【霊夢】\n？" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, false, false );
				ui.ShowText( "【魔理沙】\nあいつら全員、霊夢目当てで来たんだろ？\n全員相手してやるって啖呵きったんだ、答えてやらないとな" );
				ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false );
				ui.ShowText( "【霊夢】\nまぁそうね" );
				ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, false, false );
				ui.ShowText( "【霊夢】\nそれじゃあ20分、凌ぎきるわよみんな！" );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true );
				var talkEnd = ui.ShowImage( -1, StoryUIImagePosition.Left, true );
				while( !ui.IsCompleted( talkEnd ) )
					yield return null;

				ui.Hide();
				ui.ResumeGame();

				World.ChangeBGM( BGMList.Battle_Hakurei );

				yield return null;
			}
			protected override IEnumerator TacticsBlue()
			{
				GotoGuardLine( PutUnit<Unit.Blue.Unique.Koishi>( Owner.blueFaction, Owner.areas["拠点生産点"], 1 ), 0 );

				foreach( var item in BuildSukimas() )
					yield return null;

				for( int i = 0; i < blueSpan * ( waitRate - 1 ); i++ )
					yield return null;

				BringBlueUnitsToLine();

				var parameters = new UnitParameter[]{
					Owner.blueFaction.GetUnitParameter( typeof( Unit.Blue.Sunny ).FullName ),
					Owner.blueFaction.GetUnitParameter( typeof( Unit.Blue.Lunar ).FullName ),
					Owner.blueFaction.GetUnitParameter( typeof( Unit.Blue.Star ).FullName ),
				};
				foreach( var item in BlueLatterTactics( blueSpan, 3, parameters ) )
					yield return null;
			}
			protected override IEnumerator TacticsRed()
			{
				var spies = new[]{
					typeof( Unit.Red.Low01 ),
				};
				var units = Enumerable.Repeat( typeof( Unit.Red.Kanako ), 3 );
				var heroins = new[]{
					typeof( Unit.Red.Unique.Remilia ),
					typeof( Unit.Red.Unique.Sanae ),
				};

				foreach( var item in EnemyTactics( Owner.redFaction, redSpan, spies, units, heroins ) )
					yield return null;
			}
			protected override IEnumerator TacticsGreen()
			{
				var spies = new[]{
					typeof( Unit.Green.Low01 ),
				};
				var units = new[]{
					typeof( Unit.Green.Komachi ),
					typeof( Unit.Green.Komachi ),
					typeof( Unit.Green.Medicine ),
					typeof( Unit.Green.Medicine ),
				};
				var heroins = new[]{
					typeof( Unit.Green.Unique.Alice ),
					typeof( Unit.Green.Unique.Suika ),
				};

				foreach( var item in EnemyTactics( Owner.greenFaction, greenSpan, spies, units, heroins ) )
					yield return null;
			}
			protected override IEnumerator TacticsBlack()
			{
				var spies = new[]{
					typeof( Unit.Black.ZombieFairy ),
				};
				var units = new[]{
					typeof( Unit.Black.Orin ),
					typeof( Unit.Black.Orin ),
					typeof( Unit.Black.Oku ),
				};
				var heroins = new[]{
					typeof( Unit.Black.Unique.Miko ),
					typeof( Unit.Black.Unique.Yuyuko ),
				};

				foreach( var item in EnemyTactics( Owner.blackFaction, blackSpan, spies, units, heroins ) )
					yield return null;
			}

			protected IEnumerable BuildSukimas()
			{
				Func<Area, Vector2D> getCenter = _ =>
					{
						var rect = _ as AreaRectangle;
						return ( rect.Start.ToVector2D() + rect.End.ToVector2D() ) / 2;
					};
				var yukariUnit = World.UnitManager.GetUnits( _ => _.Parameter.GetType() == typeof( Unit.Blue.Unique.Yukari ) ).First();
				var buildAreas = new[] { Owner.areas["スキマ１"], Owner.areas["スキマ２"], Owner.areas["スキマ３"] };

				Owner.sukimas = new IBuilding[buildAreas.Length];

				for( int j = 0; j < Owner.sukimas.Length; j++ )
				{
					World.UnitManager.SelectTarget(
						yukariUnit.ToSequence(),
						getCenter( SukimaProductArea[j] ) * Def.MapTipLengthOnField,
						false );

					for( int i = 0; i < 150; i++ )
						yield return null;

					yukariUnit.Stop( false );

					var building = World.BuildingManager.Create(
						typeof( Building.Blue.SukimaBuilding ).FullName,
						Owner.blueFaction,
						getCenter( buildAreas[j] ).ToPoint2D().X,
						getCenter( buildAreas[j] ).ToPoint2D().Y,
						0,
						null );
					building.StartConstruction( null, 0 );

					Owner.sukimas[j] = building;
				}

				World.UnitManager.AttackMove(
					yukariUnit.UniqueID.ToSequence(),
					getCenter( Owner.areas["拠点生産点"] ) * Def.MapTipLengthOnField,
					false );
				yield return null;
			}

			protected IEnumerable EnemyTactics(
				IFaction faction,
				int callSpan,
				IEnumerable<Type> spies,
				IEnumerable<Type> units,
				IEnumerable<Type> heroins )
			{
				UnitParameter[] unitParameters = units.Select( _ => faction.GetUnitParameter( _.FullName ) ).ToArray();
				var heroinParameters = new Queue<UnitParameter>( heroins.Select( _ => faction.GetUnitParameter( _.FullName ) ) );

				for( int i = 0; ; i++ )
				{
					if( Owner.restTime < Def.FPS * 60 * 10 )
					{
						unitParameters.ForEach( _ => PutAttakingEnemy( faction, _, 1 ) );
						if( heroinParameters.Any() )
						{
							PutAttakingEnemy( faction, heroinParameters.Dequeue(), 1 );
						}
					}
					else if( i >= waitRate )
					{
						spies.ForEach( _ => PutAttakingEnemy( faction, faction.GetUnitParameter( _.FullName ), 1 ) );
						PutAttakingEnemy( faction, unitParameters.First(), 1 );
					}
					else if( i >= 3 )
					{
						spies.ForEach( _ => PutAttakingEnemy( faction, faction.GetUnitParameter( _.FullName ), 1 ) );
					}
					for( int j = 0; j < callSpan; j++ )
						yield return null;
				}
			}
		}

		class LastState : StoryState
		{
			static readonly int blueSpan = 1700;
			static readonly int enemySpan = 1800;
			static readonly int enemyAmount = 3;

			public LastState( StoryWhite04 owner, IWorld world )
				: base( owner, world )
			{
			}

			protected override IEnumerator Talk()
			{
				ui.PauseGame();
				ui.Show();
				ui.ShowImage( Alice[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【アリス】\n不味いわね、思ってた以上に守りが堅いわ" );
				ui.ShowImage( Mokou[2], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【妹紅】\n輝夜のところの式神が面倒ね、弾幕は厚いし回復もする" );
				ui.ShowImage( Miko[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【神子】\n毘沙門天の硬さも厄介ですね" );
				ui.ShowImage( Remilia[2], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【レミリア】\n咲夜！時間は！？" );
				ui.ShowImage( Sakuya[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【咲夜】\n間もなく15分が経過、\n残念ですがお嬢様の宣言は疾うの昔に過ぎてしまっています" );
				ui.ShowImage( Yuka[0], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【幽香】\n半分で突破だったかしら？" );
				ui.ShowImage( Remilia[1], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【レミリア】\nふん、その口ぶりは自分が突破できてから言うことね" );
				ui.ShowImage( Aya[2], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【文】\n始まる前から想像ついてましたが足並み揃ってませんね" );
				ui.ShowImage( Yuyuko[1], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【幽々子】\n紫もそれを見越しての提案だったかもしれないわ" );
				ui.ShowImage( Yuyuko[0], StoryUIImagePosition.Left, false, false );
				ui.ShowText( "【幽々子】\nね、せっかくみんなで組んでるわけだし\n足並み揃えて霊夢や紫を驚かしてみない？" );
				ui.ShowImage( Alice[2], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【アリス】\nまぁ、このままやってもジリ貧なのは確かね" );
				ui.ShowImage( Suika[1], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【萃香】\nこれで紫の一人勝ち、というのは癪だね" );
				ui.ShowImage( Yuyuko[1], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【幽々子】\nそういうこと、お二人もどうかしら？" );
				ui.ShowImage( Yuka[0], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【幽香】\nついてこれるなら、別に構わないわ" );
				ui.ShowImage( Remilia[1], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【レミリア】\n合わせるような繊細なことはできないと言っているようなものだな\nいいわ、私が合わせてみせようじゃない" );
				ui.ShowImage( Miko[1], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【神子】\n私は構いませんよ、そちらの兵の聞き分け程度たやすいことです" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true );
				ui.ShowImage( -1, StoryUIImagePosition.Right, true );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true );
				ui.ShowImage( Koishi[1], StoryUIImagePosition.Front, true, false );
				ui.ShowText( "【こいし】\nとか言ってたわよ？" );
				ui.ShowImage( -1, StoryUIImagePosition.Front, true );
				ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, true, false );
				ui.ShowText( "【霊夢】\nなんだかんだで仲良くやってるじゃないの" );
				ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, true, false );
				ui.ShowText( "【魔理沙】\n遊びというのは仲良く楽しむものだ\nそら、もうすぐ来るぞこっちも準備しないとな" );
				ui.ShowImage( -1, StoryUIImagePosition.Left, true );
				var talkEnd = ui.ShowImage( -1, StoryUIImagePosition.Right, true );
				while( !ui.IsCompleted( talkEnd ) )
					yield return null;

				ui.Hide();
				ui.ResumeGame();

				yield return null;
			}
			protected override IEnumerator TacticsBlue()
			{
				{
					var heroins = new List<IUnit>();
					heroins.AddRange( PutUnit<Unit.Blue.Unique.Chirno>( Owner.blueFaction, Owner.areas["拠点生産点"], 1 ) );
					heroins.AddRange( PutUnit<Unit.Blue.Unique.Tenshi>( Owner.blueFaction, Owner.areas["拠点生産点"], 1 ) );
					heroins.ForEach( _ => _.AddExperience( 3000 ) );
				}

				BringBlueUnitsToLine();

				var parameters = new UnitParameter[]{
					Owner.blueFaction.GetUnitParameter( typeof( Unit.Blue.Iku ).FullName ),
					Owner.blueFaction.GetUnitParameter( typeof( Unit.Blue.Sunny ).FullName ),
					Owner.blueFaction.GetUnitParameter( typeof( Unit.Blue.Star ).FullName ),
					Owner.blueFaction.GetUnitParameter( typeof( Unit.Blue.Lunar ).FullName ),
				};
				foreach( var item in BlueLatterTactics( blueSpan, 5, parameters ) )
					yield return null;
			}
			protected override IEnumerator TacticsRed()
			{
				var heroins = new[]
				{
					typeof( Unit.Red.Unique.Remilia ),
					typeof( Unit.Red.Unique.Sanae ),
					typeof( Unit.Red.Unique.Aya ),
					typeof( Unit.Red.Unique.Moko ),
					typeof( Unit.Red.Unique.Sakuya ),
					typeof( Unit.Red.Unique.Patchouli ),
				};
				var units = new[]
				{
					typeof( Unit.Red.Flandre ),
					typeof( Unit.Red.Suwako ),
				};

				foreach( var item in EnemyTactics( Owner.redFaction, heroins, units ) )
					yield return null;
			}
			protected override IEnumerator TacticsGreen()
			{
				var heroins = new[]
				{
					//typeof( Unit.Green.Unique.Yuka ),
					typeof( Unit.Green.Unique.Alice ),
					typeof( Unit.Green.Unique.Suika ),
					typeof( Unit.Green.Unique.Kasen ),
					typeof( Unit.Green.Unique.Lyrica ),
					typeof( Unit.Green.Unique.Minoriko ),
				};
				var units = new[]
				{
					typeof( Unit.Green.SuikaSmall ),
					typeof( Unit.Green.Medicine ),
				};

				foreach( var item in EnemyTactics( Owner.greenFaction, heroins, units ) )
					yield return null;
			}
			protected override IEnumerator TacticsBlack()
			{
				var heroins = new[]
				{
					typeof( Unit.Black.Unique.Miko ),
					typeof( Unit.Black.Unique.Yuyuko ),
					typeof( Unit.Black.Unique.Yamame ),
					typeof( Unit.Black.Unique.Seiga ),
					typeof( Unit.Black.Unique.Utsuho ),
					typeof( Unit.Black.Unique.Rin ),
				};
				var units = new[]
				{
					typeof( Unit.Black.Satori ),
					typeof( Unit.Black.Youmu ),
				};

				foreach( var item in EnemyTactics( Owner.blackFaction, heroins, units ) )
					yield return null;
			}

			private IEnumerable EnemyTactics(
				IFaction faction,
				IEnumerable<Type> heroinTypes,
				IEnumerable<Type> unitTypes )
			{
				var heroinQueue = new Queue<UnitParameter>( heroinTypes.Select( _ => faction.GetUnitParameter( _.FullName ) ) );
				var parameters = unitTypes.Select( _ => faction.GetUnitParameter( _.FullName ) );

				World.UnitManager.GetUnits( _ => heroinQueue.Contains( _.Parameter ) ).ForEach( _ => _.HP = 0 );

				while( true )
				{
					parameters.ForEach( _ => PutAttakingEnemy( faction, _, enemyAmount ) );
					if( heroinQueue.Any() )
					{
						var h = PutAttakingEnemy( faction, heroinQueue.Dequeue(), 1 ).FirstOrDefault();
						if( h != null )
						{
							h.AddExperience( 3000 );
						}
					}

					for( int i = 0; i < enemySpan; i++ )
						yield return null;
				}
			}
		}

		#endregion

		#region フィールド
		static readonly int timeLimit = Def.FPS * 60 * 20;
		static readonly int blueUnitMax = 20;
		static readonly int[] FirstResoure = new List<int> { 500, 200, 200 }.ToArray();
		static readonly float[] HPRate = new List<float> { 0.5f, 1, 1.2f }.ToArray();

		int[] Reimu = null;
		int[] Marisa = null;
		int[] Akyu = null;
		int[] Hijiri = null;
		int[] Ichirin = null;

		int[] Aya = null;
		int[] Sanae = null;
		int[] Remilia = null;
		int[] Mokou = null;
		int[] Sakuya = null;

		int[] Yamame = null;
		int[] Yuyuko = null;
		int[] Miko = null;

		int[] Suika = null;
		int[] Yuka = null;
		int[] Alice = null;

		int[] Tenshi = null;
		int[] Koishi = null;
		int[] Yukari = null;

		int count = 0;
		int restTime
		{
			get { return timeLimit - count; }
		}
		StoryState state { get; set; }
		Dictionary<string, Area> areas = new Dictionary<string, Area>();

		IBuilding whiteBase;
		IBuilding blueBase;
		IBuilding[] sukimas;

		IFaction blueFaction;
		IFaction redFaction;
		IFaction greenFaction;
		IFaction blackFaction;
		#endregion

		public override string[] GetSelectableFactions()
		{
			return new string[]{
				typeof( Faction.White ).FullName,
			};
		}
		public override string[] GetEnableFactions()
		{
			return new string[]{
				typeof( Faction.White ).FullName,
				typeof( Faction.Red ).FullName,
				typeof( Faction.Blue ).FullName,
				typeof( Faction.Green ).FullName,
				typeof( Faction.Black ).FullName,
			};
		}
		public override string[] GetSelectableMaps()
		{
			return new string[]{
				"LotusCraft/Story/StoryWhite04.map"
			};
		}

		public StoryWhite04()
			: base()
		{
			Name = "白4話";
			Description = "白4話専用マップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		public override void OnInitialize( IWorld world )
		{
			var ui = world.StoryUI;

			blueFaction = world.Factions[1];
			redFaction = world.Factions[2];
			greenFaction = world.Factions[3];
			blackFaction = world.Factions[4];

			world.Factions[1].PlayerName = "青勢力";
			world.Factions[2].PlayerName = "赤勢力";
			world.Factions[3].PlayerName = "緑勢力";
			world.Factions[4].PlayerName = "黒勢力";

			world.Factions[0].DoesNotLose = true;
			world.Factions[1].DoesNotLose = true;
			world.Factions[2].DoesNotLose = true;
			world.Factions[3].DoesNotLose = true;
			world.Factions[4].DoesNotLose = true;

			// 本拠の位置を初期視点にする
			var basePos = world.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( LotusCraft.Building.White.HomeBase ) ).First().Pos;
			// 初期視点位置
			world.Factions[0].Parameter.DefaultScreenPosition = new Point2D( (int)basePos.X, (int)basePos.Y );

			Reimu = LoadTachies( world, "LotusCraft/Image/Tachie/", "Reimu_N", "Reimu_S", "Reimu_W" );
			Marisa = LoadTachies( world, "LotusCraft/Image/Tachie/", "Marisa_N", "Marisa_S", "Marisa_W" );
			Akyu = LoadTachies( world, "LotusCraft/Image/Tachie/", null, "Akyu_S", "Akyu_W" );
			Hijiri = LoadTachies( world, "LotusCraft/Image/Tachie/", "Hijiri_N", "Hijiri_S", "Hijiri_W" );
			Ichirin = LoadTachies( world, "LotusCraft/Image/Tachie/", null, "Ichirin_S", null );

			Aya = LoadTachies( world, "LotusCraft/Image/Tachie/", "Aya_N", null, "Aya_W" );
			Sanae = LoadTachies( world, "LotusCraft/Image/Tachie/", null, "Sanae_S", "Sanae_W" );
			Remilia = LoadTachies( world, "LotusCraft/Image/Tachie/", null, "Remilia_S", "Remilia_W" );
			Mokou = LoadTachies( world, "LotusCraft/Image/Tachie/", "Mokou_N", null, "Mokou_W" );
			Sakuya = LoadTachies( world, "LotusCraft/Image/Tachie/", null, null, "Sakuya_W" );

			Suika = LoadTachies( world, "LotusCraft/Image/Tachie/", "Suika_N", "Suika_S", "Suika_W" );
			Yuka = LoadTachies( world, "LotusCraft/Image/Tachie/", "Yuka_N", null, null );
			Alice = LoadTachies( world, "LotusCraft/Image/Tachie/", null, "Alice_S", "Alice_W" );

			Yamame = LoadTachies( world, "LotusCraft/Image/Tachie/", null, "Yamame_S", null );
			Yuyuko = LoadTachies( world, "LotusCraft/Image/Tachie/", "Yuyuko_N", "Yuyuko_S", null );
			Miko = LoadTachies( world, "LotusCraft/Image/Tachie/", "Miko_N", "Miko_S", "Miko_W" );

			Tenshi = LoadTachies( world, "LotusCraft/Image/Tachie/", null, "Tenshi_S", null );
			Koishi = LoadTachies( world, "LotusCraft/Image/Tachie/", null, "Koishi_S", null );
			Yukari = LoadTachies( world, "LotusCraft/Image/Tachie/", "Yukari_N", "Yukari_S", null );

			world.AllPurposeArea.ForEach( _ => areas[_.Name] = _ );

			var sukimaParam = blueFaction.GetBuildingParameter( typeof( Building.Blue.SukimaBuilding ).FullName );
			sukimaParam.Name = "紫のとっておきスキマ";
			sukimaParam.HP = 1500;

			SetDifficulty( world );

			base.OnInitialize( world );
		}

		public override IEnumerable Update( IWorld world )
		{
			var ui = world.StoryUI;
			whiteBase = GetBuildings<Building.White.HomeBase>( world ).First();
			blueBase = GetBuildings<Building.Blue.HomeBase>( world ).First();
			sukimas = new List<IBuilding>( GetBuildings<Building.Blue.SukimaBuilding>( world ) ).ToArray();
			state = new IntroductionState( this, world );

			while( true )
			{
				if( state != null )
				{
					state.Update();
				}

				if( count == Def.FPS * 60 * 15 )
				{
					++count;
					state = new LastState( this, world );
				}

				if( count >= timeLimit )
				{
					state = null;
					foreach( var item2 in Talk勝利( world ) )
					{
						yield return null;
					}
					world.Factions[2].DoesNotLose = false;
					world.Factions[3].DoesNotLose = false;
					world.Factions[4].DoesNotLose = false;
					world.Factions[2].Lose();
					world.Factions[3].Lose();
					world.Factions[4].Lose();
					world.StoryUtilities.SetDataToSaveData( GetClearDataName( world ), "Clear" );
					world.StoryUtilities.SetDataToSaveData("LotusCraft.White.04", "Clear");
					break;
				}
				else if( !whiteBase.Exists() && !blueBase.Exists() )
				{
					state = null;
					foreach( var item2 in Talk敗北( world ) )
					{
						yield return null;
					}
					world.Factions[0].DoesNotLose = false;
					world.Factions[1].DoesNotLose = false;
					world.Factions[0].Lose();
					world.Factions[1].Lose();
					break;
				}

				yield return null;
			}

			yield return null;
		}

		private void SetDifficulty( IWorld world )
		{
			int difficulty = world.StoryUtilities.GetDifficulty();

			// 初期資源を配布
			world.Factions[0].IncResource( 0, FirstResoure[difficulty] );
			world.Factions[0].IncResource( 1, FirstResoure[difficulty] );

			var redTypes = new[]{
				typeof( Unit.Red.Low01 ), typeof( Unit.Red.Low02 ),
				typeof( Unit.Red.Suwako ), typeof( Unit.Red.Kanako ), typeof( Unit.Red.Flandre ),
				typeof( Unit.Red.Unique.Remilia ), typeof( Unit.Red.Unique.Sanae ), typeof( Unit.Red.Unique.Patchouli ),
				typeof( Unit.Red.Unique.Aya ), typeof( Unit.Red.Unique.Moko ), typeof( Unit.Red.Unique.Sakuya ),
			};
			var greenTypes = new[]{
				typeof( Unit.Green.Low01 ),
				typeof( Unit.Green.Komachi ), typeof( Unit.Green.Medicine ), typeof( Unit.Green.SuikaSmall ),
				typeof( Unit.Green.Unique.Alice ), typeof( Unit.Green.Unique.Suika ), typeof( Unit.Green.Unique.Lyrica ),
				typeof( Unit.Green.Unique.Kasen ), typeof( Unit.Green.Unique.Minoriko ),
			};
			var blackTypes = new[]{
				typeof( Unit.Black.ZombieFairy ), typeof( Unit.Black.Orin ), typeof( Unit.Black.Oku ),
				typeof( Unit.Black.Satori ), typeof( Unit.Black.Youmu ),
				typeof( Unit.Black.Unique.Miko ), typeof( Unit.Black.Unique.Seiga ), typeof( Unit.Black.Unique.Yuyuko ),
				typeof( Unit.Black.Unique.Rin ), typeof( Unit.Black.Unique.Utsuho ), typeof( Unit.Black.Unique.Yamame )
			};

			var redParams = redTypes.Select( _ => redFaction.GetUnitParameter( _.FullName ) );
			var greenParams = greenTypes.Select( _ => greenFaction.GetUnitParameter( _.FullName ) );
			var blackParams = blackTypes.Select( _ => blackFaction.GetUnitParameter( _.FullName ) );
			redParams.Concat( greenParams ).Concat( blackParams )
				.ForEach( _ => _.HP = (int)( _.HP * HPRate[difficulty] ) );
		}

		private string GetClearDataName( IWorld world )
		{
			switch( world.StoryUtilities.GetDifficulty() )
			{
			case 0: return "LotusCraft.White.04.Easy";
			case 1: return "LotusCraft.White.04.Normal";
			case 2: return "LotusCraft.White.04.Hard";
			default: throw new InvalidOperationException();
			}
		}


		IEnumerable Talk勝利( IWorld world )
		{
			var ui = world.StoryUI;
			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false );
			ui.ShowText( "【霊夢】\n私たちの勝ちみたいね" );
			ui.ShowImage( Marisa[0], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【魔理沙】\nどうやらそのようだな" );
			ui.ShowImage( Hijiri[0], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【白蓮】\n悪は滅びたのです……" );
			ui.ShowImage( Ichirin[1], StoryUIImagePosition.Left, true, false );
			ui.ShowText( "【一輪】\n姐さんとの愛の力で" );
			ui.ShowImage( Miko[2], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【神子】\n滅びてませんけどね。今回は勝ちを譲りましょう。" );
			ui.ShowImage( Miko[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【神子】\nけれど、まだ終わりませんよね？" );
			ui.ShowImage( Yuka[0], StoryUIImagePosition.Left, true, false );
			ui.ShowText( "【幽香】\nじゃあ次は弾幕ごっこでもしましょうか" );
			ui.ShowImage( Alice[2], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【アリス】\n元気ね。私はパスだわ" );
			ui.ShowImage( Remilia[1], StoryUIImagePosition.Left, true, false );
			ui.ShowText( "【レミリア】\nそうよ！　こんなゴチャ混ぜ混成軍で、しかも耐えればそっちの勝ちなんて納得行かないわ！\nもう一回、もう一回よ！" );
			ui.ShowImage( Mokou[0], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【妹紅】\n私はお金が貰えればなんでもいいんだが" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true );
			ui.ShowImage( Reimu[2], StoryUIImagePosition.Front, true, false );
			ui.ShowText( "【霊夢】\nそれよりも、いい考えがあるわ" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Front, false, false );
			ui.ShowText( "【霊夢】\n飲みましょう。遊ぶのは、またそれからでいいわ" );
			ui.ShowImage( -1, StoryUIImagePosition.Front, true );
			ui.ShowText( "こうして霊夢の懸念は外れ、思った以上に広がりはしたものの、大きな異変にもならず一応の決着を見ることになった。" );
			ui.ShowText( "が、暇に怯える妖怪たちは、博麗の巫女を怒らせない程度にはこの遊びを広めたいと考えているのだった。" );
			ui.ShowText( "長い時を生きる妖怪にとって、何よりも恐ろしいものは退屈である。" );
			ui.ShowText( "気安く遊びに誘えない者同士が、理屈を捏ねては遊びに連れ出して、飽きれば酒を酌み交わす。" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Front, false, false );
			ui.ShowText( "まぁ、何もないならそれでも良いか。" );
			ui.ShowText( "そう思い、今しばらく静観を決め込むことにした霊夢であった。" );
			var talkEnd = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd ) )
				yield return null;

			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}

		IEnumerable Talk敗北( IWorld world )
		{
			var ui = world.StoryUI;
			yield return null;
		}


		int[] LoadTachies( IStoryUI ui, string path_N, string path_S, string path_W )
		{
			return new int[3]{
				ui.LoadImage( path_N ),
				ui.LoadImage( path_S ),
				ui.LoadImage( path_W )
			};
		}

		int[] LoadTachies( IWorld world, string directory, string file_N, string file_S, string file_W, string file_A = null )
		{
			var ui = world.StoryUI;
			var list = new List<int>();

			if( !directory.EndsWith( "/" ) )
				directory += "/";

			Action<string> addPath = _ => list.Add( _ != null ? ui.LoadImage( directory + _ ) : -1 );

			addPath( file_N );
			addPath( file_S );
			addPath( file_W );
			addPath( file_A );
			return list.ToArray();
		}

		IEnumerable<IBuilding> GetBuildings<T>( IWorld world ) where T : BuildingParameter
		{
			return world.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( T ) );
		}
	}
}

#endif