﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
namespace LotusCraft.Technology
{
	public class Technology : thrts.TechnologyParameter<ITargetObject>
	{
		public Technology(string name,string iconPath, int techTime, string[] techTargets, Func<UnitParameter, bool> techUnitAction=null, Func<BuildingParameter, bool> techBuildingAction=null, Func<BulletParameter, bool> techBulletAction=null)
		{
 			this.Name = name;
			this.IconPath = iconPath;
			this.TechTime = techTime;
			this.TechTargets = techTargets;
			this.UnitTechAction = techUnitAction;
			this.BuildingTechAction = techBuildingAction;
			this.BulletTechAction = techBulletAction;
		}
		public override void OnComplete(ITargetObject holder)
		{
			var faction = holder.Faction;
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.EndResearch).FullName, holder.Pos);
			base.OnComplete(holder);
		}
	}

	public class UnitArmorTech : thrts.TechnologyParameter<ITargetObject>
	{
		protected int armorInc = 2;
		public UnitArmorTech()
		{
			this.Name = "服飾";
			this.ShortcutKeyword = "TechInfantryArmor";
			this.Description = "下級魔法陣で生産するユニットの防御力を上昇させる。";
			this.ButtonDescription = this.Description;
			this.UnitTechAction = TechUnit;
			this.ButtonPosX = ButtonPosition._0;
			this.ButtonPosY = ButtonPosition._1;
			this.TechTime = 220*30;
		}
		public bool TechUnit(thrts.UnitParameter param)
		{
			param.Armor += armorInc;
			return true;
		}
		public override void OnComplete(ITargetObject holder)
		{
			var faction = holder.Faction;
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.EndResearch).FullName, holder.Pos);
			base.OnComplete(holder);
		}
	}
	public class GunTech : thrts.TechnologyParameter<ITargetObject>
	{
		protected int atkInc = 2;
		public GunTech()
		{
			this.Name = "弾幕指南";
			this.ShortcutKeyword = "TechInfantryWeapon";
			this.Description = "下級魔法陣で生産するユニットの攻撃力を上昇させる。";
			this.ButtonDescription = this.Description;
			this.BulletTechAction = TechBullet;
			this.ButtonPosX = ButtonPosition._0;
			this.ButtonPosY = ButtonPosition._0;
			this.TechTime =2230*30;
		}
		public bool TechBullet(thrts.BulletParameter param)
		{
			param.Attack += atkInc;
			return true;
		}
		public override void OnComplete(ITargetObject holder)
		{
			var faction = holder.Faction;
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.EndResearch).FullName, holder.Pos);
			base.OnComplete(holder);
		}
	}

	public class CannonArmorTech : thrts.TechnologyParameter<ITargetObject>
	{
		protected int armorInc = 2;
		public CannonArmorTech()
		{
			this.Name = "板金";
			this.ShortcutKeyword = "TechSeniorArmor";
			this.Description = "上級魔法陣で生産するユニットの防御力を上昇させる。";
			this.ButtonDescription = this.Description;
			this.UnitTechAction = TechUnit;
			this.ButtonPosX = ButtonPosition._1;
			this.ButtonPosY = ButtonPosition._1;
			this.TechTime = 220*30;
		}
		public bool TechUnit(thrts.UnitParameter param)
		{
			param.Armor += armorInc;
			return true;
		}
		public override void OnComplete(ITargetObject holder)
		{
			var faction = holder.Faction;
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.EndResearch).FullName, holder.Pos);
			base.OnComplete(holder);
		}
	}
	public class CannonAtkTech : thrts.TechnologyParameter<ITargetObject>
	{
		protected int atkInc = 2;
		public CannonAtkTech()
		{
			this.Name = "鍛冶";
			this.ShortcutKeyword = "TechSeniorWeapon";
			this.Description = "上級魔法陣で生産するユニットの攻撃力を上昇させる。";
			this.ButtonDescription = this.Description;
			this.BulletTechAction = TechBullet;
			this.ButtonPosX = ButtonPosition._1;
			this.ButtonPosY = ButtonPosition._0;
			this.TechTime = 220*30;
		}
		public bool TechBullet(thrts.BulletParameter param)
		{
			param.Attack += atkInc;
			return true;
		}
		public override void OnComplete(ITargetObject holder)
		{
			var faction = holder.Faction;
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.EndResearch).FullName, holder.Pos);
			base.OnComplete(holder);
		}
	}

	public class BuildingHPTech : thrts.TechnologyParameter<ITargetObject>
	{
		protected float hpIncRatio = 0.4f;
		public BuildingHPTech()
		{
			this.Name = "改装";
			this.ShortcutKeyword = "TechBuildingHP";
			this.Description = "建物のHPを上昇させます";
			this.ButtonDescription = this.Description;
			this.BuildingTechAction = TechAction;
			this.ButtonPosX = ButtonPosition._2;
			this.ButtonPosY = ButtonPosition._2;
			this.TechTime = 120*30;
		}
		public bool TechAction(thrts.BuildingParameter param)
		{
			var up = (int)(param.HP*hpIncRatio/50);
			param.HP += up*50;
			return true;
		}
		public override void OnComplete(ITargetObject holder)
		{
			var faction = holder.Faction;
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.EndResearch).FullName, holder.Pos);
			base.OnComplete(holder);
		}

	}
	public class BuildingArmorTech : thrts.TechnologyParameter<ITargetObject>
	{
		protected int armorInc = 2;
		public BuildingArmorTech()
		{
			this.Name = "材質強化";
			this.ShortcutKeyword = "TechBuildingArmor";
			this.Description = "建物の防御力を上昇させます";
			this.ButtonDescription = this.Description;
			this.BuildingTechAction = TechAction;
			this.ButtonPosX = ButtonPosition._1;
			this.ButtonPosY = ButtonPosition._2;
			this.TechTime = 150*30;
		}
		public bool TechAction(thrts.BuildingParameter param)
		{
			param.Armor += armorInc;
			return true;
		}
		public override void OnComplete(ITargetObject holder)
		{
			var faction = holder.Faction;
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.EndResearch).FullName, holder.Pos);
			base.OnComplete(holder);
		}
	}
	public class SpecialSkillTech : thrts.TechnologyParameter<ITargetObject>
	{
		protected string skillName = null;
		public SpecialSkillTech()
		{
			this.Name = "修練";
			this.Description = "スキルを習得します";
			this.ButtonDescription = this.Description;
			this.FreeTechAction = TechAction;
			this.ButtonPosX = ButtonPosition._1;
			this.ButtonPosY = ButtonPosition._2;
			this.TechTime = 60*30;
		}
		public bool TechAction(IWorld world,IFaction faction)
		{
			if(skillName != null)
				faction.LvUpSpellWithName(skillName);

			return true;
		}
		public override void OnComplete(ITargetObject holder)
		{
			var faction = holder.Faction;
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.EndResearch).FullName, holder.Pos);
			base.OnComplete(holder);
		}
	}

	public class ReactorTech : thrts.TechnologyParameter<ITargetObject>
	{
		string kindName = "";
		public ReactorTech(string kindName,string unitName,int cost0 = 0,int cost1 =  25,int cost2 = 0,int cost3 =  0)
		{
			this.Name = "二重魔法陣";
			this.Description = unitName+"が二体同時に生産出来るようになる。";
			this.ButtonDescription = this.Description;
			this.ButtonPosX = ButtonPosition._0;
			this.ButtonPosY = ButtonPosition._2;
			this.TechTime = 25*Def.FPS;
			this.IconPath = "LotusCraft/Image/Button/Technology/BarracksReactor";
			this.kindName = kindName;
			this.Costs = new int[4]{cost0, cost1, cost2, cost3};
			this.TechTime = 40*Def.FPS;
			this.DoesOnlyTarget = true;
		}
		public override void OnComplete (ITargetObject holder)
		{
			var faction = holder.Faction;
			var b = holder as IBuilding;
			if (null != b) {
				if (!b.UserValues.ContainsKey (BuildingParameter.ReactorKey))
					b.UserValues.Add (BuildingParameter.ReactorKey, kindName);
				else
					b.UserValues[BuildingParameter.ReactorKey] = kindName;
			}
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.EndResearch).FullName, holder.Pos);
			base.OnComplete(holder);
		}
	}
}
