﻿ using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Spell.Blue;

namespace LotusCraft.Unit.Blue
{
	public class HighFairy : General.Special
	{
		public HighFairy()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Blue/HighFairy";
			#endregion

			#region UnitParams
			Name = "大妖精(式神)";
			HP = 360;
			Armor = 4;
			Radius = (float)Utls.UnitSize.Medium;
			Speed = 0.7f * Speed;

			ViewRange = 14 * 32;

			MakingTime = 55 * 30;
			/// mine,gas,sup,?
			Costs = new [] {200,250,0,0};

			PopulationWeight = 6;
            ButtonDescription = "飛行・対地ユニット\n長い射程、高い攻撃力と耐久力を持つ";
			
			MoveType |= UnitMoveType.Air;

			CanState = false;

			 #endregion



            #region Animation
            this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Blue/HighFairy");
            #endregion

			RequireBuildings = new string[]{
				typeof(Building.Blue.Airport).FullName,
			};
			
			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion
			
			#region Bullets
			Bullets = new string[] { typeof(HighFairyBullet).FullName };
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{

			#region Spells
			SpellParams = new SpellParameter<IUnit>[]
			{
				new HighFairySkill(),
			};
			#endregion
			#region AutoCasts
			AutocasterParams = new AutoCastParameter<IUnit>[]
			{
				SpellParams[0].DefaultAutoCast,
			};
			#endregion
			#region Buttons
			ButtonParams = new ButtonParameterHolder<IUnit>[]
			{
				new ButtonParameterHolder<IUnit>(SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
			ButtonParams = ButtonParams.Concat(faction.Parameter.CommonUnitButton).ToArray();
			#endregion
		}
		public override void OnAppear (IUnit unit)
		{
			if(unit.Faction.World.CurrentTime > 10)
				unit.Faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.CreatedUnit).FullName,unit.Pos);

			base.OnAppear (unit);
		}
		public override void OnVanish (IUnit unit)
		{
            var faction = unit.Faction;
            var offset = new Vector3D(-2, -2, 8);
            var effect = faction.World.EffectManager.CreateEffect(typeof(DeadUnitEffect).FullName, unit.Pos + offset);
            var scale = new Vector3D(8, 8, 8);
            effect.SetScale(scale);
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.PCH).FullName, unit.Pos);
			base.OnVanish (unit);
		}
	}
}
namespace LotusCraft.Bullet.Blue
{
	public class HighFairyBullet : General.LongRangeBullet
	{
		public HighFairyBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;

			DoesHitOnlyTarget = true;

			this.Attack = 70;
			HasSplash = true;
			IsSplashMissHit = true;
			SplashDamageRates.Add(0.7f*Def.MapTipLengthOnField,0.75f);
			SplashDamageRates.Add(1.0f*Def.MapTipLengthOnField,0.5f);

			Speed = 10.0f * BasisSpeed;
			Gravity = 0f;
			MaxRange = 7 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackSpeed = (int)(2.5f * Def.FPS);

			OnHit = OnHitEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
			TextureAnimation.Offset.X = 128 * 2 + 16 * 3;
			TextureAnimation.Offset.Y = 128 * 0 + 16 * 0;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
            LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalRed);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
		}
	}
}

namespace LotusCraft.Spell.Blue
{
	class HighFairySkill:thrts.SpellParameter<thrts.IUnit>
	{
		const int duration = 30*Def.FPS;
		public HighFairySkill()
		{
			DefaultLevel = 1 ;
			MaxLevel = 1;
			Name = "妖精達のお姉さん";
			string basePath = "Blue/Spell/HighFairySkill/";
			
			CoolTimes = new int[2];
			CoolTimes[0] = 60*30;
			CoolTimes[1] = 60*30;
			
			Description = "隠れているヒロイン以外のユニットを発見する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				duration/Def.FPS + "秒間",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		

		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;
			
			/// reset cooltime
			selfUnit.GetSpellWithName (Name).ResetCoolTime ();
			
			var world = selfUnit.Faction.World;
			
			/// show effect
			if (selfUnit.AbilityManager.ContainsAbility (this.Name + "Effect")) {
				selfUnit.AbilityManager.ExtendLifetimeWithName(duration,this.Name + "Effect",duration);
			} else {
				var offset = new Vector3D (2, 2, 2);
				var effect = world.EffectManager.CreateEffect (typeof(Effect.Common.HighFairyEffect).FullName, selfUnit.Pos+ offset);
				var scale = selfUnit.ViewRange;
				effect.SetScale (new Vector3D (scale, scale, scale));
				var ability = new HighFairySkillAbility(effect, offset, duration);
				if (!selfUnit.AbilityManager.AppendAbility (ability))
					effect.Stop ();
			}
			

			yield return false;
		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class HighFairySkillAbility  : FollowEffectAbility
	{
		public const string AbilityName =  "妖精達のお姉さん";
		public HighFairySkillAbility(int durationTime,IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = durationTime;
			this.Effect = effect;
		}
		public HighFairySkillAbility(IEffect effect,Vector3D offset,int durationTime)
		{
			this.Name = AbilityName;
			this.Effect = effect;
			this.Offset = offset;
			this.DurationTime = durationTime;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.IsDetector = true;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
	
}