﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Blue;
using LotusCraft.Effect.Blue;
using LotusCraft.Spell.Blue;
using LotusCraft.Building.Blue;

namespace LotusCraft.Unit.Blue
{
	public class Lunar : General.Special
	{
		public Lunar ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Blue/Lunar";
			#endregion

			#region UnitParams
			Name = "ルナチャイルド(式神)";
			HP = 220;
			Radius = (float)Utls.UnitSize.Large;
			Speed = 1.2f * BasisSpeed;

			ViewRange = 12*Def.MapTipLengthOnField;
			PopulationWeight = 2;
            ButtonDescription = "飛行・対地ユニット\n味方のみ通行可能の障害物を作る";

			UnitType = new TargetObjectType[]{TargetObjectType.Clay,};

			MoveType |= UnitMoveType.Air;

			Costs = new [] {125,75,0,0};

			MakingTime = 42 * Def.FPS;

			RequireBuildings = new string[]{
				typeof(BigTree).FullName,
			};

			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Blue/Lunar");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(LunarBullet).FullName };
			#endregion
		}

		public override void OnLoad (IFaction faction)
		{
			SpellParams = new thrts.SpellParameter<thrts.IUnit>[]
			{
				new LunarSkill (),
			};
			AutocasterParams = new AutoCastParameter<IUnit>[]
			{
				SpellParams[0].DefaultAutoCast,
			};
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit> (SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
			#endregion
			
			base.OnLoad(faction);
		}
	}
}

namespace LotusCraft.Bullet.Blue
{
	public class LunarBullet : General.MiddleRangeBullet
	{
		public LunarBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			DoesHitOnlyTarget = true;
			this.Attack = 15;
			Speed = 10.0f * BasisSpeed;
			MaxRange = 5 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(1.0f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			DamageBonus[(int)TargetObjectType.Water] = 1.2f;
			DamageBonus[(int)TargetObjectType.Structure] = 1.4f;

			OnHit = OnHitEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 0;
			TextureAnimation.Center.Y = 0;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 1;
			TextureAnimation.Offset.X = 128 * 2 + 16 * 0;
			TextureAnimation.Offset.Y = 128 * 1 + 16 * 6;
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
		}
        static void OnHitEffect(IBullet bullet, ITargetObject target)
        {
            LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalYellow);
        }
	}
}
