﻿ using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Ability;
using LotusCraft.Ability.Common;
using LotusCraft.Effect.Common;
using LotusCraft.Bullet.Common;
using LotusCraft.Spell.Common;

namespace LotusCraft.Unit.Common
{
	public class Eiki : General.Special
	{
		int duration = 15*Def.FPS;
		public Eiki()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Common/Eiki";
			#endregion

			#region UnitParams
			Name = "四季映姫・ヤマザナドゥ(式神)";
			HP = 500;
			IsSummon = true;
			Radius = (float)Utls.UnitSize.Medium;
			Speed = 1.0f * Speed;

			ViewRange = 9 * 32;

			MakingTime = 25 * 30;
			/// mine,gas,sup,?
			Costs = new [] {0,0,0,0};

			Exp = 0;

			 #endregion

			#region Description
			ButtonDescription += "一定時間共に戦う増援ユニットです。\nスペルを使う事で敵ユニットの移動速度を下げる事ができます";

			#endregion

            #region Animation
            this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Common/Eiki");
            #endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion
			
			#region Bullets
			Bullets = new string[] { typeof(EikiBullet).FullName };
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{

			#region Spells
			SpellParams = new SpellParameter<IUnit>[]
			{
				new EikiSkill(),
			};
			#endregion
			#region AutoCasts
			AutocasterParams = new AutoCastParameter<IUnit>[]
			{
				SpellParams[0].DefaultAutoCast,
			};
			#endregion
			#region Buttons
			ButtonParams = new ButtonParameterHolder<IUnit>[]
			{
				new ButtonParameterHolder<IUnit>(SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
			ButtonParams = ButtonParams.Concat(faction.Parameter.CommonUnitButton).ToArray();
			#endregion
		}
		public override void OnAppear (IUnit unit)
		{
			if(unit.Faction.World.CurrentTime > 10)
				unit.Faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.CreatedUnit).FullName,unit.Pos);

			/// 自動死亡のアビリティを追加
			unit.AbilityManager.AppendAbility(new SuicideAbility(null,duration,this.Name));

			base.OnAppear (unit);
		}
		public override void OnVanish (IUnit unit)
		{
            var faction = unit.Faction;
            var offset = new Vector3D(-2, -2, 8);
            var effect = faction.World.EffectManager.CreateEffect(typeof(DeadUnitEffect).FullName, unit.Pos + offset);
            var scale = new Vector3D(8, 8, 8);
            effect.SetScale(scale);
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.PCH).FullName, unit.Pos);
			base.OnVanish (unit);
		}
	}
}
namespace LotusCraft.Bullet.Common
{
	public class EikiBullet : General.LongRangeBullet
	{
		public EikiBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			DoesHitOnlyTarget = true;
			this.Attack = 30;
			Speed = 10.0f * BasisSpeed;
			Gravity = 0f;
			MaxRange = 6 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackRigor = (int)(1.0f * Def.FPS);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/kaeiduka";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 0;
			TextureAnimation.Center.Y = 0;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
            LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalRed);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
		}
	}
}

namespace LotusCraft.Spell.Common
{
	class EikiSkill:thrts.SpellParameter<thrts.IUnit>
	{
		const int interval = 10;
		public EikiSkill()
		{
			DefaultLevel = 1 ;
			MaxLevel = 1;
			Name = "審判「ギルティ・オワ・ノットギルティ」";
			string basePath = "Common/Spell/EikiSkill/";
			
			CoolTimes = new int[2];
			CoolTimes[0] = 75*30;
			CoolTimes[1] = 75*30;
			
			Description = "周囲の敵の移動速度を低下させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"移動速度-0.25",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new SpellAutoCast<IUnit>(this,interval,null),
				new SpellAutoCast<IUnit>(this,interval,(IUnit unit)=>Spelling (unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			var affectRadius = 7;
			
			/// reset cooltime
			selfUnit.GetSpellWithName (Name).ResetCoolTime ();
			
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			
			/// show effect
			if (selfUnit.AbilityManager.ContainsAbility (this.Name + "Effect")) {
				selfUnit.AbilityManager.ExtendLifetimeWithName(interval,this.Name + "Effect");
			} else {
				var offset = new Vector3D (2, 2, 2);
				var effect = world.EffectManager.CreateEffect (typeof(CircleAreaEffect).FullName, selfUnit.Pos + offset);
				var scale = affectRadius * 5;
				effect.SetScale (new Vector3D (scale, scale, scale));
				if (!selfUnit.AbilityManager.AppendAbility (new Ability.FollowEffectAbility (effect, offset, interval*2, this.Name + "Effect")))
					effect.Stop ();
			}
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).Where (_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>!selfFaction.World.IsSameTeam(selfFaction,_.Faction))) {
				if(u.AbilityManager.ContainsAbility(EikiSkillAbility.AbilityName))
				{
					u.AbilityManager.ExtendLifetimeWithName(interval,EikiSkillAbility.AbilityName,2*Def.FPS);
				}
				else
				{
				var debuffEffect = world.EffectManager.CreateEffect(typeof(DebuffEffect).FullName,u.Pos);
				debuffEffect.SetScale(new Vector3D(12,12,12));
				if(!u.AbilityManager.AppendAbility(new EikiSkillAbility(level,debuffEffect)))
				{
					debuffEffect.Stop();
				}
				}
			}

			return true;
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;
			

			yield return false;
		}
	}
}

namespace LotusCraft.Ability.Common
{
	class EikiSkillAbility  : FollowEffectAbility
	{
		public float DecMove;
		public const string AbilityName =  "審判「ギルティ・オワ・ノットギルティ」";
		public EikiSkillAbility(int level,IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = 2*Def.FPS;
			this.Effect = effect;
			switch (level)
			{
			case 1:
				DecMove = 0.25f;
				break;
			default:
				DecMove = 0.0f;
				break;
			}
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Speed -= DecMove*UnitParameter.BasisSpeed;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
	
}