﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Common;
using LotusCraft.Effect.Common;

namespace LotusCraft.Unit.Common
{
	public class TheBook : General.Special
	{
		public TheBook ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Common/TheBook";
			#endregion

			#region UnitParams
			Name = "魔導書";
			HP = 250;
			Radius = (float)Utls.UnitSize.Small;
			Speed = 0.8f*BasisSpeed;

			UnitType = new TargetObjectType[]{TargetObjectType.Clay};

			PopulationWeight = 2;

			MoveType |= UnitMoveType.Air;

			MakingTime = 20 * Def.FPS;
			/// mine,gas
			Costs = new [] {135,0,0,0};

			IsSummon = true;
			CanState = false;
			// 
			Exp =  20;
			#endregion

			#region Animation
			this.SetBookUnitAnimation("LotusCraft/Image/Unit/Common/TheBook");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Bullets
			Bullets = new string[] {
				typeof(BookBullet).FullName,
			};
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			#region Buttons
			#endregion
			
			base.OnLoad (faction);
		}

        public override void OnVanish(IUnit unit)
        {
            var faction = unit.Faction;
            var offset = new Vector3D(-2, -2, 8);
            var effect = faction.World.EffectManager.CreateEffect(typeof(DeadUnitEffect).FullName, unit.Pos + offset);
            var scale = new Vector3D(8, 8, 8);
            effect.SetScale(scale);
            base.OnVanish(unit);
        }
		public override void OnAppear (IUnit unit)
		{
			base.OnAppear (unit);
		}
	}
}

namespace LotusCraft.Bullet.Common
{
	public class BookBullet : General.ShortRangeBullet
	{
		public BookBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;

			DoesHitOnlyTarget = true;

			this.Attack = 28;
			Speed = 10.0f * BasisSpeed;
			
			MaxRange = 7 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.2f * 30);
			AttackRigor = (int)(0.0f * 30);
			AttackSpeed = (int)(2.0f * 30);

			HasSplash = true;
			IsSplashMissHit = true;
			SplashDamageRates.Add(0.4f * Def.MapTipLengthOnField, 1.0f);
			SplashDamageRates.Add(0.7f * Def.MapTipLengthOnField, 0.75f);
			SplashDamageRates.Add(1.1f * Def.MapTipLengthOnField, 0.5f);

			OnHit = OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/ball_blue";
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
			TextureAnimation.Width = 32;
			TextureAnimation.Height = 32;
			TextureAnimation.Center.X = 16;
			TextureAnimation.Center.Y = 16;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 4;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 4;
		}
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			var unit = bullet.Generator as IUnit;
			if (unit != null &&
			    unit.Exists() )
			{
				LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalBlue);
			}
		}
	}
}
