﻿ using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.Unit.General
{
	public class Fly : thrts.UnitParameter
	{
		public Fly()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/chen";
			#endregion

			#region UnitParams
			Name = "飛行";
			HP = 400;
			Radius = (float)Utls.UnitSize.Large;
			Speed = 1.0f * Speed;

			ViewRange = 9 * 32;

			MakingTime = 45 * 30;
			/// mine,gas,sup,?
			Costs = new [] {100,200,0,0};
			SelectPriority = 40;
			MoveType |= UnitMoveType.Air;
			UnitType = new TargetObjectType[]{TargetObjectType.Water,};

			#endregion

			#region Description
			ButtonDescription += "崖や谷などの地形を無視して移動する事ができます。";

			#endregion
			/*
			#region Animation
			Animation = new TextureAnimation();
			Animation.Path = "LotusCraft/Image/Unit/unit";
			Animation.Width = 32;
			Animation.Height = 32;
			Animation.Center.X = 0;
			Animation.Center.Y = 10;
			Animation.CenterPosition = CenterPosition.MiddleMiddle;
			Animation.FrameCount = 4;
			Animation.FrameTime = 4;
			Animation.HasDirection = true;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 4;
			Animation.Count.Y = 8;
			#endregion

			#region BuildableObjcets
			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion
			
			#region Bullets
			Bullets = new string[] { "LotusCraft.Bullet.Melee" };
			#endregion
			*/
		}
		public override void OnLoad (IFaction faction)
		{
			if(ConcurrentProductions > 1)
				Exp = Costs.Sum()/(3*ConcurrentProductions);
			else
				Exp = Costs.Sum()/3;

			#region Buttons
			ButtonParams = ButtonParams.Concat(faction.Parameter.CommonUnitButton).ToArray();
			#endregion

		}
		public override void OnAppear (IUnit unit)
		{
			if (unit.Faction.World.CurrentTime > 10)
			{
				unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.CreatedUnit).FullName, unit.Pos);

				// 生成エフェクト
				var effect = unit.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateUnitEffect).FullName, unit.Pos);
				effect.SetScale(new Vector3D(3, 3, 3));
			}

			base.OnAppear (unit);
		}
		public override void OnVanish (IUnit unit)
		{
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.PCH).FullName, unit.Pos);
			base.OnVanish (unit);
		}
	}
}

namespace LotusCraft.Ability.Common
{
	class UFOCloakAbility : AbilityTemplate
	{
		public const string AbilityName = "未確認飛行物体";
		
		public IEffectCommon effect = null;
		public Vector3D offset;
		public UFOCloakAbility ()
		{
			this.Name = AbilityName;
			this.DurationTime = 999;
			this.effect = null;
			this.offset = new Vector3D();
		}
		public UFOCloakAbility (IEffectCommon effect,Vector3D offset)
		{
			this.Name = AbilityName;
			this.DurationTime = 999;
			this.effect = effect;
			this.offset = offset;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				// 不可視系の設定は定数増加で行う
				// 近くにDetectorが居たら透明になれない
				if(unit.Faction.World.GetObjectFromCircle(new Circle2D(16*Def.MapTipLengthOnField,new Vector2D(unit.Pos.X,unit.Pos.Y)))
				   .OfType<IUnit>()
				   .Any(_=>!unit.Faction.World.IsSameTeam(_.Faction,unit.Faction) && _.IsDetector && Utl.InCircle(unit.Pos,_.Pos,_.ViewRange) ) )
				{
					unit.IsPreventedFromFindingEnemy = false;
				}
				else
				{
					unit.IsPreventedFromFindingEnemy = true;
				}
				unit.IsSemitransparent = true;

				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if (effect != null)
					effect.SetPosition(unit.Pos + offset);
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish (unit);
		}
		public override void DecDurationTime ()
		{
			// no dec -> inf ability
//			base.DecDurationTime ();
		}
	}
}