﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.Unit.General
{
	public class ShortRangeUnique : Unique
	{
		public ShortRangeUnique ()
		{
			HPs[0] = 1100;
			HPs[1] = 1200;
			HPs[2] = 1300;
			HPs[3] = 1400;
			HPs[4] = 1500;
			HPs[5] = 1600;
			HPs[6] = 1700;
			HPs[7] = 1800;
			HPs[8] = 1900;
			HPs[9] = 2000;

			Armors[0] = 2;
			Armors[1] = 2;
			Armors[2] = 4;
			Armors[3] = 4;
			Armors[4] = 6;
			Armors[5] = 6;
			Armors[6] = 8;
			Armors[7] = 8;
			Armors[8] = 10;
			Armors[9] = 10;

			ATKs[0] = 25;
			ATKs[1] = 35;
			ATKs[2] = 45;
			ATKs[3] = 55;
			ATKs[4] = 63;
			ATKs[5] = 71;
			ATKs[6] = 79;
			ATKs[7] = 85;
			ATKs[8] = 91;
			ATKs[9] = 95;

		}
	}
	public class Unique : thrts.UnitParameter
	{
		/// <summary>
		/// レベルごとのHP
		/// </summary>
		public int[] HPs = new int[10];
		/// <summary>
		/// レベルごとのアーマー
		/// </summary>
		public int[] Armors = new int[10];
		/// <summary>
		/// レベルごとの攻撃力
		/// </summary>
		public int[] ATKs = new int[10];
		/// <summary>
		/// レベルごとの攻撃範囲
		/// </summary>
		public float[] Ranges = new float[10];
		/// <summary>
		/// レベルごとの移動速度
		/// </summary>
		public float[] Speeds = new float[10];

		protected int[] defaultInitCosts = new int[4];
		protected int[] incInitCosts = new int[4];
		protected int[] defaultCosts = new int[4];
		protected int[] incCosts = new int[4];

		protected int defaultMakingTime = 28*Def.FPS;
		protected int incMakingTime = 3*Def.FPS;
		const string spellCooltimeKey="spell_cooltimes";
		public Unique()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/chen";
			#endregion

			#region UnitParams
			Name = "testunit";
			HP = 1350;
			Radius = (float)Utls.UnitSize.Large;
			Speed = 1.0f*BasisSpeed;

			ViewRange = 14*Def.MapTipLengthOnField;

			PopulationWeight = 10;
			StationWeight = -1;
			MakingTime = defaultMakingTime;

			Costs = new [] {350,200,0,0};
			defaultCosts = new [] {350,200,0,0};
			incCosts = new []{10,5,0,0};

			InitCosts = new [] {150,0,0,0};
			defaultInitCosts = new [] {150,0,0,0};
			incInitCosts = new []{5,0,0,0};

			MaximumExists = 1;
			MaximumExistsInQueue = 1;
			IsUnique = true; 

			IsDetector = true;

			SelectPriority = 10;
			UnitType = new TargetObjectType[]{TargetObjectType.Hero,};

			Exp += 0;
			#endregion
/*
			#region Animation
			this.SetBoushuenSizeAnimation("LotusCraft/Image/Unit/unit.png");
			#endregion

			#region BuildableObjcets
			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Bullets
			Bullets = new string[] { "LotusCraft.Bullet.Gun" };
			#endregion
			*/

		}

		public override void OnLoad (IFaction faction)
		{
			if(StationWeight < 0)
				StationWeight = PopulationWeight;

			#region Buttons
			ButtonParams = ButtonParams.Concat(faction.Parameter.CommonUnitButton).ToArray();
			#endregion
			
		}
		public override void OnLvUp (thrts.IUnit unit)
		{
			if( null == unit || !unit.Exists() )
				return;

			int lv = unit.GetLevel ();
			HP = HPs[ Math.Min(lv , HPs.Length) - 1];
			Armor = Armors[Math.Min(lv , Armors.Length ) - 1];
			unit.Faction.GetBulletParameter(Bullets[0]).Attack = (int)ATKs[Math.Min(lv, ATKs.Length) - 1];
//			unit.Faction.GetBulletParameter(Bullets[0]).MaxRange = (int)Ranges[Math.Min(lv , Ranges.Length) - 1];
//			Speed = Speeds[Math.Min(lv , Speeds.Length ) - 1];

			// inc exp.
			Exp = 100 + lv * 50;
			// inc make time
			unit.Parameter.MakingTime = defaultMakingTime + incMakingTime*(lv - 1);

			// inc costs
			for(int i=0;i<unit.Parameter.Costs.Length;++i)
			{
				unit.Parameter.Costs[i] = defaultCosts[i] + incCosts[i] * (lv - 1);
			}
			for(int i=0;i<unit.Parameter.InitCosts.Length;++i)
			{
				unit.Parameter.InitCosts[i] = defaultInitCosts[i] + incInitCosts[i] * (lv - 1);
			}

			unit.SetParameter(unit.Parameter);

			if (lv > 1)
			{

				var hpafter = HPs[Math.Min(lv, HPs.Length) - 1];
				var hpbefore = HPs[Math.Min(lv, HPs.Length) - 2];
				unit.HP = unit.HP + (hpafter - hpbefore);
			}

			if (lv > 1 && unit.TimeAfterCreated > 5)
			{
				var offset = new Vector3D(1, 1, 10);
				var lvUpEffect = unit.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.LvUpEffect).FullName, unit.Pos + offset);
				var scale = 8.0f;
				lvUpEffect.SetScale(new Vector3D(scale, scale, scale));
				unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.LvUp).FullName, unit.Pos);
			}

			unit.StartLoadCutin();
		}
		public override void OnAppear (IUnit unit)
		{
			if (unit.Faction.World.CurrentTime > 10) {
				unit.Faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.CreatedUnit).FullName, unit.Pos);

				// 生成エフェクト
				var effect = unit.Faction.World.EffectManager.CreateEffect (typeof(Effect.Common.CreateUnitEffect).FullName, unit.Pos);
				effect.SetScale (new Vector3D (3, 3, 3));
			}

			if (unit.Faction.UserValues.ContainsKey (spellCooltimeKey + unit.Parameter.GetType ().FullName)) {
				Dictionary<string,int> spellCooltimes = unit.Faction.UserValues [spellCooltimeKey + unit.Parameter.GetType ().FullName] as Dictionary<string,int>;
				if (null != spellCooltimes) {
			
					foreach (var sp in unit.Parameter.SpellParams) {
						if(null == sp)
							continue;
						var spell = unit.GetSpellWithName (sp.Name);
						if (null == spell)
							continue;
				
						if (spellCooltimes.ContainsKey (sp.Name))
							spell.CoolTimeCounter = spellCooltimes [sp.Name];
					}
				}
			}
			unit.Vanishing += SaveCooltimes;
			base.OnAppear (unit);
		}
		public void SaveCooltimes (IUnit unit)
		{
			if (null == unit)
				return;
				
			if (!unit.Faction.UserValues.ContainsKey (spellCooltimeKey + unit.Parameter.GetType ().FullName))
				unit.Faction.UserValues.Add (spellCooltimeKey + unit.Parameter.GetType ().FullName, new Dictionary<string,int> ());
				
			Dictionary<string,int> spellCooltimes = unit.Faction.UserValues [spellCooltimeKey + unit.Parameter.GetType ().FullName] as Dictionary<string,int>;
			if (null == spellCooltimes) {
				spellCooltimes = new Dictionary<string,int> ();
			}
			{
					
				foreach (var sp in unit.Parameter.SpellParams) {
					if (null == sp)
						continue;
					if (sp.RequireLevels[0] < 5)
						continue;

					var spell = unit.GetSpellWithName (sp.Name);
					if (null == spell)
						continue;
						
					if (!spellCooltimes.ContainsKey (sp.Name))
						spellCooltimes.Add (sp.Name, spell.CoolTimeCounter);
					else
						spellCooltimes [sp.Name] = spell.CoolTimeCounter;
				}
				unit.Faction.UserValues [spellCooltimeKey + unit.Parameter.GetType ().FullName] = spellCooltimes;
			}
		}
		public override void OnVanish (IUnit unit)
		{
			unit.Vanishing -= SaveCooltimes;
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.PCH).FullName, unit.Pos);
			base.OnVanish (unit);
		}
	}
}

namespace LotusCraft.Bullet.General
{
	public class HeroBullet : thrts.BulletParameter
	{
		public HeroBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			this.Attack = 25;
			Speed = 10.0f * BasisSpeed;
			MaxRange = 9 * Def.MapTipLengthOnField;

			DamageBonus = new float[AttackBonusSet.Hero.Length];
			for(int i=0;i<this.DamageBonus.Length;++i)
				DamageBonus[i] = AttackBonusSet.Hero[i];

			AttackDelay = (int)(0.5f * 30);
			AttackRigor = (int)(0.5f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
		}
	}
}

