﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Green;
using LotusCraft.Building.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Unit.Green
{
	public class SuikaSmall  : General.Special
	{
		public SuikaSmall ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Green/Suika_small";
			#endregion

			#region UnitParams
			Name = "ちび萃香(式神)";
			HP = 60; 
			Radius = (float)Utls.UnitSize.VerySmall;
			Speed = 1.1f*BasisSpeed;
			Armor = 0;

			PopulationWeight = 1;
			ViewRange = 9*Def.MapTipLengthOnField;
			MakingTime = (20) * Def.FPS;
			ConcurrentProductions = 1;
			Costs = new []{25,25,0,0};
			ButtonDescription = "自爆ユニット。\n広い範囲に大ダメージを与える。";
			Description = _ => ButtonDescription.Split('\n').Select(s => new ColorTextLine(s));
			UnitType = new TargetObjectType[]{TargetObjectType.Water,};
			#endregion

			#region Animation
			this.SetMiniSuikaAnimation("LotusCraft/Image/Unit/Green/Suika_small");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(SuikaSmallBullet).FullName };
			#endregion

			#region Requires
			RequireBuildings = new string[]{
				typeof(Hyoutan).FullName,
			};
			#endregion
		}
		public override void OnAppear (IUnit unit)
		{
			unit.AbilityManager.AppendAbility(new Ability.Green.OnGardenEffectAbility());
			base.OnAppear (unit);
		}
	}
}

namespace LotusCraft.Bullet.Green
{
	public class SuikaSmallBullet : General.ShortRangeBullet
	{
		public SuikaSmallBullet()
		{
			Type = BulletType.ShortRange;
			SpaceType = BulletSpaceType.Earth;
			
			HasSplash = true;
			SplashDamageRates.Add(1.5f*Def.MapTipLengthOnField,1.0f);
			SplashDamageRates.Add(1.9f*Def.MapTipLengthOnField,0.8f);
			SplashDamageRates.Add(2.4f*Def.MapTipLengthOnField,0.5f);
			SplashDamageRates.Add(3.0f*Def.MapTipLengthOnField,0.25f);

			this.Attack = 75;
            Speed = 10.0f * BasisSpeed;
			DamageBonus[(int)TargetObjectType.Water] = 1.4f;
			DamageBonus[(int)TargetObjectType.Structure] = 4.0f;
			
//			MaxRange = 0.5f * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(1.0f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
			
		}
		static void OnHitEffect (IBullet bullet, ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			var offset = new Vector3D (1, 2, 8);
			var effect = faction.World.EffectManager.CreateEffect (typeof(ParadeHitEffect).FullName, target.Pos + offset);
			var scale = new Vector3D (8, 8, 8);
			effect.SetScale (scale);
			faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.Bom01).FullName, target.Pos);
			
			// suicide unit
			if(bullet.Generator is IUnit)
				(bullet.Generator as IUnit).HP = 0;
			
		}
	}
}
