﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Neutral;
using LotusCraft.Effect.Neutral;

namespace LotusCraft.Unit.Neutral
{
	public class NeutralBase : thrts.UnitParameter
	{
		public override void OnLoad (IFaction faction)
		{
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.MoveButton ()),
				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.StopButton ()),
				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.HoldPositionButton ()),
//				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.PatrolButton ()),
//				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.AttackButton ()),
//				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.StationButton ()),
			};
			#endregion

		}
		public override void OnAppear (IUnit unit)
		{
			var initPos = new Vector2D(unit.Pos.X,unit.Pos.Y);
			int counter = 0;
			int timelimit = 3*Def.FPS;
			float cautionLimit = 12*Def.FPS;
			float returnLimit = 16*Def.FPS;
			unit.Updating += () => 
			{
				if(unit.ActualAction == UnitAction.Free || unit.ActualAction == UnitAction.Move)
				{
					counter++;
				}
				else
				{
					counter = 0;
				}

				var dx=unit.Pos.X - initPos.X;
				var dy=unit.Pos.Y - initPos.Y;
				// 警戒距離を超えたら索敵を停止
				if(dx*dx + dy*dy > cautionLimit*cautionLimit)
				{
					unit.DoFindEnemyAutomatically = false;
				}
				else
				{
					unit.DoFindEnemyAutomatically = true;
				}

				if(counter > timelimit)
				{
					if(dx*dx + dy*dy > returnLimit*returnLimit)
					{
						unit.Faction.World.UnitManager.SelectTarget (new[]{unit},initPos,false);
					}
					else if(dx*dx + dy*dy > Def.MapTipLengthOnField*Def.MapTipLengthOnField)
					{
						unit.Faction.World.UnitManager.AttackMove(new []{unit.UniqueID},initPos,false);
					}

					counter = 0;
				}
			};

			base.OnAppear (unit);
		}
		public override void OnVanish(IUnit unit)
		{
			var faction = unit.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(DeadUnitEffect).FullName, faction, unit.Pos + offset);
			var scale = new Vector3D(8, 8, 8);
			effect.SetScale(scale);
			base.OnVanish(unit);
		}
	}
}