﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
/// <summary>
/// To debug
/// </summary>
using LotusCraft.Bullet.White;
using LotusCraft.Effect.White;
using LotusCraft.Building.White;

namespace LotusCraft.Unit.White
{
	public class Kaguya : LotusCraft.Unit.General.Special
	{
		public Kaguya ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/White/Kaguya";
			#endregion

			#region UnitParams
			Name = "蓬莱山 輝夜(式神)";
			HP = 460;
			Radius = (float)Utls.UnitSize.Large;
			Armor = 2;
			Speed = 1.0f * BasisSpeed;

			PopulationWeight = 5  ;

			ViewRange = 14 * Def.MapTipLengthOnField;

			MakingTime = 50*Def.FPS;

			/// mine,gas,sup,?
			Costs = new [] {150,200,0,0};

			ButtonDescription = "遠距離砲撃ユニット\n空中を攻撃できないかわり、長い射程と高い攻撃力を持つ\n砲撃状態に切り替えることで更に攻撃力が上昇する";
			Description = _ => ButtonDescription.Split('\n').Select(s => new ColorTextLine(s));

			UnitType = new TargetObjectType[]{TargetObjectType.Clay,};


			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/White/Kaguya");
			#endregion

			#region BuildableObjcets
			#endregion

			SpellParams = new thrts.SpellParameter<thrts.IUnit>[]
			{
				new LotusCraft.Spell.White.KaguyaSkill (),
			};

			AutocasterParams = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				/// mode change autocast
				SpellParams[0].SpellAutoCasts[2],
			};

			#region Bullets
			Bullets = new string[] {
				typeof(KaguyaMobBullet).FullName,
				typeof(KaguyaMobBigBullet).FullName,
			};
			#endregion
		}

		public override void OnLoad(IFaction faction)
		{
			// set pram from init param
			Ability.White.AjaModeAbility.AjaModeDecArmor = this.Armor;

			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit>(SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
			#endregion

			base.OnLoad(faction);
		}
	}
}

namespace LotusCraft.Bullet.White
{
	public class KaguyaMobBullet : General.CannonBullet
	{
		public KaguyaMobBullet()
		{
			Type = BulletType.LongRange;
			Attack = 30;
			DamageBonus[(int)TargetObjectType.Clay] = 1.8f;
			DamageBonus[(int)TargetObjectType.Structure] = 1.5f;
			DoesHitOnlyTarget = false;
			Speed = 8.0f * BasisSpeed;
			//			Gravity = 0;

			MaxRange = 7 * Def.MapTipLengthOnField;

			AttackDelay = (int)(0.8f * Def.FPS);
			AttackRigor = (int)(0.0f * Def.FPS);
			AttackSpeed = (int)(1*Def.FPS);

			OnHit += OnHitEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/ball_yellow";
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
			TextureAnimation.Width = 32;
			TextureAnimation.Height = 32;
			TextureAnimation.Center.X = 16;
			TextureAnimation.Center.Y = 16;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 4;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 4;
		}
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			var offset = new Vector3D(-8, -8, 3);
			var effect = faction.World.EffectManager.CreateEffect(typeof(KaguyaMobHitEffect).FullName, target.Pos + offset);
			var scale = new Vector3D(8, 8, 8);
			effect.SetScale(scale);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom01).FullName, target.Pos);
		}
	}
	public class KaguyaMobBigBullet : General.CannonBullet
	{
		public KaguyaMobBigBullet()
		{
			Type = BulletType.LongRange;
			DoesHitOnlyTarget = false;

			HasSplash = true;
			IsSplashMissHit = true;
			SplashDamageRates.Add(0.4f * Def.MapTipLengthOnField, 1.0f);
			SplashDamageRates.Add(0.7f * Def.MapTipLengthOnField, 0.75f);
			SplashDamageRates.Add(1.1f * Def.MapTipLengthOnField, 0.5f);
			SplashDamageRates.Add(1.5f * Def.MapTipLengthOnField, 0.25f);
			Attack = 75;
			DamageBonus[(int)TargetObjectType.Clay] = 1.4f;
			DamageBonus[(int)TargetObjectType.Structure] = 1.6f;

			Speed = 5.0f * BasisSpeed;
			//Gravity = 5;
			MaxRange = 13 * Def.MapTipLengthOnField;
			MinRange = 4 * Def.MapTipLengthOnField;

			AttackDelay = (int)(0.0f * Def.FPS);
			AttackRigor = (int)(0.0f * Def.FPS);
			AttackSpeed = (int)(3.0f * Def.FPS);

			OnHit += OnHitEffect;
			OnMiss += OnMissEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/kinkakuji";
			TextureAnimation.Width = 128;
			TextureAnimation.Height = 128;
			TextureAnimation.Center.X = 64;
			TextureAnimation.Center.Y = 64;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 16;
			TextureAnimation.FrameTime = 2;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 4;
		}
		void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			float scale = (SplashDamageRates.Max(_ => _.Key) / Def.MapTipLengthOnField) * 5;
			CreateEffect(faction, target.Pos, scale);
		}
		
		void OnMissEffect(IBullet bullet)
		{
			var faction = bullet.Generator.Faction;
			float scale = (SplashDamageRates.Max(_ => _.Key) / Def.MapTipLengthOnField) * 5;
			CreateEffect(faction, bullet.Position, scale);
		}
		
		public static void CreateEffect(IFaction faction, Vector3D pos, float scale)
		{
			var offset = new Vector3D(0, 0, 4);
			var effect = faction.World.EffectManager.CreateEffect(typeof(KaguyaMobHitBigEffect).FullName, pos + offset);
			effect.SetScale(new Vector3D(scale,scale,scale));
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom12).FullName, pos);
		}
	}
}
