﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

using LotusCraft.Faction;

namespace LotusCraft
{
	public class LotusCraft1v1MissionForMultiplayer : Mission
	{
		const int preparatoryPeriodTime = 60 * Def.FPS;

		const int checkHpInterval = Def.FPS;
		int checkHpCounter = 0;
		const int hpTimelimit = 10*60*Def.FPS / checkHpInterval; // 10 min
		bool isMapOpened = false;
		// HP check dic;
		Dictionary<IFaction,int> factionHPs = new Dictionary<IFaction, int>();
		Dictionary<IFaction,int> chkHpCnts = new Dictionary<IFaction, int>();
		IFaction[] keys = new IFaction[0];

		public override string[] GetSelectableFactions()
		{
			return new string[] { 
				typeof(LotusCraft.Faction.White).FullName, 
				typeof(LotusCraft.Faction.Red).FullName, 
#if !_TRIAL
				typeof(LotusCraft.Faction.Green).FullName, 
				typeof(LotusCraft.Faction.Blue).FullName, 
				typeof(LotusCraft.Faction.Black).FullName, 
#endif
			};
		}

		public override string[] GetEnableFactions()
		{
			return new string[] { 
				typeof(LotusCraft.Faction.White).FullName, 
				typeof(LotusCraft.Faction.Red).FullName, 
				typeof(LotusCraft.Faction.Green).FullName, 
#if !_TRIAL
				typeof(LotusCraft.Faction.Blue).FullName, 
				typeof(LotusCraft.Faction.Black).FullName, 
#endif
				typeof(LotusCraft.Faction.Neutral).FullName, 
			};
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] { 
				"LotusCraft/Map/1vs1_5/garden_1v1.map", 
				"LotusCraft/Map/1vs1_3/mountain_1v1.map",
#if !_TRIAL
				"LotusCraft/Map/1vs1_5/forest_1v1.map",
				"LotusCraft/Map/1vs1_5/koumakan_1v1.map",
				"LotusCraft/Map/1vs1_2/saisinomiti_1v1.map",
				"LotusCraft/Map/1vs1_5/sandou_1v1.map",
				"LotusCraft/Map/1vs1_2/tenkai_1v1.map",
				"LotusCraft/Map/1vs1_5/tikurin_1v1.map",
				"LotusCraft/Map/1vs1_5/koumakan_lobby_1v1.map",
				"LotusCraft/Map/1vs1_5/senkai_1v1.map",
#endif
			};
		}

		public override string[] GetSelectableBombs()
		{
			return new string[]{
				typeof(LotusCraft.Bomb.Reinforce).FullName,
				typeof(LotusCraft.Bomb.Fast).FullName,
				typeof(LotusCraft.Bomb.Increase).FullName,
				typeof(LotusCraft.Bomb.Attack).FullName,
				typeof(LotusCraft.Bomb.Revive).FullName,
				typeof(LotusCraft.Bomb.TownPortal).FullName,
			};
		}

		public override Dictionary<string, string> GetCOMs()
		{
			var coms = new Dictionary<string, string>();
			coms.Add(typeof(Faction.White).FullName, typeof(COM.White).FullName);
			coms.Add(typeof(Faction.Red).FullName, typeof(COM.Red).FullName);
            coms.Add(typeof(Faction.Green).FullName, typeof(COM.Green).FullName);
#if !_TRIAL
            coms.Add(typeof(Faction.Blue).FullName, typeof(COM.Blue).FullName);
            coms.Add(typeof(Faction.Black).FullName, typeof(COM.Black).FullName);
#endif
			return coms;
		}

		public LotusCraft1v1MissionForMultiplayer()
			: base()
		{
			Name = "通常対戦(1VS1)";
			Description = "1v1用の対戦モードです。";
			Mode = MissionMode.VS;

			// 準備時間を設定
			this.PreparatoryPeriodTime = preparatoryPeriodTime;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 1;
			this.TheNumberOfSelectableTeams = 2;
			this.HasMapSeen = true;

			isMapOpened = false;

		}

		public override void OnInitialize(IWorld world)
		{
			// 中立処理
			var neutrals = world.Factions.Where(_ => _.Parameter.GetType() == typeof(LotusCraft.Faction.Neutral));
			foreach (var neutral in neutrals)
			{
				neutral.IsNeutral = true;
			}

			var purposeAreas = world.AllPurposeArea;

			if (purposeAreas.Length < 1 + TheNumberOfSelectableTeams * 3)
				return;


			// bases per 3 areas
			// 1. homebase pos
			// 2. first alchol pos
			// 3. default farmer pos
			int num = purposeAreas.Count(_ => _.Name.StartsWith("HomeBase_") && !_.Name.StartsWith("HomeBase_Sub_"));

			// alloc points
			Point2D[] resPos = new Point2D[num];
			Point2D[] basePos = new Point2D[num];
			RectF[] farmerPos = new RectF[num];
			int farmerNum = 2;

			Point2D mapsize = new Point2D(world.Map.Count, world.Map.Count);

			foreach (var a in purposeAreas)
			{
				var nameElements = a.Name.Split('_');

				if (nameElements.Length > 1 && nameElements.Last().Length > 0)
				{
					var rectArea = a as AreaRectangle;
					if (rectArea == null)
						return;

					int index = 0;
					if (!int.TryParse(nameElements.Last(), out index) || index >= num)
						continue;

					if (nameElements.First().StartsWith("HomeBase") && !nameElements.Contains("Sub"))
					{
						basePos[index] = new Point2D((rectArea.Start.X + rectArea.End.X) / 2, (rectArea.Start.Y + rectArea.End.Y) / 2);
					}
					else if (nameElements.First().StartsWith("Alchol"))
					{
						resPos[index] = new Point2D((rectArea.Start.X + rectArea.End.X) / 2, (rectArea.Start.Y + rectArea.End.Y) / 2);
					}
					else if (nameElements.First().StartsWith("Farmer"))
					{
						farmerPos[index] = new RectF(Def.MapTipLengthOnField * Math.Min(rectArea.Start.X, rectArea.End.X),
													 Def.MapTipLengthOnField * Math.Min(rectArea.Start.Y, rectArea.End.Y),
													 Def.MapTipLengthOnField * Math.Abs(rectArea.End.X - rectArea.Start.X),
													 Def.MapTipLengthOnField * Math.Abs(rectArea.End.Y - rectArea.Start.Y));
					}
				}

			}

			// dry run rand
			for (int c = 0; c < 1000; ++c)
				world.NextRand();

			// shuffle pos
			Point2D tmp;
			RectF tmp_rf;
			for (int c = 0; c < 10; ++c)
			{
				int i = world.NextRand(num);
				tmp = basePos[i];
				basePos[i] = basePos[0];
				basePos[0] = tmp;

				tmp = resPos[i];
				resPos[i] = resPos[0];
				resPos[0] = tmp;

				tmp_rf = farmerPos[i];
				farmerPos[i] = farmerPos[0];
				farmerPos[0] = tmp_rf;
			}

			for (int factID = 0; factID < world.Factions.Count && factID < num; factID++)
			{
				// 初期資源を配布 200
				world.Factions[factID].IncResource(0, 200);
				farmerNum = 0;

				// set init sight
				world.Factions[factID].Parameter.DefaultScreenPosition
					= new Point2D((int)(basePos[factID].X * Def.MapTipLengthOnField), (int)(basePos[factID].Y * Def.MapTipLengthOnField));

				string homebaseName = string.Empty;
				string produceResType0Name = string.Empty;
				string farmerName = string.Empty;

				bool isPlayer = false;
				farmerNum += 5;

				if (world.Factions[factID].Parameter.GetType().FullName == typeof(White).FullName)
				{
					homebaseName = typeof(Building.White.HomeBase).FullName;
					produceResType0Name = typeof(Building.White.ProduceResType0).FullName;
					farmerName = typeof(Unit.White.Farmer).FullName;
					farmerNum += 0;
					isPlayer = true;
				}
				else if (world.Factions[factID].Parameter.GetType().FullName == typeof(Red).FullName)
				{
					homebaseName = typeof(Building.Red.HomeBase).FullName;
					produceResType0Name = typeof(Building.Red.ProduceResType0).FullName;
					farmerName = typeof(Unit.Red.Farmer).FullName;
					farmerNum += -1;
					isPlayer = true;
				}
				else if (world.Factions[factID].Parameter.GetType().FullName == typeof(Green).FullName)
				{
					homebaseName = typeof(Building.Green.HomeBase).FullName;
					produceResType0Name = typeof(Building.Green.ProduceResType0).FullName;
					farmerName = typeof(Unit.Green.Farmer).FullName;
					farmerNum += -2;
					isPlayer = true;
				}
				else if (world.Factions[factID].Parameter.GetType().FullName == typeof(Blue).FullName)
				{
					homebaseName = typeof(Building.Blue.HomeBase).FullName;
					produceResType0Name = typeof(Building.Blue.ProduceResType0).FullName;
					farmerName = typeof(Unit.Blue.Farmer).FullName;
					farmerNum += -1;
					isPlayer = true;
				}
				else if (world.Factions[factID].Parameter.GetType().FullName == typeof(Black).FullName)
				{
					homebaseName = typeof(Building.Black.HomeBase).FullName;
					produceResType0Name = typeof(Building.Black.ProduceResType0).FullName;
					farmerName = typeof(Unit.Black.Farmer).FullName;
					farmerNum += 0;
					isPlayer = true;
				}
				else
				{
					isPlayer = false;
				}

				if (isPlayer)
				{
					var homeBasePos = new Point2D(basePos[factID].X, basePos[factID].Y);
					var home = world.BuildingManager.Create(homebaseName, world.Factions[factID], homeBasePos.X, homeBasePos.Y, 0.0f, world.Factions[factID]);
					home.UserValues.Add(Building.General.BaseStructure.FirstBuildingKey,null);
					world.Factions[factID].UserValues.Add(Building.General.BaseStructure.FirstBuildingKey,home);
					home.UpdateProgress(100.0f);

					var prodResType0Pos = new Point2D(resPos[factID].X, resPos[factID].Y);
					world.BuildingManager.Create(produceResType0Name, world.Factions[factID], prodResType0Pos.X, prodResType0Pos.Y, 100.0f, world.Factions[factID]);

					if(farmerNum > 0)
					{
					var angle = 45 * (farmerPos[factID].Width > farmerPos[factID].Height ? -1 : 1);
					var d = Def.MapTipLengthOnField;
					MissionUtls.putLine_Units(farmerPos[factID].X + farmerPos[factID].Width / 2.0f,
											  farmerPos[factID].Y + farmerPos[factID].Height / 2.0f,
											  d,
											  angle,
											  farmerNum,
											  ref world,
											  factID,
											  farmerName);
					}

					// add check hp factions
					if(!factionHPs.ContainsKey(world.Factions[factID]))
						factionHPs.Add(world.Factions[factID],0);
					if(!chkHpCnts.ContainsKey(world.Factions[factID]))
						chkHpCnts.Add(world.Factions[factID],0);
				}
			}
			// hpチェックを始める為にisMapOpenedフラグを寝かせる
			keys = factionHPs.Keys.ToArray();

		}

		public override System.Collections.IEnumerable Update (IWorld world)
		{
			while (true) {
				if (!isMapOpened && ++checkHpCounter >= checkHpInterval) {
					checkHpCounter = 0;
					foreach (var f in keys) {
						if (f.State != FactionState.Normal)
							continue;
						if (!chkHpCnts.ContainsKey (f))
							continue;
						var hpSum = world.UnitManager.GetUnits (_ => null != _ && _.Exists () && _.Faction == f).Sum (__ => __.HP) 
							+ world.BuildingManager.GetBuildings (_ => null != _ && _.Exists () && _.IsComplete && _.Faction == f).Sum (__ => __.HP);

						if (factionHPs [f] == hpSum) {
							chkHpCnts [f]++;
						} else {
							factionHPs [f] = hpSum;
							chkHpCnts [f] = 0;
						}

						if (chkHpCnts [f] > hpTimelimit) {
							foreach (var ff in keys) {
								if (ff.State == FactionState.Normal && ff != f)
									world.Map.OpenAll (ff);
							}
							isMapOpened = true;
							yield return false;
							yield break;
						}
					}
				}
				yield return true;
			}

			yield return false;
			yield break;
		}
	}
}
