﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

using LotusCraft.Faction;

namespace LotusCraft
{
	public class LotusCraft2v2MissionForMultiplayer : Mission
	{
		const int preparatoryPeriodTime = 60 * Def.FPS;
		public override string[] GetSelectableFactions()
		{
			return new string[] { 
				typeof(LotusCraft.Faction.White).FullName, 
				typeof(LotusCraft.Faction.Red).FullName, 
#if !_TRIAL
				typeof(LotusCraft.Faction.Green).FullName, 
				typeof(LotusCraft.Faction.Blue).FullName, 
				typeof(LotusCraft.Faction.Black).FullName, 
#endif
			};
		}

		public override string[] GetEnableFactions()
		{
			return new string[] { 
				typeof(LotusCraft.Faction.White).FullName, 
				typeof(LotusCraft.Faction.Red).FullName, 
				typeof(LotusCraft.Faction.Green).FullName, 
#if !_TRIAL
				typeof(LotusCraft.Faction.Blue).FullName, 
				typeof(LotusCraft.Faction.Black).FullName, 
#endif
				typeof(LotusCraft.Faction.Neutral).FullName, 
			};
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] { 
				"LotusCraft/Map/2vs2_2/garden_2v2.map",
#if !_TRIAL
				"LotusCraft/Map/2vs2_2/kyuuto_2v2.map",
				"LotusCraft/Map/2vs2_2/sandou_2v2.map",
				"LotusCraft/Map/2vs2_2/senkai_2v2.map",
				"LotusCraft/Map/2vs2_2/tenkai_2v2.map",
				"LotusCraft/Map/2vs2_2/tikurin_2v2.map",
#endif
			};
		}

		public override string[] GetSelectableBombs()
		{
			return new string[]{
				typeof(LotusCraft.Bomb.Reinforce).FullName,
				typeof(LotusCraft.Bomb.Fast).FullName,
				typeof(LotusCraft.Bomb.Increase).FullName,
				typeof(LotusCraft.Bomb.Attack).FullName,
				typeof(LotusCraft.Bomb.Revive).FullName,
				typeof(LotusCraft.Bomb.TownPortal).FullName,
			};
		}

		public override Dictionary<string, string> GetCOMs()
		{
			var coms = new Dictionary<string, string>();
			coms.Add(typeof(Faction.White).FullName, typeof(COM.White).FullName);
			coms.Add(typeof(Faction.Red).FullName, typeof(COM.Red).FullName);
            coms.Add(typeof(Faction.Green).FullName, typeof(COM.Green).FullName);
#if !_TRIAL
            coms.Add(typeof(Faction.Blue).FullName, typeof(COM.Blue).FullName);
            coms.Add(typeof(Faction.Black).FullName, typeof(COM.Black).FullName);
#endif
			return coms;
		}

		public LotusCraft2v2MissionForMultiplayer()
			: base()
		{
			Name = "通常対戦(2VS2)";
			Description = "2v2用の対戦モードです。";
			Mode = MissionMode.VS;

			// 準備時間を設定
			this.PreparatoryPeriodTime = preparatoryPeriodTime;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 2;
			this.TheNumberOfSelectableTeams = 2;
			this.HasMapSeen = true;
		}

		public override void OnInitialize (IWorld world)
		{
			// 中立処理
			var neutrals = world.Factions.Where (_ => _.Parameter.GetType () == typeof(LotusCraft.Faction.Neutral));
			foreach (var neutral in neutrals) {
				neutral.IsNeutral = true;
			}

			var purposeAreas = world.AllPurposeArea;

			if (purposeAreas.Length < TheNumberOfSelectableTeams * 3)
				return;


			/// bases per 3 areas
			/// 1. homebase pos
			/// 2. first alchol pos
			/// 3. default farmer pos
			int num = purposeAreas.Count(_ => _.Name.StartsWith("HomeBase_") && !_.Name.StartsWith("HomeBase_Sub_"));

			/// alloc points
			Point2D[] resPos = new Point2D[4];
			Point2D[] basePos = new Point2D[4];
			RectF[] farmerPos = new RectF[4];
			int farmerNum = 2;

			Point2D mapsize = new Point2D(world.Map.Count, world.Map.Count);

			foreach (var a in purposeAreas) {
				var nameElements = a.Name.Split('_');
				if( a.Name == "MapEnd" )
				{
				}
				else if(nameElements.Length > 1 && nameElements.Last().Length > 0)
				{
					var rectArea = a as AreaRectangle;
					if( rectArea == null )
						return;

					int index = 0;
					if( !int.TryParse(nameElements.Last(), out index) || index >= num)
						continue;

					if (nameElements.First().StartsWith("HomeBase") && !nameElements.Contains("Sub"))
					{
						basePos[index] = new Point2D( (rectArea.Start.X + rectArea.End.X)/2 , (rectArea.Start.Y + rectArea.End.Y)/2);
					}
					else if(nameElements.First().StartsWith("Alchol"))
					{
						resPos[index] = new Point2D( (rectArea.Start.X + rectArea.End.X)/2 , (rectArea.Start.Y + rectArea.End.Y)/2);
					}
					else if(nameElements.First().StartsWith("Farmer"))
					{
						farmerPos[index] = new RectF( Def.MapTipLengthOnField*Math.Min(rectArea.Start.X , rectArea.End.X),
						                             Def.MapTipLengthOnField*Math.Min(rectArea.Start.Y , rectArea.End.Y), 
						                             Def.MapTipLengthOnField*Math.Abs(rectArea.End.X - rectArea.Start.X),
						                             Def.MapTipLengthOnField*Math.Abs(rectArea.End.Y - rectArea.Start.Y));
					}
				}

			}

			int[] rands = new int[world.Factions.Count];
			for(int i=0;i<rands.Length;++i)
			{
				int r = 0;
				do
				{
					r = world.NextRand();
				}while(rands.Contains(r));
				
				rands[i] = r;
			}
			var factions = world.Factions.OrderBy(_ => _.Team).ThenBy(f=>rands[f.Index]).ToArray();
			for(int factID = 0 ; factID < factions.Length && factID < num ; factID++)
			{
				// 初期資源を配布 800
				factions[factID].IncResource(0, 200);
				farmerNum = 0;

				// set init sight
				factions[factID].Parameter.DefaultScreenPosition
					= new Point2D((int)(basePos[factID].X*Def.MapTipLengthOnField), (int)(basePos[factID].Y*Def.MapTipLengthOnField));

				string homebaseName = string.Empty;
				string produceResType0Name = string.Empty;
				string farmerName = string.Empty;

				bool isPlayer = false;
				farmerNum += 5;

				if (factions[factID].Parameter.GetType().FullName == typeof(White).FullName)
				{
					homebaseName = typeof(Building.White.HomeBase).FullName;
					produceResType0Name = typeof(Building.White.ProduceResType0).FullName;
					farmerName = typeof(Unit.White.Farmer).FullName;
					farmerNum += 0;
					isPlayer = true;
				}
				else if (factions[factID].Parameter.GetType().FullName == typeof(Red).FullName)
				{
					homebaseName = typeof(Building.Red.HomeBase).FullName;
					produceResType0Name = typeof(Building.Red.ProduceResType0).FullName;
					farmerName = typeof(Unit.Red.Farmer).FullName;
					farmerNum += -1;
					isPlayer = true;
				}
				else if (factions[factID].Parameter.GetType().FullName == typeof(Green).FullName)
				{
					homebaseName = typeof(Building.Green.HomeBase).FullName;
					produceResType0Name = typeof(Building.Green.ProduceResType0).FullName;
					farmerName = typeof(Unit.Green.Farmer).FullName;
					farmerNum += -2;
					isPlayer = true;
				}
				else if (factions[factID].Parameter.GetType().FullName == typeof(Blue).FullName)
				{
					homebaseName = typeof(Building.Blue.HomeBase).FullName;
					produceResType0Name = typeof(Building.Blue.ProduceResType0).FullName;
					farmerName = typeof(Unit.Blue.Farmer).FullName;
					farmerNum += -1;
					isPlayer = true;
				}
				else if (factions[factID].Parameter.GetType().FullName == typeof(Black).FullName)
				{
					homebaseName = typeof(Building.Black.HomeBase).FullName;
					produceResType0Name = typeof(Building.Black.ProduceResType0).FullName;
					farmerName = typeof(Unit.Black.Farmer).FullName;
					farmerNum += 0;
					isPlayer = true;
				}
				else
				{
					isPlayer = false;
				}

				if(isPlayer)
				{
					var homeBasePos =  new Point2D(basePos[factID].X,basePos[factID].Y);
					var home = world.BuildingManager.Create (homebaseName, factions [factID], homeBasePos.X, homeBasePos.Y, 0.0f, factions [factID]);
					home.UserValues.Add(Building.General.BaseStructure.FirstBuildingKey,null);
					factions[factID].UserValues.Add(Building.General.BaseStructure.FirstBuildingKey,home);
					home.UpdateProgress(100.0f);

					var prodResType0Pos =  new Point2D(resPos[factID].X,resPos[factID].Y);
					world.BuildingManager.Create (produceResType0Name, factions [factID], prodResType0Pos.X, prodResType0Pos.Y, 100.0f, factions [factID]);

					if(farmerNum > 0)
					{
					var angle = 45*(farmerPos[factID].Width >farmerPos[factID].Height?-1:1);
					var d = Def.MapTipLengthOnField;
					MissionUtls.putLine_Units(farmerPos[factID].X + farmerPos[factID].Width / 2.0f, 
					                          farmerPos[factID].Y + farmerPos[factID].Height/2.0f, 
					                          d, 
					                          angle , 
					                          farmerNum, 
					                          ref world, 
					                          factions[factID].Index,
					                          farmerName);
					}
				}
			}

			
		}

		public override System.Collections.IEnumerable Update(IWorld world)
		{
			return base.Update(world);
		}
	}
}
